/** \file @macro.h
	  
*/

#ifndef __MACRO_H__
#define __MACRO_H__
#pragma once
////////////////////////////////////////////////////////////////////////////


///  \@macro.h
#define VERSION_MACRO   "09.09.2019"


///////////////////////////////////////////////////////////////////////
/// \defgroup gComMacro #  

///////////////////////////////////////////////////////////////////////
/** \defgroup g000   
	,     .
	\ingroup gComMacro
	@{
*/
/// \def _bit
///       \c bit.
/// \def _is_bit
///    \c bit   \c num.
/// \def _is_no_bit
///    \c bit   \c num.
/// \def _set_bit
///   \c bit   \c num.
/// \def _remove_bit
///   \c bit   \c num.
/// \def _inv_bit
///   \c bit   \c num.
/// \def _check_bit
///  \c is  \c TRUE,    \c bit   \c num,    .
#define _bit(bit)					( (DWORD)(1 << (bit)) )
#define _is_bit(num, bit)			( !!(num & _bit(bit)) )
#define _is_no_bit(num, bit)		( !(num & _bit(bit)) )
#define _set_bit(num, bit)			( num |= _bit(bit) )
#define _remove_bit(num, bit)		( num &= ~_bit(bit) )
#define _inv_bit(num, bit)			( num ^= _bit(bit) )
#define _check_bit(is, num, bit)	( num = (is) ? num | _bit(bit) : num & ~_bit(bit) )
/// @}

///////////////////////////////////////////////////////////////////////
/** \defgroup g001   
	,    .
	\ingroup gComMacro
	@{
*/
/// \def _is_flag
///    \c flag   \c num.
/// \def _is_no_flag
///    \c flag   \c num.
/// \def _set_flag
///   \c flag   \c num.
/// \def _remove_flag
///   \c flag   \c num.
/// \def _inv_flag
///   \c flag   \c num.
/// \def _check_flag
///  \c is  \c TRUE,    \c flag   \c num,    .
#define _is_flag(num, flag)			( !!(num & (flag)) )
#define _is_no_flag(num, flag)		( !(num & (flag)) )
#define _set_flag(num, flag)		( num |= (flag) )
#define _remove_flag(num, flag)		( num &= ~(flag) )
#define _inv_flag(num, flag)		( num ^= (flag) )
#define _check_flag(is, num, flag)	( num = (is) ? num | (flag) : num & ~(flag) )
/// @}

///////////////////////////////////////////////////////////////////////
/** \defgroup g002   
	   (  )   .
	\ingroup gComMacro
	@{
*/
/// \def __rand_num
///	      <code>[0, n[</code>.
/// \def __rand_range
///	      <code>[a, b]</code>.
/// \def __rand_step
///	      <code>[a+0&times;h, a+1&times;h, a+2&times;h,  ,a+(c1)&times;h]</code>.
/// \def __rand_real
///	      <code>[0.0, 1.0[</code>.
#define __rand_num(n)			( rand() % (n) )
#define __rand_range(a, b)		( rand() % ((b) - (a) + 1) + (a) )
#define __rand_step(a, b, h)	( (a) + (rand() % (b)) * (h) )
#define __rand_real()			( (double)rand() / (double)RAND_MAX )
/// @}

///////////////////////////////////////////////////////////////////////
/** \defgroup g003  
	  .
	\ingroup gComMacro
	@{
*/
/// \def _not
///	 \c x   .
/// \def _and
///	 \c x   \c AND  \c y.
/// \def _xor
///	 \c x   \c XOR  \c y.
/// \def _or
///	 \c x   \c OR  \c y.
#define _not(x)			x = !x
#define _and(x, y)		x = x && (y)
#define _xor(x, y)		x = !!(x ^ (y))
#define _or(x, y)		x = x || (y)
/// @}

///////////////////////////////////////////////////////////////////////
/** \defgroup g004   
	   .
	\ingroup gComMacro
	@{
*/
/// \def _bnot
///	 \c x   .
/// \def _band
///	 \c x   \c AND  \c y.
/// \def _bxor
///	 \c x   \c XOR  \c y.
/// \def _bor
///	 \c x   \c OR  \c y.
#define _bnot(x)		x = ~x
#define _band(x, y)		x = x & (y)
#define _bxor(x, y)		x = x ^ (y)
#define _bor(x, y)		x = x | (y)
/// @}

///////////////////////////////////////////////////////////////////////
/** \defgroup g005      
	      .
	\ingroup gComMacro
	@{
*/
/// \def max3
///	     \c x1, \c x2  \c x3.
/// \def min3
///	     \c x1, \c x2  \c x3.
#define max3(x1, x2, x3) ( max( (x1), max( (x2), (x3) )) )
#define min3(x1, x2, x3) ( min( (x1), min( (x2), (x3) )) )
/// @}

///////////////////////////////////////////////////////////////////////
/** \defgroup g006     
	     ( 2  15).
	 \c mul_N ( \c N:  2  15)   \c x    \c N.
	\ingroup gComMacro
	@{
*/
/// <code>2x</code>
#define mul_2(x)	( (x) << 1 )
/// <code>2x + x</code>
#define mul_3(x)	( ((x) << 1) + (x) )
/// <code>4x</code>
#define mul_4(x)	( (x) << 2 )
/// <code>4x + x</code>
#define mul_5(x)	( ((x) << 2) + (x) )
/// <code>4x + 2x</code>
#define mul_6(x)	( ((x) << 2) + ((x) << 1) )
/// <code>8x  x</code>
#define mul_7(x)	( ((x) << 3) - (x) )
/// <code>8x</code>
#define mul_8(x)	( (x) << 3 )
/// <code>8x + x</code>
#define mul_9(x)	( ((x) << 3) + (x) )
/// <code>8x + 2x</code>
#define mul_10(x)	( ((x) << 3) + ((x) << 1) )
/// <code>8x + 2x + x</code>
#define mul_11(x)	( ((x) << 3) + ((x) << 1) + (x) )
/// <code>8x + 4x</code>
#define mul_12(x)	( ((x) << 3) + ((x) << 2) )
/// <code>8x + 4x + x</code>
#define mul_13(x)	( ((x) << 3) + ((x) << 2) + (x) )
/// <code>16x  2x</code>
#define mul_14(x)	( ((x) << 4) - ((x) << 1) )
/// <code>16x  x</code>
#define mul_15(x)	( ((x) << 4) - (x) )
/// @}

///////////////////////////////////////////////////////////////////////
/** \defgroup g007   
	       .
	\ingroup gComMacro
	@{
*/
/// \def const01
///  \c x   <code>sqrt(2)/2</code>.
/// \def const02
///  \c x   <code>sqrt(3)/2</code>.
/// \def const03
///  \c x   <code>&pi;/180</code>.
/// \def const04
///  \c x   <code>180/&pi;</code>.
#define const01(x)	(  0.707106781186548 * (double)(x) )
#define const02(x)	(  0.866025403784439 * (double)(x) )
#define const03(x)	(  0.017453292519943 * (double)(x) )
#define const04(x)	( 57.295779513082321 * (double)(x) )
/// @}

///////////////////////////////////////////////////////////////////////
/** \defgroup g008  
	   ,    .
	\ingroup gComMacro
	@{
*/
/// \def log2
///  <code>log<sub>2</sub>(x)</code>.
/// \def pow2
///  <code>2<sup>x</sup></code>.
#define log2(x)		( log((double)x) / 0.693147180559945 )
#define pow2(x)		( 1 << (x) )
/// @}

///////////////////////////////////////////////////////////////////////
/** \defgroup g009   
	   A-      .
	\ingroup gComMacro
	@{
*/
/// \def _supp
///       \c x  A- .
/// \def _even
///   \c x  A- .
/// \def _supp4
///       \c x  4- .
/// \def _even4
///   \c x  4- .
/// \def __even
///    \c x  A-  (\c A    2).
/// \def __supp4
///        \c x  4- .
/// \def __even4
///    \c x  4- .
#define _supp(x, A)  ( ((A) - (x) % (A)) % (A) )
#define _even(x, A)  ( (x) + _supp((x), (A)) )
#define _supp4(x)    ( _supp( (x), 4) )
#define _even4(x)    ( _even( (x), 4) )
#define __even(x, A) ( ((x) + ((A) - 1)) & ~((A) - 1) )
#define __supp4(x)   ( (x) - (((x) + 3) & 0xFFFFFFFC) )
#define __even4(x)   ( ((x) + 3) & 0xFFFFFFFC )
/// @}

///////////////////////////////////////////////////////////////////////
/** \defgroup g010    
	  \c x   <code>(a, b)</code>          .
	      \c __inNM  \c N  \c M (  0  1)  : \n  (1)
	  (0)   (\c N)   (\c M)  .    (<code>a &gt; b</code>)
	   FALSE.
	 |  |  | 
	------ | -------- | ------ | -------
	__in00 | (a,b)    | .  | .
	__in10 | [a,b)    | .  | .
	__in01 | (a,b]    | .  | .
	__in11 | [a,b]    | .  | .
	\ingroup gComMacro
	@{
*/
#define __in00(x, a, b)		( ((x) >  (a)) && ((x) <  (b)) )
#define __in10(x, a, b)		( ((x) >= (a)) && ((x) <  (b)) )
#define __in01(x, a, b)		( ((x) >  (a)) && ((x) <= (b)) )
#define __in11(x, a, b)		( ((x) >= (a)) && ((x) <= (b)) )
/// @}

///////////////////////////////////////////////////////////////////////
/** \defgroup g011   
	   ,  .
	\ingroup gComMacro
	@{
*/
/// \def _array_count
///     \c array      \c TYPE.
/// \def _array_size
///     \c array.
/// \def _array_delete
///   \c array     \c array.
#define _array_count(array, TYPE)	( sizeof(array) / sizeof(TYPE) )
#define _array_size(array)			( sizeof(array) / sizeof(array[0]) )
#define _array_delete(array)		if(array) { delete [] array; array = NULL; }
/// @}

///////////////////////////////////////////////////////////////////////
/** \defgroup g012    
	     ,    .
	\ingroup gComMacro
	@{
*/
/// \def swap_byte
///      \c x.
/// \def swap_word
///      \c x.
/// \def swap_dword
///       \c x.
#define swap_byte(x)	( (((x) & 0x0F) << 4) | (((x) & 0xF0) >> 4) )
#define swap_word(x)	( (((x) & 0x00FF) << 8) | (((x) & 0xFF00) >> 8) )
#define swap_dword(x)	( (((x) & 0x000000FF) << 24) | (((x) & 0xFF000000) >> 24) | (((x) & 0x0000FF00) << 8) | (((x) & 0x00FF0000) >> 8) )
/// @}

///////////////////////////////////////////////////////////////////////
/** \defgroup g013    
	      .
	\ingroup gComMacro
	@{
*/
/// \def _lower
///    \c ch   <code>[A-Za-z]</code>   .
/// \def _upper
///    \c ch   <code>[A-Za-z]</code>   .
#define _lower(ch)	ch += 'a' - 'A'
#define _upper(ch)	ch += 'A' - 'a'
/// @}

///////////////////////////////////////////////////////////////////////
/** \defgroup g014     
	     .
	\ingroup gComMacro
	@{
*/
/// \def _is_even
///     \c x.
/// \def _is_odd
///     \c x.
#define _is_even(x)  ( !((x) & 1) )
#define _is_odd(x)   ( !_is_even(x) )
/// @}

///////////////////////////////////////////////////////////////////////
/** \defgroup g015  
	 ,     \c if   .
	   ifstrXXX     ,   {}. 
	\ingroup gComMacro
	@{
*/
/// \def ifstrcmp
///  \c str  \c cmp  .
/// \def ifstricmp
///  \c str  \c cmp     .
/// \def ifstrncmp
///  \c n  \c str  \c cmp  .
/// \def ifstrnicmp
///  \c n  \c str  \c cmp     .
#define ifstrcmp(str, cmp)		if(strcmp(str, cmp) == 0)
#define ifstricmp(str, cmp)		if(_stricmp(str, cmp) == 0)
#define ifstrncmp(str, cmp, n)	if(strncmp(str, cmp, n) == 0)
#define ifstrnicmp(str, cmp, n)	if(_strnicmp(str, cmp, n) == 0)
/// @}

///////////////////////////////////////////////////////////////////////
/** \defgroup g016   
	    ()  ,   .
	\ingroup gComMacro
	@{
*/
/// \def is_ptrr
///       \c TYPE     \c ptr.
/// \def is_ptrw
///       \c TYPE     \c ptr.
/// \def assign_ptr
///      \c ptr  \c data.
#define is_ptrr(ptr, TYPE)		( !::IsBadReadPtr(ptr, sizeof(TYPE)) )
#define is_ptrw(ptr, TYPE)		( !::IsBadWritePtr(ptr, sizeof(TYPE)) )
#define assign_ptr(pptr, data)	if(pptr) *pptr = (data)
/// @}

///////////////////////////////////////////////////////////////////////
/** \defgroup g017   naked 
	 naked ,    .
	 Visual C++    \c __LOCAL_SIZE       .
	          .
	  (\c ebp)     \c ebp.      <code>ebp2</code>.
	     ,    \c ebp     \c ebp.
	\ingroup gComMacro
	@{
*/
/// \def __naked
///  naked .
/// \def __prolog
///  naked .
/// \def __epilog
///  naked .
#define __naked __declspec(naked)

#define __prolog \
	__asm	push   ebp \
	__asm	mov    ebp, esp \
	__asm	sub    esp, __LOCAL_SIZE

#define __epilog(n) \
	__asm	mov    esp, ebp \
	__asm	pop    ebp \
	__asm	retn   n
/// @}


///////////////////////////////////////////////////////////////////////
/** \defgroup g018      
	       .
	\ingroup gComMacro
	@{
*/
/// \def to_radian
///  \c n   : <code>n*&pi;/180</code>.
/// \def to_degree
///  \c n   : <code>n*180/&pi;</code>.
#define	to_radian(n)	( 0.017453292519943 * (double)(n) )
#define	to_degree(n)	( 57.295779513082321 * (double)(n) )
/// @}


///////////////////////////////////////////////////////////////////////
/** \defgroup g019      
	  \c x   <code>(a, b)</code>   \c x ( ).
	  \c in_rotate,        ,     .
	  \c in_shift,        ,      .
	\ingroup gComMacro
	@{
*/
#define range_rotate(x, a, b)	if(x < a) x = b; if(x > b) x = a
#define range_stop(x, a, b)		if(x < a) x = a; if(x > b) x = b
/// @}





////////////////////////////////////////////////////////////////////////////
#endif // __MACRO_H__
