// __Stack.h : include file
//

///////////////////////////
//////// ECXSTATIC ////////
///////////////////////////

#ifndef ___STACK_H__
#define ___STACK_H__
#pragma once
/////////////////////////////////////////////////////////////////////////////



/////////////////////////////////////////////////////////////////////////////
//   
/////////////////////////////////////////////////////////////////////////////
enum _TypeClass
{
	TPC_SYMPLE,		//   
	TPC_STRING,		// 
	TPC_ARRAY		// 
};



/////////////////////////////////////////////////////////////////////////////
//  
/////////////////////////////////////////////////////////////////////////////
typedef struct _Item  // item
{
	_Item* next;		//  
	int type_size;		//  
	void* data;			// 

} _ITEM, *_PITEM;


/////////////////////////////////////////////////////////////////////////////
//  callback-  
/////////////////////////////////////////////////////////////////////////////
typedef void (CALLBACK *ENUMSTACKPROC) (_PITEM, void*);


//////////////////////////////////////////////////////////////////// stack //
//   stack
/////////////////////////////////////////////////////////////////////////////
template <class TYPE, int nTypeClass = TPC_SYMPLE>
class stack
{
public:

	typedef _PITEM iterator;

	stack();
	~stack();

	void push(const TYPE& data, int size = 0);
	void pop();
	bool remove(_PITEM pitem);
	void clear();
	bool find(const TYPE& data, _PITEM* ppitem);
	void enumerate(ENUMSTACKPROC pEnumStackProc, void* pParam);
	TYPE data(_PITEM pitem);

	inline TYPE top() { return data(m_StackTop); };
	inline bool empty() { return m_StackTop == NULL; };
	inline int count() { return m_nCount; };
	inline _PITEM top_item() { return m_StackTop; };

	inline int type_size() { return m_StackTop->type_size; };
	

private:

	_PITEM m_StackTop;  //  
	int m_nCount;
};



//////////////////////////////////////////////////////////////////// stack //
// 
/////////////////////////////////////////////////////////////////////////////
template <class TYPE, int nTypeClass>
stack<TYPE, nTypeClass>::stack()
{
	//  ;  
	m_StackTop = NULL;
	m_nCount = 0;
}


//////////////////////////////////////////////////////////////////// stack //
// 
/////////////////////////////////////////////////////////////////////////////
template <class TYPE, int nTypeClass>
stack<TYPE, nTypeClass>::~stack()
{
	//  
	clear();
}


//////////////////////////////////////////////////////////////////// stack //
//     
/////////////////////////////////////////////////////////////////////////////
template <class TYPE, int nTypeClass>
void stack<TYPE, nTypeClass>::push(const TYPE& data, int size)
{
	void* tmp;

	//     
	_PITEM pitem = (_PITEM)_EcxAlloc(sizeof(_ITEM));

	//  ;  
	pitem->next = m_StackTop;
	m_StackTop = pitem;

	//   
	switch(nTypeClass)
	{
		//   
		case TPC_SYMPLE:
			size = sizeof(TYPE);
			tmp = (void*)&data;
			break;

		// 
		case TPC_STRING:
			size = strlen(*(char**)&data) + 1;

		// 
		case TPC_ARRAY:
			tmp = *(void**)&data;
			break;
	}

	//  
	pitem->data = _EcxAlloc(size);
	memcpy((void*)pitem->data, tmp, size);

	//  
	m_StackTop->type_size = size;

	//   
	m_nCount++;
}


//////////////////////////////////////////////////////////////////// stack //
//     _Item.data
/////////////////////////////////////////////////////////////////////////////
template <class TYPE, int nTypeClass>
TYPE stack<TYPE, nTypeClass>::data(_PITEM pitem)
{
	//  
	if(pitem == NULL) return *(TYPE*)NULL;

	//     
	return *(TYPE*)((nTypeClass == TPC_SYMPLE) ? pitem->data : &pitem->data);
}


//////////////////////////////////////////////////////////////////// stack //
//     
/////////////////////////////////////////////////////////////////////////////
template <class TYPE, int nTypeClass>
void stack<TYPE, nTypeClass>::pop()
{
	//     
	if(m_StackTop != NULL)
	{
		//  
		_EcxFree(m_StackTop->data);

		//   
		m_nCount--;

		//  
		_PITEM pitem = m_StackTop->next;

		//   ;     
		_EcxFree(m_StackTop);
		m_StackTop = pitem;
	}
}


//////////////////////////////////////////////////////////////////// stack //
//    
/////////////////////////////////////////////////////////////////////////////
template <class TYPE, int nTypeClass>
bool stack<TYPE, nTypeClass>::find(const TYPE& data, _PITEM* ppitem)
{
	_PITEM pitem = m_StackTop;
	bool res = false;

	//     
	while(pitem)
	{
		//   ;   
		if(nTypeClass == TPC_SYMPLE)
			res = (*(TYPE*)pitem->data == data);
		else
			res = !memcmp((void*)pitem->data, *(void**)&data, pitem->type_size);

		//  
		if(res) break;

		//  
		pitem = pitem->next;
	}

	*ppitem = pitem;
	return res;
}


//////////////////////////////////////////////////////////////////// stack //
//  
/////////////////////////////////////////////////////////////////////////////
template <class TYPE, int nTypeClass>
bool stack<TYPE, nTypeClass>::remove(_PITEM pitem)
{
	_ITEM* pprev = NULL;
	_ITEM* pcurr = m_StackTop;

	//     
	while(pcurr)
	{
		//  
		if(pcurr == pitem)
		{
			//      
			if(pprev != NULL) pprev->next = pcurr->next;
			else m_StackTop = pcurr->next;

			//   
			m_nCount--;

			//   
			if(nTypeClass != TPC_SYMPLE)
				_EcxFree((void*)pcurr->data);

			//   
			_EcxFree(pcurr);
			return true;
		}

		//  
		pprev = pcurr;
		pcurr = pcurr->next;
	}

	return false;
}


//////////////////////////////////////////////////////////////////// stack //
//   
/////////////////////////////////////////////////////////////////////////////
template <class TYPE, int nTypeClass>
void stack<TYPE, nTypeClass>::enumerate(ENUMSTACKPROC pEnumStackProc, void* pParam)
{
	_PITEM pitem = m_StackTop;

	//     
	while(pitem)
	{
		//  callback-
		pEnumStackProc(pitem, pParam);
		
		//  
		pitem = pitem->next;
	}
}


//////////////////////////////////////////////////////////////////// stack //
//  
/////////////////////////////////////////////////////////////////////////////
template <class TYPE, int nTypeClass>
void stack<TYPE, nTypeClass>::clear()
{
	//     
	while(m_StackTop)
	{
		//  
		_PITEM pitem = m_StackTop->next;

		//  
		_EcxFree(m_StackTop->data);

		//   ;     
		_EcxFree(m_StackTop);
		m_StackTop = pitem;
	}
}


/////////////////////////////////////////////////////////////////////////////
#endif // ___STACK_H__
