// __EString.h : header file
//

///////////////////////////
//////// ECXSTATIC ////////
///////////////////////////

#ifndef __ESTRING_H__
#define __ESTRING_H__
#pragma once
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
//  __EString.h
/////////////////////////////////////////////////////////////////////////////
#define VERSION___ESTRING   "26.10.2020"


// LPSTR = CHAR* = char*
// LPCSTR = CONST CHAR* = const char*
// LPWSTR = WCHAR* = wchar_t* = unsigned short*
// LPCWSTR = CONST WCHAR* = const wchar_t* = const unsigned short*
// BSTR = OLECHAR* = WCHAR* = wchar_t* = unsigned short*

class EString;

extern LPCTSTR _EcxPchNil;
#define _EcxEmptyString ((EString&)*(EString*)&_EcxPchNil)


/////////////////////////////////////////////////////////////////////////////
// Globals
/////////////////////////////////////////////////////////////////////////////
DWORD _EcxHashProc(LPCSTR lpszSrc, int nMode = 0);
EString _EcxStringFromCLSID(GUID guid);


//////////////////////////////////////////////////////////// EStringDataEx //
//       EString
/////////////////////////////////////////////////////////////////////////////
typedef struct EStringDataEx  // sde
{
	int nCodePage;
	BYTE nData[32];

} ESTRINGDATAEX;


////////////////////////////////////////////////////////////// EStringData //
//       EString
/////////////////////////////////////////////////////////////////////////////
struct EStringData  // data
{
	int nStringLength;
	int nBufferLength;
	ESTRINGDATAEX sde;
};


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
#define INSERT_BEFOR	0
#define INSERT_AFTER	1
#define __str_info(v)	(LPCSTR)v.info(#v)



////////////////////////////////////////////////////////////////// EString //
//   EString : str
/////////////////////////////////////////////////////////////////////////////
class EString
{
public:

	//   
	EString();
	EString(char ch, int count = 1);
	EString(LPCSTR src);
	EString(const EString& src);
	EString(const EString* src);
	EString(int nID);

//	EString(LPCSTR src, int nLength);
//	EString(LPCWSTR src, int nLength);
//	EString(const unsigned char* src);
//	EString(int count);
//	EString(LPWSTR src);

	~EString();


	//    
	operator LPSTR();
	operator LPCSTR();

	//    
	int operator~();

	//  
	const EString& operator=(const EString& src);
	const EString& operator=(const EString* src);
	const EString& operator=(char ch);
	const EString& operator=(LPCSTR src);

	//   ()
	friend EString __stdcall operator+(const EString& str1, const EString& str2);
	friend EString __stdcall operator+(const EString& str1, char ch);
	friend EString __stdcall operator+(char ch, const EString& str2);
	friend EString __stdcall operator+(const EString& str1, LPCSTR str2);
	friend EString __stdcall operator+(LPCSTR str1, const EString& str2);

	//    
	const EString& operator+=(char ch);
	const EString& operator+=(LPCSTR src);
	const EString& operator+=(const EString& src);
	const EString& operator+=(const EString* src);

	//  
	friend EString __stdcall operator*(const EString& str, int count);
	friend EString __stdcall operator*(int count, const EString& str);

	//    
	const EString& operator*=(int count);

	//  
	friend BOOL __stdcall operator==(EString& str1, EString& str2);
	friend BOOL __stdcall operator==(EString& str1, LPCSTR str2);
	friend BOOL __stdcall operator==(LPCSTR str1, EString& str2);

	friend BOOL __stdcall operator!=(EString& str1, EString& str2);
	friend BOOL __stdcall operator!=(EString& str1, LPCSTR str2);
	friend BOOL __stdcall operator!=(LPCSTR str1, EString& str2);
	
	friend BOOL __stdcall operator<(EString& str1, EString& str2);
	friend BOOL __stdcall operator<(EString& str1, LPCSTR str2);
	friend BOOL __stdcall operator<(LPCSTR str1, EString& str2);
	
	friend BOOL __stdcall operator>(EString& str1, EString& str2);
	friend BOOL __stdcall operator>(EString& str1, LPCSTR str2);
	friend BOOL __stdcall operator>(LPCSTR str1, EString& str2);
	
	friend BOOL __stdcall operator<=(EString& str1, EString& str2);
	friend BOOL __stdcall operator<=(EString& str1, LPCSTR str2);
	friend BOOL __stdcall operator<=(LPCSTR str1, EString& str2);
	
	friend BOOL __stdcall operator>=(EString& str1, EString& str2);
	friend BOOL __stdcall operator>=(EString& str1, LPCSTR str2);
	friend BOOL __stdcall operator>=(LPCSTR str1, EString& str2);

	int Compare(LPCSTR src);
	int Compare(LPCSTR src, int nCount);
	int CompareNoCase(LPCSTR src);
	int CompareNoCase(LPCSTR src, int nCount);
	int CompareCase(LPCSTR src, BOOL isCase);

	int Collate(LPCSTR src);
	int CollateNoCase(LPCSTR src);


	//      
	int& operator[](int nIndex);
	const EString operator()(int nIndex1, int nIndex2);


	EString* SetString(LPCSTR src, int nNull = 0);
	EString* Concat(LPCSTR str1, ...);

	EString Left(int nCount);
	EString Right(int nCount);
	EString Mid(int nIndex, int nCount);
	EString Mid(int nIndex);
	EString LeftTo(char chFind, int nFindNum = 1);
//	EString RightTo(char chFind, int nFindNum = 1);
	EString* _Left(int nCount);
	EString* _Right(int nCount);
	EString* _Mid(int nIndex, int nCount);
	EString* _Mid(int nIndex);

	void Format(LPCSTR lpszFormat, ...);
	void Format(int nFormatID, ...);
	void FormatV(LPCSTR lpszFormat, va_list argList);

	int Scan(LPCSTR lpszFormat, ...);
	int Scan(int nFormatID, ...);
	int ScanV(LPCSTR lpszFormat, va_list argList);

	int LoadString(int nID);

	EString* MakeUpper();
	EString* MakeLower();
	EString* MakeReverse();

	EString* AnsiToOem();
	EString* OemToAnsi();

	int& GetAt(int nIndex);
	void SetAt(int nIndex, char ch);
	void SetAt(int nIndex, wchar_t ch);

	int Find(char ch, int nIndex = 0);
	int FindRev(char ch);
	int FindClass(DWORD dwMask, int nIndex = 0);
	int FindClassRev(DWORD dwMask);
	int Find(LPCSTR str, int nIndex = 0);
	int FindRev(LPCSTR str);
	int FindOneOf(LPCSTR lpszCharSet);
	

	EString* Delete(int nIndex, int nCount);
	
	EString* Empty();
	BOOL IsEmpty();
	
	EString* Insert(int nMode, int nIndex, char ch);
	EString* Insert(int nMode, int nIndex, LPCSTR src);
	
	EString* Remove(char ch);
	EString* Remove(LPCSTR lpszTargets);
	EString* RemoveClass(DWORD dwMask);

	int Replace(char chOld, char chNew, int nMaxCount = -1);
	int Replace(LPCSTR lpszOld, LPCSTR lpszNew, int nMaxCount = -1);
	int Replace(LPCSTR lpszOld, int nNew);
	int Replace(LPCSTR lpszOld, double dNew, int nPrecision = 2);
	int ReplaceClass(DWORD dwMask, char chNew, int nMaxCount = -1);
	int ReplaceSalt(LPCSTR lpszOld, LPCSTR lpszNew, int nSalt);

	int ExpandTabs(int nTabs = 4);
	int Translate(LPCSTR lpszTable1, LPCSTR lpszTable2);

	int Count(char ch);
	int Count(LPCSTR lpsz);
	
	EString SpanIncluding(LPCSTR lpszCharSet);
	EString SpanExcluding(LPCSTR lpszCharSet);

	BOOL Read(LPCSTR lpszFileName);
	BOOL Write(LPCSTR lpszFileName);


	void* ToBuffer(int count);
	EString* ToStrW();
	void Release(int nLength);

	char GetLastChar();
	int GetLength();
	void FormatChar(BOOL isUpper, int nOffset = 0);

	//  
	void TrimLeft(LPCSTR lpszTargets);
	void TrimLeft(char chTarget);
	void TrimLeft();
	void TrimRight(LPCSTR lpszTargets);
	void TrimRight(char chTarget);
	void TrimRight();
	void Trim(LPCSTR lpszTargets);
	void Trim(char chTarget);
	void Trim();
//	void TrimLeftTo(char chFind, BOOL isDeleteFind = TRUE, int nFindNum = 1)
//	void TrimLeftTo(LPCSTR lpszFind, BOOL isDeleteFind = TRUE, int nFindNum = 1)

	//   
	void Quote(BOOL isQuote, LPCSTR lpszQt1, LPCSTR lpszQt2);
	void Quote(BOOL isQuote, char chQt1, char chQt2);
	void Quote(BOOL isQuote = TRUE);

	//     
	BOOL IsPrefix(LPCSTR lpszPrefix, BOOL isCase = FALSE);
	BOOL IsSuffix(LPCSTR lpszSuffix, BOOL isCase = FALSE);
	void AddPrefix(LPCSTR lpszPrefix);
	void AddSuffix(LPCSTR lpszSuffix);
	BOOL DeletePrefix(LPCSTR lpszPrefix, BOOL isCase = TRUE);
	BOOL DeleteSuffix(LPCSTR lpszSuffix, BOOL isCase = TRUE);

//	BSTR AllocSysString() const;
//	BSTR SetSysString(BSTR* pbstr) const;


	DWORD HashProc();


public:

	char* m_pBuffer;

	BOOL ic_is_invalid_index(int nIndex);
	BOOL ic_normalize_index(int& nIndex);
	void ic_normalize_index_count(int& nIndex, int& nCount);
	void ic_normalize_count(int& nCount);

	int str_len(void* src);
	void str_set(int ch, int count);
	void str_set(void* src, int count);
	void str_cpy(void* src);
	void str_ncpy(void* src, int length);
	EString* str_cat(void* src);
	void str_ncat(void* src, int length);
	void str_release(int length);

	EString info(char* var);
	EStringData* GetData(char* pBuf = NULL) const;

protected:
	BOOL mem_alloc(int nLength, BOOL isFree = TRUE);
	BOOL mem_append(int nLength);
	BOOL mem_change(int nLength);
	BOOL mem_copy(int nOffset, const void* pSrc, int nLength);
	void mem_cpy(void* pBuf, int nOffset, const void* pSrc, int nSrcOffset, int nLength);
	void mem_free(char* pBuf);
	void data_copy(char* pSrc);

	void* get_buffer_offset(int offset);

	
};

template <class T> inline int tmpRemove(T ch, T** ppBuf)
{
	int count = 0;
	T *tmp0 = *ppBuf, *tmp1 = *ppBuf;
	while(*tmp0)
	{
		if(*tmp0 == ch)
		{
			tmp0++;
			continue;
		}
		count++;
		*tmp1 = *tmp0;
		tmp0++;
		tmp1++;
	}
	return count;
}
template<class T> inline int tmpRemove(T* pTargets, T** ppBuf)
{
	int count = 0;
	T *tmp0 = *ppBuf, *tmp1 = *ppBuf;
	const char* rw = typeid(T).raw_name();
	while(*tmp0)
	{
		if((rw[1] == 'D') ? (void*)strchr((char*)pTargets, (int)*tmp0) : (void*)wcschr((wchar_t*)pTargets, (wint_t)*tmp0))
		{
			tmp0++;
			continue;
		}
		count++;
		*tmp1 = *tmp0;
		tmp0++;
		tmp1++;
	}
	return count;
}



////////////////////////////////////////////////////////////////// EString //
// Inlines
/////////////////////////////////////////////////////////////////////////////
inline EStringData* EString::GetData(char* pBuf) const
	{ return (pBuf == NULL) ? (EStringData*)m_pBuffer - 1 : (EStringData*)pBuf - 1; }

inline EString::operator LPSTR()
	{ return m_pBuffer; }
inline EString::operator LPCSTR()
	{ return m_pBuffer; }
inline int EString::operator~()
	{ return GetData()->nStringLength; }
inline void EString::Release(int nLength)
	{ str_release(nLength); }

// ==
inline BOOL __stdcall operator==(EString& str1, EString& str2)
	{ return str1.Compare(str2.m_pBuffer) == 0; }
inline BOOL __stdcall operator==(EString& str1, LPCSTR str2)
	{ return str1.Compare(str2) == 0; }
inline BOOL __stdcall operator==(LPCSTR str1, EString& str2)
	{ return str2.Compare(str1) == 0; }
// !=
inline BOOL __stdcall operator!=(EString& str1, EString& str2)
	{ return str1.Compare(str2.m_pBuffer) != 0; }
inline BOOL __stdcall operator!=(EString& str1, LPCSTR str2)
	{ return str1.Compare(str2) != 0; }
inline BOOL __stdcall operator!=(LPCSTR str1, EString& str2)
	{ return str2.Compare(str1) != 0; }
// <
inline BOOL __stdcall operator<(EString& str1, EString& str2)
	{ return str1.Compare(str2.m_pBuffer) < 0; }
inline BOOL __stdcall operator<(EString& str1, LPCSTR str2)
	{ return str1.Compare(str2) < 0; }
inline BOOL __stdcall operator<(LPCSTR str1, EString& str2)
	{ return str2.Compare(str1) > 0; }
// >
inline BOOL __stdcall operator>(EString& str1, EString& str2)
	{ return str1.Compare(str2.m_pBuffer) > 0; }
inline BOOL __stdcall operator>(EString& str1, LPCSTR str2)
	{ return str1.Compare(str2) > 0; }
inline BOOL __stdcall operator>(LPCSTR str1, EString& str2)
	{ return str2.Compare(str1) < 0; }
// <=
inline BOOL __stdcall operator<=(EString& str1, EString& str2)
	{ return str1.Compare(str2.m_pBuffer) <= 0; }
inline BOOL __stdcall operator<=(EString& str1, LPCSTR str2)
	{ return str1.Compare(str2) <= 0; }
inline BOOL __stdcall operator<=(LPCSTR str1, EString& str2)
	{ return str2.Compare(str1) >= 0; }
// >=
inline BOOL __stdcall operator>=(EString& str1, EString& str2)
	{ return str1.Compare(str2.m_pBuffer) >= 0; }
inline BOOL __stdcall operator>=(EString& str1, LPCSTR str2)
	{ return str1.Compare(str2) >= 0; }
inline BOOL __stdcall operator>=(LPCSTR str1, EString& str2)
	{ return str2.Compare(str1) <= 0; }

inline EString* EString::MakeUpper()
	{ ::CharUpper(m_pBuffer); return this; }
inline EString* EString::MakeLower()
	{ ::CharLower(m_pBuffer); return this; }
inline EString* EString::MakeReverse()
	{ _strrev(m_pBuffer); return this; }
inline EString* EString::AnsiToOem()
	{ ::CharToOem(m_pBuffer, m_pBuffer); return this; }
inline EString* EString::OemToAnsi()
	{ ::OemToChar(m_pBuffer, m_pBuffer); return this; }
inline int& EString::GetAt(int nIndex)
	{ return (int&)m_pBuffer[nIndex]; }
inline void EString::SetAt(int nIndex, char ch)
	{ m_pBuffer[nIndex] = ch; }

inline int EString::Compare(LPCSTR src)
	{ return strcmp(m_pBuffer, src); }
inline int EString::Compare(LPCSTR src, int nCount)
	{ return strncmp(m_pBuffer, src, nCount); }

inline int EString::CompareNoCase(LPCSTR src)
	{ return _stricmp(m_pBuffer, src); }
inline int EString::CompareNoCase(LPCSTR src, int nCount)
	{ return _strnicmp(m_pBuffer, src, nCount); }

inline int EString::Collate(LPCSTR src)
	{ return strcoll(m_pBuffer, src); }
inline int EString::CollateNoCase(LPCSTR src)
	{ return _stricoll(m_pBuffer, src); }

inline BOOL EString::IsEmpty()
	{ return GetData()->nStringLength == 0; }
inline EString EString::SpanIncluding(LPCSTR lpszCharSet)
	{ return Left(strspn(m_pBuffer, lpszCharSet)); }
inline EString EString::SpanExcluding(LPCSTR lpszCharSet)
	{ return Left(strcspn(m_pBuffer, lpszCharSet)); }
inline void EString::TrimLeft()
	{ TrimLeft(" "); }
inline void EString::TrimRight()
	{ TrimRight(" "); }
inline DWORD EString::HashProc()
	{ return _EcxHashProc(m_pBuffer); }





/////////////////////////////////////////////////////////////////////////////
#endif // __ESTRING_H__
