// EArray.h : header file
//

#ifndef __EARRAY_H__
#define __EARRAY_H__
#pragma once
/////////////////////////////////////////////////////////////////////////////


#include "EMemory.h"

/////////////////////////////////////////////////////////////////////////////
//  EArray.h
/////////////////////////////////////////////////////////////////////////////
#define VERSION_EARRAY   "23.09.2013"



/////////////////////////////////////////////////////////////////////////////
//   EArray
/////////////////////////////////////////////////////////////////////////////
template <class TYPE>
class EArray : public EMemory
{
public:

	EArray();
	EArray(EArray& src);
	EArray(int nCount, TYPE* ptData);

	int GetCount() { return m_nCount; };
	int GetTypeSize() { return sizeof(TYPE); };
	int GetSize() { return GetData()->nArrayLength; };

	TYPE* Add(const TYPE& tData);
	int Append(int nCount, TYPE* ptData);
	int Append(EArray& src);
	void Copy(EArray& src);

	TYPE GetAt(int nIndex);               
	void SetAt(int nIndex, const TYPE& tData);
	TYPE& GetItem(int nIndex);
	TYPE* GetItemPtr(int nIndex);
	TYPE& operator[](int nIndex);

	void Insert(int nIndex, const TYPE& tData, int nCount = 1);
	void Insert(int nIndex, EArray* pArray);
	void Remove(int nIndex, int nCount = 1);
	void Swap(int nIndex1, int nIndex2);

	void SetCount(int nCount);



	void RemoveAll();



//private:

	int _corr_index(int nIndex);

	int m_nCount;
	int m_nCurrIndex, m_nIndex[2];
};





/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
template <class TYPE>
EArray<TYPE>::EArray()
{
	RemoveAll();
}

template <class TYPE>
EArray<TYPE>::EArray(EArray& src)
{
	RemoveAll();
	Append(src);
}

template <class TYPE>
EArray<TYPE>::EArray(int nCount, TYPE* ptData)
{
	RemoveAll();
	Append(nCount, ptData);
}


/////////////////////////////////////////////////////////////////////////////
//  
/////////////////////////////////////////////////////////////////////////////
template <class TYPE>
TYPE* EArray<TYPE>::Add(const TYPE& tData)
{
	MemAppend(sizeof(TYPE));
	*((TYPE*)m_pBuffer + m_nCount) = tData;
	m_nCurrIndex = m_nCount;
	m_nCount++;
	return (TYPE*)m_pBuffer + m_nCurrIndex;
}


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
template <class TYPE>
int EArray<TYPE>::Append(int nCount, TYPE* ptData)
{
	if(nCount == 0) return -1;

	int nArrayLength = nCount * sizeof(TYPE);
	MemAppend(nArrayLength);
	memcpy((TYPE*)m_pBuffer + m_nCount, ptData, nArrayLength);

	DWORD dwCount = m_nCount;
	m_nCount += nCount;

	return dwCount + 1;
}


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
template <class TYPE>
int EArray<TYPE>::Append(EArray& src)
{
	if(src.m_nCount == 0) return -1;

	MemAppend(src.GetData()->nArrayLength);
	memcpy((TYPE*)m_pBuffer + m_nCount, src.m_pBuffer, src.GetData()->nArrayLength);

	DWORD dwCount = m_nCount;
	m_nCount += src.m_nCount;

	return dwCount + 1;
}


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
template <class TYPE>
void EArray<TYPE>::Copy(EArray& src)
{
	if(src.m_nCount == 0) return;

	MemAlloc(src.GetData()->nArrayLength);
	memcpy(m_pBuffer, src.m_pBuffer, src.GetData()->nArrayLength);

	m_nCount = src.m_nCount;
}


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
template <class TYPE>
TYPE EArray<TYPE>::GetAt(int nIndex)
{
	m_nCurrIndex = _corr_index(nIndex);
	return *((TYPE*)m_pBuffer + m_nCurrIndex);
}


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
template <class TYPE>
void EArray<TYPE>::SetAt(int nIndex, const TYPE& tData)
{
	m_nCurrIndex = _corr_index(nIndex);
	*((TYPE*)m_pBuffer + m_nCurrIndex) = tData;
}


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
template<class TYPE>                          
TYPE& EArray<TYPE>::GetItem(int nIndex)
{
	m_nCurrIndex = _corr_index(nIndex);
	return (TYPE&)(*((TYPE*)m_pBuffer + m_nCurrIndex));
}                     


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
template<class TYPE>                          
TYPE* EArray<TYPE>::GetItemPtr(int nIndex)
{
	m_nCurrIndex = _corr_index(nIndex);
	return (TYPE*)m_pBuffer + m_nCurrIndex;
}

/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
template<class TYPE>
TYPE& EArray<TYPE>::operator[](int nIndex)
{
	return GetItem(nIndex);
}


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
template<class TYPE>
void EArray<TYPE>::Insert(int nIndex, const TYPE& tData, int nCount)
{
	if(nCount == 0) return;

	m_nCurrIndex = _corr_index(nIndex);

	MemAppend(nCount * sizeof(TYPE));
	memmove((TYPE*)m_pBuffer + m_nCurrIndex + nCount, (TYPE*)m_pBuffer + m_nCurrIndex, (m_nCount - m_nCurrIndex) * sizeof(TYPE));

	for(int i=0; i<nCount; i++)
		*((TYPE*)m_pBuffer + m_nCurrIndex + i) = tData;
	
	m_nCount += nCount;
}


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
template<class TYPE>
void EArray<TYPE>::Insert(int nIndex, EArray* pArray)
{
	if(pArray->m_nCount == 0) return;

	m_nCurrIndex = _corr_index(nIndex);

	MemAppend(pArray->GetData()->nArrayLength);
	memmove((TYPE*)m_pBuffer + m_nCurrIndex + pArray->m_nCount, (TYPE*)m_pBuffer + m_nCurrIndex, (m_nCount - m_nCurrIndex) * sizeof(TYPE));
	memcpy((TYPE*)m_pBuffer + m_nCurrIndex, pArray->m_pBuffer, pArray->GetData()->nArrayLength);

	m_nCount += pArray->m_nCount;
}


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
template<class TYPE>
void EArray<TYPE>::Remove(int nIndex, int nCount)
{
	if(nCount == 0) return;

	m_nCurrIndex = _corr_index(nIndex);
	memmove((TYPE*)m_pBuffer + m_nCurrIndex, (TYPE*)m_pBuffer + m_nCurrIndex + nCount, (m_nCount - m_nCurrIndex - nCount) * sizeof(TYPE));

	SetCount(m_nCount - nCount);
}


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
template<class TYPE>
void EArray<TYPE>::Swap(int nIndex1, int nIndex2)
{
	TYPE tData = *((TYPE*)m_pBuffer + nIndex1);
	*((TYPE*)m_pBuffer + nIndex1) = *((TYPE*)m_pBuffer + nIndex2);
	*((TYPE*)m_pBuffer + nIndex2) = tData;
}


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
template <class TYPE>
void EArray<TYPE>::SetCount(int nCount)
{
	m_nCount = nCount;
	MemChange(nCount * sizeof(TYPE));
}


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
template <class TYPE>
void EArray<TYPE>::RemoveAll()
{
	MemFree();

	m_nCount = 0;
	MemAlloc(0);
}


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
template <class TYPE>
int EArray<TYPE>::_corr_index(int nIndex)
{
	if(nIndex < 0) nIndex = 0;
	if(nIndex >= m_nCount) nIndex = m_nCount - 1;
	return nIndex;
}






/////////////////////////////////////////////////////////////////////////
#endif // __EARRAY_H__