// EJson.h : header file
//

#ifndef __EJSON_H__
#define __EJSON_H__
#pragma once



#include "EJsonPath.h"


#ifdef USED_JSON
/////////////////////////////////////////////////////////////////////////////
//  EJson.h
/////////////////////////////////////////////////////////////////////////////
#define VERSION_EJSON   "05.12.2016"










// http://json.org

/*

json = object | array

object = begin-object [ member (value-separator member)* ] end-object
array = begin-array [ value (value-separator value)* ] end-array

member = string name-separator value
value = false | null | true | object | array | number | string

string = quotation-mark char* quotation-mark

char = unescaped | escape
escape = %x5C (%x22 | %x5C | %x2F | %x62 | %x66 | %x6E | %x72 | %x74 | %x75 hexdig{1,4})
quotation-mark = %x22
unescaped = %x20-21 | %x23-5B | %x5D-10FFFF

space = (%x20 | %x09 | %x0A | %x0D)*

name-separator  = space %x3A space   ; : colon
value-separator = space %x2C space   ; , comma

false = %x66.61.6C.73.65   ; false
null  = %x6E.75.6C.6C      ; null
true  = %x74.72.75.65      ; true

begin-object    = space %x7B space   ; { left curly bracket
end-object      = space %x7D space   ; } right curly bracket

begin-array     = space %x5B space   ; [ left square bracket
end-array       = space %x5D space   ; ] right square bracket

number = [ minus ] int [ frac ] [ exp ]

decimal-point = '.'
e = 'e' | 'E'
exp = e (minus | plus)* digit{1}
frac = decimal-point digit{1}
int = zero | ( digit digit{1,9} )

minus = '-'
plus = '+'
zero = '0'

hexdig = [0-9a-fA-F]
digit = [0-9]


*/






/////////////////////////////////////////////////////////////////////////////
//   (JSON_ - JavaScript Object Notation)
/////////////////////////////////////////////////////////////////////////////
enum _JsonEntryType
{
	JSON_UNKNOWN,	//
	JSON_OBJECT,	//  n:{}  {}
	JSON_ARRAY,		//  n:[]  []
	JSON_MEMBER,	//  n:d
	JSON_VALUE	,	//  d
	JSON_SECTION,	//  JSON_OBJECT | JSON_ARRAY (   IsType)
	JSON_ROOT,		//
};

static char* _JsonEntryTypeStr[] =
{
	"JSON_UNKNOWN",
	"JSON_OBJECT",
	"JSON_ARRAY",
	"JSON_MEMBER",
	"JSON_VALUE",
	"",
	"",
};


/////////////////////////////////////////////////////////////////////////////
//  
/////////////////////////////////////////////////////////////////////////////
enum _JsonValueType
{
	JSON_VALUE_EMPTY,	//
	JSON_VALUE_OBJECT,	//  {}
	JSON_VALUE_ARRAY,	//  []
	JSON_VALUE_NULL,	// null
	JSON_VALUE_STRING,	// "..."
	JSON_VALUE_INTEGER,	//
	JSON_VALUE_DOUBLE,	//
	JSON_VALUE_BOOLEAN,	// true | false
	JSON_VALUE_UNKNOWN,	//
};

static char* _JsonValueTypeStr[] =
{
	"JSON_VALUE_EMPTY",
	"JSON_VALUE_OBJECT",
	"JSON_VALUE_ARRAY",
	"JSON_VALUE_NULL",
	"JSON_VALUE_STRING",
	"JSON_VALUE_INTEGER",
	"JSON_VALUE_DOUBLE",
	"JSON_VALUE_BOOLEAN",
	"JSON_VALUE_UNKNOWN",
};






////////////////////////////////////////////////////////////////////////////
//     
////////////////////////////////////////////////////////////////////////////
#define FIELD_NAME	0
#define FIELD_DATA	1


////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////




////////////////////////////////////////////////////////////////////////////
//    (PARSING_)
////////////////////////////////////////////////////////////////////////////
#define PARSING_TEXT		0	// P0
#define PARSING_OBJECT		1	// P1
#define PARSING_ARRAY		2	// P2

////////////////////////////////////////////////////////////////////////////
//    (WAITING_)
////////////////////////////////////////////////////////////////////////////
#define WAITING_EOT			0	// W0
#define WAITING_BOS			1	// W1
#define WAITING_NAME		2	// W2
#define WAITING_COLON		3	// W3
#define WAITING_COMMA		4	// W4
#define WAITING_VALUE		5	// W5
#define WAITING_NONE		6	// W6

////////////////////////////////////////////////////////////////////////////
//   (ACTION_)
////////////////////////////////////////////////////////////////////////////
#define ACTION_NONE			0	// A00
#define ACTION_GET_SECTION	1	// A01
#define ACTION_EOS_EXPLOR	2	// A02
#define ACTION_GET_VALUE	3	// A03
//		ACTION_NEW_NONE		4	// A10
#define ACTION_NEW_SECTION	5	// A13
#define ACTION_NEW_MEMBER	6	// A11
#define ACTION_NEW_VALUE	7	// A12

////////////////////////////////////////////////////////////////////////////
//   (JSON_CHAR_)
////////////////////////////////////////////////////////////////////////////
#define JSON_CHAR_UNKNOWN	0	// C0
#define JSON_CHAR_BOS		1	// C1
#define JSON_CHAR_EOS		2	// C2
#define JSON_CHAR_COLON		3	// C3
#define JSON_CHAR_COMMA		4	// C4
#define JSON_CHAR_QUOTE		5	// C5

////////////////////////////////////////////////////////////////////////////
//    (READ_)
////////////////////////////////////////////////////////////////////////////
#define READ_NAME			0
#define READ_DATA			1







////////////////////////////////////////////////////////////////////////////
//   nMode  InsertEntry
////////////////////////////////////////////////////////////////////////////
#define INSERT_BEFOR	0
#define INSERT_AFTER	1


////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////
#define JSON_MAX_LENGTH		MAX_PATH * 100




/////////////////////////////////////////////////////////////////////////////
//     
/////////////////////////////////////////////////////////////////////////////
struct _JsonEntry;


/////////////////////////////////////////////////////////////////////////////
//   
/////////////////////////////////////////////////////////////////////////////
typedef struct _JsonChild
{
	_JsonEntry* first;	//   
	_JsonEntry* last;	//   
	int nCount;			//    
	int nNode;			//   

} JSONCHILD;



/////////////////////////////////////////////////////////////////////////////
//   
/////////////////////////////////////////////////////////////////////////////
typedef struct _JsonEntry  // je
{
	int nType;				//  
	int nLevel;				//  

	_JsonEntry* prev;		//  
	_JsonEntry* next;		//  
	_JsonEntry* parent;		//   

	JSONCHILD child;		//   
	_JsonEntry** hash;		//   


	char* szEntryName;		//  
	char* szEntryData;		//  
	int nNameType;
	int nDataType;
	int nNameLength;		//   
	int nDataLength;		//   


	//  / 
	_JsonEntry()
	{
		memset(this, 0, sizeof(_JsonEntry));
	}

	~_JsonEntry()
	{
		if(szEntryName != NULL) delete [] szEntryName;
		if(szEntryData != NULL) delete [] szEntryData;
		if(hash != NULL) delete [] hash;
	}



	//    
	BOOL IsRoot()
		{ return parent == NULL; }

	//    
	BOOL IsChild()
		{ return parent != NULL; }

	//     
	BOOL IsChildFirst()
		{ return (parent != NULL) && (prev == NULL); }

	//     
	BOOL IsChildLast()
		{ return (parent != NULL) && (next == NULL); }

	//   
	BOOL IsNode()
		{ return (child.first != NULL); }

	//      lpszName
	BOOL IsName(LPTSTR lpszName)
		{ return (_EcxStrCompare(szEntryName, lpszName) == 0) || (IsRoot() && strcmp(lpszName, "root") == 0); }

	//      nCmpType
	BOOL IsType(int nCmpType)
	{
		BOOL res = (nType == nCmpType);
		if(nCmpType == JSON_SECTION) res = (nType == JSON_OBJECT) || (nType == JSON_ARRAY);
		return res;
	}

	//
	int GetEntryType()
	{
		if(parent == NULL) return JSON_ROOT;
		else
		{
			if(parent->nType == JSON_OBJECT) return JSON_MEMBER;
			if(parent->nType == JSON_ARRAY) return JSON_VALUE;
		}
		return JSON_UNKNOWN;
	}

	//      nItem
	_JsonEntry* GetArrayItem(int nItem)
	{
		if(IsType(JSON_ARRAY))
		{
			if(__in10(nItem, 0, child.nCount))
			{
				return hash[nItem];
			}
		}
		return NULL;
	}




	//     (JSON_)
	EString GetEntryTypeStr(BOOL isShort)
	{
		EString s1 = _JsonEntryTypeStr[nType];
		if(isShort) s1 = s1.Mid(5);
		return s1;
	}

	//     (JSON_TYPE_)
	EString GetDataTypeStr(int nField, BOOL isShort)
	{
		EString s1 = _JsonValueTypeStr[(nField == FIELD_NAME) ? nNameType : nDataType];
		if(isShort) s1 = s1.Mid(11);
		return s1;
	}

	//        : ENTRYTYPE (NAMETYPE, DATATYPE)
	EString GetEntryTypeDataStr(BOOL isShort = TRUE)
	{
		EString s1;
		s1.Format("%s (%s, %s)", GetEntryTypeStr(isShort), GetDataTypeStr(FIELD_NAME, isShort), GetDataTypeStr(FIELD_DATA, isShort));
		return s1;
	}

	//
	EString GetEntryNameDataStr(int nField, int nMode = 0)
	{
		EString str;
		char* sz = (nField == FIELD_NAME) ? szEntryName : szEntryData;
		int type = (nField == FIELD_NAME) ? nNameType : nDataType;
		if(sz == NULL)
		{
			str = (nMode) ? ((IsRoot()) ? "(root)" : "(empty)") : "NULL";
		}
		else
		{
			str = sz;
			if(type == JSON_VALUE_STRING) str.Quote();
			if(type == JSON_VALUE_NULL) str = "null";
			if(type == JSON_VALUE_EMPTY) str = (nType == JSON_VALUE) ? "(undefined)" : "(empty ++)";
		}
		return str;
	}

	//
	EString GetPropertyStr()
	{
		EString str;
		EString s1 = GetEntryNameDataStr(FIELD_NAME);
		EString s2 = GetEntryNameDataStr(FIELD_DATA);
		if(nType == JSON_MEMBER)
			str.Format("%s: %s", s1, s2);
		else
			str = s2;
		return str;
	}

	//
	static EString GetEntryPtrStr(_JsonEntry* pJe)
	{
		EString str = "NULL";
		if(pJe != NULL)
		{
			str.Format("0x%.8X:  %s", (DWORD)pJe, pJe->GetEntryStr(1));
		}
		return str;
	}

	//
	EString GetEntryStr(int nMode)
	{
		EString str;
		EString strEntryName = GetEntryNameDataStr(FIELD_NAME, nMode);
		EString strEntryData = GetEntryNameDataStr(FIELD_DATA, nMode);

		switch(nMode)
		{
			//     
			case 0:
			{
				EString p1, p2, p3, p4, p5, p6, br = "\r\n";
				EString a1, a2, a3, a4, a5, a6, a7, a8, n1, n2;
				EString arr, sa;

				p1.Format("0x%.8X\r\n", this);

				p2.Format("prev         = %s\r\n", GetEntryPtrStr(prev));
				p3.Format("next         = %s\r\n", GetEntryPtrStr(next));
				p4.Format("parent       = %s\r\n", GetEntryPtrStr(parent));
				p5.Format("child.first  = %s\r\n", GetEntryPtrStr(child.first));
				p6.Format("child.last   = %s\r\n", GetEntryPtrStr(child.last));
				a3.Format("child.nCount = %d\r\n", child.nCount);
				a4.Format("child.nNode  = %d\r\n", child.nNode);

				if(child.nCount > 0)
				{
					arr = "hash:\r\n";
					int i = 0;
					JSONENTRY* tmp = child.first;
					do
					{
						sa.Format("%3d) %s\r\n", i, GetEntryPtrStr(hash[i]));
						arr += sa;
						i++;
						tmp = tmp->next;

					} while(tmp);
				}
				else
					arr = "hash         = NULL\r\n";

				a1.Format("nType        = %s\r\n", GetEntryTypeStr(FALSE));
				a2.Format("nLevel       = %d\r\n", nLevel);

				a5.Format("szEntryName  = %s\r\n", strEntryName);
				a6.Format("szEntryData  = %s\r\n", strEntryData);
				a7.Format("nNameType    = %d (%s)\r\n", nNameType, GetDataTypeStr(FIELD_NAME, FALSE));
				a8.Format("nDataType    = %d (%s)\r\n", nDataType, GetDataTypeStr(FIELD_DATA, FALSE));
				n1.Format("nNameLength  = %d\r\n", nNameLength);
				n2.Format("nDataLength  = %d\r\n", nDataLength);

				EString b1, b2;
				b1.Format("%s:  ", GetEntryTypeDataStr(TRUE));
				b2.Format("%s\r\n\r\n", GetEntryStr(1));

				str = p1 + b1 + b2 + a1 + a2 + br + p2 + p3 + p4 + p5 + p6 + a3 + a4 + arr + br + a5 + a6 + a7 + a8 + n1 + n2;
				break;
			}

			//  
			case 1:
			{
				EString strEqual = ": ";

				switch(nType)
				{
					case JSON_MEMBER:
						str.Format("%s: %s", strEntryName, strEntryData);
						break;
					case JSON_VALUE:
						str.Format("%s", strEntryData);
						break;

					case JSON_OBJECT:
					case JSON_ARRAY:
						if(szEntryData == NULL)
							str.Format("%s", strEntryName);
						else
							str.Format("%s %s%s", strEntryName, strEqual, strEntryData);
						break;
				}
				break;
			}
		}
		return str;
	}


} JSONENTRY;



////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////
#define EnumArrayItems(i, arr)	for(int i = 0; i < arr->child.nCount; i++)







////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////

#define MESSAGE_JSON_ERROR_TEXT		WM_USER + 8732

enum _JsonErrorCode
{
	JSON_ERROR_NOERROR,
	JSON_ERROR_BAD_SYNTAX,
	JSON_ERROR_WAITING_SECTION,
	JSON_ERROR_MULTYPLE_ROOT,
	JSON_ERROR_BEGIN_SECTION,
	JSON_ERROR_NOT_STRING,
	JSON_ERROR_NOT_COLON,
	JSON_ERROR_NOT_COMMA,
	JSON_ERROR_NOT_VALUE,

	JSON_ERROR_NULL_POINTER,
	JSON_ERROR_MAX_LENGTH,
};

static char* _JsonErrorCodeStr[] =
{
	" ",
	"  JSON ",
	"   ",
	"     ",
	"     ",
	"   (   )",
	"    ()",
	"    ()",
	"  ",

	" ",
	"  ",
};

// ::SendMessage(m_pWnd, MESSAGE_JSON_ERROR_TEXT, (WPARAM)&m_strError, (LPARAM)&m_pLa);
//#define _STR_ERR(buf, errorcode) \
//	m_strError.Format(" : %d\r\n%s\r\n%s\r\n\r\n%s", errorcode, #errorcode, _JsonErrorCodeStr[errorcode], m_pLa->GetPosition(_Pos(buf))); \
//	::MessageBox(NULL, m_strError, " _STR_ERR", MB_OK|MB_ICONWARNING|MB_TASKMODAL); \
//	::SendMessage(m_pWnd, MESSAGE_JSON_ERROR_TEXT, (WPARAM)&m_strError, (LPARAM)&m_pLa);


#define _STR_ERR(buf, errorcode) \
	m_strError.Format("line %d\r\n\r\n%s\r\n%s\r\n\r\n%s", __LINE__, #errorcode, _JsonErrorCodeStr[errorcode], m_pLa->GetPosition(_Pos(buf))); \
	::SendMessage(m_pWnd, MESSAGE_JSON_ERROR_TEXT, (WPARAM)&m_strError, (LPARAM)&m_pLa);

#ifdef _DEBUG
	#define JsonError(errorcode)  _ERROR(errorcode, FALSE, (char**)&_JsonErrorCodeStr)
	#define JsonWarning(buf, errorcode) \
{ \
	_STR_ERR(buf, errorcode); \
	nError = errorcode; \
	goto LABEL_END; \
}

#else
	#define JsonError(errorcode) ( (void)0 )
	#define JsonWarning(buf, errorcode)  ( (void)0 )
#endif



////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////
typedef struct _JsonError // err
{
	int nError;
	char szFunction[32];

	////////////////////////////////////////////////////////////////////////
	// 
	////////////////////////////////////////////////////////////////////////
	_JsonError()
	{
		memset(this, 0, sizeof(_JsonError));
	}

} JSON_ERROR;




////////////////////////////////////////////////////////////////////////////
//    :
// +-------------------+--------------------------------------+
// |      nRead        |                                 |
// +-------------------+----------+---------------------------+
// |     READ_NAME     |  ppName  |  JSONENTRY.szEntryName    |
// |                   |  pnType  |  JSONENTRY.nNameType      |
// +-------------------+----------+---------------------------+
// |     READ_DATA     |  ppName  |  JSONENTRY.szEntryData    |
// |                   |  pnType  |  JSONENTRY.nDataType      |
// +-------------------+----------+---------------------------+
//    _GetNameTypePtr.
////////////////////////////////////////////////////////////////////////////
typedef struct _JsonNameTypePtr  // ntp
{
	int nRead;
	char** ppName;
	int* pnType;

} JSONNAMETYPEPTR;








/////////////////////////////////////////////////////////////////////////////
//   
/////////////////////////////////////////////////////////////////////////////
class EJson  // cfg
{
// Construction
public:
	EJson();
	~EJson();


	int Parse(char** ppData, JSONENTRY* pJe, int nLength, int& nError);
	void _NewEntry(JSONENTRY** ppJe, JSONENTRY* parent, int& nCount, int& nLevel, BOOL& isAlt, int nType);


	JSONENTRY* AddEntry(JSONENTRY* pJe, int nType, LPCSTR lpszName = NULL);
	void Remove();
	void Clear(JSONENTRY* pJe);
	BOOL Delete(JSONENTRY* pJe);

	void Print(JSONENTRY* pJe, int nMode);
	void _Print(JSONENTRY* pJe, int nMode);
#ifdef __AFXCMN_H__
	void SetTree(JSONENTRY* pJe, CTreeCtrl* pTreeCfg);
	void _SetTree(JSONENTRY* pJe, CTreeCtrl* pTreeCfg, HTREEITEM hParent);
#endif

	void _AddCounts(JSONENTRY* pJe, int nType);
	void _SubCounts(JSONENTRY* pJe, int nType);
	void _SetChildHash(JSONENTRY* pJe);
	int _GetChildHash(JSONENTRY* pJe);
	EString _GetPath(JSONENTRY* pJe);


	BOOL IsIncorrectEntryType(int nType);


	EString GetSysVarStr();


	JSONENTRY* FindEntry(char* szPath, JSONENTRY* pJe = NULL); //+



	JSONENTRY* GetEntry(char* szPath, int nNumber);

	BOOL GetBoolean(char* szPath, JSONENTRY* pJe = NULL);
	int GetInteger(char* szPath, JSONENTRY* pJe = NULL);
	double GetDouble(char* szPath, JSONENTRY* pJe = NULL); //+
	char* GetString(char* szPath, JSONENTRY* pJe = NULL);

	char* GetEntryName(char* szPath, int nNumber);
	char* GetEntryData(char* szPath, int nNumber);
	char* GetEntryString(char* szPath, int nNumber, int nStringType);


	JSONENTRY* _FindPath(JSONENTRY* pJe, int nDeltaLevel); //+

	JSONENTRY* _FindEntry(char* szName, JSONENTRY* pJe, int nEntryType);
	JSONENTRY* _FindEntryUp(char* szName, JSONENTRY* pJe, int nEntryType);

	JSONENTRY* _GetEntry(int nNumber, JSONENTRY* pJe);


	void _GetNameData(char** ppBuf, JSONENTRY* pJe, int nRead);
	void _GetNameTypePtr(JSONENTRY* pJe, JSONNAMETYPEPTR* pNtp);



	////////////////////////////////////////////////////////////////////////
	//
	////////////////////////////////////////////////////////////////////////
	JSONCHILD m_base;		//  ()

	char* m_szJsonText;
	int m_nJsonLength;		//   
//	BOOL m_nVersion;

	ELineArray* m_pLa;
	JSON_ERROR m_err;
	HWND m_pWnd;
	EString m_strError;

	EJsonPath m_path;
	EString m_str;
#ifdef __AFXCMN_H__
	CTreeCtrl* m_pTreeCfg;
#endif

	int _GetJsonCharType(char ch)
	{
		if(ch == '{' || ch == '[') return JSON_CHAR_BOS;
		if(ch == '}' || ch == ']') return JSON_CHAR_EOS;
		if(ch == ':') return JSON_CHAR_COLON;
		if(ch == ',') return JSON_CHAR_COMMA;
		if(ch == 0x22) return JSON_CHAR_QUOTE;
		return JSON_CHAR_UNKNOWN;
	}

	int _GetJsonListTypeL(char br)
	{
		if(br == '{') return JSON_OBJECT;
		if(br == '[') return JSON_ARRAY;
		return JSON_UNKNOWN;
	}

	int _GetJsonListTypeR(char br)
	{
		if(br == '}') return JSON_OBJECT;
		if(br == ']') return JSON_ARRAY;
		return JSON_UNKNOWN;
	}

	int _Pos(char* pBuf)
	{
		return pBuf - m_szJsonText;
	}

};




////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////
typedef struct _JsonDatas  // cds
{
	JSONENTRY* pJe;
	EJson* pConfig;

} JSONDATA;



////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline EJson::EJson()
{
	memset(&m_base, 0, sizeof(JSONCHILD));
	m_pLa = new ELineArray;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline EJson::~EJson()
{
	Remove();
	delete m_pLa;
}




////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////
inline void EJson::_NewEntry(JSONENTRY** ppJe, JSONENTRY* parent, int& nCount, int& nLevel, BOOL& isAlt, int nType)
{
	//  
	JSONENTRY* pCeNew = new JSONENTRY;
	nCount++;

	//   
	if(*ppJe == NULL)
	{
		m_base.first = pCeNew;
	}
	else
	{
		//   
		if(isAlt)
			(*ppJe)->child.first = pCeNew;
		else
		{
			pCeNew->prev = *ppJe;
			(*ppJe)->next = pCeNew;
		}
	}

	//  
	*ppJe = pCeNew;
	if(nLevel == 0) m_base.last = pCeNew;
	(*ppJe)->parent = parent;
	if(parent != NULL) parent->child.last = pCeNew;
	isAlt = FALSE;

	//   
	(*ppJe)->nType = nType;
	(*ppJe)->nLevel = nLevel;
}


////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////
inline int EJson::Parse(char** ppData, JSONENTRY* pJe, int nLength, int& nError)
{
	BOOL isAlt = FALSE, isComma = FALSE;
	char *p0, *tmp = *ppData;


	//   
	if(nLength != 0)
	{
		m_szJsonText = tmp;
		m_nJsonLength = nLength;
	}


	JSONENTRY* parent = pJe;
	int nCount = 0, nLevel = 0, nCharType;//, nType;
	int nParsing, nWaiting, nAction = ACTION_NONE;
	nError = JSON_ERROR_NOERROR;


	//   
	//   
	nParsing = (parent == NULL) ? PARSING_TEXT : ((parent->nType == JSON_OBJECT) ? PARSING_OBJECT : PARSING_ARRAY);
	nWaiting = (nParsing == PARSING_TEXT) ? WAITING_BOS : ((nParsing == PARSING_OBJECT) ? WAITING_NAME : WAITING_VALUE);


	//     
	if(pJe != NULL)
	{
		isAlt = TRUE;
		nLevel = pJe->nLevel + 1;
	}


	//   
	while(1)
	{
		//   
		if(_EcxSkipNonInformation(&tmp)) break;

		//  
		nCharType = _GetJsonCharType(*tmp);



		//////////////////////////////////////////////////////////////////////
		// W0
		//////////////////////////////////////////////////////////////////////
		if(nWaiting == WAITING_EOT)
			JsonWarning(tmp, JSON_ERROR_MULTYPLE_ROOT);

		//////////////////////////////////////////////////////////////////////
		// W1
		//////////////////////////////////////////////////////////////////////
		if(nWaiting == WAITING_BOS)
		{
			if(nCharType == JSON_CHAR_BOS)
				nAction = ACTION_NEW_SECTION; // A11
			else
				JsonWarning(tmp, JSON_ERROR_BEGIN_SECTION);
		}

		//////////////////////////////////////////////////////////////////////
		// W2
		//////////////////////////////////////////////////////////////////////
		if(nWaiting == WAITING_NAME)
		{
			if(nCharType == JSON_CHAR_EOS)
				nAction = ACTION_EOS_EXPLOR; // A02
			else
				if(nCharType == JSON_CHAR_QUOTE)
					nAction = ACTION_NEW_MEMBER; // A12
				else
					JsonWarning(tmp, JSON_ERROR_NOT_STRING);
		}

		//////////////////////////////////////////////////////////////////////
		// W3
		//////////////////////////////////////////////////////////////////////
		if(nWaiting == WAITING_COLON)
		{
			if(nCharType == JSON_CHAR_COLON)
			{
				nWaiting = WAITING_VALUE;
				tmp++;
				continue;
			}
			else
				JsonWarning(tmp, JSON_ERROR_NOT_COLON);
		}

		//////////////////////////////////////////////////////////////////////
		// W4
		//////////////////////////////////////////////////////////////////////
		if(nWaiting == WAITING_COMMA)
		{
			if(nCharType == JSON_CHAR_EOS)
				nAction = ACTION_EOS_EXPLOR; // A02
			else
				if(nCharType == JSON_CHAR_COMMA)
				{
					isComma = TRUE;
					nWaiting = (nParsing == PARSING_OBJECT) ? WAITING_NAME : WAITING_VALUE;
					tmp++;
					continue;
				}
				else
					JsonWarning(tmp, JSON_ERROR_NOT_COMMA);
		}

		//////////////////////////////////////////////////////////////////////
		// W5
		//////////////////////////////////////////////////////////////////////
		if(nWaiting == WAITING_VALUE)
		{
			switch(nCharType)
			{
				case JSON_CHAR_UNKNOWN:
				case JSON_CHAR_QUOTE:
					nAction = ACTION_NEW_VALUE; // A13
					break;
				case JSON_CHAR_BOS:
					nAction = ACTION_NEW_SECTION; // A11
					break;
				case JSON_CHAR_EOS:
					if(nParsing == PARSING_OBJECT) JsonWarning(tmp, JSON_ERROR_NOT_VALUE);
					nAction = ACTION_EOS_EXPLOR; // A02
					break;
				case JSON_CHAR_COLON:
					JsonWarning(tmp, JSON_ERROR_BAD_SYNTAX);
					break;
				case JSON_CHAR_COMMA:
					if(nParsing == PARSING_OBJECT) JsonWarning(tmp, JSON_ERROR_NOT_VALUE);

//					tmp++;
					isComma = TRUE;
					nWaiting = WAITING_NONE;
					nAction = ACTION_NEW_VALUE; // A13
//					continue;
			}

			if(nParsing == PARSING_OBJECT) nAction &= 3;
		}

		//////////////////////////////////////////////////////////////////////
		// A02
		//////////////////////////////////////////////////////////////////////
		if(nAction == ACTION_EOS_EXPLOR)
		{
			if(isComma)
			{
				nAction = ACTION_NEW_VALUE; // A13
			}
			else
			{
				// } for PARSING_OBJECT
				// ] for PARSING_ARRAY
				if(_GetJsonListTypeR(*tmp) == nParsing)
				{
					tmp++;
					goto LABEL_END;
				}
			}
		}

		//
		if(nWaiting != WAITING_NONE) isComma = FALSE;


		//////////////////////////////////////////////////////////////////////
		//   (   )
		//////////////////////////////////////////////////////////////////////
		if(nAction > 4)
			_NewEntry(&pJe, parent, nCount, nLevel, isAlt, JSON_VALUE_EMPTY);


		//////////////////////////////////////////////////////////////////////
		// A01
		//////////////////////////////////////////////////////////////////////
		if((nAction & 3) == ACTION_GET_SECTION)
		{
			//  
			pJe->nType = _GetJsonListTypeL(*tmp);

			//   
			if(parent != NULL) parent->child.nNode++;

			//      
			tmp++;
			pJe->child.nCount = Parse(&tmp, pJe, 0, nError);
			if(nError) return 0;

			//    
//			pJe->nDataType = JSON_VALUE_EMPTY;
			if(pJe->child.nCount > 0)
			{
				//  
				pJe->nDataType = pJe->nType;
				_SetChildHash(pJe);
			}

			//     
			nWaiting = (nParsing == PARSING_TEXT) ? WAITING_EOT : WAITING_COMMA; // W0 | W4
			nAction = ACTION_NONE; // A00
			continue;
		}


		//////////////////////////////////////////////////////////////////////
		// A12
		//////////////////////////////////////////////////////////////////////
		if(nAction == ACTION_NEW_MEMBER)
		{
			pJe->nType = JSON_MEMBER;

			//  
			_GetNameData(&tmp, pJe, READ_NAME);
			pJe->nNameLength = strlen(pJe->szEntryName);

			//  
			nWaiting = WAITING_COLON;
			nAction = ACTION_NONE; // A00
			continue;
		}


		//////////////////////////////////////////////////////////////////////
		// A03
		//////////////////////////////////////////////////////////////////////
		if((nAction & 3) == ACTION_GET_VALUE)
		{
			if(nParsing == PARSING_ARRAY) pJe->nType = JSON_VALUE;

			//  
			p0 = tmp;
			_GetNameData(&tmp, pJe, READ_DATA);
			pJe->nDataLength = strlen(pJe->szEntryData);

			if(pJe->nDataType == JSON_VALUE_UNKNOWN)
				JsonWarning(p0, JSON_ERROR_NOT_VALUE);

			//  
			nWaiting--;
			if(nWaiting == WAITING_VALUE) tmp++;
			nAction = ACTION_NONE; // A00
			continue;
		}


		//////////////////////////////////////////////////////////////////////
		//
		//////////////////////////////////////////////////////////////////////
		JsonWarning(tmp, JSON_ERROR_BAD_SYNTAX);
		tmp++;

	}

LABEL_END:

	//    
	if(parent)
	{
		*ppData = tmp;
	}


	//   
	m_base.nCount += nCount;
	return nCount;
}


////////////////////////////////////////////////////////////////////////////
//        
//       
////////////////////////////////////////////////////////////////////////////
inline void EJson::_SetChildHash(JSONENTRY* pJe)
{
	//     
	if(pJe->hash) delete [] pJe->hash;

	//       
	pJe->hash = new JSONENTRY*[pJe->child.nCount];
	int i = 0;
	JSONENTRY* tmp = pJe->child.first;
	do
	{
		pJe->hash[i] = tmp;
		i++;
		tmp = tmp->next;

	} while(tmp);
}


////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////
inline int EJson::_GetChildHash(JSONENTRY* pJe)
{
	//    
	if(pJe->IsChild())
	{
		for(int i=0; i<pJe->parent->child.nCount; i++)
		{
			if(pJe->parent->hash[i] == pJe) return i;
		}
	}
	return -1;
}


////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////
inline EString EJson::_GetPath(JSONENTRY* pJe)
{
	EString s1, s2;
	JSONENTRY* tmp = pJe;
	do
	{
		switch(tmp->GetEntryType())
		{
			case JSON_ROOT:
				s2 = "root";
				break;
			case JSON_MEMBER:
				if(_EcxStrClass(tmp->szEntryName, CCM_SPACE))
					s2.Format("[\"%s\"]", tmp->szEntryName);
				else
					s2.Format(".%s", tmp->szEntryName);
				break;
			case JSON_VALUE:
				s2.Format("[%d]", _GetChildHash(tmp));
				break;
		}
		s1 = s2 + s1;
		tmp = tmp->parent;
	}
	while(tmp);

	return s1;
}


////////////////////////////////////////////////////////////////////////////
//   (szEntryName, nNameType)  (szEntryData, nDataType)
//      READ_NAME  READ_DATA
////////////////////////////////////////////////////////////////////////////
inline void EJson::_GetNameTypePtr(JSONENTRY* pJe, JSONNAMETYPEPTR* pNtp)
{
	if(pNtp->nRead == READ_NAME)
	{
		pNtp->ppName = &pJe->szEntryName;
		pNtp->pnType = &pJe->nNameType;
	}
	else
	{
		pNtp->ppName = &pJe->szEntryData;
		pNtp->pnType = &pJe->nDataType;
	}
}


////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////
inline void EJson::_GetNameData(char** ppBuf, JSONENTRY* pJe, int nRead)
{
	int nType = JSON_VALUE_UNKNOWN;
	int nLength;
	char *p0, *tmp;
	p0 = tmp = *ppBuf;


	//     
	JSONDATA cds = { pJe, this };

	//   (szEntryName, nNameType)  (szEntryData, nDataType)
	JSONNAMETYPEPTR ntp = { nRead, NULL, NULL };
	_GetNameTypePtr(pJe, &ntp);

	//   
	if(*tmp == 0x22)
	{
		_EcxSkipString(&tmp, FALSE);
		nType = JSON_VALUE_STRING;
	}
	else
	{
		//    
		while(!is_eot(tmp))
		{
			if(*tmp == ',') break;
			if(is_char(*tmp, CCM_BRACKET_R)) break;
			tmp++;
		}
	}
	nLength = tmp - p0;

	//    
	if(nType == JSON_VALUE_STRING)
	{
		p0++;
		nLength -= 2;
	}

	//    
	_EcxStrDup(ntp.ppName, p0, nLength);
	nLength = _EcxStrTrimRight(ntp.ppName);


	//   
	if(nType != JSON_VALUE_STRING)
	{
		if(nLength > 0)
		{
			ifstrcmp(*ntp.ppName, "null") nType = JSON_VALUE_NULL;
			ifstrcmp(*ntp.ppName, "true") nType = JSON_VALUE_BOOLEAN;
			ifstrcmp(*ntp.ppName, "false") nType = JSON_VALUE_BOOLEAN;

			char* num = *ntp.ppName;
			__strtol(num);
			if(nLength - (num - *ntp.ppName) == 0)
				nType = JSON_VALUE_INTEGER;
			else
			{
				num = *ntp.ppName;
				__strtod(num);
				if(nLength - (num - *ntp.ppName) == 0)
					nType = JSON_VALUE_DOUBLE;
			}

			if(**ntp.ppName == '+')
				nType = JSON_VALUE_UNKNOWN;
		}
		if(nLength == 0) nType = JSON_VALUE_EMPTY;
	}

	*ppBuf = tmp;
	*ntp.pnType = nType;
}


////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////
inline void EJson::Remove()
{
	if(m_base.first == NULL) return;
	Clear(m_base.first);
	memset(&m_base, 0, sizeof(JSONCHILD));
	memset(&m_err, 0, sizeof(JSON_ERROR));
	m_path.Clear();
}


////////////////////////////////////////////////////////////////////////////
//   ,    pJe
////////////////////////////////////////////////////////////////////////////
inline void EJson::Clear(JSONENTRY* pJe)
{
	if(pJe == NULL) return;
	JSONENTRY* tmp = pJe;
	do
	{
		JSONENTRY* next = tmp->next;
		if(tmp->child.first != NULL) Clear(tmp->child.first);
		delete tmp;
		m_base.nCount--;
		tmp = next;

	} while(tmp);
}


////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////
inline BOOL EJson::Delete(JSONENTRY* pJe)
{
	if(pJe == NULL) return FALSE;

	if(pJe->next != NULL) pJe->next->prev = pJe->prev;
	if(pJe->prev != NULL) pJe->prev->next = pJe->next;

	if(pJe == m_base.last) m_base.last = pJe->prev;
	if(pJe == m_base.first) m_base.first = pJe->next;

	//   
	Clear(pJe->child.first);

	if(pJe->parent != NULL)
	{
		if(pJe->prev == NULL) pJe->parent->child.first = pJe->next;
		if(pJe->next == NULL) pJe->parent->child.last = pJe->prev;

		//  
		_SubCounts(pJe->parent, pJe->nType);
	}
	delete pJe;

	m_base.nCount--;
	return TRUE;
}


////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////
inline JSONENTRY* EJson::AddEntry(JSONENTRY* pJe, int nType, LPCSTR lpszName)
{
	//  
	#define ADD_ENTRY 0
	#define ADD_CHILD 1


	//     
//	if(IsIncorrectEntryType(nType)) JsonError(ERROR_INCORRECT_ENTRY_TYPE);

	//   
	int nMode = ADD_ENTRY;

	//     
	if(pJe != NULL)
	{
		//   
		if(pJe->IsType(JSON_SECTION))
		{
			//   
			nMode = ADD_CHILD;
		}

		//   
		else
		{
			//    
			if(pJe->IsChild())
			{
				//      -  (.. )
				nMode = ADD_CHILD;
				pJe = pJe->parent;
			}
		}
	}

	//   
	JSONENTRY* pCeAdd = new JSONENTRY;
	pCeAdd->nType = nType;
	m_base.nCount++;

	//  
	if(lpszName != NULL)
	{
		int nLength = strlen(lpszName);
		pCeAdd->szEntryName = new char[nLength + 1];
		memcpy(pCeAdd->szEntryName, lpszName, nLength);
		pCeAdd->szEntryName[nLength] = '\0';
	}


	//   
	if(nMode == ADD_ENTRY)
	{
		//   
		if(m_base.first == NULL)
		{
			m_base.first = pCeAdd;
		}
		else
		{
			//    
			pCeAdd->prev = m_base.last;
			m_base.last->next = pCeAdd;
		}

		//   
		m_base.last = pCeAdd;
	}

	//   
	else
	{
		//   alt-
		if(pJe->child.first == NULL)
		{
			pJe->child.first = pCeAdd;
		}
		else
		{
			//    
			pCeAdd->prev = pJe->child.last;
			pJe->child.last->next = pCeAdd;
		}

		//   
		pJe->child.last = pCeAdd;

		//   
		pCeAdd->parent = pJe;
		pCeAdd->nLevel = pJe->nLevel + 1;

		//  
		_AddCounts(pJe, nType);
	}

	//  
	return pCeAdd;
}



////////////////////////////////////////////////////////////////////////////
//       
////////////////////////////////////////////////////////////////////////////
inline void EJson::_AddCounts(JSONENTRY* pJe, int nType)
{
	if(pJe == NULL) return;
	pJe->child.nCount++;
	if(nType == JSON_OBJECT || nType == JSON_ARRAY) pJe->child.nNode++;
}


////////////////////////////////////////////////////////////////////////////
//       
////////////////////////////////////////////////////////////////////////////
inline void EJson::_SubCounts(JSONENTRY* pJe, int nType)
{
	if(pJe == NULL) return;
	pJe->child.nCount--;
	if(nType == JSON_OBJECT || nType == JSON_ARRAY) pJe->child.nNode--;
}




////////////////////////////////////////////////////////////////////////////
//      
////////////////////////////////////////////////////////////////////////////
inline JSONENTRY* EJson::FindEntry(char* szPath, JSONENTRY* pJe)
{
	JSONENTRY* tmp = (pJe) ? pJe->child.first : m_base.first;
	if(tmp == NULL) return NULL;

	//       
	if(!m_path.Split(szPath)) return NULL;

	//    
	return _FindPath(tmp, tmp->nLevel);
}


////////////////////////////////////////////////////////////////////////////
//    
// pJe -    
////////////////////////////////////////////////////////////////////////////
inline JSONENTRY* EJson::_FindPath(JSONENTRY* pJe, int nDeltaLevel)
{
	JSONENTRY *alt, *fnd;

	// 
	JSONENTRY* tmp = pJe;
	if(pJe == NULL) return NULL;

	//       . 
	if(tmp->nLevel - nDeltaLevel >= m_path.m_nCount) return NULL;

	//  
	char* name = m_path.m_ppEntry[tmp->nLevel - nDeltaLevel];

	//   
	do
	{
		JSONENTRY* next = tmp->next;

		if(m_path.IsACD(name))
		{
			if(pJe->parent && pJe->parent->IsType(JSON_ARRAY))
			{
				int n = m_path.GetACD(name);
				if(n >= pJe->parent->child.nCount) return NULL;
				tmp = pJe->parent->hash[n];
				goto LAB_01;
			}
		}

		//        
		if(tmp->IsName(name))
		{
LAB_01:
			//    
			if(tmp->nLevel - nDeltaLevel == m_path.m_nCount - 1)
			{
				return tmp;
			}

			//   - 
			if(tmp->IsType(JSON_SECTION))
			{
				//  
				alt = tmp->child.first;
				if(alt != NULL)
				{
					//  
					fnd = _FindPath(tmp->child.first, nDeltaLevel);
					if(fnd != NULL) return fnd;
				}
			}
		}

		tmp = next;

	} while(tmp);

	return NULL;
}


////////////////////////////////////////////////////////////////////////////
//     
////////////////////////////////////////////////////////////////////////////
inline JSONENTRY* EJson::_FindEntry(char* szName, JSONENTRY* pJe, int nEntryType)
{
	JSONENTRY* fnd = NULL;

	//   
	JSONENTRY *next, *tmp = pJe;
	if(tmp == NULL) return NULL;

	if(szName == NULL) return NULL;
	if(szName[0] == '\0') return NULL;

	//   
	do
	{
		next = tmp->next;

		//       
		if(tmp->IsName(szName))
		{
			//       
			if(tmp->IsType(nEntryType))
			{
				fnd = tmp;
				break;
			}
		}

		tmp = next;

	} while(tmp);

	return fnd;
}


////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////
inline JSONENTRY* EJson::_FindEntryUp(char* szName, JSONENTRY* pJe, int nEntryType)
{
	JSONENTRY* fnd = NULL;

	//   
	JSONENTRY *prev, *tmp = pJe;
	if(tmp == NULL) return NULL;
	if(_EcxStrIsNull(szName)) return NULL;

	//   
	while(1)
	{
		prev = tmp->prev;
		if(prev == NULL) prev = tmp->parent;
		if(prev == NULL) break;

		//       
		if(prev->IsName(szName))
		{
			//       
			if(prev->IsType(nEntryType))
			{
				fnd = prev;
				break;
			}
		}

		tmp = prev;
	}

	return fnd;
}


////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////
inline JSONENTRY* EJson::_GetEntry(int nNumber, JSONENTRY* pJe)
{
	JSONENTRY* tmp = pJe->child.first;
	if(tmp == NULL) return NULL;
	if(nNumber >= pJe->child.nCount) return NULL;

	int i = 0;
	JSONENTRY* fnd = NULL;

	do
	{
		JSONENTRY* next = tmp->next;

		if(i == nNumber)
		{
			fnd = tmp;
			break;
		}

		tmp = next;
		i++;

	} while(tmp);

	return fnd;
}


////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////
inline JSONENTRY* EJson::GetEntry(char* szPath, int nNumber)
{
	JSONENTRY *pce0, *pce1;

//	pce0 = FindEntry(szPath, NULL, CET_SECTION);
	if(pce0 == NULL) return NULL;
	pce1 = _GetEntry(nNumber, pce0);
	return pce1;
}


////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////
inline char* EJson::GetEntryName(char* szPath, int nNumber)
{
	JSONENTRY* pce = GetEntry(szPath, nNumber);
	if(pce == NULL) return NULL;

	return pce->szEntryName;
}


////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////
inline char* EJson::GetEntryData(char* szPath, int nNumber)
{
	JSONENTRY* pce = GetEntry(szPath, nNumber);
	if(pce == NULL) return NULL;

	return pce->szEntryData;
}


////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////
inline char* EJson::GetEntryString(char* szPath, int nNumber, int nStringType)
{
	switch(nStringType)
	{
		case FIELD_NAME:
			return GetEntryName(szPath, nNumber);
			break;

		case FIELD_DATA:
			return GetEntryData(szPath, nNumber);
			break;
	}

	return NULL;
}


////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////
inline BOOL EJson::GetBoolean(char* szPath, JSONENTRY* pJe)
{
	BOOL value = FALSE;
	JSONENTRY* pce = FindEntry(szPath, pJe);
	if(pce != NULL)
	{
		char* tmp = pce->szEntryData;
		if(is_char(*tmp, CCM_ALPHA))
		{
			int len = strlen(tmp);
			char* psz = new char[len + 1];
			strcpy(psz, tmp);
			::CharUpperBuff(psz, len);

			if(strcmp(psz, "TRUE") == 0) value = TRUE;
			if(strcmp(psz, "YES") == 0) value = TRUE;
			if(strcmp(psz, "FALSE") == 0) value = FALSE;
			if(strcmp(psz, "NO") == 0) value = FALSE;

			delete [] psz;
		}
		else
		{
			int i = __strtol(tmp);
			value = (i == 0) ? FALSE : TRUE;
		}
	}
	return value;
}


////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////
inline int EJson::GetInteger(char* szPath, JSONENTRY* pJe)
{
	int value = 0;
	JSONENTRY* pce = FindEntry(szPath, pJe);
	if(pce != NULL)
	{
		char* tmp = pce->szEntryData;
		value = __strtol(tmp);
	}
	return value;
}


////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////
inline double EJson::GetDouble(char* szPath, JSONENTRY* pJe)
{
	double value = 0.0;
	JSONENTRY* pce = FindEntry(szPath, pJe);
	if(pce != NULL)
	{
		char* tmp = pce->szEntryData;
		value = __strtod(tmp);
	}
	return value;
}


////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////
inline char* EJson::GetString(char* szPath, JSONENTRY* pJe)
{
	char* value = NULL;
	JSONENTRY* pce = FindEntry(szPath, pJe);
	if(pce != NULL)
	{
		value = pce->szEntryData;
	}
	return value;
}


////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////
inline void EJson::Print(JSONENTRY* pJe, int nMode)
{
	m_str = "";
	_Print(pJe, nMode);
}


////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////
inline void EJson::_Print(JSONENTRY* pJe, int nMode)
{
	if(pJe == NULL) pJe = m_base.first;
	if(pJe == NULL) return;

	EString s1, s2, rnsp;
	JSONENTRY* tmp = pJe;

	do
	{
		EString spaces(' ', 4 * tmp->nLevel);
		s2 = "";
		rnsp = "\r\n" + spaces;

		EString strEntryName = tmp->GetEntryNameDataStr(FIELD_NAME);
		EString strEntryData = tmp->GetEntryNameDataStr(FIELD_DATA);

		char brL = (tmp->nType == JSON_OBJECT) ? '{' : '[';
		char brR = (tmp->nType == JSON_OBJECT) ? '}' : ']';

		//   cfg-
		switch(nMode)
		{
			// 
			case 1:
			{
				switch(tmp->nType)
				{
					case JSON_MEMBER:
					case JSON_VALUE:
						s1.Format("%s\r\n", tmp->GetEntryTypeDataStr(TRUE));
						break;

					case JSON_OBJECT:
					case JSON_ARRAY:
					{
						EString sp;
						sp.Format("\r\n%s", spaces);
						s1.Format("%s : %d,%d\r\n%s%c\r\n", tmp->GetEntryTypeDataStr(TRUE), tmp->child.nCount, tmp->child.nNode, spaces, brL);
						if(tmp->prev != NULL) s1 = sp + s1;
						s2.Format("%s%c\r\n", spaces, brR);
						break;
					}
				}
				break;
			}

			//  
			case 0:
			{
				switch(tmp->nType)
				{
					case JSON_MEMBER:
					case JSON_VALUE:
					{
						s1.Format("%s%s\r\n", tmp->GetPropertyStr(), (tmp->next) ? "," : "");
						break;
					}


					case JSON_OBJECT:
					case JSON_ARRAY:
					{
//						EString sp;
//						sp.Format("\r\n%s", spaces);
						s1.Format("%c", brL);
						EString sn = ": ";
						if(tmp->child.nCount) sn = ":\r\n" + spaces;
						if(tmp->szEntryName) s1 = strEntryName + sn + s1;
//						if(tmp->prev != NULL) s1 = sp + s1;
						s2.Format("%c%s\r\n", brR, (tmp->next) ? "," : "");

						if(tmp->child.nCount)
						{
							s1 += "\r\n";
							s2 = spaces + s2;
						}
						else
							s1 += " ";
						break;
					}
				}
				break;
			}

			// 
			case 2:
			{
				break;
			}

		}

		m_str += spaces + s1;
		if(tmp->child.first != NULL) _Print(tmp->child.first, nMode);
		m_str += s2;

		tmp = tmp->next;

	} while(tmp);
}

#ifdef __AFXCMN_H__
////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////
inline void EJson::SetTree(JSONENTRY* pJe, CTreeCtrl* pTreeCfg)
{
	if(pJe == NULL) pJe = m_base.first;
	if(pJe == NULL) return;

	pTreeCfg->DeleteAllItems();
	_SetTree(pJe, pTreeCfg, TVI_ROOT);
}


////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////
inline void EJson::_SetTree(JSONENTRY* pJe, CTreeCtrl* pTreeCfg, HTREEITEM hParent)
{
	JSONENTRY* tmp = pJe;
	HTREEITEM hCurr;

	do
	{
		hCurr = pTreeCfg->InsertItem(tmp->GetEntryStr(1), hParent);
//		pTreeCfg->SetItemImage(hCurr, tmp->nType, tmp->nType);

		int n = tmp->nType;
		if(n >= JSON_MEMBER) n = (tmp->nDataType == JSON_VALUE_EMPTY) ? 4 : tmp->nDataType + 2;

		pTreeCfg->SetItemImage(hCurr, n, n);
		pTreeCfg->SetItemData(hCurr, (DWORD)tmp);

		if(tmp->child.first != NULL) _SetTree(tmp->child.first, pTreeCfg, hCurr);
		tmp = tmp->next;

	} while(tmp);
}
#endif

////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////
inline EString EJson::GetSysVarStr()
{
	EString p1, p2;
	EString a1, a2;

	p1.Format("m_base\r\n   first  = %s\r\n", _JsonEntry::GetEntryPtrStr(m_base.first));
	p2.Format("   last   = %s\r\n", _JsonEntry::GetEntryPtrStr(m_base.last));
	a1.Format("   nCount = %d\r\n\r\n", m_base.nCount);
	a2.Format("m_nJsonLength = %d\r\n", m_nJsonLength);

	EString str = p1 + p2 + a1 + a2;
	return str;
}



////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////
inline BOOL EJson::IsIncorrectEntryType(int nType)
{
//	return (nType < 0) || (nType > CET_LIST_LIST);
	return 0;
}





#endif //USED_JSON
/////////////////////////////////////////////////////////////////////////////
#endif // __EJSON_H__

