// EMemory.h : header file
//

#ifndef ___EMEMORY_H__
#define ___EMEMORY_H__
#pragma once
//////////////////////////////////////////////////////////////////////////////



/////////////////////////////////////////////////////////////////////////////
//  EMemory.h
/////////////////////////////////////////////////////////////////////////////
#define VERSION_EMEMORY   "23.09.2013"



/////////////////////////////////////////////////////////////////////////////
//      EMemory
/////////////////////////////////////////////////////////////////////////////
struct EMemoryData  // data
{
	int nArrayLength;		//   
	int nBufferLength;		//   
};


//////////////////////////////////////////////////////////////////////////////
//    EMemory
//////////////////////////////////////////////////////////////////////////////
class EMemory  // mem
{
public:

	EMemory();
	EMemory(int nLength);
	EMemory(EMemory* pMem);
	EMemory(EMemory& mem);
	~EMemory();

	BOOL MemAlloc(int nLength, BOOL isZeroInit = FALSE);
	BOOL MemAppend(int nLength, BOOL isZeroInit = FALSE);
	BOOL MemChange(int nLength, BOOL isZeroInit = FALSE);
	BOOL MemDup(EMemory* pMem);
	EMemory& operator=(EMemory& mem);
	void MemFree();
	EMemoryData* GetData(BYTE* pBuf = NULL) const;

	//        
	operator LPBYTE();
	int operator~();
	BYTE& operator[](int dwIndex);
	BYTE& GetAt(int dwIndex);
	void SetAt(int dwIndex, BYTE n);
	int GetLength();

	BYTE* m_pBuffer;			//   

private:
	BOOL __Mem_Alloc(int nLength, BOOL isZeroInit, BOOL isMemFree);
};




//////////////////////////////////////////////////////////////////////////////
//  / 
//////////////////////////////////////////////////////////////////////////////
inline EMemory::EMemory()
	{ m_pBuffer = NULL; }
inline EMemory::EMemory(int nLength)
	{ m_pBuffer = NULL; MemAlloc(nLength); }
inline EMemory::EMemory(EMemory* pMem)
	{ m_pBuffer = NULL; MemDup(pMem); }
inline EMemory::EMemory(EMemory& mem)
	{ m_pBuffer = NULL; MemDup(&mem); }
inline EMemory::~EMemory()
	{ MemFree(); }

//////////////////////////////////////////////////////////////////////////////
//    nLength
//////////////////////////////////////////////////////////////////////////////
inline BOOL EMemory::MemAlloc(int nLength, BOOL isZeroInit)
{
	return __Mem_Alloc(nLength, isZeroInit, TRUE);
}

inline BOOL EMemory::__Mem_Alloc(int nLength, BOOL isZeroInit, BOOL isMemFree)
{
	if(isMemFree) MemFree();
	int nBufferLength = _even4(nLength);
	EMemoryData* pData = (EMemoryData*)new BYTE[nBufferLength + sizeof(EMemoryData)];
	if(isZeroInit) memset(pData, 0, nBufferLength + sizeof(EMemoryData));
	if(pData == NULL) return FALSE;
	m_pBuffer = (BYTE*)(pData + 1);
	pData->nBufferLength = nBufferLength;
	pData->nArrayLength = nLength;
	return TRUE;
}

//////////////////////////////////////////////////////////////////////////////
//  
//////////////////////////////////////////////////////////////////////////////
inline BOOL EMemory::MemAppend(int nLength, BOOL isZeroInit)
{
	if(m_pBuffer == NULL) return FALSE;
	int dwLen0 = GetData()->nArrayLength;
	int dwLen1 = dwLen0 + nLength;
	GetData()->nArrayLength = dwLen1;

	if(dwLen1 > GetData()->nBufferLength)
	{
		BYTE* tmp = m_pBuffer;
		if(__Mem_Alloc(2 * dwLen1, isZeroInit, FALSE))
		{
			memcpy(m_pBuffer, tmp, dwLen0);
			GetData()->nArrayLength = dwLen1;
			EMemoryData* pData = (EMemoryData*)tmp - 1;
			delete [] pData;
			return TRUE;
		}
	}
	return FALSE;
}

//////////////////////////////////////////////////////////////////////////////
//  
//////////////////////////////////////////////////////////////////////////////
inline BOOL EMemory::MemChange(int nLength, BOOL isZeroInit)
{
	if(m_pBuffer == NULL) return FALSE;
	BYTE* tmp = m_pBuffer;
	if(__Mem_Alloc(nLength, isZeroInit, FALSE))
	{
		EMemoryData* pData = (EMemoryData*)tmp - 1;
		delete [] pData;
		return TRUE;
	}
	return FALSE;
}

//////////////////////////////////////////////////////////////////////////////
//  
//////////////////////////////////////////////////////////////////////////////
inline BOOL EMemory::MemDup(EMemory* pMem)
{
	if(pMem != NULL)
	{
		EMemoryData* pData = (EMemoryData*)pMem->m_pBuffer - 1;
		MemAlloc(pData->nBufferLength);
		memcpy(m_pBuffer, pMem->m_pBuffer, pData->nArrayLength);
		return TRUE;
	}
	return FALSE;
}

//////////////////////////////////////////////////////////////////////////////
//  
//////////////////////////////////////////////////////////////////////////////
inline EMemory& EMemory::operator=(EMemory& mem)
{
	MemDup(&mem);
	return *this;
}

//////////////////////////////////////////////////////////////////////////////
//  
//////////////////////////////////////////////////////////////////////////////
inline void EMemory::MemFree()
{
	if(m_pBuffer)
	{
		EMemoryData* pData = (EMemoryData*)m_pBuffer - 1;
		delete [] pData;
		m_pBuffer = NULL;
	}
}

//////////////////////////////////////////////////////////////////////////////
//  
//////////////////////////////////////////////////////////////////////////////
inline EMemoryData* EMemory::GetData(BYTE* pBuf) const
	{ return (pBuf == NULL) ? (EMemoryData*)m_pBuffer - 1 : (EMemoryData*)pBuf - 1; }

inline EMemory::operator LPBYTE()
	{ return m_pBuffer; }

inline int EMemory::operator~()
	{ return GetData()->nArrayLength; }

inline BYTE& EMemory::operator[](int dwIndex)
	{ return m_pBuffer[dwIndex]; }

inline BYTE& EMemory::GetAt(int dwIndex)
	{ return m_pBuffer[dwIndex]; }

inline void EMemory::SetAt(int dwIndex, BYTE n)
	{ m_pBuffer[dwIndex] = n; }

inline int EMemory::GetLength()
	{ return GetData()->nArrayLength; }



/////////////////////////////////////////////////////////////////////////////
#endif // ___EMEMORY_H__
