// ERangeArray.h : header file
//

#ifndef __ERANGEARRAY_H__
#define __ERANGEARRAY_H__
#pragma once
////////////////////////////////////////////////////////////////////////////

#include "EArray.h"
#include "..\GDI\Bitmaps\ECanvas.h"

/////////////////////////////////////////////////////////////////////////////
//  ERangeArray.h
/////////////////////////////////////////////////////////////////////////////
#define VERSION_ERANGE   "05.04.2012"




///////////////////////////////////////////////////////////////////
// 
///////////////////////////////////////////////////////////////////
#define MIN_INFINITE	INT_MIN
#define MAX_INFINITE	INT_MAX
static char br[] = { '(', ')', '[', ']' };

///////////////////////////////////////////////////////////////////
//     
///////////////////////////////////////////////////////////////////
typedef struct _EndOfRange
{
	int nVal;
	BOOL isClosed;
	int nBr;

	

	///////////////////////////////////////////////////////////////
	// 
	///////////////////////////////////////////////////////////////
	void Clear() { memset(this, 0, sizeof(_EndOfRange)); };
	char Br() { return br[isClosed * 2 + nBr]; }
	_EndOfRange& operator~() { isClosed = !isClosed; return *this; }
	_EndOfRange& operator!() { nVal += 1 - (isClosed ^ nBr) * 2; isClosed = !isClosed; return *this; }

	///////////////////////////////////////////////////////////////
	// 
	///////////////////////////////////////////////////////////////
	friend BOOL operator==(_EndOfRange x, _EndOfRange y) { return x.nVal == y.nVal; }
	friend BOOL operator!=(_EndOfRange x, _EndOfRange y) { return x.nVal != y.nVal; }
	friend BOOL operator<(_EndOfRange x, _EndOfRange y)  { return x.nVal < y.nVal; }
	friend BOOL operator>(_EndOfRange x, _EndOfRange y)  { return x.nVal > y.nVal; }
	friend BOOL operator<=(_EndOfRange x, _EndOfRange y) { return x.nVal <= y.nVal; }
	friend BOOL operator>=(_EndOfRange x, _EndOfRange y) { return x.nVal >= y.nVal; }

	///////////////////////////////////////////////////////////////
	// 
	///////////////////////////////////////////////////////////////
	friend _EndOfRange operator+(_EndOfRange x, int n) { _EndOfRange tmp = x; tmp.nVal = tmp.nVal + n; return tmp; }
	friend _EndOfRange operator+(int n, _EndOfRange x) { _EndOfRange tmp = x; tmp.nVal = n + tmp.nVal; return tmp; }
	friend _EndOfRange operator-(_EndOfRange x, int n) { _EndOfRange tmp = x; tmp.nVal = tmp.nVal - n; return tmp; }
	friend _EndOfRange operator-(int n, _EndOfRange x) { _EndOfRange tmp = x; tmp.nVal = n - tmp.nVal; return tmp; }

	///////////////////////////////////////////////////////////////
	// 
	///////////////////////////////////////////////////////////////
	friend BOOL Eq(_EndOfRange x, _EndOfRange y) { return (x.nVal == y.nVal) && (x.isClosed == y.isClosed); }
	friend BOOL NotEq(_EndOfRange x, _EndOfRange y) { return !Eq(x, y); }

} EOR;


///////////////////////////////////////////////////////////////////
// 
///////////////////////////////////////////////////////////////////
enum _CompareRange
{
	CR_ERROR,
	CR_BELOW,
	CR_NOT_BELOW,
	CR_A_IN_B,
	CR_B_IN_A,
	CR_NOT_ABOVE,
	CR_ABOVE,
};

static char* _CompareRangeStr[] =
{
	"???",
	"CR_BELOW",
	"CR_NOT_BELOW",
	"CR_A_IN_B",
	"CR_B_IN_A",
	"CR_NOT_ABOVE",
	"CR_ABOVE",
};


///////////////////////////////////////////////////////////////////
//    
///////////////////////////////////////////////////////////////////
typedef struct _Range  // r
{
	EOR a;
	EOR b;
	char szName[32];
	DWORD dwData;

	///////////////////////////////////////////////////////////////
	// 
	///////////////////////////////////////////////////////////////
	_Range()
	{
		Empty();
		dwData = 0;
		set_name(NULL);
	}

	_Range(LPCSTR lpszName)
	{
		Empty();
		dwData = 0;
		set_name(lpszName);
	}

	///////////////////////////////////////////////////////////////
	// 
	///////////////////////////////////////////////////////////////
	~_Range()
	{
	}

	///////////////////////////////////////////////////////////////
	//   
	///////////////////////////////////////////////////////////////
	void Empty()
	{
		a.Clear();
		b.Clear();
		a.nBr = 0;
		b.nBr = 1;
	}

	///////////////////////////////////////////////////////////////
	//    
	///////////////////////////////////////////////////////////////
	BOOL IsEmpty()
	{
		if(a > b) return TRUE;
		if((a == b) && !(a.isClosed & b.isClosed)) return TRUE;
		if((a + 1 == b) && !(a.isClosed | b.isClosed)) return TRUE;
		return FALSE;
	}

	///////////////////////////////////////////////////////////////
	//   |a,b|  ,  :
	// 1)    : []
	// 2)  : a <= b
	///////////////////////////////////////////////////////////////
	BOOL IsNormalized()
	{
		return (a <= b) && (a.isClosed & b.isClosed);
	}

	///////////////////////////////////////////////////////////////
	//     
	///////////////////////////////////////////////////////////////
	void Normalize()
	{
//		open2closed();
		if(a > b) Empty();
	}

	///////////////////////////////////////////////////////////////
	// 
	///////////////////////////////////////////////////////////////
	int GetLength()
	{
		_Range tmp = *this;
		tmp.Normalize();
		return tmp.b.nVal - tmp.a.nVal;
	}

	///////////////////////////////////////////////////////////////
	//     
	///////////////////////////////////////////////////////////////
	void Set(EOR eor_a, EOR eor_b, BOOL isNormalizing = FALSE)
	{
		a.nVal = eor_a.nVal;
		b.nVal = eor_b.nVal;
		a.isClosed = eor_a.isClosed;
		b.isClosed = eor_b.isClosed;
		if(isNormalizing) Normalize();
	}

	void Set(int nVal_a, int nVal_b, BOOL isClosed_a, BOOL isClosed_b, BOOL isNormalizing = FALSE)
	{
		a.nVal = nVal_a;
		b.nVal = nVal_b;
		a.isClosed = isClosed_a;
		b.isClosed = isClosed_b;
		if(isNormalizing) Normalize();
	}

	void Set(char nBr_a, int nVal_a, int nVal_b, char nBr_b, BOOL isNormalizing = FALSE)
	{
		a.nVal = nVal_a;
		b.nVal = nVal_b;
		a.isClosed = (nBr_a == '[');
		b.isClosed = (nBr_b == ']');
		if(isNormalizing) Normalize();
	}

	void Set(int nVal_a, int nVal_b, char* pBr, BOOL isNormalizing = FALSE)
	{
		a.nVal = nVal_a;
		b.nVal = nVal_b;
		a.isClosed = (pBr[0] == '[');
		b.isClosed = (pBr[1] == ']');
		if(isNormalizing) Normalize();
	}

	///////////////////////////////////////////////////////////////
	//      r2
	///////////////////////////////////////////////////////////////
	int Compare(_Range r2)
	{
		_Range r1 = *this;

		r1.o2c();
		r2.o2c();

		if(r1.b < r2.a) return CR_BELOW;
		if(r1.a > r2.b) return CR_ABOVE;

		BOOL n1 = (r1.a >= r2.a) && (r1.a <= r2.b);
		BOOL n2 = (r1.b <= r2.b) && (r1.b >= r2.a);

		if((r1.a <= r2.a) && (r1.b >= r2.b)) return CR_B_IN_A;
		if(n1 && n2) return CR_A_IN_B;

		if((r1.a < r2.a) && n2) return CR_NOT_BELOW;
		if(n1 && (r1.b > r2.b)) return CR_NOT_ABOVE;

		r1.c2o();
		r2.c2o();

		return CR_ERROR;
	}

	///////////////////////////////////////////////////////////////
	//    r1  r2
	///////////////////////////////////////////////////////////////
	friend BOOL operator==(_Range r1, _Range r2) { return Eq(r1.a, r2.a) && Eq(r1.b, r2.b); }
	friend BOOL operator!=(_Range r1, _Range r2) { return !(r1 == r2); }
	friend BOOL operator<(_Range r1, _Range r2)  { return r1.Compare(r2) == CR_BELOW; }
	friend BOOL operator<=(_Range r1, _Range r2) { return r1.Compare(r2) == CR_NOT_BELOW; }
	friend BOOL operator>(_Range r1, _Range r2)  { return r1.Compare(r2) == CR_ABOVE; }
	friend BOOL operator>=(_Range r1, _Range r2) { return r1.Compare(r2) == CR_NOT_ABOVE; }
	friend BOOL operator>>(_Range r1, _Range r2) { return (r1.Compare(r2) == CR_A_IN_B) || (r1 == r2); }
	friend BOOL operator<<(_Range r1, _Range r2) { return (r1.Compare(r2) == CR_B_IN_A) || (r1 == r2); }

	///////////////////////////////////////////////////////////////
	//       
	///////////////////////////////////////////////////////////////
	void open2closed()
	{
		if(a.isClosed == FALSE) a.nVal++;
		if(b.isClosed == FALSE) b.nVal--;
		a.isClosed = b.isClosed = TRUE;
	}

	void closed2open()
	{
		if(a.isClosed == TRUE) a.nVal--;
		if(b.isClosed == TRUE) b.nVal++;
		a.isClosed = b.isClosed = FALSE;
	}

	////////////////////////////////////////////////////////////////////////
	//      lpszName
	////////////////////////////////////////////////////////////////////////
	BOOL IsName(LPCSTR lpszName)
	{
		return strcmp(szName, lpszName) == 0;
	}

	////////////////////////////////////////////////////////////////////////
	//   
	////////////////////////////////////////////////////////////////////////
	void SetName(LPCSTR lpszName)
	{
		set_name(lpszName);
	}

private:

	void o2c() { if(!a.isClosed) a.nVal++; if(!b.isClosed) b.nVal--; }
	void c2o() { if(!a.isClosed) a.nVal--; if(!b.isClosed) b.nVal++; }

	void set_name(LPCSTR lpszName)
	{
		if(lpszName == NULL)
		{
			szName[0] = '\0';
			return;
		}
		strcpy(szName, lpszName);
	}

} RANGE;




////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
class ERangeArray : public EArray<RANGE>  // ra
{
public:


	///////////////////////////////////////////////////////////////
	// 
	///////////////////////////////////////////////////////////////
	ERangeArray() {}
	ERangeArray(RANGE r) { Add(r); }
	ERangeArray(ERangeArray& src) { Append(src); }
	
	///////////////////////////////////////////////////////////////
	// 
	///////////////////////////////////////////////////////////////
	int GetRangeSize(int nIndex)
	{
		RANGE r = GetAt(nIndex);
		return r.b.nVal - r.a.nVal + 1 - (r.a.isClosed + r.b.isClosed);
	}

	///////////////////////////////////////////////////////////////
	// 
	///////////////////////////////////////////////////////////////
	void Normalize()
	{
		for(int i=0; i<GetCount(); i++)
		{
			GetItem(i).Normalize();
		}
	}

	///////////////////////////////////////////////////////////////
	// 
	///////////////////////////////////////////////////////////////
	void DeNormalize()
	{
		for(int i=0; i<GetCount(); i++)
		{
			GetItem(i).closed2open();
		}
	}

	///////////////////////////////////////////////////////////////
	// 
	///////////////////////////////////////////////////////////////
	int GetCountEmpty()
	{
		int empty = 0;
		for(int i=0; i<GetCount(); i++)
		{
			if(GetAt(i).IsEmpty()) empty++;
		}
		return empty;
	}

	///////////////////////////////////////////////////////////////
	// 
	///////////////////////////////////////////////////////////////
	void RemoveEmpty()
	{
		ERangeArray tmp(*this);

		for(int i=0; i<GetCount(); i++)
		{
			if(tmp.GetAt(i).IsEmpty()) tmp.Remove(i);
		}

		RemoveAll();
		Append(tmp);
	}

	///////////////////////////////////////////////////////////////
	// 
	///////////////////////////////////////////////////////////////
	const ERangeArray& operator=(ERangeArray& ra)	{ RemoveAll(); Append(ra); return *this; }
	const ERangeArray& operator=(RANGE r)			{ RemoveAll(); Add(r); return *this; }

	///////////////////////////////////////////////////////////////
	// 
	///////////////////////////////////////////////////////////////
	const ERangeArray& operator+=(ERangeArray& ra)	{ Append(ra); return *this; }
	const ERangeArray& operator+=(RANGE r)			{ Add(r); return *this; }

	///////////////////////////////////////////////////////////////
	// 
	///////////////////////////////////////////////////////////////
	friend ERangeArray operator+(ERangeArray& ra1, ERangeArray& ra2)	{ ERangeArray tmp = ra1; tmp.Append(ra2); return tmp; }
	friend ERangeArray operator+(ERangeArray& ra1, RANGE r2)			{ ERangeArray tmp = ra1; tmp.Add(r2); return tmp; }
	friend ERangeArray operator+(RANGE r1, ERangeArray& ra2)			{ ERangeArray tmp = ra2; tmp.Add(r1); return tmp; }
	friend ERangeArray operator+(RANGE r1, RANGE r2)					{ ERangeArray tmp; tmp.Add(r1); tmp.Add(r2); return tmp; }

	///////////////////////////////////////////////////////////////
	//  - Union
	///////////////////////////////////////////////////////////////
	friend ERangeArray Ux(RANGE r1, RANGE r2)
	{
		RANGE r;
		ERangeArray tmp;
		switch(r1.Compare(r2))
		{
			case CR_BELOW: tmp.Add(r1); tmp.Add(r2); break;
			case CR_NOT_BELOW: r.Set(r1.a, r2.b); tmp.Add(r); break;
			case CR_A_IN_B: tmp.Add(r2); break;
			case CR_B_IN_A: tmp.Add(r1); break;
			case CR_NOT_ABOVE: r.Set(r2.a, r1.b); tmp.Add(r); break;
			case CR_ABOVE: tmp.Add(r2); tmp.Add(r1); break;
		}
		tmp.Normalize();
		return tmp;
	}

	///////////////////////////////////////////////////////////////
	//  - Intersection
	///////////////////////////////////////////////////////////////
	friend ERangeArray Ix(RANGE r1, RANGE r2)
	{
		RANGE r;
		ERangeArray tmp;
		switch(r1.Compare(r2))
		{
			case CR_BELOW: tmp.Add(r); break;
			case CR_NOT_BELOW: r.Set(r2.a, r1.b); tmp.Add(r); break;
			case CR_A_IN_B: tmp.Add(r1); break;
			case CR_B_IN_A: tmp.Add(r2); break;
			case CR_NOT_ABOVE: r.Set(r1.a, r2.b); tmp.Add(r); break;
			case CR_ABOVE: tmp.Add(r); break;
		}
		tmp.Normalize();
		return tmp;
	}

	///////////////////////////////////////////////////////////////
	//  - Difference
	///////////////////////////////////////////////////////////////
	friend ERangeArray Dx(RANGE r1, RANGE r2)
	{
		RANGE r;
		ERangeArray tmp;
		switch(r1.Compare(r2))
		{
			case CR_BELOW: tmp.Add(r1); break;
			case CR_NOT_BELOW: r.Set(r1.a, ~r2.a); tmp.Add(r); break;
			case CR_A_IN_B: tmp.Add(r); break;
			case CR_B_IN_A: r.Set(r1.a, ~r2.a); tmp.Add(r); r.Set(~r2.b, r1.b); tmp.Add(r); break;
			case CR_NOT_ABOVE: r.Set(~r2.b, r1.b); tmp.Add(r); break;
			case CR_ABOVE: tmp.Add(r1); break;
		}
		tmp.Normalize();
		return tmp;
	}

	///////////////////////////////////////////////////////////////
	//   - Symmetric difference
	///////////////////////////////////////////////////////////////
	friend ERangeArray Sx(RANGE r1, RANGE r2)
	{
		RANGE r;
		ERangeArray tmp;
		switch(r1.Compare(r2))
		{
			case CR_BELOW: tmp.Add(r1); tmp.Add(r2); break;
			case CR_NOT_BELOW: r.Set(r1.a, ~r2.a); tmp.Add(r); r.Set(~r1.b, r2.b); tmp.Add(r); break;
			case CR_A_IN_B: r.Set(r2.a, ~r1.a); tmp.Add(r); r.Set(~r1.b, r2.b); tmp.Add(r); break;
			case CR_B_IN_A: r.Set(r1.a, ~r2.a); tmp.Add(r); r.Set(~r2.b, r1.b); tmp.Add(r); break;
			case CR_NOT_ABOVE: r.Set(r2.a, ~r1.a); tmp.Add(r); r.Set(~r2.b, r1.b); tmp.Add(r); break;
			case CR_ABOVE: tmp.Add(r2); tmp.Add(r1); break;
		}
		tmp.Normalize();
		return tmp;
	}

	///////////////////////////////////////////////////////////////
	//  - Multiplication
	///////////////////////////////////////////////////////////////
	friend ERangeArray Mx(RANGE r1, RANGE r2)
	{
		RANGE r;
		ERangeArray tmp;
		switch(r1.Compare(r2))
		{
			case CR_BELOW: r.Set(r1.a, r2.b); tmp.Add(r); break;
			case CR_NOT_BELOW:
			case CR_A_IN_B:
			case CR_B_IN_A:
			case CR_NOT_ABOVE: tmp = Ux(r1, r2); break;
			case CR_ABOVE: r.Set(r2.a, r1.b); tmp.Add(r); break;
		}
		tmp.Normalize();
		return tmp;
	}

	///////////////////////////////////////////////////////////////
	//  - Partition
	///////////////////////////////////////////////////////////////
	friend ERangeArray Px(RANGE r1, RANGE r2)
	{
		RANGE r;
		ERangeArray tmp;
		switch(r1.Compare(r2))
		{
			case CR_BELOW: r.Set(r1.b, r2.a); tmp.Add(r); break;
			case CR_NOT_BELOW:
			case CR_A_IN_B:
			case CR_B_IN_A:
			case CR_NOT_ABOVE: tmp = Ix(r1, r2); break;
			case CR_ABOVE: r.Set(r2.b, r1.a); tmp.Add(r); break;
		}
		tmp.Normalize();
		return tmp;
	}

	///////////////////////////////////////////////////////////////
	//  - Between
	///////////////////////////////////////////////////////////////
	friend ERangeArray Bx(RANGE r1, RANGE r2)
	{
		RANGE r;
		ERangeArray tmp;
		switch(r1.Compare(r2))
		{
			case CR_BELOW: r.Set(~r1.b, ~r2.a); tmp.Add(r); break;
			case CR_NOT_BELOW:
			case CR_A_IN_B:
			case CR_B_IN_A:
			case CR_NOT_ABOVE: tmp.Add(r); break;
			case CR_ABOVE: r.Set(~r2.b, ~r1.a); tmp.Add(r); break;
		}
		tmp.Normalize();
		return tmp;
	}

	///////////////////////////////////////////////////////////////
	//
	///////////////////////////////////////////////////////////////
	void _Br(HDC hDC, EOR br, int x, int y, int cx, int cy, COLORREF cr)
	{
		if(br.isClosed) return;

		int xx = x - cx * br.nBr;

		_EcxFastColorRect(hDC, xx, y, cx, cy, cr);
	}

	///////////////////////////////////////////////////////////////
	//
	///////////////////////////////////////////////////////////////
	void DrawEmpty(HDC hDC, int x, int y, int cx, int nHeight)
	{
		HFONT hFont = ::CreateFont(nHeight, 0, 0, 0, FW_NORMAL, 0, 0, 0, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, PROOF_QUALITY, DEFAULT_PITCH|FF_DONTCARE, "Symbol");
		HFONT hFontOld = (HFONT)::SelectObject(hDC, hFont);
		char str[2]; str[0] = (char)0xC6; str[1] = 0x00;
		SIZE siz;
		::GetTextExtentPoint32(hDC, str, 1, &siz);
		int ddx = (cx - siz.cx) / 2;
		::SetBkMode(hDC, TRANSPARENT);
		::TextOut(hDC, x + ddx, y - 13, str, 1);
		::SelectObject(hDC, hFontOld);
		::DeleteObject(hFont);
	}

	///////////////////////////////////////////////////////////////
	//
	///////////////////////////////////////////////////////////////
	void Draw(HDC hDC, int x, int y, int r, ERangeArray* pRa = NULL)
	{
		char str[64];
		HFONT hFont;
		int i, dx, dy, delta;
		dx = 20; dy = 20; delta = 32;

		int rcx = delta * r + 2 * dx;
		int rcy = 280;

		ECanvas cnv;
		cnv.Create(hDC, rcx, rcy);


		hFont = ::CreateFont(-10, 0, 0, 0, FW_NORMAL, 0, 0, 0, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, PROOF_QUALITY, DEFAULT_PITCH|FF_DONTCARE, "Times New Roman");
		HFONT hFontOld = (HFONT)::SelectObject(cnv.m_hDC, hFont);


		_EcxFastColorRect(cnv.m_hDC, 0, 0, rcx, rcy, RGB(0,0,0));
		_EcxFastColorRect(cnv.m_hDC, 1, 1, rcx - 2, rcy - 2, RGB(255,255,255));
		_EcxFastColorRect(cnv.m_hDC, dx, dy, delta * r, 2, RGB(0,0,0));
		for(i=0; i<=r; i++)
		{
			if(i < r)
			{
				sprintf(str, "%d", i);
				int len = strlen(str);
				::SetBkColor(cnv.m_hDC, RGB(255,255,255));
				SIZE siz;
				::GetTextExtentPoint32(cnv.m_hDC, str, len, &siz);
				int ddx = (delta - siz.cx) / 2;
				::TextOut(cnv.m_hDC, dx + i * delta + ddx, dy - 13, str, len);
			}

			_EcxFastColorRect(cnv.m_hDC, dx + i * delta, dy - 4, 1, 9, RGB(0,0,0));
		}

		::SetBkMode(cnv.m_hDC, TRANSPARENT);

		for(i=0; i<GetCount(); i++)
		{
			RANGE range = GetAt(i);
			int x = dx + range.a.nVal * delta;
			int y = dy + 23 + i * 20;
			int cx = (range.b.nVal - range.a.nVal + 1) * delta + 1;
			int cy = 5;

			if(!range.IsEmpty())
			{
				sprintf(str, "%s %c%d, %d%c", range.szName, range.a.Br(), range.a.nVal, range.b.nVal, range.b.Br());
				int len = strlen(str);
				SIZE siz;
				::GetTextExtentPoint32(cnv.m_hDC, str, len, &siz);
				int ddx = (cx - siz.cx) / 2;
				::SetBkColor(cnv.m_hDC, RGB(255,255,255));
				::TextOut(cnv.m_hDC, x + ddx, y - 13, str, len);
			}
			else
				DrawEmpty(cnv.m_hDC, x, y, cx, -10);


			int r = __rand_step(160, 9, 10);
			int g = __rand_step(160, 9, 10);
			int b = __rand_step(160, 9, 10);

//			sprintf(str, "RGB(%d,%d,%d)", r, g, b);
//			int len = strlen(str);
//			SIZE siz;
//			::GetTextExtentPoint32(cnv.m_hDC, str, len, &siz);
//			int dk = (siz.cx <= x) ? -siz.cx - 3 : cx + 3;
//			::TextOut(cnv.m_hDC, x + dk, y - 4, str, len);

			int r1 = r + 10;
			int g1 = g + 10;
			int b1 = b + 10;

			_EcxFastColorRect(cnv.m_hDC, x, y, cx, rcy, RGB(r, g, b));
			if(!range.a.isClosed)
				_EcxFastColorRect(cnv.m_hDC, x, y, delta + 1, rcy, RGB(r1, g1, b1));
			if(!range.b.isClosed)
				_EcxFastColorRect(cnv.m_hDC, x + cx - delta - 1, y, delta + 1, rcy, RGB(r1, g1, b1));
			_EcxFastColorRect(cnv.m_hDC, x, y, cx, cy, RGB(148,148,148));
			int dd = delta;
			if(range.a == range.b) dd -= 2;
			_Br(cnv.m_hDC, range.a, x + 1, y + 1, dd, 3, RGB(255,255,255));
			_Br(cnv.m_hDC, range.b, x + cx - 1, y + 1, dd, 3, RGB(255,255,255));

		}


		if(pRa != NULL)
		{
			int k = GetCount() + 2;

			for(i=0; i<pRa->GetCount(); i++)
			{
				RANGE range = pRa->GetAt(i);
				int x = dx + range.a.nVal * delta;
				int y = dy + 23 + (i + k) * 20;
				int cx = (range.b.nVal - range.a.nVal + 1) * delta + 1;
				int cy = 5;

				if(!range.IsEmpty())
				{
					sprintf(str, "%c%d, %d%c", range.a.Br(), range.a.nVal, range.b.nVal, range.b.Br());
					int len = strlen(str);
					SIZE siz;
					::GetTextExtentPoint32(cnv.m_hDC, str, len, &siz);
					int ddx = (cx - siz.cx) / 2;
					::SetBkColor(cnv.m_hDC, RGB(255,255,255));
					::TextOut(cnv.m_hDC, x + ddx, y - 13, str, len);
				}
				else
					DrawEmpty(cnv.m_hDC, x, y, cx, -10);


				int r = __rand_step(160, 9, 10);
				int g = __rand_step(160, 9, 10);
				int b = __rand_step(160, 9, 10);

//				sprintf(str, "RGB(%d,%d,%d)", r, g, b);
//				int len = strlen(str);
//				SIZE siz;
//				::GetTextExtentPoint32(cnv.m_hDC, str, len, &siz);
//				int dk = (siz.cx <= x) ? -siz.cx - 3 : cx + 3;
//				::TextOut(cnv.m_hDC, x + dk, y - 4, str, len);

				int r1 = r + 10;
				int g1 = g + 10;
				int b1 = b + 10;

				_EcxFastColorRect(cnv.m_hDC, x, y, cx, rcy, RGB(r, g, b));
				if(!range.a.isClosed)
					_EcxFastColorRect(cnv.m_hDC, x, y, delta + 1, rcy, RGB(r1, g1, b1));
				if(!range.b.isClosed)
					_EcxFastColorRect(cnv.m_hDC, x + cx - delta - 1, y, delta + 1, rcy, RGB(r1, g1, b1));
				_EcxFastColorRect(cnv.m_hDC, x, y, cx, cy, RGB(196,148,128));
				int dd = delta;
				if(range.a == range.b) dd -= 2;
				_Br(cnv.m_hDC, range.a, x + 1, y + 1, dd, 3, RGB(255,255,255));
				_Br(cnv.m_hDC, range.b, x + cx - 1, y + 1, dd, 3, RGB(255,255,255));

			}
		}



		cnv.Draw(x, y);

		::SelectObject(cnv.m_hDC, hFontOld);
		::DeleteObject(hFont);
	}



};



////////////////////////////////////////////////////////////////////////////
#endif // __ERANGEARRAY_H__
