// ETree.h : header file
//

#ifndef __ETREE_H__
#define __ETREE_H__
#pragma once


/////////////////////////////////////////////////////////////////////////////
//   
/////////////////////////////////////////////////////////////////////////////
#include "shellapi.h"

#include "..\FileInputOutput\FileInputOutput.h"
#include "..\FileInputOutput\FileInputOutputFunc.h"
#include "..\FileInputOutput\EPath.h"

#include "..\Shell\CommonControls\TreeViewControls\TreeViewControls.h"
#include "..\Shell\CommonControls\TreeViewControls\TreeViewControlsFunc.h"




#ifdef USED_TREE
/////////////////////////////////////////////////////////////////////////////
//  ETree.h
/////////////////////////////////////////////////////////////////////////////
#define VERSION_ETREE   "21.10.2013"



/////////////////////////////////////////////////////////////////////////////
//  det  nType1  nType2
// det = 4 * nType1 + nType2
/////////////////////////////////////////////////////////////////////////////
//     +-----+--------+--------+---------------+
//     | det | nType1 | nType2 |  |
//     +-----+--------+--------+---------------+
//     |   0 |   L    |   L    | identical     |
//     |   1 |   L    |   N    |               |
//     |   2 |   L    |   A    | incompatible  |
//     |   3 |        |        |               |
//     +-----+--------+--------+---------------+
//     |   4 |   N    |   L    |               |
//     |   5 |   N    |   N    | identical     |
//     |   6 |   N    |   A    | incompatible  |
//     |   7 |        |        |               |
//     +-----+--------+--------+---------------+
//     |   8 |   A    |   L    | incompatible  |
//     |   9 |   A    |   N    | incompatible  |
//     |  10 |   A    |   A    | identical     |
//     |  11 |        |        |               |
//     +-----+--------+--------+---------------+
//     |  12 |        |        |               |
//     |  13 |        |        |               |
//     |  14 |        |        |               |
//     |  15 |        |        |               |
//     +-----+--------+--------+---------------+


/////////////////////////////////////////////////////////////////////////////
//  
/////////////////////////////////////////////////////////////////////////////
enum _TreeEntryType
{
	TET_LEAF,   // L - 
	TET_NODE,   // N - 
	TET_ATTR,   // A - 
};

static char* _TreeEntryTypeStr[] =
{
	"LEAF",
	"NODE",
	"ATTR",
};




/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
enum _TreeErrorType
{
	ERR_OK,		// 
	ERR_NOT_PARENT_FOR_ATTR,	// 
	ERR_INCORRECT_INSERT,	// 
	ERR_INVALID_SWAP,	// 
	ERR_4,	// 
	ERR_5,	// 
};

static char* _TreeErrorTypeStr[] =
{
	"ERR_OK",
	"ERR_NOT_PARENT_FOR_ATTR",
	"ERR_INCORRECT_INSERT",
	"ERR_INVALID_SWAP",
	"4",
	"5",
};


/////////////////////////////////////////////////////////////////////////////
//   
/////////////////////////////////////////////////////////////////////////////
enum _TreePrintMode
{
	TPM_STRUCTURE,	// 
	TPM_XML,		// 
	TPM_CUSTOM,		// 
	TPM_CONFIG,		// 
};


/////////////////////////////////////////////////////////////////////////////
//  
/////////////////////////////////////////////////////////////////////////////
enum _TreeInsertMode
{
	TIM_BEFOR,		// 
	TIM_AFTER,		// 
};


/////////////////////////////////////////////////////////////////////////////
//  
/////////////////////////////////////////////////////////////////////////////
enum _TreeSortMode
{
	TSM_NO_SORT		= 0x00000000,		// 
	TSM_NODE_LEAF	= 0x00000001,		// NL
	TSM_LEAF_NODE	= 0x00000002,		// LN
	TSM_ASC_NAME	= 0x00000010,		// a-z
	TSM_DESC_NAME	= 0x00000020,		// z-a
	TSM_ASC_DATA	= 0x00000100,		// a-z
	TSM_DESC_DATA	= 0x00000200,		// z-a
};


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////










/////////////////////////////////////////////////////////////////////////////
//     
/////////////////////////////////////////////////////////////////////////////
struct _TreeEntry;


/////////////////////////////////////////////////////////////////////////////
//   
/////////////////////////////////////////////////////////////////////////////
typedef struct _AltEntry  // ae
{
	_TreeEntry* first;	//   
	_TreeEntry* last;	//   
	int nCount;			//    
	int nNode;			//   

} ALTENTRY;



/////////////////////////////////////////////////////////////////////////////
//     
/////////////////////////////////////////////////////////////////////////////
typedef   void (CALLBACK* PFN_DELETE_DATA) (_TreeEntry*);


/////////////////////////////////////////////////////////////////////////////
//   
/////////////////////////////////////////////////////////////////////////////
typedef struct _TreeEntry  // te
{
	int nType;					//  
	int nLevel;					//  

	_TreeEntry* prev;			//  
	_TreeEntry* next;			//  
	_TreeEntry* parent;			//   

	ALTENTRY child;				//   
	ALTENTRY attr;				//  

	char* szEntryName;			//  
	char* szEntryData;			//  
	
	void* pData;				//  
	int nDataSize;				//   
	PFN_DELETE_DATA pfnDelete;	//    
	
	
	int nUserData;				//


	////////////////////////////////////////////////////////////////////////
	// 
	////////////////////////////////////////////////////////////////////////
	_TreeEntry()
	{
		memset(this, 0, sizeof(_TreeEntry));
	}

	////////////////////////////////////////////////////////////////////////
	// .
	////////////////////////////////////////////////////////////////////////
	~_TreeEntry()
	{
		if(szEntryName != NULL) delete [] szEntryName;
		if(szEntryData != NULL) delete [] szEntryData;
		if(pfnDelete != NULL) pfnDelete(this);
	}


	////////////////////////////////////////////////////////////////////////
	//     
	////////////////////////////////////////////////////////////////////////
	BOOL IsNode()		{ return child.first != NULL; }
	BOOL IsFirst()		{ return prev == NULL; }
	BOOL IsLast()		{ return next == NULL; }
	BOOL IsOnce()		{ return (prev == NULL) && (next == NULL); }
	BOOL IsChild()		{ return parent != NULL; }
	BOOL IsChildFirst()	{ return (parent != NULL) && (prev == NULL); }
	BOOL IsChildLast()	{ return (parent != NULL) && (next == NULL); }


	////////////////////////////////////////////////////////////////////////
	//      lpszName
	////////////////////////////////////////////////////////////////////////
	BOOL IsName(LPCTSTR lpszName)
	{
		return strcmp(szEntryName, lpszName) == 0;
	}

	////////////////////////////////////////////////////////////////////////
	//  
	////////////////////////////////////////////////////////////////////////
	void Copy(_TreeEntry* pTe)
	{
		if(pTe == NULL) return;
		nType = pTe->nType;
		_EcxStrDup(&szEntryName, pTe->szEntryName);
		_EcxStrDup(&szEntryData, pTe->szEntryData);
		_EcxMemDup((BYTE**)&pData, (BYTE*)pTe->pData, pTe->nDataSize);
		nDataSize = pTe->nDataSize;
		pfnDelete = pTe->pfnDelete;
	}

} TREEENTRY;



////////////////////////////////////////////////////////////////////////////
//     
////////////////////////////////////////////////////////////////////////////
#define FIELD_NAME	0
#define FIELD_DATA	1


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
#define ENTRY_GET_NAME	0
#define ENTRY_GET_DATA	1


////////////////////////////////////////////////////////////////////////////
//   nMode  InsertEntry
////////////////////////////////////////////////////////////////////////////
#define INSERT_BEFOR	0
#define INSERT_AFTER	1


/////////////////////////////////////////////////////////////////////////////
//  -
/////////////////////////////////////////////////////////////////////////////
typedef   BOOL (CALLBACK* PFN_TREE_ENUM_PROC)	(TREEENTRY*, void**);
typedef   BOOL (CALLBACK* PFN_TREE_SORT_PROC)	(TREEENTRY*, TREEENTRY*, DWORD, void**);
typedef   BOOL (CALLBACK* PFN_TREE_PRINT_PROC)	(TREEENTRY*, void**);


////////////////////////////////////////////////////////////////////////////
//   dwEnumMode   ETree::Enum
////////////////////////////////////////////////////////////////////////////
#define ENUM_FIRST_ENTRY		0x00000001	// 1
#define ENUM_NEXTS_ENTRY		0x00000002	// 2,3,...
#define ENUM_ADD_ENTRIES		0x00000004	//

#define ENUM_FIRST_CHILD		0x00000010	// 1
#define ENUM_NEXTS_CHILD		0x00000020	// 2,3,...
#define ENUM_ADD_CHILDS			0x00000040	//

#define ENUM_ATTR_ENTRIES		0x00000100	//
#define ENUM_ATTR_CHILDS		0x00000200	//

#define ENUM_ALL_LEVELS			0x00010000	//


#define ENUMERATE_ENTRIES		ENUM_FIRST_ENTRY | ENUM_NEXTS_ENTRY | ENUM_ADD_ENTRIES
#define ENUMERATE_CHILDS		ENUM_FIRST_CHILD | ENUM_NEXTS_CHILD | ENUM_ADD_CHILDS
#define ENUMERATE_ALL			0x3FFFFFFF




/////////////////////////////////////////////////////////////////////////////
//  
/////////////////////////////////////////////////////////////////////////////
class ETree  // tree
{
// Construction
public:
	ETree();
	~ETree();


	BOOL Enum(TREEENTRY* pTe, DWORD dwEnumMode, PFN_TREE_ENUM_PROC lpEnumProc = EnumProc, PFN_TREE_ENUM_PROC lpEqProc = EqProc, void** ppData = NULL);
	friend BOOL CALLBACK EnumProc(TREEENTRY* pTe, void** ppData);
	friend BOOL CALLBACK EqProc(TREEENTRY* pTe, void** ppData);
	int m_nLevel;

	TREEENTRY* Add(TREEENTRY* pTe, TREEENTRY* pTeSrc);
	TREEENTRY* Add(TREEENTRY* pTe, int nType, LPCSTR lpszEntryName = NULL, LPCSTR lpszEntryData = NULL);
	TREEENTRY* Insert(TREEENTRY* pTe, int nMode, TREEENTRY* pTeSrc);
	TREEENTRY* Insert(TREEENTRY* pTe, int nMode, int nType, LPCSTR lpszEntryName = NULL, LPCSTR lpszEntryData = NULL);

	BOOL Swap(TREEENTRY* pTe1, TREEENTRY* pTe2);
	void Sort(TREEENTRY* pTe, DWORD dwSortMode, PFN_TREE_SORT_PROC lpSortProc = SortProc, void** ppData = NULL);
	void SortChild(TREEENTRY* pTe, DWORD dwSortMode, PFN_TREE_SORT_PROC lpSortProc = SortProc, void** ppData = NULL);
	void SortTree(TREEENTRY* pTe = NULL, DWORD dwSortMode = TSM_NODE_LEAF|TSM_ASC_NAME);
	friend BOOL CALLBACK SortProc(TREEENTRY* pTe1, TREEENTRY* pTe2, DWORD dwSortMode, void** ppData);
	
	void Level(TREEENTRY* pTe, int nDelta);
	void Remove();
	void Clear(TREEENTRY* pTe);
	BOOL Delete(TREEENTRY* pTe);

	void Print(TREEENTRY* pTe, int nMode, PFN_TREE_PRINT_PROC lpPrintProc = PrintProc, void** ppData = NULL);
	void _Print(TREEENTRY* pTe, int nMode, PFN_TREE_PRINT_PROC lpPrintProc = NULL, void** ppData = NULL);
	friend BOOL CALLBACK PrintProc(TREEENTRY* pTe, void** ppData);

	void SetTreeCtrl(TREEENTRY* pTe, HWND hTreeCtrl, HIMAGELIST hImageList);
	void _SetTreeCtrl(TREEENTRY* pTe, HWND hTreeCtrl, HTREEITEM hParent, HIMAGELIST hImageList, BOOL isAttrChild);

	void _AddCounts(ALTENTRY* pAe, int nType);
	void _SubCounts(ALTENTRY* pAe, int nType);
	
	
	EString GetEntryStr(TREEENTRY* pTe);
	EString GetEntryPtrStr(TREEENTRY* pTe);
	EString GetSysVarStr();


	TREEENTRY* FindEntry(char* szPath, char* szName, int nEntryType);
	TREEENTRY* GetEntry(char* szPath, int nNumber);

	BOOL GetBoolean(char* szPath, char* szName);
	int GetInteger(char* szPath, char* szName);
	double GetDouble(char* szPath, char* szName);
	char* GetString(char* szPath, char* szName);
	char* GetEntryName(char* szPath, int nNumber);
	char* GetEntryData(char* szPath, int nNumber);
	char* GetEntryString(char* szPath, int nNumber, int nStringType);


	TREEENTRY* _FindPath(TREEENTRY* pTe);
	TREEENTRY* _FindEntry(char* szName, TREEENTRY* pTe, int nEntryType);
	TREEENTRY* _GetEntry(int nNumber, TREEENTRY* pTe);


	BOOL _GetName(char** ppBuf, char** ppName, int* pnType, int nGetType);
	BOOL _GetDada(char** ppBuf, char** ppData, int* pnType, BOOL isClearNonInformation);

	EString GetEntryNameStr(TREEENTRY* pTe, int nNull = 0, BOOL isQuote = FALSE);
	EString GetEntryDataStr(TREEENTRY* pTe, int nNull = 0, BOOL isQuote = FALSE);




	ALTENTRY m_base;		//  ()
	int m_nCount;

	BOOL m_isAttrFirst;
	void _SetAttrChild(TREEENTRY* pTe, TREEENTRY** ppTe1, TREEENTRY** ppTe2);



	EPath m_path;
	EString m_str;


	int m_nError;



//private:




	/////////////////////////////////////////////////////////////
	//    nType1  nType2
	/////////////////////////////////////////////////////////////
	inline int _GetDet(int nType1, int nType2)
	{
		return 4 * nType1 + nType2;
	}

	/////////////////////////////////////////////////////////////
	// TRUE -    (LA, NA, AL, AN)
	/////////////////////////////////////////////////////////////
	inline BOOL _IsIncompatibleTypes(int nType1, int nType2)
	{
		int det = _GetDet(nType1, nType2);
		return (det == 2 || det == 6 || det == 8 || det == 9);
	}

	/////////////////////////////////////////////////////////////
	// TRUE -    (LL, NN, AA)
	/////////////////////////////////////////////////////////////
	inline BOOL _IsIdenticalTypes(int nType1, int nType2)
	{
		int det = _GetDet(nType1, nType2);
		return (det == 0 || det == 5 || det == 10);
	}

	/////////////////////////////////////////////////////////////
	//    
	/////////////////////////////////////////////////////////////
	inline ALTENTRY* _GetBranch(TREEENTRY* pTe)
	{
		return (pTe->parent) ? ((pTe->nType == TET_ATTR) ? &pTe->parent->attr : &pTe->parent->child) : &m_base;
	}

};



////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline ETree::ETree()
{
	memset(&m_base, 0, sizeof(ALTENTRY));
	m_nCount = 0;
	m_nError = ERR_OK;
	m_isAttrFirst = TRUE;
	m_nLevel = -1;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline ETree::~ETree()
{
	Remove();
}


////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////
inline BOOL ETree::Enum(TREEENTRY* pTe, DWORD dwEnumMode, PFN_TREE_ENUM_PROC lpEnumProc, PFN_TREE_ENUM_PROC lpEqProc, void** ppData)
{
	if(pTe == NULL) pTe = m_base.first;
	if(pTe == NULL) return FALSE;
	TREEENTRY* tmp = pTe;
	int nNumber = 0;

	if(dwEnumMode == 0) return FALSE;

	//    -
	BOOL isCallProc = FALSE;
	BOOL isTranslate, isEq;
	if(lpEnumProc == NULL) lpEnumProc = EnumProc;
	if(lpEqProc == NULL) lpEqProc = EqProc;

	while(tmp)
	{
		isTranslate = FALSE;
		tmp->nUserData = nNumber;


		//   
		isEq = lpEqProc(tmp, ppData);


		//      
		if(_is_flag(dwEnumMode, ENUM_FIRST_ENTRY) && nNumber == 0)
		{
			isTranslate = TRUE;
		}

		//      
		if(_is_flag(dwEnumMode, ENUM_NEXTS_ENTRY) && nNumber > 0)
		{
			isTranslate = TRUE;
		}

		//    
		if(isTranslate)
		{
			//    
			if(isEq)
			{
				//     
				if(_is_flag(dwEnumMode, ENUM_ADD_ENTRIES))
				{
					//   -
					isCallProc = TRUE;

					//  -
					if(!lpEnumProc(tmp, ppData)) break;
				}
			}
		}


		//    
		if(isTranslate)
		{
			//  
			BOOL is = FALSE;

			//    
			if(m_isAttrFirst)
			{
label_attr:
				//     
				//   
				if(_is_flag(dwEnumMode, ENUM_ATTR_ENTRIES) && tmp->attr.first != NULL)
				{
					//  
					isCallProc |= Enum(tmp->attr.first, ENUMERATE_ENTRIES, lpEnumProc, lpEqProc, ppData);
				}

				//      
				//   
				//   
				if(_is_flag(dwEnumMode, ENUM_ATTR_CHILDS) && _is_flag(dwEnumMode, 0x80000000) && tmp->attr.first != NULL)
				{
					//    
					isCallProc |= Enum(tmp->attr.first, ENUMERATE_ENTRIES, lpEnumProc, lpEqProc, ppData);
				}
			}

			//   
			if(is) goto label_end;

			//     
			if(_is_flag(dwEnumMode, ENUM_FIRST_CHILD|ENUM_NEXTS_CHILD) && tmp->child.first != NULL)
			{
				DWORD dwMode = dwEnumMode & 0x00F00000;
				if(_is_flag(dwEnumMode, ENUM_FIRST_CHILD)) _set_flag(dwMode, ENUM_FIRST_ENTRY);
				if(_is_flag(dwEnumMode, ENUM_NEXTS_CHILD)) _set_flag(dwMode, ENUM_NEXTS_ENTRY);
				if(_is_flag(dwEnumMode, ENUM_ATTR_CHILDS)) _set_flag(dwMode, ENUM_ATTR_CHILDS|0x80000000);
				if(_is_flag(dwEnumMode, ENUM_ALL_LEVELS))
				{
					if(_is_flag(dwEnumMode, ENUM_FIRST_CHILD)) _set_flag(dwMode, ENUM_FIRST_CHILD|ENUM_ALL_LEVELS);
					if(_is_flag(dwEnumMode, ENUM_NEXTS_CHILD)) _set_flag(dwMode, ENUM_NEXTS_CHILD|ENUM_ALL_LEVELS);
				}

				//     
				if(_is_flag(dwEnumMode, ENUM_ADD_CHILDS))
				{
					_set_flag(dwMode, ENUM_ADD_ENTRIES);
					if(_is_flag(dwEnumMode, ENUM_ALL_LEVELS)) _set_flag(dwMode, ENUM_ADD_CHILDS);
				}
			
				dwMode |= 0x00200000;

				//   
				isCallProc |= Enum(tmp->child.first, dwMode, lpEnumProc, lpEqProc, ppData);
			}

			//    
			if(!m_isAttrFirst)
			{
				//  
				is = TRUE;
				goto label_attr;
			}

label_end:
			;
		}


		//  
		tmp = tmp->next;
		nNumber++;
		continue;
	}

	return isCallProc;
}


////////////////////////////////////////////////////////////////////////////
// -     ETree::Enum
////////////////////////////////////////////////////////////////////////////
inline BOOL CALLBACK EnumProc(TREEENTRY* pTe, void** ppData)
{
	ETree* pTree = (ETree*)ppData;
	TREEENTRY* tmp = pTe;

	static int i = 1;
	EString s1, s2;
	EString spaces((tmp->nType == TET_NODE) ? ' ' : ' ', 4 * tmp->nLevel);

	s1.Format("%d) %s(%d.%d) : %s\r\n", i, _TreeEntryTypeStr[tmp->nType], tmp->nLevel, tmp->nUserData, pTree->GetEntryNameStr(tmp, 2, TRUE));
	pTree->m_str += spaces + s1;
	i++;

	return TRUE;
}


////////////////////////////////////////////////////////////////////////////
//        ETree::Enum
////////////////////////////////////////////////////////////////////////////
inline BOOL CALLBACK EqProc(TREEENTRY* pTe, void** ppData)
{
	ETree* pTree = (ETree*)ppData;
	TREEENTRY* tmp = pTe;

//	EString s1;
//	EString spaces((tmp->nType == TET_NODE) ? '-' : ' ', 4 * tmp->nLevel);
//	s1.Format("[%d]\r\n", tmp->nLevel);
//	pTree->m_str += spaces + s1;

	if(pTree->m_nLevel == -1) return TRUE;
	return tmp->nLevel == pTree->m_nLevel;
}


////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////
inline void ETree::Remove()
{
	if(m_base.first == NULL) return;
	Clear(m_base.first);
	memset(&m_base, 0, sizeof(ALTENTRY));
	m_nCount = 0;
	m_path.Clear();
}


////////////////////////////////////////////////////////////////////////////
//   ,    pTe
////////////////////////////////////////////////////////////////////////////
inline void ETree::Clear(TREEENTRY* pTe)
{
	if(pTe == NULL) return;
	TREEENTRY* tmp = pTe;
	do
	{
		TREEENTRY* next = tmp->next;
		if(tmp->attr.first != NULL) Clear(tmp->attr.first);
		if(tmp->child.first != NULL) Clear(tmp->child.first);
//		_SubCounts(&m_base, tmp->nType);
		m_nCount--;
		delete tmp;
		tmp = next;

	} while(tmp);
}


////////////////////////////////////////////////////////////////////////////
//       nDelta
////////////////////////////////////////////////////////////////////////////
inline void ETree::Level(TREEENTRY* pTe, int nDelta)
{
	if(pTe == NULL) return;
	if(pTe->attr.first != NULL) Level(pTe->attr.first, nDelta);
	if(pTe->child.first != NULL) Level(pTe->child.first, nDelta);
	pTe->nLevel += nDelta;
}


////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////
inline BOOL ETree::Delete(TREEENTRY* pTe)
{
	if(pTe == NULL) return FALSE;

	// 
	if(pTe->next != NULL) pTe->next->prev = pTe->prev;
	if(pTe->prev != NULL) pTe->prev->next = pTe->next;

	//   
	Clear(pTe->attr.first);
	Clear(pTe->child.first);

	//    
	ALTENTRY* pAe = _GetBranch(pTe);
	
	//    
	if(pTe->prev == NULL) pAe->first = pTe->next;
	if(pTe->next == NULL) pAe->last = pTe->prev;

	//  
	_SubCounts(pAe, pTe->nType);
	m_nCount--;

	//   
	delete pTe;
	return TRUE;
}


////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////
inline TREEENTRY* ETree::Add(TREEENTRY* pTe, TREEENTRY* pTeSrc)
{
	// ,    
	m_nError = ERR_OK;

	if(pTeSrc == NULL) return NULL;

	//     
//	if(IsIncorrectEntryType(nType))
//	{
//		m_nError = ERR_INCORRECT_ENTRY_TYPE;
//		return NULL;
//	}

	//     
	if(pTe)
	{
		//   
		if(!__in11(_GetDet(pTe->nType, pTeSrc->nType), 2, 6))
		{
			// -
			pTe = pTe->parent;
		}
	}
	else
	{
		//    
		if(pTeSrc->nType == TET_ATTR)
		{
			m_nError = ERR_NOT_PARENT_FOR_ATTR;
			return NULL;
		}
	}

	//   
	TREEENTRY* pTeAdd = new TREEENTRY;
	pTeAdd->nType = pTeSrc->nType;

	//    
	pTeAdd->Copy(pTeSrc);


	//    
	ALTENTRY* pAe = (pTe) ? ((pTeSrc->nType == TET_ATTR) ? &pTe->attr : &pTe->child) : &m_base;

	//      
	if(pAe->first == NULL)
	{
		pAe->first = pTeAdd;
	}
	else
	{
		//    
		pTeAdd->prev = pAe->last;
		pAe->last->next = pTeAdd;
	}

	//     
	pAe->last = pTeAdd;

	//    
	if(pTe)
	{
		//   
		pTeAdd->parent = pTe;
		pTeAdd->nLevel = pTe->nLevel + 1;
	}

	//  
	_AddCounts(pAe, pTeSrc->nType);
	m_nCount++;

	//  
	return pTeAdd;
}



inline TREEENTRY* ETree::Add(TREEENTRY* pTe, int nType, LPCSTR lpszEntryName, LPCSTR lpszEntryData)
{
	TREEENTRY te;
	te.nType = nType;
	te.szEntryName = (char*)lpszEntryName;
	te.szEntryData = (char*)lpszEntryData;

	pTe = Add(pTe, &te);

	te.szEntryName = NULL;
	te.szEntryData = NULL;

	return pTe;
}


////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////
inline TREEENTRY* ETree::Insert(TREEENTRY* pTe, int nMode, TREEENTRY* pTeSrc)
{
	// ,    
	m_nError = ERR_OK;

	if(pTeSrc == NULL) return NULL;

	//     
//	if(IsIncorrectEntryType(nType))
//	{
//		m_nError = ERR_INCORRECT_ENTRY_TYPE;
//		return NULL;
//	}

	//     
	if(pTe)
	{
		//   
		if(_IsIncompatibleTypes(pTe->nType, pTeSrc->nType))
		{
			m_nError = ERR_INCORRECT_INSERT;
			return NULL;
		}
	}

	//   
	TREEENTRY* pTeIns = new TREEENTRY;
	pTeIns->nType = pTeSrc->nType;

	//    
	pTeIns->Copy(pTeSrc);

	//    
	ALTENTRY* pAe = _GetBranch(pTe);

	//  
	if(nMode == TIM_BEFOR)
	{
		pTeIns->next = pTe;
		pTeIns->prev = pTe->prev;
		if(pTe->prev)
			pTe->prev->next = pTeIns;
		else
			pAe->first = pTeIns;
		pTe->prev = pTeIns;
	}

	//  
	else
	{
		pTeIns->next = pTe->next;
		pTeIns->prev = pTe;
		if(pTe->next)
			pTe->next->prev = pTeIns;
		else
			pAe->last = pTeIns;
		pTe->next = pTeIns;
	}

	//   
	pTeIns->parent = pTe;
	pTeIns->nLevel = pTe->nLevel;

	//  
	_AddCounts(pAe, pTeSrc->nType);
	m_nCount++;

	//  
	return pTeIns;
}


inline TREEENTRY* ETree::Insert(TREEENTRY* pTe, int nMode, int nType, LPCSTR lpszEntryName, LPCSTR lpszEntryData)
{
	TREEENTRY te;
	te.nType = nType;
	te.szEntryName = (char*)lpszEntryName;
	te.szEntryData = (char*)lpszEntryData;

	pTe = Insert(pTe, nMode, &te);

	te.szEntryName = NULL;
	te.szEntryData = NULL;

	return pTe;
}


////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////
inline BOOL ETree::Swap(TREEENTRY* pTe1, TREEENTRY* pTe2)
{
	//  
	m_nError = ERR_INVALID_SWAP;

	if(pTe1 == pTe2) return FALSE;
	if(pTe1 == NULL || pTe2 == NULL) return FALSE;

	//   
	if(_IsIncompatibleTypes(pTe1->nType, pTe2->nType)) return FALSE;

	m_nError = ERR_OK;


	//    
	if(pTe1->nLevel != pTe2->nLevel)
	{
		int k1 = pTe2->nLevel - pTe1->nLevel;
		Level(pTe1, k1);
		Level(pTe2, -k1);
	}

	//    
	ALTENTRY* a1 = _GetBranch(pTe1);
	ALTENTRY* a2 = _GetBranch(pTe2);

	//  
	_EcxSwap(&pTe1->parent, &pTe2->parent);

	//   
	TREEENTRY* e11 = pTe1->prev;
	TREEENTRY* e12 = pTe1->next;
	TREEENTRY* e21 = pTe2->prev;
	TREEENTRY* e22 = pTe2->next;

	//   
	if((pTe1->next == pTe2) || (pTe2->next == pTe1))
	{
		e12 = pTe1;
		e21 = pTe2;
	}
	
	//  
	if(e11)
		e11->next = pTe2;
	else
		a1->first = pTe2;

	if(e12)
		e12->prev = pTe2;
	else
		a1->last = pTe2;

	pTe1->next = e22;
	pTe1->prev = e21;

	//  
	if(e21)
		e21->next = pTe1;
	else
		a2->first = pTe1;

	if(e22)
		e22->prev = pTe1;
	else
		a2->last = pTe1;

	pTe2->next = e12;
	pTe2->prev = e11;

	return TRUE;
}


////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////
inline void ETree::Sort(TREEENTRY* pTe, DWORD dwSortMode, PFN_TREE_SORT_PROC lpSortProc, void** ppData)
{
	if(pTe == NULL) pTe = m_base.first;
	if(pTe == NULL) return;

	if(dwSortMode == TSM_NO_SORT) return;
	if(lpSortProc == NULL) lpSortProc = SortProc;

	//    
	ALTENTRY* pAe = _GetBranch(pTe);
	pTe = pAe->first;
	TREEENTRY* tmp = pTe;
	int nCount = pAe->nCount;

	//  
	BOOL isSorted = TRUE;
	BOOL isSwap = FALSE;

	do
	{
		pAe = _GetBranch(pTe);
		tmp = pAe->first;
		isSwap = FALSE;

		for(int i=0; i < nCount - 1; i++)
		{
			//  -
			if(lpSortProc(tmp, tmp->next, dwSortMode, ppData))
			{
				Swap(tmp, tmp->next);
				isSwap = TRUE;
				continue;
			}
			tmp = tmp->next;
		}
		isSorted = !isSwap;

	} while(!isSorted);
}


////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////
inline void ETree::SortChild(TREEENTRY* pTe, DWORD dwSortMode, PFN_TREE_SORT_PROC lpSortProc, void** ppData)
{
	if(pTe == NULL) return;
	Sort(pTe->child.first, dwSortMode, lpSortProc, ppData);
}


////////////////////////////////////////////////////////////////////////////
//   ,    pTe
////////////////////////////////////////////////////////////////////////////
inline void ETree::SortTree(TREEENTRY* pTe, DWORD dwSortMode)
{
	if(pTe == NULL) pTe = m_base.first;
	if(pTe == NULL) return;
	TREEENTRY* tmp = pTe;

	do
	{
		TREEENTRY* next = tmp->next;
		if(tmp->attr.first != NULL) SortTree(tmp->attr.first, dwSortMode);
		if(tmp->child.first != NULL) SortTree(tmp->child.first, dwSortMode);
		Sort(tmp, dwSortMode);
		tmp = next;

	} while(tmp);
}


////////////////////////////////////////////////////////////////////////////
// -     ETree::Sort
////////////////////////////////////////////////////////////////////////////
inline BOOL CALLBACK SortProc(TREEENTRY* pTe1, TREEENTRY* pTe2, DWORD dwSortMode, void** ppData)
{
	ETree* pTree = (ETree*)ppData;
	if(dwSortMode == TSM_NO_SORT) return FALSE;
	if(pTree->_IsIncompatibleTypes(pTe1->nType, pTe2->nType)) return FALSE;

	BOOL res = FALSE;
	int i, p = pTree->_GetDet(pTe1->nType, pTe2->nType);

	//   NL   LN
	if(_is_flag(dwSortMode, TSM_NODE_LEAF) && (p == 1)) return TRUE;

	//   LN   NL
	if(_is_flag(dwSortMode, TSM_LEAF_NODE) && (p == 4)) return TRUE;

	//   NL  LN    
	if(_is_flag(dwSortMode, TSM_NODE_LEAF|TSM_LEAF_NODE) && !pTree->_IsIdenticalTypes(pTe1->nType, pTe2->nType)) return FALSE;

	//        
	if(_is_flag(dwSortMode, TSM_ASC_DATA|TSM_ASC_DATA))
	{
		if(p == 0)
		{
			if((i = _EcxStrCompare(pTe1->szEntryData, pTe2->szEntryData, TRUE)) != 0)
			{
				res = (i > 0);
				if(_is_flag(dwSortMode, TSM_DESC_DATA)) _not(res);
				return res;
			}
		}
	}

	//        
	if(_is_flag(dwSortMode, TSM_ASC_NAME|TSM_DESC_NAME))
	{
		if((i = _EcxStrCompare(pTe1->szEntryName, pTe2->szEntryName, TRUE)) == 0) return FALSE;
		res = (i > 0);
		if(_is_flag(dwSortMode, TSM_DESC_NAME)) _not(res);
	}

	return res;
}


////////////////////////////////////////////////////////////////////////////
//       
////////////////////////////////////////////////////////////////////////////
inline void ETree::_AddCounts(ALTENTRY* pAe, int nType)
{
	pAe->nCount++;
	if(nType == TET_NODE) pAe->nNode++;
}


////////////////////////////////////////////////////////////////////////////
//       
////////////////////////////////////////////////////////////////////////////
inline void ETree::_SubCounts(ALTENTRY* pAe, int nType)
{
	pAe->nCount--;
	if(nType == TET_NODE) pAe->nNode--;
}




////////////////////////////////////////////////////////////////////////////
//    ,   
////////////////////////////////////////////////////////////////////////////
inline TREEENTRY* ETree::FindEntry(char* szPath, char* szName, int nEntryType)
{
	TREEENTRY* fnd = NULL;

	//      
//	if(nEntryType > CET_WORD_LIST) return NULL;

	// 
	TREEENTRY* pTe = m_base.first;
	if(pTe == NULL) return NULL;

	//    
	BOOL isSplited = m_path.Split(szPath);

	//    
	if(isSplited)
	{
		//    
		fnd = _FindPath(pTe);
		
		//      
		if(szName == NULL) return fnd;
		if(szName[0] == '\0') return fnd;

		//  
//		if(fnd != NULL) fnd = _FindEntry(szName, fnd->first, nEntryType);
		if(fnd != NULL) fnd = _FindEntry(szName, fnd->child.first, nEntryType);
	}

	return fnd;
}


////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////
inline TREEENTRY* ETree::_FindPath(TREEENTRY* pTe)
{
	TREEENTRY* fnd = NULL;

	// 
	TREEENTRY* tmp = pTe;
	if(pTe == NULL) return NULL;

	//       .  
	if(tmp->nLevel >= m_path.m_nCount) return NULL;

	//  
	char* name = m_path.m_ppEntry[tmp->nLevel];

	//   
	do
	{
		TREEENTRY* next = tmp->next;

		//        
		if(strstr(tmp->szEntryName, name) != NULL)
		{
			//   - 
			if(tmp->nType == TET_NODE)
			{
				//   
				TREEENTRY* alt = tmp->child.first;
				if(alt != NULL)
				{
					//    
					if(alt->nLevel == m_path.m_nCount)
					{
						fnd = tmp;
						break;
					}

					//  
//					fnd = _FindPath(tmp->first);
					fnd = _FindPath(tmp->child.first);
					if(fnd != NULL) break;
				}
			}
		}

		tmp = next;

	} while(tmp);

	return fnd;
}


////////////////////////////////////////////////////////////////////////////
//     
////////////////////////////////////////////////////////////////////////////
inline TREEENTRY* ETree::_FindEntry(char* szName, TREEENTRY* pTe, int nEntryType)
{
	TREEENTRY* fnd = NULL;

	//   
	TREEENTRY* tmp = pTe;
	if(tmp == NULL) return NULL;

	if(szName == NULL) return NULL;
	if(szName[0] == '\0') return NULL;

	//   
	do
	{
		TREEENTRY* next = tmp->next;

		//       
		if(strstr(tmp->szEntryName, szName) != NULL)
		{
			//       
			if(tmp->nType == nEntryType)
			{
				fnd = tmp;
				break;
			}
		}

		tmp = next;

	} while(tmp);

	return fnd;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline TREEENTRY* ETree::_GetEntry(int nNumber, TREEENTRY* pTe)
{
//	TREEENTRY* tmp = pTe->first;
	TREEENTRY* tmp = pTe->child.first;
	if(tmp == NULL) return NULL;
//	if(nNumber >= pTe->nEntryCount) return NULL;

	int i = 0;
	TREEENTRY* fnd = NULL;

	do
	{
		TREEENTRY* next = tmp->next;

		if(i == nNumber)
		{
			fnd = tmp;
			break;
		}

		tmp = next;
		i++;

	} while(tmp);

	return fnd;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline TREEENTRY* ETree::GetEntry(char* szPath, int nNumber)
{
	TREEENTRY *pte0, *pte1;

	pte0 = FindEntry(szPath, NULL, TET_NODE);
	if(pte0 == NULL) return NULL;
	pte1 = _GetEntry(nNumber, pte0);
	return pte1;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline char* ETree::GetEntryName(char* szPath, int nNumber)
{
	TREEENTRY* pte = GetEntry(szPath, nNumber);
	if(pte == NULL) return NULL;

	return pte->szEntryName;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline char* ETree::GetEntryData(char* szPath, int nNumber)
{
	TREEENTRY* pte = GetEntry(szPath, nNumber);
	if(pte == NULL) return NULL;

	return pte->szEntryData;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline char* ETree::GetEntryString(char* szPath, int nNumber, int nStringType)
{
	switch(nStringType)
	{
		case FIELD_NAME:
			return GetEntryName(szPath, nNumber);
			break;

		case FIELD_DATA:
			return GetEntryData(szPath, nNumber);
			break;
	}

	return NULL;
}


////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////
inline BOOL ETree::GetBoolean(char* szPath, char* szName)
{
	BOOL value = FALSE;
	TREEENTRY* pte = FindEntry(szPath, szName, 1);
	if(pte != NULL)
	{
		char* tmp = pte->szEntryData;
		if(is_char(*tmp, CCM_ALPHA))
		{
			int len = strlen(tmp);
			char* psz = new char[len + 1];
			strcpy(psz, tmp);
			::CharUpperBuff(psz, len);

			if(strcmp(psz, "TRUE") == 0) value = TRUE;
			if(strcmp(psz, "YES") == 0) value = TRUE;
			if(strcmp(psz, "FALSE") == 0) value = FALSE;
			if(strcmp(psz, "NO") == 0) value = FALSE;

			delete [] psz;
		}
		else
		{
			int i = __strtol(tmp);
			value = (i == 0) ? FALSE : TRUE;
		}
	}
	return value;
}


////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////
inline int ETree::GetInteger(char* szPath, char* szName)
{
	int value = 0;
	TREEENTRY* pte = FindEntry(szPath, szName, 1);
	if(pte != NULL)
	{
		char* tmp = pte->szEntryData;
		value = __strtol(tmp);
	}
	return value;
}


////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////
inline double ETree::GetDouble(char* szPath, char* szName)
{
	double value = 0.0;
	TREEENTRY* pte = FindEntry(szPath, szName, 1);
	if(pte != NULL)
	{
		char* tmp = pte->szEntryData;
		value = __strtod(tmp);
	}
	return value;
}


////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////
inline char* ETree::GetString(char* szPath, char* szName)
{
	char* value = NULL;
	TREEENTRY* pte = FindEntry(szPath, szName, 1);
	if(pte != NULL)
	{
		value = pte->szEntryData;
	}
	return value;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline EString ETree::GetEntryNameStr(TREEENTRY* pTe, int nNull, BOOL isQuote)
{
	if(pTe == NULL) return "";
	EString str;
	str.SetString(pTe->szEntryName, nNull);
	if(pTe->szEntryName != NULL) str.Quote(isQuote);
	return str;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline EString ETree::GetEntryDataStr(TREEENTRY* pTe, int nNull, BOOL isQuote)
{
	if(pTe == NULL) return "";
	EString str;
	str.SetString(pTe->szEntryData, nNull);
	if(pTe->szEntryData != NULL) str.Quote(isQuote);
	return str;
}




////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////
inline void ETree::Print(TREEENTRY* pTe, int nMode, PFN_TREE_PRINT_PROC lpPrintProc, void** ppData)
{
	m_str = "";
	_Print(pTe, nMode, lpPrintProc, ppData);
}


////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////
inline void ETree::_Print(TREEENTRY* pTe, int nMode, PFN_TREE_PRINT_PROC lpPrintProc, void** ppData)
{
	if(pTe == NULL) pTe = m_base.first;
	if(pTe == NULL) return;

	EString spaces, s1, s2, s3, s11;
	TREEENTRY* tmp = pTe;
	if(lpPrintProc == NULL) lpPrintProc = PrintProc;


	switch(nMode)
	{
		//////////////////////////////////////////////////////////////
		// 
		//////////////////////////////////////////////////////////////
		case TPM_STRUCTURE:
		{
			do
			{
				spaces.str_set(' ', 4 * tmp->nLevel);
				s2 = "";
				
				if(tmp->nType == TET_NODE)
				{
					if(m_isAttrFirst)
						s2.Format("(%d,A%d,N%d)", tmp->child.nCount, tmp->attr.nCount, tmp->child.nNode);
					else
						s2.Format("(%d,N%d,A%d)", tmp->child.nCount, tmp->child.nNode, tmp->attr.nCount);
				}

				s1.Format("%d.%s%s: %s = %s\r\n", tmp->nLevel, _TreeEntryTypeStr[tmp->nType], s2, GetEntryNameStr(tmp, 1, TRUE), GetEntryDataStr(tmp, 1, TRUE));
				m_str += spaces + s1;

				TREEENTRY *pTe1, *pTe2;
				_SetAttrChild(tmp, &pTe1, &pTe2);
				if(pTe1 != NULL) _Print(pTe1, nMode, lpPrintProc, ppData);
				if(pTe2 != NULL) _Print(pTe2, nMode, lpPrintProc, ppData);

				tmp = tmp->next;

			} while(tmp);

			break;
		}


		//////////////////////////////////////////////////////////////
		// XML
		//////////////////////////////////////////////////////////////
		case TPM_XML:
		{
			do
			{
				spaces.str_set(' ', 4 * tmp->nLevel);
				s2 = "";
				s3 = "";

				EString strEntryName = GetEntryNameStr(tmp, 2, FALSE);
				EString strEntryData = GetEntryDataStr(tmp, 2, TRUE);

				
				switch(tmp->nType)
				{
					case TET_LEAF:
					{
						if(tmp->attr.first)
						{
							s1.Format("<%s ", strEntryName);
							s2.Format(">%s</%s>\r\n", strEntryData, strEntryName);
						}
						else
						{
							s1.Format("<%s>%s</%s>\r\n", strEntryName, strEntryData, strEntryName);
						}
						break;
					}

					case TET_NODE:
					{
						if(tmp->attr.first)
						{
							s1.Format("%s<%s ", spaces, strEntryName);
							s2.Format(">\r\n");
							s3.Format("%s</%s>\r\n", spaces, strEntryName);
							spaces = "";
						}
						else
						{
							s1.Format("<%s>\r\n", strEntryName);
							s2.Format("%s</%s>\r\n", spaces, strEntryName);
						}
						break;
					}

					case TET_ATTR:
					{
						spaces = "";
						if(tmp->szEntryData)
							s1.Format("%s=%s ", strEntryName, strEntryData);
						else
							s1.Format("%s ", strEntryName);
						break;
					}
				}
				

				m_str += spaces + s1;
				if(tmp->attr.first)
				{
					_Print(tmp->attr.first, nMode, lpPrintProc, ppData);
					m_str.TrimRight();
					m_str += s2;
					s2 = "";
				}
				if(tmp->child.first) _Print(tmp->child.first, nMode, lpPrintProc, ppData);
				m_str += s2;
				m_str += s3;

				tmp = tmp->next;

			} while(tmp);

			break;
		}


		//////////////////////////////////////////////////////////////
		// PrintProc
		//////////////////////////////////////////////////////////////
		case TPM_CUSTOM:
		{
			lpPrintProc(tmp, ppData);
			break;
		}


		//////////////////////////////////////////////////////////////
		// 
		//////////////////////////////////////////////////////////////
		case TPM_CONFIG:
		{
			m_str = "";

/*			do
			{
				spaces.str_set(' ', 4 * tmp->nLevel);
				s2 = "";

				EString strEntryName = GetEntryNameStr(tmp, 1, FALSE);
				EString strEntryData = GetEntryDataStr(tmp, 1, TRUE);


				EString strEqual = (m_isSetEqualSign) ? "= " : "";

				switch(tmp->nType)
				{
					case CET_WORD_WORD:
						s1.Format("%s %s%s;\r\n", strEntryName, strEqual, strEntryData);
						break;

					case CET_WORD_LIST:
						s1.Format("%s %s{%s}\r\n", strEntryName, strEqual, strEntryData);
						break;

					case CET_LIST_WORD:
						s1.Format("{%s} %s%s;\r\n", strEntryName, strEqual, strEntryData);
						break;

					case CET_LIST_LIST:
						s1.Format("{%s} %s{%s}\r\n", strEntryName, strEqual, strEntryData);
						break;

					case CET_SECTION:
						if(tmp->pCePrev != NULL)
							s1.Format("\r\n%s%s\r\n%s{\r\n", spaces, strEntryName, spaces);
						else
							s1.Format("%s\r\n%s{\r\n", strEntryName, spaces);
						s2 = spaces + "}\r\n";
						break;
				}



				switch(tmp->nType)
				{
					case TET_LEAF:
					{
						s1.Format("%s = %s\r\n", strEntryName, strEntryData);
						break;
					}

					case TET_NODE:
					{
						s1.Format("\r\n%s%s\r\n%s{\r\n", spaces, strEntryName, spaces);
						s2 = spaces + "}\r\n";break;
					}
				}

				m_str += spaces + s1;
				if(tmp->child.first != NULL) _Print(tmp->child.first, nMode, lpPrintProc, ppData);
				m_str += s2;

				tmp = tmp->next;

			} while(tmp);
*/
			break;
		}
	}

}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline BOOL CALLBACK PrintProc(TREEENTRY* pTe, void** ppData)
{
	ETree* pTree = (ETree*)ppData;
	TREEENTRY* tmp = pTe;
	EString spaces, s1, s2, s3, s4, s5;

	do
	{

		spaces.str_set(' ', 3 * tmp->nLevel);
		s2 = ""; s3 = ""; s4 = ""; s5 = "";

		if(tmp->attr.first)
		{
			s3 = spaces + "<";
			s4 = ">\r\n";
		}

		EString strEntryName = pTree->GetEntryNameStr(tmp, 3, FALSE);
		strEntryName.ReplaceClass(CCM_WORD|CCM_INV, '_');
		EString strEntryData = pTree->GetEntryDataStr(tmp, 3, TRUE);
		if(tmp->szEntryName == NULL) goto LABEL_NEXT;

		switch(tmp->nType)
		{
			case TET_LEAF:
			{
				s1.Format("%s = %s\r\n", strEntryName, strEntryData);
				break;
			}

			case TET_ATTR:
			{
				spaces = "";
				if(tmp->szEntryData)
					s1.Format("%s = %s ", strEntryName, strEntryData);
				else
					s1.Format("%s ", strEntryName);
				break;
			}

			case TET_NODE:
			{
				s5.Format("%s{\r\n", spaces);
				if(tmp->prev != NULL)
					s1.Format("\r\n%s%s = %s\r\n", spaces, strEntryName, strEntryData);
				else
					s1.Format("%s = %s\r\n", strEntryName, strEntryData);
				s2 = spaces + "}\r\n";
				break;
			}
		}


		pTree->m_str += spaces + s1;
		pTree->m_str += s3;
		
		if(tmp->attr.first)
		{
			PrintProc(tmp->attr.first, ppData);
			pTree->m_str.TrimRight();
		}
		pTree->m_str += s4;
		pTree->m_str += s5;

		if(tmp->child.first) PrintProc(tmp->child.first, ppData);
		pTree->m_str += s2;

LABEL_NEXT:
		tmp = tmp->next;

	} while(tmp);

	return TRUE;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline void ETree::SetTreeCtrl(TREEENTRY* pTe, HWND hTreeCtrl, HIMAGELIST hImageList)
{	
	if(pTe == NULL) pTe = m_base.first;
	if(pTe == NULL) return;

	_EcxTreeView_DeleteAllItems(hTreeCtrl);
	_SetTreeCtrl(pTe, hTreeCtrl, TVI_ROOT, hImageList, TRUE);
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline void ETree::_SetTreeCtrl(TREEENTRY* pTe, HWND hTreeCtrl, HTREEITEM hParent, HIMAGELIST hImageList, BOOL isAttrChild)
{	

	TREEENTRY* tmp = pTe;
	if(tmp == NULL) return;
	HTREEITEM hCurr;
	int nIcon = 0, nIconSel = 0;
	EString path = pTe->szEntryData;
	EString name;


	do
	{
		name = tmp->szEntryName;

/**/
		if(tmp->parent && tmp->szEntryName)
		{
			path.Format("%s%s", tmp->parent->szEntryData, tmp->szEntryName);
		}

		SHFILEINFO shinfo;
		HICON hIcon, hIconSel;

		DWORD dwFileAttributes = 0;
		UINT uFlags = SHGFI_DISPLAYNAME | SHGFI_ICON | SHGFI_SMALLICON;
		if(tmp->nType == TET_LEAF)
		{
			dwFileAttributes = FILE_ATTRIBUTE_NORMAL;
			uFlags |= SHGFI_USEFILEATTRIBUTES;
		}

		SHGetFileInfo(path, dwFileAttributes, &shinfo, sizeof(SHFILEINFO), uFlags);
		hIcon = shinfo.hIcon;

		SHGetFileInfo(path, dwFileAttributes, &shinfo, sizeof(SHFILEINFO), uFlags | SHGFI_OPENICON);
		hIconSel = shinfo.hIcon;

		if(tmp->parent)
		{
//			name = GetEntryNameStr(tmp, 1, FALSE);
		}
		else
		{
//			name = GetEntryNameStr(tmp, 1, FALSE);
//			name = shinfo.szDisplayName;
		}

		if(hImageList)
		{
			nIcon = ImageList_AddIcon(hImageList, hIcon);
			nIconSel = ImageList_AddIcon(hImageList, hIconSel);
			TreeView_SetImageList(hTreeCtrl, hImageList, TVSIL_NORMAL);
		}

		hCurr = _EcxTreeView_InsertItem(hTreeCtrl, name, hParent);
		_EcxTreeView_SetItemData(hTreeCtrl, hCurr, (DWORD)tmp);

//		_EcxTreeView_SetItemImage(hTreeCtrl, hCurr, nIcon, nIcon);
		_EcxTreeView_SetItemImage(hTreeCtrl, hCurr, nIcon, nIconSel);
//		_EcxTreeView_SetItemImage(hTreeCtrl, hCurr, tmp->nType, tmp->nType);



// TREEENTRY* pTe, HWND hTreeCtrl, HTREEITEM hCurr

		if(isAttrChild)
		{
			TREEENTRY *pTe1, *pTe2;
			_SetAttrChild(tmp, &pTe1, &pTe2);
			if(pTe1 != NULL) _SetTreeCtrl(pTe1, hTreeCtrl, hCurr, hImageList, TRUE);
			if(pTe2 != NULL) _SetTreeCtrl(pTe2, hTreeCtrl, hCurr, hImageList, TRUE);
		}

		tmp = tmp->next;

	} while(tmp);
}


////////////////////////////////////////////////////////////////////////////
//  m_isAttrFirst  TRUE,  ppTe1   pTe->attr.first,
//  ppTe2   pTe->child.first.   m_isAttrFirst  FALSE,
//     
////////////////////////////////////////////////////////////////////////////
inline void ETree::_SetAttrChild(TREEENTRY* pTe, TREEENTRY** ppTe1, TREEENTRY** ppTe2)
{
	if(m_isAttrFirst)
	{
		// : ,  
		*ppTe1 = pTe->attr.first;
		*ppTe2 = pTe->child.first;
	}
	else
	{
		// :  , 
		*ppTe1 = pTe->child.first;
		*ppTe2 = pTe->attr.first;
	}
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline EString ETree::GetEntryStr(TREEENTRY* pTe)
{
	if(pTe == NULL) pTe = m_base.first;
	if(pTe == NULL) return "";

	EString a1, a2, p0, p1, p2, p3;
	EString p40, p41, p42, a43, a44, p50, p51, p52, a53, q1, q2;

	a1.Format("nType    = %s\r\n", _TreeEntryTypeStr[pTe->nType]);
	a2.Format("nLevel   = %d\r\n", pTe->nLevel);

	p0.Format("0x%.8X\r\n\r\n", pTe);
	p1.Format("prev     = %s\r\n", GetEntryPtrStr(pTe->prev));
	p2.Format("next     = %s\r\n", GetEntryPtrStr(pTe->next));
	p3.Format("parent   = %s\r\n\r\n", GetEntryPtrStr(pTe->parent));
	
	p40 = "child:\r\n";
	p41.Format("  first  = %s\r\n", GetEntryPtrStr(pTe->child.first));
	p42.Format("  last   = %s\r\n", GetEntryPtrStr(pTe->child.last));
	a43.Format("  nCount = %d\r\n", pTe->child.nCount);
	a44.Format("  nNode  = %d\r\n\r\n", pTe->child.nNode);
	
	p50 = "attr:\r\n";
	p51.Format("  first  = %s\r\n", GetEntryPtrStr(pTe->attr.first));
	p52.Format("  last   = %s\r\n", GetEntryPtrStr(pTe->attr.last));
	a53.Format("  nCount = %d\r\n", pTe->attr.nCount);

	q1.Format("szEntryName  = %s\r\n", GetEntryNameStr(pTe, TRUE, TRUE));
	q2.Format("szEntryData  = %s\r\n", GetEntryDataStr(pTe, TRUE, TRUE));

	EString str;
	str = p0 + a1 + a2 + p1 + p2 + p3 + p40 + p41 + p42 + a43 + a44 + p50 + p51 + p52 + a53 + "\r\n" + q1 + q2;
	return str;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline EString ETree::GetEntryPtrStr(TREEENTRY* pTe)
{
	if(pTe == NULL) return "NULL";
	EString str;
	str.Format("0x%.8X:  %s:  %s", (DWORD)pTe, _TreeEntryTypeStr[pTe->nType], GetEntryNameStr(pTe, 1, TRUE));
	return str;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline EString ETree::GetSysVarStr()
{
	EString p40, p41, p42, a43, a44, a45;
	EString a1, a2;
	p40 = "m_base:\r\n";
	p41.Format("  first  = %s\r\n", GetEntryPtrStr(m_base.first));
	p42.Format("  last   = %s\r\n", GetEntryPtrStr(m_base.last));
	a43.Format("  nCount = %d\r\n", m_base.nCount);
	a44.Format("  nNode  = %d\r\n\r\n", m_base.nNode);
	a1.Format("m_nCount  = %d\r\n", m_nCount);
	a2.Format("m_nError  = %s\r\n", _TreeErrorTypeStr[m_nError]);
	EString str = a1 + p40 + p41 + p42 + a43 + a44 + a2;
	return str;
}


#endif //USED_TREE
/////////////////////////////////////////////////////////////////////////////
#endif // __ETREE_H__
