// Hash.h : header file
//

#ifndef __HASH_H__
#define __HASH_H__
#pragma once
/////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////
//  Hash.h
/////////////////////////////////////////////////////////////////////////////
#define VERSION_HASH   "16.01.2013"


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
#ifdef USED_HASH


/////////////////////////////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////////////////////////////
typedef struct _HashFunctionParams  // hfp
{
	int nHashFunction;
	char szHashFunctionName[24];
	int nBufferSize;
	int nResultSize;
	int nBlockSize;
	BOOL isSpaces;

} HASHFUNCTIONPARAMS;


/////////////////////////////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////////////////////////////
enum _HashFunctions
{
	HASH_RS,
	HASH_JS,
	HASH_PJW32,
	HASH_BKDR,
	HASH_MABKDR,
	HASH_SDBM,
	HASH_DJB,
	HASH_DEK,
	HASH_AP,
	HASH_LY,
	HASH_ROT13,
	HASH_FAQ6,
	HASH_LOOKUP3,
	HASH_FNV,
	HASH_MAFNV,
	HASH_MURMUR2,
	HASH_MURMUR2A,
	HASH_ADLER32,

	SUMM_08,
	SUMM_16,
	SUMM_32,
	SUMM_64,

	DIGEST_MD5,
	DIGEST_SHA224,
	DIGEST_SHA256,
};


#define DECLARE_HASH(hash, bfs, res, bls, sp)	{ hash, #hash, bfs, res, bls, sp },


static HASHFUNCTIONPARAMS g_hfpHashFunctions[] =
{
	DECLARE_HASH( HASH_RS,           4,     4,     4,     FALSE )
	DECLARE_HASH( HASH_JS,           4,     4,     4,     FALSE )
	DECLARE_HASH( HASH_PJW32,        4,     4,     4,     FALSE )
	DECLARE_HASH( HASH_BKDR,         4,     4,     4,     FALSE )
	DECLARE_HASH( HASH_MABKDR,       4,     4,     4,     FALSE )
	DECLARE_HASH( HASH_SDBM,         4,     4,     4,     FALSE )
	DECLARE_HASH( HASH_DJB,          4,     4,     4,     FALSE )
	DECLARE_HASH( HASH_DEK,          4,     4,     4,     FALSE )
	DECLARE_HASH( HASH_AP,           4,     4,     4,     FALSE )
	DECLARE_HASH( HASH_LY,           4,     4,     4,     FALSE )
	DECLARE_HASH( HASH_ROT13,        4,     4,     4,     FALSE )
	DECLARE_HASH( HASH_FAQ6,         4,     4,     4,     FALSE )
	DECLARE_HASH( HASH_LOOKUP3,      4,     4,     4,     FALSE )
	DECLARE_HASH( HASH_FNV,          4,     4,     4,     FALSE )
	DECLARE_HASH( HASH_MAFNV,        4,     4,     4,     FALSE )
	DECLARE_HASH( HASH_MURMUR2,      4,     4,     4,     FALSE )
	DECLARE_HASH( HASH_MURMUR2A,     4,     4,     4,     FALSE )
	DECLARE_HASH( HASH_ADLER32,      4,     4,     4,     FALSE )

	DECLARE_HASH( SUMM_08,           1,     1,     1,     FALSE )
	DECLARE_HASH( SUMM_16,           2,     2,     2,     FALSE )
	DECLARE_HASH( SUMM_32,           4,     4,     4,     FALSE )
	DECLARE_HASH( SUMM_64,           8,     8,     8,     FALSE )

	DECLARE_HASH( DIGEST_MD5,       16,    16,     1,     FALSE )
	DECLARE_HASH( DIGEST_SHA224,    32,    28,     4,     TRUE  )
	DECLARE_HASH( DIGEST_SHA256,    32,    32,     4,     TRUE  )

};


/////////////////////////////////////////////////////////////////////////////
// RS Hash Function
//---------------------------------------------------------------------------
// RS   -   , 
// Robert Sedgwicks.     .
/////////////////////////////////////////////////////////////////////////////
inline DWORD _EcxHash_RS(BYTE* pBuf, DWORD dwSize, BYTE* pResult)
{
	DWORD b = 378551;
	DWORD a = 63689;
	DWORD i, dwHash = 0;

	for(i = 0; i < dwSize; pBuf++, i++)
	{
		dwHash = dwHash * a + *pBuf;
		a = a * b;
	}
	if(pResult) memcpy(pResult, &dwHash, 4);
	return dwHash;
}


/////////////////////////////////////////////////////////////////////////////
// JS Hash Function
//---------------------------------------------------------------------------
//
/////////////////////////////////////////////////////////////////////////////
inline DWORD _EcxHash_JS(BYTE* pBuf, DWORD dwSize, BYTE* pResult)
{
	DWORD i, dwHash = 1315423911;

	for(i = 0; i < dwSize; pBuf++, i++)
	{
		dwHash ^= ((dwHash << 5) + *pBuf + (dwHash >> 2));
	}
	if(pResult) memcpy(pResult, &dwHash, 4);
	return dwHash;
}

/////////////////////////////////////////////////////////////////////////////
// P. J. Weinberger Hash Function
//---------------------------------------------------------------------------
// PJW-32 - -,   
// (Peter J. Weinberger)  AT&T Bell Laboratories.  
//   {}  32- -,
//  {-} .    
// -x   ,     
//      .  
//        ELF Linux, 
//     Unix- .
/////////////////////////////////////////////////////////////////////////////
inline DWORD _EcxHash_PJW32(BYTE* pBuf, DWORD dwSize, BYTE* pResult)
{
	DWORD test = 0;
	DWORD i, dwHash = 0;

	for(i = 0; i < dwSize; pBuf++, i++)
	{
		dwHash = (dwHash << 4) + *pBuf;
		if((test = dwHash & 0xf0000000) != 0)
		{
			dwHash = ((dwHash ^ (test >> 24)) & (0xFFFFFFF));
		}
	}
	if(pResult) memcpy(pResult, &dwHash, 4);
	return dwHash;
}

/////////////////////////////////////////////////////////////////////////////
// BKDR Hash Function
// MABKDR Hash Function
//---------------------------------------------------------------------------
// BKDR -  -   , 
//     ,    
// "  ".    .
// maBKDR -  BKDR -    .
// ,  maBKDR,  ..  
//         (
//   )
/////////////////////////////////////////////////////////////////////////////
inline DWORD _EcxHash_BKDR(BYTE* pBuf, DWORD dwSize, BOOL isMa, BYTE* pResult)
{
	DWORD seed = 131313;   // 31 131 1313 13131 131313 etc..
	DWORD i, dwHash = 0;

	for(i = 0; i < dwSize; pBuf++, i++)
	{
		dwHash = (dwHash * seed) + *pBuf + ((isMa) ? i : 0);
	}
	if(pResult) memcpy(pResult, &dwHash, 4);
	return dwHash;
}

/////////////////////////////////////////////////////////////////////////////
// SDBM Hash Function
//---------------------------------------------------------------------------
// SDBM     -  , 
//   SDBM.   -, 
// 32-  .     
//     .
/////////////////////////////////////////////////////////////////////////////
inline DWORD _EcxHash_SDBM(BYTE* pBuf, DWORD dwSize, BYTE* pResult)
{
	DWORD i, dwHash = 0;

	for(i = 0; i < dwSize; pBuf++, i++)
	{
		dwHash = *pBuf + (dwHash << 6) + (dwHash << 16) - dwHash;
	}
	if(pResult) memcpy(pResult, &dwHash, 4);
	return dwHash;
}

/////////////////////////////////////////////////////////////////////////////
// DJB Hash Function
//---------------------------------------------------------------------------
// DJB     -  , 
//   ,    .
//   -,  32-
//     -.
/////////////////////////////////////////////////////////////////////////////
inline DWORD _EcxHash_DJB(BYTE* pBuf, DWORD dwSize, BYTE* pResult)
{
	DWORD i, dwHash = 5381;

	for(i = 0; i < dwSize; pBuf++, i++)
	{
		dwHash = ((dwHash << 5) + dwHash) + *pBuf;
	}
	if(pResult) memcpy(pResult, &dwHash, 4);
	return dwHash;
}

/////////////////////////////////////////////////////////////////////////////
// DEK Hash Function
//---------------------------------------------------------------------------
//
/////////////////////////////////////////////////////////////////////////////
inline DWORD _EcxHash_DEK(BYTE* pBuf, DWORD dwSize, BYTE* pResult)
{
	DWORD i, dwHash = dwSize;

	for(i = 0; i < dwSize; pBuf++, i++)
	{
		dwHash = ((dwHash << 5) ^ (dwHash >> 27)) ^ (*pBuf);
	}
	if(pResult) memcpy(pResult, &dwHash, 4);
	return dwHash;
}

/////////////////////////////////////////////////////////////////////////////
// AP Hash Function
//---------------------------------------------------------------------------
// AP     -  ,  .
// Arash Partow.   -, 
// 32-  .   ,   
//    ,   
//    LFSR.
/////////////////////////////////////////////////////////////////////////////
inline DWORD _EcxHash_AP(BYTE* pBuf, DWORD dwSize, BYTE* pResult)
{
	DWORD i, dwHash = 0;

	for(i = 0; i < dwSize; pBuf++, i++)
	{
		dwHash ^= ((i & 1) == 0)
			? ((dwHash << 7) ^ (*pBuf) ^ (dwHash >> 3))
			: (~((dwHash << 11) ^ (*pBuf) ^ (dwHash >> 5)));
	}
	if(pResult) memcpy(pResult, &dwHash, 4);
	return dwHash;
}

/////////////////////////////////////////////////////////////////////////////
// LY Hash Function
//---------------------------------------------------------------------------
// Congruential generator proposed by Leonid Yuriev. Multiplier constant
// suggested by M.Lavaux & F.Janssens.
/////////////////////////////////////////////////////////////////////////////
inline DWORD _EcxHash_LY(BYTE* pBuf, DWORD dwSize, BYTE* pResult)
{
	DWORD i, dwHash = 0;

	for(i = 0; i < dwSize; pBuf++, i++)
	{
		dwHash = (dwHash * 1664525) + *pBuf + 1013904223;
	}
	if(pResult) memcpy(pResult, &dwHash, 4);
	return dwHash;
}

/////////////////////////////////////////////////////////////////////////////
// ROT13 Hash Function
//---------------------------------------------------------------------------
// ROT13     -  ,  
//     ,   .
//   -,  32-
//  .
/////////////////////////////////////////////////////////////////////////////
inline DWORD _EcxHash_ROT13(BYTE* pBuf, DWORD dwSize, BYTE* pResult)
{
	DWORD i, dwHash = 0;

	for(i=0; i < dwSize; pBuf++, i++)
	{
		dwHash += *pBuf;
		dwHash -= _lrotl(dwHash, 13);
	}
	if(pResult) memcpy(pResult, &dwHash, 4);
	return dwHash;
}

/////////////////////////////////////////////////////////////////////////////
// FAQ6 Hash Function
//---------------------------------------------------------------------------
// FAQ6     -  , 
//  .   -, 
// 32-  .
// http://burtleburtle.net/bob/hash/hashfaq.html
/////////////////////////////////////////////////////////////////////////////
inline DWORD _EcxHash_FAQ6(BYTE* pBuf, DWORD dwSize, BYTE* pResult)
{
	DWORD i, dwHash = 0;

	for(i = 0; i < dwSize; pBuf++, i++)
	{
		dwHash += *pBuf;
		dwHash += (dwHash << 10);
		dwHash ^= (dwHash >> 6);
	}
	dwHash += (dwHash << 3);
	dwHash ^= (dwHash >> 11);
	dwHash += (dwHash << 15);
	if(pResult) memcpy(pResult, &dwHash, 4);
	return dwHash;
}

/////////////////////////////////////////////////////////////////////////////
// LOOKUP3 Hash Function
//---------------------------------------------------------------------------
// LOOKUP3  ,    -  ,
//   .   -,
//  32-  .
// http://burtleburtle.net/bob/c/lookup3.c
/////////////////////////////////////////////////////////////////////////////
#define rot(x, k) (((x) << (k)) ^ ((x) >> (32 - (k))))
 
#define mix(a, b, c) { \
	a -= c;  a ^= rot (c,  4);  c += b; \
	b -= a;  b ^= rot (a,  6);  a += c; \
	c -= b;  c ^= rot (b,  8);  b += a; \
	a -= c;  a ^= rot (c, 16);  c += b; \
	b -= a;  b ^= rot (a, 19);  a += c; \
	c -= b;  c ^= rot (b,  4);  b += a; }
 
#define final(a, b, c) { \
	c ^= b; c -= rot (b, 14); \
	a ^= c; a -= rot (c, 11); \
	b ^= a; b -= rot (a, 25); \
	c ^= b; c -= rot (b, 16); \
	a ^= c; a -= rot (c, 4);  \
	b ^= a; b -= rot (a, 14); \
	c ^= b; c -= rot (b, 24); }
 
inline DWORD _EcxHash_LOOKUP3(BYTE* pBuf, DWORD dwSize, BYTE* pResult)
{
	BYTE* tmp = new BYTE[dwSize + 3];
	memcpy(tmp, pBuf, dwSize);

	DWORD a, b, c;

	// the key, an array of DWORD values
	DWORD* k = (DWORD*)tmp;

	// the length of the key, in DWORDs
	DWORD length = (dwSize + 3) / 4;

	// tail pad with zeros
	tmp[dwSize] = 0;
	tmp[dwSize + 1] = 0;
	tmp[dwSize + 2] = 0;

	// Set up the internal state
	a = b = c = 0xDEADBEEF + dwSize;

	// handle most of the key
	while(length > 3)
	{
		a += k[0];
		b += k[1];
		c += k[2];
		mix(a, b, c);
		length -= 3;
		k += 3;
	}

	// handle the last 3 unsigned ints
	switch(length)
	{
		case 3:
			c += k[2];
		case 2:
			b += k[1];
		case 1:
			a += k[0];
			final (a, b, c);
		case 0:
			// nothing left to add
			break;
	}

	delete [] tmp;
	if(pResult) memcpy(pResult, &c, 4);
	return c;
}

/////////////////////////////////////////////////////////////////////////////
// FNV Hash Function
// MAFNV Hash Function
//---------------------------------------------------------------------------
// FNV (. Fowler-Noll-Vo)   -   ,
//   ,      .  
//  .    32-, 64-, 128-, 256-,
// 512-  1024  .   , 
//   .  ,   .  
//       .  , 
//   (   ).
/////////////////////////////////////////////////////////////////////////////
inline DWORD _EcxHash_FNV(BYTE* pBuf, DWORD dwSize, BOOL isMa, BYTE* pResult)
{
	DWORD dwHash = 0x811C9DC5;
	BYTE* bp = (BYTE*)pBuf;	
	BYTE* be = bp + dwSize;		

	while(bp < be)
	{
		if(isMa)
		{
			dwHash *= 0x01000193;
			dwHash ^= (DWORD)*bp++;
		}
		else
		{
			dwHash ^= (DWORD)*bp++;
			dwHash *= 0x01000193;
		}
	}
	if(pResult) memcpy(pResult, &dwHash, 4);
	return dwHash;
}

/////////////////////////////////////////////////////////////////////////////
// MURMUR2 Hash Function
// MURMUR2A Hash Function
//---------------------------------------------------------------------------
// Murmur2     -  , 
//  .   -, 
// 32-  .     
// ,  ,   ,  
//      .   
//   Intel- .   -
//   .  ,     
// .      Merkle-Damgard,
//    ( 20 %),    .
/////////////////////////////////////////////////////////////////////////////
#define mmix(h,k) { k *= m; k ^= k >> r; k *= m; h *= m; h ^= k; }

inline DWORD _EcxHash_MURMUR2(BYTE* pBuf, DWORD dwSize, BOOL isMa, BYTE* pResult)
{
	DWORD dwHash, k;
	DWORD m = 0x5BD1E995;
	DWORD seed = 0;
	int r = 24;
	BYTE* data = pBuf;

	if(isMa)
	{
		DWORD l = dwSize;
		dwHash = seed;

		while(dwSize >= 4)
		{
			k = *(unsigned int*)data;
			mmix(dwHash, k);

			data += 4;
			dwSize -= 4;
		}

		DWORD t = 0;
		switch(dwSize)
		{
			case 3: t ^= data[2] << 16;
			case 2: t ^= data[1] << 8;
			case 1: t ^= data[0];
		};

		mmix(dwHash, t);
		mmix(dwHash, l);
	}
	else
	{
		dwHash = seed ^ dwSize;

		while (dwSize >= 4)
		{
			k  = data[0];
			k |= data[1] << 8;
			k |= data[2] << 16;
			k |= data[3] << 24;

			k *= m;
			k ^= k >> r;
			k *= m;

			dwHash *= m;
			dwHash ^= k;

			data += 4;
			dwSize -= 4;
		}

		switch(dwSize)
		{
			case 3:
				dwHash ^= data[2] << 16;
			case 2:
				dwHash ^= data[1] << 8;
			case 1:
				dwHash ^= data[0];
				dwHash *= m;
		}
	}

	dwHash ^= dwHash >> 13;
	dwHash *= m;
	dwHash ^= dwHash >> 15;

 	if(pResult) memcpy(pResult, &dwHash, 4);
	return dwHash;
}






/////////////////////////////////////////////////////////////////////////////
// 
//---------------------------------------------------------------------------
//
/////////////////////////////////////////////////////////////////////////////
//inline DWORD _EcxHash_(BYTE* pBuf, DWORD dwSize, BYTE* pResult)
//{
//	DWORD i, dwHash = dwSize;
//	f(pResult) memcpy(pResult, &dwHash, 4);
//	return dwHash;
//}


/////////////////////////////////////////////////////////////////////////////
// Adler-32 Hash Function
//---------------------------------------------------------------------------
// Adler-32 - -,   .  
//   Fletcher.     
//   RFC 1950      .  
//      CRC32 .
// Adler-32    Zlib.
/////////////////////////////////////////////////////////////////////////////
inline DWORD _EcxHash_ADLER32(BYTE* pBuf, DWORD dwSize, DWORD dwInit, BYTE* pResult)
{
	DWORD i, dwHash = 1;
	if(pBuf != NULL)
	{
		DWORD s1 = dwInit & 0x0000FFFF;
		DWORD s2 = (dwInit >> 16) & 0x0000FFFF;

		for(i=0; i < dwSize; i++)
		{
			s1 = (s1 + pBuf[i]) % 65521;
			s2 = (s2 + s1) % 65521;
		}
		dwHash = (s2 << 16) + s1;
	}

	if(pResult) memcpy(pResult, &dwHash, 4);
	return dwHash;
}

/////////////////////////////////////////////////////////////////////////////
// SUMM Hash Function
//---------------------------------------------------------------------------
// SUMM -     pBuf.
/////////////////////////////////////////////////////////////////////////////
template <class TYPE>
inline TYPE _EcxHash_SUMM(BYTE* pBuf, DWORD dwSize, TYPE tInit, BYTE* pResult)
{
	BYTE* tmp = pBuf;
	TYPE tChecksum = tInit;

	while(dwSize--)
	{
		tChecksum += *tmp;
		tmp++;
	}
	if(pResult) memcpy(pResult, &tChecksum, sizeof(TYPE));
	return tChecksum;
}

/////////////////////////////////////////////////////////////////////////////
// MD5 Digest Function
//---------------------------------------------------------------------------
// MD5 (. Message Digest 5)  128-  , 
//   .     
// (Massachusetts Institute of Technology, MIT)  1991 .  
//  ""  ""   . 
//      MD4.   RFC-1321.
/////////////////////////////////////////////////////////////////////////////
typedef struct _MD5		// md5
{
	DWORD dwABCD[4];		//  md5 (16 )
	DWORD dwCount[2];		// number of bits, modulo 2^64 (lsb first)
	BYTE nBuffer[64];		// input buffer

} MD5;

const DWORD _F[] =
{
	0xD76AA478, 0xE8C7B756, 0x242070DB, 0xC1BDCEEE,
	0xF57C0FAF, 0x4787C62A, 0xA8304613, 0xFD469501,
	0x698098D8, 0x8B44F7AF, 0xFFFF5BB1, 0x895CD7BE,
	0x6B901122, 0xFD987193, 0xA679438E, 0x49B40821
};

const DWORD _G[] =
{
	0xF61E2562, 0xC040B340, 0x265E5A51, 0xE9B6C7AA,
	0xD62F105D, 0x02441453, 0xD8A1E681, 0xE7D3FBC8,
	0x21E1CDE6, 0xC33707D6, 0xF4D50D87, 0x455A14ED,
	0xA9E3E905, 0xFCEFA3F8, 0x676F02D9, 0x8D2A4C8A
};

const DWORD _H[] =
{
	0xFFFA3942, 0x8771F681, 0x6D9D6122, 0xFDE5380C,
	0xA4BEEA44, 0x4BDECFA9, 0xF6BB4B60, 0xBEBFBC70,
	0x289B7EC6, 0xEAA127FA, 0xD4EF3085, 0x04881D05,
	0xD9D4D039, 0xE6DB99E5, 0x1FA27CF8, 0xC4AC5665
};

const DWORD _I[] =
{
	0xF4292244, 0x432AFF97, 0xAB9423A7, 0xFC93A039,
	0x655B59C3, 0x8F0CCC92, 0xFFEFF47D, 0x85845DD1,
	0x6FA87E4F, 0xFE2CE6E0, 0xA3014314, 0x4E0811A1,
	0xF7537E82, 0xBD3AF235, 0x2AD7D2BB, 0xEB86D391
};


//     2-, 3-, 4- 
const UINT _g[] = { 1, 6, 11,  0,  5, 10, 15,  4,  9, 14, 3,  8, 13,  2,  7, 12 };
const UINT _h[] = { 5, 8, 11, 14,  1,  4,  7, 10, 13,  0, 3,  6,  9, 12, 15,  2 };
const UINT _i[] = { 0, 7, 14,  5, 12,  3, 10,  1,  8, 15, 6, 13,  4, 11,  2,  9 };


static BYTE g_nPadding[64] =
{
	0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};

// F, G, H and I are basic MD5 functions
#define F(x, y, z) ( ((x) & (y)) | ((~x) & (z)) )
#define G(x, y, z) ( ((x) & (z)) | ((y) & (~z)) )
#define H(x, y, z) ( (x) ^ (y) ^ (z) )
#define I(x, y, z) ( (y) ^ ((x) | (~z)) )

// _rotate_left rotates x left n bits
#define _rotate_left(x, n) ( ((x) << (n)) | ((x) >> (32 - (n))) )

// FF, GG, HH, and II transformations for rounds 1, 2, 3, and 4.
// Rotation is separate from addition to prevent recomputation.

#define FF(a, b, c, d, x, s, ac) { \
	(a) += F ((b), (c), (d)) + (x) + (DWORD)(ac); \
	(a) = _rotate_left((a), (s)); \
	(a) += (b); }

#define GG(a, b, c, d, x, s, ac) { \
	(a) += G ((b), (c), (d)) + (x) + (DWORD)(ac); \
	(a) = _rotate_left((a), (s)); \
	(a) += (b); }

#define HH(a, b, c, d, x, s, ac) { \
	(a) += H ((b), (c), (d)) + (x) + (DWORD)(ac); \
	(a) = _rotate_left((a), (s)); \
	(a) += (b); }

#define II(a, b, c, d, x, s, ac) { \
	(a) += I ((b), (c), (d)) + (x) + (DWORD)(ac); \
	(a) = _rotate_left((a), (s)); \
	(a) += (b); }


// MD5 basic transformation. Transforms dwABCD based on block.
inline void _md5_transform(DWORD dwABCD[4], BYTE block[64])
{
	int i;
	DWORD a = dwABCD[0], b = dwABCD[1], c = dwABCD[2], d = dwABCD[3], x[16];

	memcpy(x, block, 64);

	// Round 1
	for(i=0; i<16; i+=4)
	{
		FF(a, b, c, d, x[i + 0],  7, _F[i + 0]);
		FF(d, a, b, c, x[i + 1], 12, _F[i + 1]);
		FF(c, d, a, b, x[i + 2], 17, _F[i + 2]);
		FF(b, c, d, a, x[i + 3], 22, _F[i + 3]);
	}

	// Round 2
	for(i=0; i<16; i+=4)
	{
		GG(a, b, c, d, x[_g[i + 0]],  5, _G[i + 0]);
		GG(d, a, b, c, x[_g[i + 1]],  9, _G[i + 1]);
		GG(c, d, a, b, x[_g[i + 2]], 14, _G[i + 2]);
		GG(b, c, d, a, x[_g[i + 3]], 20, _G[i + 3]);
	}

	// Round 3
	for(i=0; i<16; i+=4)
	{
		HH(a, b, c, d, x[_h[i + 0]],  4, _H[i + 0]);
		HH(d, a, b, c, x[_h[i + 1]], 11, _H[i + 1]);
		HH(c, d, a, b, x[_h[i + 2]], 16, _H[i + 2]);
		HH(b, c, d, a, x[_h[i + 3]], 23, _H[i + 3]);
	}

	// Round 4
	for(i=0; i<16; i+=4)
	{
		II(a, b, c, d, x[_i[i + 0]],  6, _I[i + 0]);
		II(d, a, b, c, x[_i[i + 1]], 10, _I[i + 1]);
		II(c, d, a, b, x[_i[i + 2]], 15, _I[i + 2]);
		II(b, c, d, a, x[_i[i + 3]], 21, _I[i + 3]);
	}

	dwABCD[0] += a;
	dwABCD[1] += b;
	dwABCD[2] += c;
	dwABCD[3] += d;

	// Zeroize sensitive information.
	memset((BYTE*)x, 0, sizeof(x));
}

// MD5 initialization. Begins an MD5 operation, writing a new md5
inline void _md5_init(MD5* pmd5)
{
	pmd5->dwCount[0] = pmd5->dwCount[1] = 0;
	
	// Load magic initialization constants
	pmd5->dwABCD[0] = 0x67452301;
	pmd5->dwABCD[1] = 0xEFCDAB89;
	pmd5->dwABCD[2] = 0x98BADCFE;
	pmd5->dwABCD[3] = 0x10325476;
}

// MD5 block update operation. Continues an MD5 message-digest
//  operation, processing another message block, and updating the md5
inline void _md5_update(MD5* pmd5, BYTE* input, UINT inputLen)
{
	UINT i, index, partLen;

	// Compute number of bytes mod 64
	index = (UINT)((pmd5->dwCount[0] >> 3) & 0x3F);

	// Update number of bits
	if((pmd5->dwCount[0] += ((DWORD)inputLen << 3)) < ((DWORD)inputLen << 3))
	pmd5->dwCount[1]++;
	pmd5->dwCount[1] += ((DWORD)inputLen >> 29);
	partLen = 64 - index;

	// Transform as many times as possible
	if(inputLen >= partLen)
	{
		memcpy(&pmd5->nBuffer[index], input, partLen);
		_md5_transform(pmd5->dwABCD, pmd5->nBuffer);

		for(i = partLen; i + 63 < inputLen; i += 64)
			_md5_transform(pmd5->dwABCD, &input[i]);

		index = 0;
	}
	else
		i = 0;

	// Buffer remaining input
	memcpy(&pmd5->nBuffer[index], &input[i], inputLen - i);
}


// MD5 finalization. Ends an MD5 message-digest operation, writing the
//  the message digest and zeroizing the md5.
inline void _md5_final(BYTE* digest, MD5* pmd5)
{
	BYTE bits[8];
	UINT index, padLen;

	// Save number of bits
	memcpy(bits, pmd5->dwCount, 8);


	// Pad out to 56 mod 64
	index = (UINT)((pmd5->dwCount[0] >> 3) & 0x3F);
	padLen = (index < 56) ? (56 - index) : (120 - index);
	_md5_update(pmd5, g_nPadding, padLen);


	// Append length (before padding)
	_md5_update(pmd5, bits, 8);

	// Store dwABCD in digest
	memcpy(digest, pmd5->dwABCD, 16);

	// Zeroize sensitive information
	memset(pmd5, 0, sizeof(MD5));
}

// Digests a string and prints the result
inline void _EcxDigest_MD5(BYTE* pBuf, DWORD dwSize, BYTE* pResult)
{
	MD5 md5;
	_md5_init(&md5);
	_md5_update(&md5, pBuf, dwSize);
	_md5_final(pResult, &md5);
}


/////////////////////////////////////////////////////////////////////////////
// SHA224, SHA256 Digest Function
//---------------------------------------------------------------------------
// 
/////////////////////////////////////////////////////////////////////////////
const ULONG h224[] = { 0xC1059ED8, 0x367CD507, 0x3070DD17, 0xF70E5939, 0xFFC00B31, 0x68581511, 0x64F98FA7, 0xBEFA4FA4 };
//  32         
const ULONG h256[] = { 0x6A09E667, 0xBB67AE85, 0x3C6EF372, 0xA54FF53A, 0x510E527F, 0x9B05688C, 0x1F83D9AB, 0x5BE0CD19 };
//  32       64-  
const ULONG k[] =
{
	0x428A2F98, 0x71374491, 0xB5C0FBCF, 0xE9B5DBA5, 0x3956C25B, 0x59F111F1, 0x923F82A4, 0xAB1C5ED5,
	0xD807AA98, 0x12835B01, 0x243185BE, 0x550C7DC3, 0x72BE5D74, 0x80DEB1FE, 0x9BDC06A7, 0xC19BF174,
	0xE49B69C1, 0xEFBE4786, 0x0FC19DC6, 0x240CA1CC, 0x2DE92C6F, 0x4A7484AA, 0x5CB0A9DC, 0x76F988DA,
	0x983E5152, 0xA831C66D, 0xB00327C8, 0xBF597FC7, 0xC6E00BF3, 0xD5A79147, 0x06CA6351, 0x14292967,
	0x27B70A85, 0x2E1B2138, 0x4D2C6DFC, 0x53380D13, 0x650A7354, 0x766A0ABB, 0x81C2C92E, 0x92722C85,
	0xA2BFE8A1, 0xA81A664B, 0xC24B8B70, 0xC76C51A3, 0xD192E819, 0xD6990624, 0xF40E3585, 0x106AA070,
	0x19A4C116, 0x1E376C08, 0x2748774C, 0x34B0BCB5, 0x391C0CB3, 0x4ED8AA4A, 0x5B9CCA4F, 0x682E6FF3,
	0x748F82EE, 0x78A5636F, 0x84C87814, 0x8CC70208, 0x90BEFFFA, 0xA4506CEB, 0xBEF9A3F7, 0xC67178F2
};

//  little-endian  big-endian -- TODO:  BSWAP!!!
inline ULONG _reverse_bytes(ULONG x)
{
	return (x >> 24) | (x >> 8) & 0xFF00 | (x << 8) & 0xFF0000 | (x << 24) & 0xFF000000;
}

//    
inline void SHA256(BYTE* pBuf, DWORD dwSize, int nType, BYTE* pResult)
{
	ULONG len2 = ((dwSize + 8 + 1) + 63) & -64;
	BYTE* t = new BYTE[len2];
	memcpy(t, pBuf, dwSize);

	int i;
	ULONG u[8];
	ULONG a, b, c, d, e, f, g, h;
//	u[0] = h0, u[1] = h1, u[2] = h2, u[3] = h3, u[4] = h4, u[5] = h5, u[6] = h6, u[7] = h7;
	ULONG* puInit;
	switch(nType)
	{
		case DIGEST_SHA224: puInit = (ULONG*)h224; break;
		case DIGEST_SHA256: puInit = (ULONG*)h256; break;
	}
	memcpy(u, puInit, 32);

	//     64 
	*(ULONG*)(t + len2 - 8) = _reverse_bytes(dwSize >> (32 - 3));
	*(ULONG*)(t + len2 - 4) = _reverse_bytes(dwSize << 3);

	//    
	*(UCHAR*)(t + dwSize) = 0x80;
	memset(t + dwSize + 1, '\0', len2 - 8 - dwSize - 1);
	
	for(ULONG* m = (ULONG*)t; m < (ULONG*)(t + len2); m += 16)
	{
		a = u[0]; b = u[1]; c = u[2]; d = u[3]; e = u[4]; f = u[5]; g = u[6]; h = u[7];
		ULONG w[64];
		
		for(i = 0; i < 16; ++i)
		{
			ULONG s0, m0, t0, s1, m1, t1;
			
			__asm
			{
				mov eax, a
				mov ebx, eax
				ror eax, 2
				mov ecx, ebx
				ror ebx, 13
				xor eax, ebx
				ror ecx, 22
				xor eax, ecx
				mov s0, eax
			}
			m0 = (a & b) | (c & (a | b));
			t0 = s0 + m0;
			
			__asm
			{
				mov eax, e
				mov ebx, eax
				ror eax, 6
				mov ecx, ebx
				ror ebx, 11
				xor eax, ebx
				ror ecx, 25
				xor eax, ecx
				mov s1, eax
			}

			__asm
			{
				mov eax, e
				mov ebx, eax
				mov ecx, eax
				ror eax, 6
				ror ebx, 11
				ror ecx, 25
				xor eax, ebx
				xor eax, ecx
				mov s1, eax
			}
			
			m1 = g ^ (e & (f ^ g));
			w[i] = _reverse_bytes(*(m + i));
			t1 = h + s1 + m1 + k[i] + w[i];
		
			h = g, g = f, f = e, e = d + t1, d = c, c = b, b = a, a = t0 + t1;
		}

		for(i = 16; i < 64; ++i)
		{
			ULONG s0, m0, t0, s1, m1, t1, v1, v2;
			
			__asm
			{
				mov eax, a
				mov ebx, eax
				ror eax, 2
				mov ecx, ebx
				ror ebx, 13
				xor eax, ebx
				ror ecx, 22
				xor eax, ecx
				mov s0, eax
			}
			
			m0 = (a & b) | (c & (a | b));
			t0 = s0 + m0;
			
			__asm
			{
				mov eax, e
				mov ebx, eax
				ror eax, 6
				mov ecx, ebx
				ror ebx, 11
				xor eax, ebx
				ror ecx, 25
				xor eax, ecx
				mov s1, eax
			}
			m1 = g ^ (e & (f ^ g));

			//  16   64 ,   48 
			ULONG g1 = w[i - 15], g2 = w[i - 2];
			
			__asm
			{
				mov eax, g1
				mov ebx, eax
				ror eax, 7
				mov ecx, ebx
				ror ebx, 18
				xor eax, ebx
				shr ecx, 3
				xor eax, ecx
				mov v1, eax

				mov eax, g2
				mov ebx, eax
				ror eax, 17
				mov ecx, ebx
				ror ebx, 19
				xor eax, ebx
				shr ecx, 10
				xor eax, ecx
				mov v2, eax
			}

			w[i] = w[i - 16] + v1 + w[i - 7] + v2;
			t1 = h + s1 + m1 + k[i] + w[i];
		
			h = g, g = f, f = e, e = d + t1, d = c, c = b, b = a, a = t0 + t1;
		}

		u[0] += a; u[1] += b; u[2] += c; u[3] += d; u[4] += e; u[5] += f; u[6] += g; u[7] += h;
	}

	delete [] t;
	if(pResult) memcpy(pResult, u, 32);
}

inline void _EcxDigest_SHA(BYTE* pBuf, DWORD dwSize, int nType, BYTE* pResult)
{
	ULONG len = strlen((char*)pBuf);
	SHA256((UCHAR*)pBuf, len, nType, pResult);
}




// D7A8FBB3 07D78094 69CA9ABC B0082E4F 8D5651E4 6D3CDB76 2D02D0BF 37C9E592













/////////////////////////////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////////////////////////////
typedef struct _HashResult  // hr
{
	int nBuffer;
	int nResult;
	BYTE* pResult;
	char* pResultStr;


	///////////////////////////////////////////////////////////
	//
	///////////////////////////////////////////////////////////
	_HashResult()
	{
		pResult = NULL;
		pResultStr = NULL;
	}

	_HashResult(int nHashFunction)
	{
		InitResult(nHashFunction);
	}

	~_HashResult()
	{
		Clear();
	}

	///////////////////////////////////////////////////////////
	//
	///////////////////////////////////////////////////////////
	void InitResult(int nHashFunction)
	{
		nBuffer = g_hfpHashFunctions[nHashFunction].nBufferSize;
		nResult = g_hfpHashFunctions[nHashFunction].nResultSize;
		
		pResult = new BYTE[nBuffer];
		pResultStr = new char[3 * nResult + 8];

		memset(pResult, 0, nBuffer);
		pResultStr[0] = '\0';
	}

	///////////////////////////////////////////////////////////
	//
	///////////////////////////////////////////////////////////
	void PrintResult(int nHashFunction)
	{
		if(pResultStr && pResult)
		{
			char s[24];
			pResultStr[0] = '\0';
			int nBlockSize = g_hfpHashFunctions[nHashFunction].nBlockSize;
			int count = nResult / nBlockSize;

			for(int i=0; i<count; i++)
			{
				switch(nBlockSize)
				{
					case 1: sprintf(s, "%.2X", *((BYTE*)pResult + i)); break;
					case 2: sprintf(s, "%.4X", *((WORD*)pResult + i)); break;
					case 4: sprintf(s, "%.8X", *((DWORD*)pResult + i)); break;
					case 8: sprintf(s, "%.16I64X", *((__int64*)pResult + i)); break;
				}

				strcat(pResultStr, s);
				if(g_hfpHashFunctions[nHashFunction].isSpaces) strcat(pResultStr, " ");
			}

			int len = strlen(pResultStr) - 1;
			if(pResultStr[len] == ' ') pResultStr[len] = '\0';
		}
	}


	///////////////////////////////////////////////////////////
	//
	///////////////////////////////////////////////////////////
	void Clear()
	{
		if(pResult) delete [] pResult;
		if(pResultStr) delete [] pResultStr;
		pResult = NULL;
		pResultStr = NULL;
	}


} HASHRESULT;


/////////////////////////////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////////////////////////////
inline void _EcxHash(HASHRESULT* pHr, int nHashFunction, BYTE* pBuf, DWORD dwSize)
{
	pHr->InitResult(nHashFunction);

	switch(nHashFunction)
	{
		case HASH_RS:			_EcxHash_RS       (pBuf, dwSize, pHr->pResult);					break;
		case HASH_JS:			_EcxHash_JS       (pBuf, dwSize, pHr->pResult);					break;
		case HASH_PJW32:		_EcxHash_PJW32    (pBuf, dwSize, pHr->pResult);					break;
		case HASH_BKDR:			_EcxHash_BKDR     (pBuf, dwSize, 0, pHr->pResult);				break;
		case HASH_MABKDR:		_EcxHash_BKDR     (pBuf, dwSize, 1, pHr->pResult);				break;
		case HASH_SDBM:			_EcxHash_SDBM     (pBuf, dwSize, pHr->pResult);					break;
		case HASH_DJB:			_EcxHash_DJB      (pBuf, dwSize, pHr->pResult);					break;
		case HASH_DEK:			_EcxHash_DEK      (pBuf, dwSize, pHr->pResult);					break;
		case HASH_AP:			_EcxHash_AP       (pBuf, dwSize, pHr->pResult);					break;
		case HASH_LY:			_EcxHash_LY       (pBuf, dwSize, pHr->pResult);					break;
		case HASH_ROT13:		_EcxHash_ROT13    (pBuf, dwSize, pHr->pResult);					break;
		case HASH_FAQ6:			_EcxHash_FAQ6     (pBuf, dwSize, pHr->pResult);					break;
		case HASH_LOOKUP3:		_EcxHash_LOOKUP3  (pBuf, dwSize, pHr->pResult);					break;
		case HASH_FNV:			_EcxHash_FNV      (pBuf, dwSize, 0, pHr->pResult);				break;
		case HASH_MAFNV:		_EcxHash_FNV      (pBuf, dwSize, 1, pHr->pResult);				break;
		case HASH_MURMUR2:		_EcxHash_MURMUR2  (pBuf, dwSize, 0, pHr->pResult);				break;
		case HASH_MURMUR2A:		_EcxHash_MURMUR2  (pBuf, dwSize, 1, pHr->pResult);				break;
		case HASH_ADLER32:		_EcxHash_ADLER32  (pBuf, dwSize, 1, pHr->pResult);				break;

		case SUMM_08:			_EcxHash_SUMM     (pBuf, dwSize, (__int8)0, pHr->pResult);		break;
		case SUMM_16:			_EcxHash_SUMM     (pBuf, dwSize, (__int16)0, pHr->pResult);		break;
		case SUMM_32:			_EcxHash_SUMM     (pBuf, dwSize, (__int32)0, pHr->pResult);		break;
		case SUMM_64:			_EcxHash_SUMM     (pBuf, dwSize, (__int64)0, pHr->pResult);		break;

		case DIGEST_MD5:		_EcxDigest_MD5    (pBuf, dwSize, pHr->pResult);					break;
		case DIGEST_SHA224:
		case DIGEST_SHA256:		_EcxDigest_SHA    (pBuf, dwSize, nHashFunction, pHr->pResult);	break;
	}
	
	pHr->PrintResult(nHashFunction);
}





/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////























////////////////////////////////////////////////////////////////////////////
// CRC32
////////////////////////////////////////////////////////////////////////////

//  CRC
#define CRC_POLYNOM		0xEDB88320
#define CRC_INIT		0xFFFFFFFF
#define QUANT 1024 //   1024  


static unsigned long t[256];

inline void CRCInit() //   t[]
{
	int i;
	unsigned long x;

	for(i=0; i<=0xFF; i++)
	{
		x = i;
		x = (x>>1) ^ (CRC_POLYNOM & (-(signed long)(x & 1)));
		x = (x>>1) ^ (CRC_POLYNOM & (-(signed long)(x & 1)));
		x = (x>>1) ^ (CRC_POLYNOM & (-(signed long)(x & 1)));
		x = (x>>1) ^ (CRC_POLYNOM & (-(signed long)(x & 1)));

		x = (x>>1) ^ (CRC_POLYNOM & (-(signed long)(x & 1)));
		x = (x>>1) ^ (CRC_POLYNOM & (-(signed long)(x & 1)));
		x = (x>>1) ^ (CRC_POLYNOM & (-(signed long)(x & 1)));
		x = (x>>1) ^ (CRC_POLYNOM & (-(signed long)(x & 1)));
		t[i] = x;
	}
}


//  CRC32   buff  bufflen
inline unsigned long CRCCount(char* buf, DWORD bufflen)
{
	unsigned long crc = CRC_INIT;
	DWORD len4 = bufflen / 4;

	while(len4--)
	{
		crc = t[(crc ^ *buf++) & 0xFF] ^ (crc >> 8);
		crc = t[(crc ^ *buf++) & 0xFF] ^ (crc >> 8);
		crc = t[(crc ^ *buf++) & 0xFF] ^ (crc >> 8);
		crc = t[(crc ^ *buf++) & 0xFF] ^ (crc >> 8);
	}

	bufflen %= 4;
	
	while(bufflen--)
		crc = t[(crc ^ *buf++) & 0xFF] ^ (crc >> 8);
	
	return ~crc;
}






/////////////////////////////////////////////////////////////////////////////
#endif // USED_HASH



static DWORD g_dwCRC32B[] =
{
	0x00000000, 0x77073096, 0xEE0E612C, 0x990951BA, 0x076DC419, 0x706AF48F, 0xE963A535, 0x9E6495A3,
	0x0EDB8832, 0x79DCB8A4, 0xE0D5E91E, 0x97D2D988, 0x09B64C2B, 0x7EB17CBD, 0xE7B82D07, 0x90BF1D91,
	0x1DB71064, 0x6AB020F2, 0xF3B97148, 0x84BE41DE, 0x1ADAD47D, 0x6DDDE4EB, 0xF4D4B551, 0x83D385C7,
	0x136C9856, 0x646BA8C0, 0xFD62F97A, 0x8A65C9EC, 0x14015C4F, 0x63066CD9, 0xFA0F3D63, 0x8D080DF5,
	0x3B6E20C8, 0x4C69105E, 0xD56041E4, 0xA2677172, 0x3C03E4D1, 0x4B04D447, 0xD20D85FD, 0xA50AB56B,
	0x35B5A8FA, 0x42B2986C, 0xDBBBC9D6, 0xACBCF940, 0x32D86CE3, 0x45DF5C75, 0xDCD60DCF, 0xABD13D59,
	0x26D930AC, 0x51DE003A, 0xC8D75180, 0xBFD06116, 0x21B4F4B5, 0x56B3C423, 0xCFBA9599, 0xB8BDA50F,
	0x2802B89E, 0x5F058808, 0xC60CD9B2, 0xB10BE924, 0x2F6F7C87, 0x58684C11, 0xC1611DAB, 0xB6662D3D,
	0x76DC4190, 0x01DB7106, 0x98D220BC, 0xEFD5102A, 0x71B18589, 0x06B6B51F, 0x9FBFE4A5, 0xE8B8D433,
	0x7807C9A2, 0x0F00F934, 0x9609A88E, 0xE10E9818, 0x7F6A0DBB, 0x086D3D2D, 0x91646C97, 0xE6635C01,
	0x6B6B51F4, 0x1C6C6162, 0x856530D8, 0xF262004E, 0x6C0695ED, 0x1B01A57B, 0x8208F4C1, 0xF50FC457,
	0x65B0D9C6, 0x12B7E950, 0x8BBEB8EA, 0xFCB9887C, 0x62DD1DDF, 0x15DA2D49, 0x8CD37CF3, 0xFBD44C65,
	0x4DB26158, 0x3AB551CE, 0xA3BC0074, 0xD4BB30E2, 0x4ADFA541, 0x3DD895D7, 0xA4D1C46D, 0xD3D6F4FB,
	0x4369E96A, 0x346ED9FC, 0xAD678846, 0xDA60B8D0, 0x44042D73, 0x33031DE5, 0xAA0A4C5F, 0xDD0D7CC9,
	0x5005713C, 0x270241AA, 0xBE0B1010, 0xC90C2086, 0x5768B525, 0x206F85B3, 0xB966D409, 0xCE61E49F,
	0x5EDEF90E, 0x29D9C998, 0xB0D09822, 0xC7D7A8B4, 0x59B33D17, 0x2EB40D81, 0xB7BD5C3B, 0xC0BA6CAD,
	0xEDB88320, 0x9ABFB3B6, 0x03B6E20C, 0x74B1D29A, 0xEAD54739, 0x9DD277AF, 0x04DB2615, 0x73DC1683,
	0xE3630B12, 0x94643B84, 0x0D6D6A3E, 0x7A6A5AA8, 0xE40ECF0B, 0x9309FF9D, 0x0A00AE27, 0x7D079EB1,
	0xF00F9344, 0x8708A3D2, 0x1E01F268, 0x6906C2FE, 0xF762575D, 0x806567CB, 0x196C3671, 0x6E6B06E7,
	0xFED41B76, 0x89D32BE0, 0x10DA7A5A, 0x67DD4ACC, 0xF9B9DF6F, 0x8EBEEFF9, 0x17B7BE43, 0x60B08ED5,
	0xD6D6A3E8, 0xA1D1937E, 0x38D8C2C4, 0x4FDFF252, 0xD1BB67F1, 0xA6BC5767, 0x3FB506DD, 0x48B2364B,
	0xD80D2BDA, 0xAF0A1B4C, 0x36034AF6, 0x41047A60, 0xDF60EFC3, 0xA867DF55, 0x316E8EEF, 0x4669BE79,
	0xCB61B38C, 0xBC66831A, 0x256FD2A0, 0x5268E236, 0xCC0C7795, 0xBB0B4703, 0x220216B9, 0x5505262F,
	0xC5BA3BBE, 0xB2BD0B28, 0x2BB45A92, 0x5CB36A04, 0xC2D7FFA7, 0xB5D0CF31, 0x2CD99E8B, 0x5BDEAE1D,
	0x9B64C2B0, 0xEC63F226, 0x756AA39C, 0x026D930A, 0x9C0906A9, 0xEB0E363F, 0x72076785, 0x05005713,
	0x95BF4A82, 0xE2B87A14, 0x7BB12BAE, 0x0CB61B38, 0x92D28E9B, 0xE5D5BE0D, 0x7CDCEFB7, 0x0BDBDF21,
	0x86D3D2D4, 0xF1D4E242, 0x68DDB3F8, 0x1FDA836E, 0x81BE16CD, 0xF6B9265B, 0x6FB077E1, 0x18B74777,
	0x88085AE6, 0xFF0F6A70, 0x66063BCA, 0x11010B5C, 0x8F659EFF, 0xF862AE69, 0x616BFFD3, 0x166CCF45,
	0xA00AE278, 0xD70DD2EE, 0x4E048354, 0x3903B3C2, 0xA7672661, 0xD06016F7, 0x4969474D, 0x3E6E77DB,
	0xAED16A4A, 0xD9D65ADC, 0x40DF0B66, 0x37D83BF0, 0xA9BCAE53, 0xDEBB9EC5, 0x47B2CF7F, 0x30B5FFE9,
	0xBDBDF21C, 0xCABAC28A, 0x53B39330, 0x24B4A3A6, 0xBAD03605, 0xCDD70693, 0x54DE5729, 0x23D967BF,
	0xB3667A2E, 0xC4614AB8, 0x5D681B02, 0x2A6F2B94, 0xB40BBE37, 0xC30C8EA1, 0x5A05DF1B, 0x2D02EF8D
};


inline __naked DWORD _EcxHash_CRC32B(BYTE* pBuf, int nBufSize)
{
	__prolog

	__asm
	{
		push  esi
		push  ebx
		mov   esi, pBuf
		mov   ecx, nBufSize
		mov   eax, 0FFFFFFFFh
		xor   ebx, ebx

		or    ecx, ecx
		jz    M0;

M2:
		mov   edx, eax
		shr   edx, 8
		and   edx, 00FFFFFFh
		mov   bl, [esi]
		xor   eax, ebx
		and   eax, 0FFh
		mov   eax, dword ptr [g_dwCRC32B + eax * 4]
		xor   eax, edx

		inc   esi
		dec   ecx
		jnz   M2

M0:
		not   eax
		pop   ebx
		pop   esi
	}

	__epilog(0)
}














/////////////////////////////////////////////////////////////////////////////
#endif // __HASH_H__