// EGuid.h : header file
//

#ifndef __EGUID_H__
#define __EGUID_H__
#pragma once
////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////
//    Microsoft Remote Procedure Call (RPC)
////////////////////////////////////////////////////////////////////////////
#include "rpcdce.h"
#pragma comment (lib, "rpcrt4.lib")


////////////////////////////////////////////////////////////////////////////
//  EGuid.h
////////////////////////////////////////////////////////////////////////////
#define VERSION_EGUID   "11.09.2013"


// RFC 4122
// A UUID URN Namespace


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////

enum _UuidVariantType
{
	VARIANT_NCS,
	VARIANT_RFC4122,
	VARIANT_MICROSOFT,
	VARIANT_FUTURE
};

//		+-------------------------------------------------------------------------+
//		|  bit                      clock_seq_hi_and_variant                      |
//		|       +---+-------------------+-----------------------------------------+
//		| 7 6 5 | N | _UuidVariantType  |                                         |
//		+-------+---+-------------------+-----------------------------------------+
//		| 0 x x | 0 | VARIANT_NCS		| NCS (Network Computing System)          |
//		| 1 0 x | 1 | VARIANT_RFC4122	| RFC4122 - UUID URN Namespace            |
//		| 1 1 0 | 2 | VARIANT_MICROSOFT | Microsoft (GUID)                        |
//		| 1 1 1 | 3 | VARIANT_FUTURE	| Reserved for future expansion           |
//		+-------+---+-------------------+-----------------------------------------+


enum _UuidVersionType 
{
	VERSION_UNKNOWN,
	VERSION_TIME_BASED,				// Time-based, by using timestamp, clock sequence, and MAC network card address (if available) for the node sections
	VERSION_DCE_SECURITY,			// DCE Security version, with embedded POSIX UUIDs
	VERSION_NAME_BASED_MD5,
	VERSION_RANDOM_NUMBER_BASED,
	VERSION_NAME_BASED_SHA1,
};



#undef uuid_t
typedef struct
{
	DWORD time_low;
	WORD  time_mid;
	WORD  time_hi_and_version;
	BYTE  clock_seq_hi_and_variant;
	BYTE  clock_seq_low;
	BYTE  node[6];

	int GetVariant(BOOL isNumber)
	{
		//  5-7
		int n, variant = clock_seq_hi_and_variant >> 5;
		if(isNumber)
		{
			if((variant & 0x04) == 0x00) n = 0;
			if((variant & 0x06) == 0x04) n = 1;
			if((variant & 0x07) == 0x06) n = 2;
			if((variant & 0x07) == 0x07) n = 3;
		}
		else
			n = variant;
		return n;
	}
	int GetVersion()
	{
		//  12-15
		return time_hi_and_version >> 12;
	}

} uuid_t;





/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
enum _GuidViewType
{
	GUID_ERROR,
	GUID_SYMPLE,
	GUID_REGISTRY,
	GUID_OLECREATE,
	GUID_DEFINEGUID,
	GUID_CONSTGUID,
	GUID_GUIDDATA1,
	GUID_GUIDDATA2,
	GUID_VALUE128,
	GUID_USERPCLIB,
	UUID_001,
};


/////////////////////////////////////////////////////////////////////////////
//     guid,   nType.
/////////////////////////////////////////////////////////////////////////////
inline EString _EcxStringFromGUID(GUID guid, int nType = GUID_REGISTRY)
{
	EString s1, s2;
	switch(nType)
	{
		case GUID_SYMPLE:
		case GUID_REGISTRY:
		case GUID_VALUE128:
			s1 = "%08X-%04X-%04X-%02X%02X-%02X%02X%02X%02X%02X%02X";
			if(nType == GUID_REGISTRY) s1.Quote(TRUE, '{', '}');
			break;

		case GUID_OLECREATE:
			s1 = "IMPLEMENT_OLECREATE(<<class_name>>, <<external_name>>,\r\n\t0x%08X, 0x%04X, 0x%04X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X);";
			break;

		case GUID_DEFINEGUID:
			s1 = "_DEFINE_GUID(<<name>>, 0x%08X, 0x%04X, 0x%04X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X);";
			break;

		case GUID_CONSTGUID:
			s1 = "static const GUID <<name>> = { 0x%08X, 0x%04X, 0x%04X, { 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X }};";
			break;

		case GUID_GUIDDATA1:
			s1 = "GUID\r\n{\r\n    DWORD Data1    = 0x%08X;\r\n    WORD  Data2    = 0x%04X;\r\n    WORD  Data3    = 0x%04X;\r\n    BYTE  Data4[8] = { 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X };\r\n}";
			break;

		case GUID_GUIDDATA2:
			s1 = "Data1    = 0x%08X;\r\nData2    = 0x%04X;\r\nData3    = 0x%04X;\r\nData4[0] = 0x%02X;\r\nData4[1] = 0x%02X;\r\nData4[2] = 0x%02X;\r\nData4[3] = 0x%02X;\r\nData4[4] = 0x%02X;\r\nData4[5] = 0x%02X;\r\nData4[6] = 0x%02X;\r\nData4[7] = 0x%02X;";
			break;
		
		case GUID_USERPCLIB:
		{
			BYTE* pStr;
			UuidToString(&guid, &pStr);
			s1 = (char*)pStr;
			RpcStringFree(&pStr);
			return s1;
		}

		case UUID_001:
		{
			s1 = "time_low = 0x%08X;\r\ntime_mid = 0x%04X;\r\ntime_hi_and_version = 0x%04X;\r\nclock_seq_hi_and_variant = 0x%02X;\r\nclock_seq_low = 0x%02X\r\nnode[0] = 0x%02X;\r\nnode[1] = 0x%02X;\r\nnode[2] = 0x%02X;\r\nnode[3] = 0x%02X;\r\nnode[4] = 0x%02X;\r\nnode[5] = 0x%02X;";
			uuid_t* pUuid = (uuid_t*)&guid;
			s2.Format(s1, pUuid->time_low, pUuid->time_mid, pUuid->time_hi_and_version, pUuid->clock_seq_hi_and_variant, pUuid->clock_seq_low, pUuid->node[0], pUuid->node[1], pUuid->node[2], pUuid->node[3], pUuid->node[4], pUuid->node[5]);

			s1.Format("\r\n\r\nvariant = 0x%02X (%d);", pUuid->GetVariant(FALSE), pUuid->GetVariant(TRUE));
			s2 += s1;
			s1.Format("\r\nversion = 0x%02X;", pUuid->GetVersion());
			s2 += s1;
			
			return s2;
			break;
		}
	}

	s2.Format(s1, guid.Data1, guid.Data2, guid.Data3, guid.Data4[0], guid.Data4[1], guid.Data4[2], guid.Data4[3], guid.Data4[4], guid.Data4[5], guid.Data4[6], guid.Data4[7]);
	if(nType == GUID_VALUE128) s2.Remove('-');
	return s2;
}


/////////////////////////////////////////////////////////////////////////////
//  GUID ( 128- ).   pGuid
//   NULL,       .
//     guid,   nType.
inline EString _EcxGenerateGUID(int nType, GUID* pGuid = NULL)
{
	EString str;
	GUID guid;
	UuidCreate(&guid);
	str = _EcxStringFromGUID(guid, nType);
	if(pGuid) *pGuid = guid;
	return str;
}


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
inline int _EcxIsGuid(char* pGuid)
{
	int nType = GUID_ERROR;

	if(!_EcxStrIsNull(pGuid))
	{
		char* tmp = pGuid;
		BOOL res = FALSE;

		int len = strlen(pGuid);

		switch(len)
		{
			// {20D04FE0-3AEA-1069-A2D8-08002B30309D}
			case 38:
				if(*tmp == '{' && *(tmp + len - 1) == '}')
				{
					nType = GUID_REGISTRY;
					tmp++;
				}
				break;

			// 20D04FE0-3AEA-1069-A2D8-08002B30309D
			case 36:
				nType = GUID_SYMPLE;
				break;

			// 0x20D04FE03AEA1069A2D808002B30309D
			// 20D04FE03AEA1069A2D808002B30309D
			case 34:
			case 32:
				nType = GUID_VALUE128;
				break;
		}

		if(nType != GUID_ERROR)
		{
			if(nType == GUID_VALUE128)
				res = _EcxIsHexSequence(&tmp, 32); //???? 0x
			else
			{
				BOOL n0, n1, n2, n3, n4, n5;
				n0 = (*(tmp + 8) == '-') && (*(tmp + 13) == '-') && (*(tmp + 18) == '-') && (*(tmp + 23) == '-');
				char* p1;
				p1 = tmp +  0; n1 = _EcxIsHexSequence(&p1, 8);
				p1 = tmp +  9; n2 = _EcxIsHexSequence(&p1, 4);
				p1 = tmp + 14; n3 = _EcxIsHexSequence(&p1, 4);
				p1 = tmp + 19; n4 = _EcxIsHexSequence(&p1, 4);
				p1 = tmp + 24; n5 = _EcxIsHexSequence(&p1, 12);
				res = n0 && n1 && n2 && n3 && n4 && n5;
			}
			if(!res) nType = GUID_ERROR;
		}
	}
	return nType;
}

// /(0[x|X])?\h{32}/
// /\h{8}(-\h{4}){3}-\h{12}/

/*
typedef struct _GUID
{
	DWORD Data1;
	WORD  Data2;
	WORD  Data3;
	BYTE  Data4[8];
} GUID;


UUID
IID
CLSID


//////////////////////////////////////////////////////////////////////////////
// 
//////////////////////////////////////////////////////////////////////////////
EString _EcxStringFromCLSID(GUID guid)
{
	char szGuid[256];
	wsprintf(szGuid, "{%08X-%04X-%04X-%02X%02X-%02X%02X%02X%02X%02X%02X}",
		guid.Data1, guid.Data2, guid.Data3,
		guid.Data4[0], guid.Data4[1], guid.Data4[2], guid.Data4[3],
		guid.Data4[4], guid.Data4[5], guid.Data4[6], guid.Data4[7]);
	return szGuid;
}


*/

////////////////////////////////////////////////////////////////////////////
//   GUID
////////////////////////////////////////////////////////////////////////////
#define _DEFINE_GUID(name, dw, w1, w2, b1, b2, b3, b4, b5, b6, b7, b8) \
			GUID name = { dw, w1, w2, { b1, b2, b3, b4, b5, b6, b7, b8 } }

#define _DEFINE_OLEGUID(name, dw, w1, w2) \
			_DEFINE_GUID(name, dw, w1, w2, 0xC0,0,0,0,0,0,0,0x46)


////////////////////////////////////////////////////////////////////////////
//     GUID
////////////////////////////////////////////////////////////////////////////
typedef GUID& RFGUID;




////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////
class EGuid
{
public:

	EGuid();
	EGuid(EGuid& src);
	EGuid(EGuid* src);
	EGuid(GUID& src);
	EGuid(GUID* src);
	EGuid(BYTE* src);
	EGuid(LPCSTR src, int nMode = GUID_REGISTRY);

	void Create(GUID* pGuid = NULL);
	void Create(BYTE* pBuf);
	void Create(DWORD dw, WORD w1, WORD w2, BYTE b1, BYTE b2, BYTE b3, BYTE b4, BYTE b5, BYTE b6, BYTE b7, BYTE b8);
	void Create(LPCSTR str, int nMode = GUID_REGISTRY);

	GUID& GetAt();
	operator RFGUID();
	operator LPGUID();

	const EGuid& operator=(EGuid& src);
	const EGuid& operator=(EGuid* src);
	const EGuid& operator=(GUID& src);
	const EGuid& operator=(GUID* src);
	const EGuid& operator=(BYTE* src);
	const EGuid& operator=(LPCSTR src);

	int Generate();
	WORD GetHash();
	EString Format(int nMode = GUID_REGISTRY);
	
	WORD IsNull();
	BOOL IsEqual(EGuid* pGuid);
	BOOL IsEqual(GUID* pGuid);
	BOOL IsEqual(BYTE* pBuf);
	BOOL IsEqual(LPCSTR str);

	//  
	friend BOOL __stdcall operator==(EGuid& guid1, EGuid& guid2);
	friend BOOL __stdcall operator==(EGuid& guid1, GUID guid2);
	friend BOOL __stdcall operator==(GUID guid1, EGuid& guid2);
	friend BOOL __stdcall operator!=(EGuid& guid1, EGuid& guid2);
	friend BOOL __stdcall operator!=(EGuid& guid1, GUID guid2);
	friend BOOL __stdcall operator!=(GUID guid1, EGuid& guid2);


	GUID m_guid;
};






////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline EGuid::EGuid()
	{ Create(); }
inline EGuid::EGuid(EGuid& src)
	{ Create(&src.m_guid); }
inline EGuid::EGuid(EGuid* src)
	{ Create(&src->m_guid); }
inline EGuid::EGuid(GUID& src)
	{ Create(&src); }
inline EGuid::EGuid(GUID* src)
	{ Create(src); }
inline EGuid::EGuid(BYTE* src)
	{ Create(src); }
inline EGuid::EGuid(LPCSTR src, int nMode)
	{ Create(src, nMode); }


////////////////////////////////////////////////////////////////////////////
//    - m_guid
////////////////////////////////////////////////////////////////////////////
inline GUID& EGuid::GetAt()
	{ return m_guid; }
inline EGuid::operator RFGUID()
	{ return m_guid; }
inline EGuid::operator LPGUID()
	{ return &m_guid; }

////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline const EGuid& EGuid::operator=(EGuid& src)
	{ Create(&src.m_guid); return *this; }
inline const EGuid& EGuid::operator=(EGuid* src)
	{ Create(&src->m_guid); return *this; }
inline const EGuid& EGuid::operator=(GUID& src)
	{ Create(&src); return *this; }
inline const EGuid& EGuid::operator=(GUID* src)
	{ Create(src); return *this; }
inline const EGuid& EGuid::operator=(BYTE* src)
	{ Create(src); return *this; }
inline const EGuid& EGuid::operator=(LPCSTR src)
	{ Create(src, GUID_REGISTRY); return *this; }


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline void EGuid::Create(GUID* pGuid)
{
	if(pGuid)
		memcpy(&m_guid, pGuid, sizeof(GUID));
	else
		memset(&m_guid, 0, sizeof(GUID));
}

////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline void EGuid::Create(BYTE* pBuf)
{
	if(::IsBadReadPtr(pBuf, sizeof(GUID)))
		Create();
	else
		m_guid = *(GUID*)pBuf;
}

////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline void EGuid::Create(DWORD dw, WORD w1, WORD w2, BYTE b1, BYTE b2, BYTE b3, BYTE b4, BYTE b5, BYTE b6, BYTE b7, BYTE b8)
{
	m_guid.Data1 = dw;
	m_guid.Data2 = w1;
	m_guid.Data3 = w2;
	m_guid.Data4[0] = b1;
	m_guid.Data4[1] = b2;
	m_guid.Data4[2] = b3;
	m_guid.Data4[3] = b4;
	m_guid.Data4[4] = b5;
	m_guid.Data4[5] = b6;
	m_guid.Data4[6] = b7;
	m_guid.Data4[7] = b8;
}

////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline void EGuid::Create(LPCSTR str, int nMode)
{
	int dw, w1, w2, b1, b2, b3, b4, b5, b6, b7, b8;
	EString strFormat;
	if(nMode == GUID_REGISTRY) strFormat = "{%08X-%04X-%04X-%02X%02X-%02X%02X%02X%02X%02X%02X}";
	if(nMode == GUID_VALUE128) strFormat = "%08X%04X%04X%02X%02X%02X%02X%02X%02X%02X%02X";
	sscanf(str, strFormat, &dw, &w1, &w2, &b1, &b2, &b3, &b4, &b5, &b6, &b7, &b8);
	Create(dw, w1, w2, b1, b2, b3, b4, b5, b6, b7, b8);
}


// RPC_S_OK					Success 
// RPC_S_UUID_LOCAL_ONLY	The UUID is guaranteed to be unique to this machine only. 
// RPC_S_UUID_NO_ADDRESS	Cannot get Ethernet or token-ring hardware address for this computer 

////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline int EGuid::Generate()
{
	return UuidCreate(&m_guid);
}

////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline EString EGuid::Format(int nMode)
{
	return _EcxStringFromGUID(m_guid, nMode);
}

////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline WORD EGuid::GetHash()
{
	RPC_STATUS status;
	return UuidHash(&m_guid, &status);
	
//		BYTE* p = (BYTE*)&m_guid;
//		WORD hash = *(WORD*)p ^ *(WORD*)(p + 2) ^ *(WORD*)(p + 4) ^ *(WORD*)(p + 6) ^ *(WORD*)(p + 8) ^ *(WORD*)(p + 10) ^ *(WORD*)(p + 12) ^ *(WORD*)(p + 14);
//		return hash;
}

////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline WORD EGuid::IsNull()
{
	RPC_STATUS status;
	return UuidIsNil(&m_guid, &status);
	
//		BYTE* p = (BYTE*)&m_guid;
//		BOOL res = !*(DWORD*)p && !*(DWORD*)(p + 4) && !*(DWORD*)(p + 8) && !*(DWORD*)(p + 12);
//		return res;
}

////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline BOOL EGuid::IsEqual(GUID* pGuid)
	{ return (_memicmp(&m_guid, pGuid, sizeof(GUID)) == 0); }

inline BOOL EGuid::IsEqual(EGuid* pGuid)
	{ return IsEqual(&pGuid->m_guid); }

inline BOOL EGuid::IsEqual(BYTE* pBuf)
	{ EGuid tmp; tmp.Create(pBuf); return tmp.IsEqual(&m_guid); }

inline BOOL EGuid::IsEqual(LPCSTR str)
	{ EGuid tmp; tmp.Create(str); return tmp.IsEqual(&m_guid); }


////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////
inline BOOL __stdcall operator==(EGuid& guid1, EGuid& guid2)
	{ return guid1.IsEqual(&guid2.m_guid); }
inline BOOL __stdcall operator==(EGuid& guid1, GUID guid2)
	{ return guid1.IsEqual(&guid2); }
inline BOOL __stdcall operator==(GUID guid1, EGuid& guid2)
	{ return guid2.IsEqual(&guid1); }
inline BOOL __stdcall operator!=(EGuid& guid1, EGuid& guid2)
	{ return !guid1.IsEqual(&guid2.m_guid); }
inline BOOL __stdcall operator!=(EGuid& guid1, GUID guid2)
	{ return !guid1.IsEqual(&guid2); }
inline BOOL __stdcall operator!=(GUID guid1, EGuid& guid2)
	{ return !guid2.IsEqual(&guid1); }





////////////////////////////////////////////////////////////////////////////
#endif // __EGUID_H__
