/** \file EBitArray.h
	  EBitArray.
*/

#ifndef __EBITARRAY_H__
#define __EBITARRAY_H__
#pragma once
////////////////////////////////////////////////////////////////////////////

#include "..\StringManipulation\StringManipulationFunc.h"
#include "..\TextParsing\TextParsingMacro.h"
#include "CharacterClassificationFunc.h"
#include "CharacterClassificationMacro.h"


///  EBitArray.h
#define VERSION_EBITARRAY   "20.03.2013"

///////////////////////////////////////////////////////////////////////
/** \defgroup hEBitArray EBitArray
	  EBitArray.
	\ingroup gCharClass
*/

//    'function' : no return value
#pragma warning (disable : 4035)

////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
#define PM_BITARRAY		0
#define PM_CHARARRAY	1


///	\ingroup hEBitArray
///   .
class EBitArray  // ba
{
public:

	/// .
	EBitArray()
	{
		Clear();
	}

	///  .
	void AddArray(EBitArray* pBa)
	{
		for(int i=0; i<8; i++) m_nBitArray[i] |= pBa->m_nBitArray[i];
	}
	///  .
	void AddArray(int* pArray)
	{
		for(int i=0; i<8; i++) m_nBitArray[i] |= pArray[i];
	}

	///   .
	void SetArray(int* pArray)
	{
		for(int i=0; i<8; i++)
			m_nBitArray[i] = (pArray == NULL) ? 0xFFFFFFFF : pArray[i];
	}

	///   .
	void SetMask(DWORD dwMask)
	{
		for(int i=0; i<256; i++)
			if(is_char(i, dwMask)) SetBit(i);
	}

	///   .
	void Clear()
	{
		int* pArray = m_nBitArray;
		__asm
		{
			cld
			mov eax, 0
			mov edi, pArray
			mov ecx, 8
			rep stosd
		}
	}

	///  .
	void SetBit(int pos)
	{
		int* pArray = m_nBitArray;
		__asm
		{
			mov ebx, pos
			mov ecx, ebx
			and ebx, 255
			shr ebx, 5				; ebx = ((pos & 255) >> 5)
			and ecx, 31
			
			mov edx, pArray
			bts [edx + 4 * ebx], ecx
		}
	}

	///  .
	void ResBit(int pos)
	{
		int* pArray = m_nBitArray;
		__asm
		{
			mov ebx, pos
			mov ecx, ebx
			and ebx, 255
			shr ebx, 5				; ebx = ((pos & 255) >> 5)
			and ecx, 31
			
			mov edx, pArray
			btr [edx + 4 * ebx], ecx
		}
	}

	///  .
	BOOL IsBit(int pos)
	{
		int* pArray = m_nBitArray;
		__asm
		{
			mov ebx, pos
			mov ecx, ebx
			and ebx, 255
			shr ebx, 5				; ebx = ((pos & 255) >> 5)
			and ecx, 31
			xor eax, eax

			mov edx, pArray
			bt [edx + 4 * ebx], ecx
			
			rcl eax, 1
		}
	}

	///  .
	void InvBit(int pos)
	{
		int* pArray = m_nBitArray;
		__asm
		{
			mov ebx, pos
			mov ecx, ebx
			and ebx, 255
			shr ebx, 5				; ebx = ((pos & 255) >> 5)
			and ecx, 31
			
			mov edx, pArray
			btc [edx + 4 * ebx], ecx
		}
	}

	///   .
	void SetRange(int pos1, int pos2)
	{
		int* pArray = m_nBitArray;
		__asm
		{
			mov esi, 0xFFFFFFFF

			mov ebx, pos1
			mov ecx, ebx
			and ebx, 255
			shr ebx, 5				; ebx = ((pos1 & 255) >> 5)
			
			and ecx, 31
			mov eax, esi
			shl eax, cl				; eax = 0xFFFFFFFF << (pos1 & 31)
			
			mov edx, pos2
			mov ecx, 31
			and edx, ecx
			sub ecx, edx
			mov edx, esi
			shr edx, cl
			mov edi, edx			; edi = 0xFFFFFFFF >> (31 - pos2 & 31)
			
			mov edx, pos2
			and edx, 255
			shr edx, 5				; edx = ((pos2 & 255) >> 5)
			mov ecx, edx
			
			cmp ebx, edx
			pushf
			jnz M1					; (pos1 >> 5) != (pos2 >> 5)
			
			and eax, edi			; eax &= 0xFFFFFFFF >> (31 - pos2 & 31)

		M1:	mov edx, pArray
			or [edx + 4 * ebx], eax

			popf
			jz M2					; (pos1 >> 5) == (pos2 >> 5)
			
			or [edx + 4 * ecx], edi
			
			cld
			mov eax, esi
			sub ecx, ebx
			dec ecx
			inc ebx
			shl ebx, 2
			add edx, ebx
			mov edi, edx
			rep stosd
		M2:
		}
	}

	/// .
	void Inverse()
	{
		for(int i=0; i<8; i++)
			m_nBitArray[i] ^= 0xFFFFFFFF;
	}

	///  .
	void Filling()
	{
		memset(m_nBitArray, 0xFFFFFFFF, 8 * sizeof(int));
	}

	///   [set].
	int ParseSet(LPCSTR set)
	{
		if(_EcxStrIsNull(set)) return 0;
		Clear();

		int ch1, ch2, inv = 0;
		char* szSet = new char[strlen(set) + 1];
		strcpy(szSet, set);
		char* tmp = szSet;
		int last = strlen(szSet) - 1;

		//      ]
		if(*(tmp + last) == ']') *(tmp + last) = '\0';
		
		//      [
		if(*tmp == '[') tmp++;
		
		//    ^
		if(*tmp == '^')
		{
			inv = 1;
			tmp++;
		}

		//   []   [-]
		if(*tmp == '\0' || *(WORD*)tmp == 0x002D)
		{
			Filling();
			goto LABEL_R0;
		}

		ch1 = ch2 = 0;

		//   [-...
		if(*tmp == '-') ch1 = 0x00000100;

		while(*tmp)
		{
			//     c1, .  [-...
			if(ch1 == 0x00000100) goto LABEL_R2;

			//   [:...
			if(*(WORD*)tmp == 0x3A5B)
			{
				DWORD dwCharClass = _EcxCharClassFromPOSIX(&tmp, TRUE);
				SetMask(dwCharClass);
				if(dwCharClass != 0) continue;
			}

			//   c1
			ch1 = _EcxEscapeToChar(&tmp);

			//   [c1-... 
			if(*tmp == '-')
			{
LABEL_R2:
				ch1 &= 0x000000FF;
				tmp++;
				ch2 = _EcxEscapeToChar(&tmp);

				//     c2, .  ...-]
				if(ch2 == 0) ch2 = 255;
				if(ch1 > ch2) _EcxSwap(&ch1, &ch2);
				
				//  
				SetRange(ch1, ch2);
				continue;
			}
			SetBit(ch1);
		}

LABEL_R0:
		if(inv) Inverse();
		delete [] szSet;
		return last + 1;
	}

	///   [set].
	int GetSetLength(LPCSTR set)
	{
		if(_EcxStrIsNull(set)) return 0;
		if(*set != '[') return 0;
		char* tmp = (char*)set;

		while(1)
		{
			//    
			if(is_char(*tmp, CCM_EOT)) return 0;
			//   :]   \]
			if(*(WORD*)tmp == 0x5D3A || *(WORD*)tmp == 0x5D5C)
			{
				tmp += 2;
				continue;
			}
			//   ]]
			if(*(WORD*)tmp == 0x5D5D) tmp++;
			if(*tmp == ']') break;
			tmp++;
		}
		tmp++;
		return tmp - set;
	}

	/// .
	char* Print(char* str, int mode)
	{
		int i, j, n;
		char ch, s[8];

		strcpy(str, "    ");
		for(i=0; i<16; i++)
		{
			sprintf(s, " %0.1X", i);
			strcat(str, s);
		}
		strcat(str, "\r\n   |--------------------------------\r\n");

		for(j=0; j<16; j++)
		{
			sprintf(s, "%.1X0 | ", j);
			strcat(str, s);

			for(i=0; i<16; i++)
			{
				n = 16 * j + i;
				switch(mode)
				{
					case PM_BITARRAY:
					{
						sprintf(s, "%.1X ", IsBit(n));
						break;
					}
					case PM_CHARARRAY:
					{
						ch = (IsBit(n)) ? ((n < 0x20) ? '\x1A' : (char)n) : ' ';
						sprintf(s, "%c ", ch);
						break;
					}
				}
				strcat(str, s);
			}
			strcat(str, "\r\n");
		}
		return str;
	}

	///    .
	typedef struct _Ranges
	{
		int nPos1;
		int nPos2;

	} RANGES;


	///    .
	void SetRanges(int nRanges, RANGES* pRange)
	{
		for(int i=0; i<nRanges; i++)
		{
			SetRange(pRange[i].nPos1, pRange[i].nPos2);
		}
	}

	///    32- ,   256 .
	int m_nBitArray[8];
};


///	\ingroup hEBitArray
///     .
#define _SetRanges(ba, arr)	 ba.SetRanges(sizeof(arr) / sizeof(EBitArray::RANGES), arr)


/** \defgroup xEBitArray   
	\ingroup hEBitArray
*/
//#ifdef USED_PREDEFINEDBITARRAY

	/** \defgroup xEBitA001 
		   .
		\ingroup xEBitArray
		@{
	*/
	static EBitArray::RANGES  RNG_CCM_WORD[]    = { 48,57, 65,90, 97,122, 95,95 };
	static EBitArray::RANGES  RNG_CCM_ALPHA[]   = { 65,90, 97,122 };
	static EBitArray::RANGES  RNG_CCM_NUMBER[]  = { 48,57 };
	static EBitArray::RANGES  RNG_CCM_HEX[]     = { 48,57, 65,70, 97,102 };
	/// @}

	/** \defgroup xEBitA002  CP1251
		     CP1251.
		\ingroup xEBitArray
		@{
	*/
	static int ARR_CCM_WORD[]   = { 0x00000000, 0x03FF0000, 0x87FFFFFE, 0x07FFFFFE, 0x00000000, 0x00000000, 0x00000000, 0x00000000 };
	static int ARR_CCM_PUNCT[]  = { 0x00000000, 0x0C001308, 0x38000000, 0x28000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000 };
	static int ARR_CCM_OPERAT[] = { 0x00000000, 0xF000ECF6, 0x40000001, 0x50000001, 0x00000000, 0x00000000, 0x00000000, 0x00000000 };
	static int ARR_CCM_SPACE[]  = { 0x00002600, 0x00000001, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000 };
	static int ARR_CCM_STOP[]   = { 0x00002401, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000 };
	static int ARR_CCM_ALPHA[]  = { 0x00000000, 0x00000000, 0x07FFFFFE, 0x07FFFFFE, 0x00000000, 0x00000000, 0x00000000, 0x00000000 };
	static int ARR_CCM_NUMBER[] = { 0x00000000, 0x03FF0000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000 };
	static int ARR_CCM_HEX[]    = { 0x00000000, 0x03FF0000, 0x0000007E, 0x0000007E, 0x00000000, 0x00000000, 0x00000000, 0x00000000 };
	static int ARR_CCM_LOWER[]  = { 0x00000000, 0x00000000, 0x00000000, 0x07FFFFFE, 0xF4010008, 0xD5180004, 0x00000000, 0xFFFFFFFF };
	static int ARR_CCM_UPPER[]  = { 0x00000000, 0x00000000, 0x07FFFFFE, 0x00000000, 0x0000F403, 0x2004852A, 0xFFFFFFFF, 0x00000000 };
	/// @}

	/** \defgroup xEBitA003  CP866
		     CP866.
		\ingroup xEBitArray
		@{
	*/
	static int ARR_OEM_RUS[] = { 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xFFFFFFFF, 0x0000FFFF, 0x00000000, 0x0003FFFF };
	static int ARR_OEM_UKR[] = { 0x00000000, 0x00000080, 0x00000000, 0x00000000, 0xD3FFFFFF, 0x0000FFFF, 0x00000000, 0x003CD3FF };
	static int ARR_OEM_BRD[] = { 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xFFF80000, 0x07FFFFFF, 0x00000000 };
	/// @}

//#endif // USED_PREDEFINEDBITARRAY


/////////////////////////////////////////////////////////////////////////////
#endif // __EBITARRAY_H__
