// DebugRoutines.h : header file
//

#ifndef __DEBUGROUTINES_H__
#define __DEBUGROUTINES_H__
#pragma once
/////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////
//  DebugRoutines.h
/////////////////////////////////////////////////////////////////////////////
#define VERSION_DEBUGROUTINES   "22.10.2012"


#define STOP_IN_ASSERT	3
#define STOP_IN_ERROR	4


/////////////////////////////////////////////////////////////////////////////
//    
/////////////////////////////////////////////////////////////////////////////
inline void _EcxTrace(LPCSTR lpszFormat, ...)
{
	va_list args;
	va_start(args, lpszFormat);

	char szBuffer[512];
	int nBuf = _vsnprintf(szBuffer, 512, lpszFormat, args);

#ifdef _DEBUG
	_CrtDbgReport(_CRT_WARN, NULL, 0, NULL, "%s", szBuffer);
#endif

	va_end(args);
}


/////////////////////////////////////////////////////////////////////////////
//   
/////////////////////////////////////////////////////////////////////////////
inline BOOL _EcxDebugInfoBox(char* lpszFileName, int nLineNum)
{
	char szName[MAX_PATH + 32], szBuf[MAX_PATH * 4];

	//  
	::GetModuleFileName(NULL, szName, MAX_PATH);
	wsprintf(szBuf, ":\n\"%s\"\n\n", szName);
	wsprintf(szName, "   :\n\"%s\"\n\n", lpszFileName);
	lstrcat(szBuf, szName);
	wsprintf(szName, " : %d\n", nLineNum);
	lstrcat(szBuf, szName);

	//   
	int nCode = ::MessageBox(NULL, szBuf, " ASSERT", MB_ABORTRETRYIGNORE|MB_ICONERROR|MB_TASKMODAL);

	// <>:  
	if(nCode == IDABORT) _exit(STOP_IN_ASSERT);

	// <>:  
	if(nCode == IDRETRY) return TRUE;

	// <>:  
	return FALSE;
}


/////////////////////////////////////////////////////////////////////////////
//   
/////////////////////////////////////////////////////////////////////////////
inline void _EcxErrorBox(DWORD dwError, char* lpszFileName, int nLineNum, BOOL isYesNo = FALSE, char** lpErrorStr = NULL)
{
	char szStr[MAX_PATH + 32], szBuf[MAX_PATH * 8];

	//  
	::GetModuleFileName(NULL, szStr, MAX_PATH);
	wsprintf(szBuf, ":\n\"%s\"\n", szStr);
	wsprintf(szStr, "   :\n\"%s\"\n", lpszFileName);
	lstrcat(szBuf, szStr);
	wsprintf(szStr, " : %d\n\n", nLineNum);
	lstrcat(szBuf, szStr);
	wsprintf(szStr, " : %d\n", dwError);
	lstrcat(szBuf, szStr);

	if(lpErrorStr == NULL)
	{
		::FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, NULL, dwError,
			MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), szStr, MAX_PATH, NULL);
	}
	else
	{
		lstrcpy(szStr, (LPCTSTR)lpErrorStr[dwError]);
	}
	lstrcat(szBuf, szStr);

	DWORD dwButtons = MB_OK;
	if(isYesNo)
	{
		dwButtons = MB_YESNO;
		lstrcat(szBuf, "\n\n ?");
	}

	//   
	int nCode = ::MessageBox(NULL, szBuf, " ERROR", dwButtons|MB_ICONERROR|MB_TASKMODAL);
	if(!isYesNo) nCode = IDYES;

	//  
	if(nCode == IDYES) _exit(STOP_IN_ERROR);
}


/////////////////////////////////////////////////////////////////////////////
//   
/////////////////////////////////////////////////////////////////////////////
inline void _EcxWarningBox(DWORD dwError, LPCSTR lpErrorStr, LPCSTR lpWarningStr = NULL)
{
	char szStr[MAX_PATH + 32], szBuf[MAX_PATH * 8];

	//  
	wsprintf(szStr, " : %d\n", dwError);
	lstrcpy(szBuf, szStr);
	wsprintf(szStr, "%s\n", lpErrorStr);
	lstrcat(szBuf, szStr);

	if(lpWarningStr != NULL)
	{
		lstrcat(szBuf, lpWarningStr);
	}

	//   
	int nCode = ::MessageBox(NULL, szBuf, " WARNING", MB_OK|MB_ICONWARNING|MB_TASKMODAL);
}




/////////////////////////////////////////////////////////////////////////////
//    
/////////////////////////////////////////////////////////////////////////////
#ifdef _DEBUG

	#define EXC_ASSERT(condition) \
		if((!(condition)) && _EcxDebugInfoBox(__FILE__, __LINE__)) _CrtDbgBreak()

	#define EXC_ERROR_(errorcode)  _EcxErrorBox(errorcode, __FILE__, __LINE__)
	#define EXC_ERROR(errorcode, isyesno, strarray)  _EcxErrorBox(errorcode, __FILE__, __LINE__, isyesno, strarray)
	#define EXC_WARNING(errorcode, strwarning)  _EcxWarningBox(errorcode, #errorcode, strwarning)
	#define EXC_TRACE  ::_EcxTrace
		
#else

	#define EXC_ASSERT(condition) ( (void)0 )
	#define EXC_ERROR_(errorcode) ( (void)0 )
	#define EXC_ERROR(errorcode, isyesno, strarray) ( (void)0 )
	#define EXC_WARNING(errorcode, strarray) ( (void)0 )
	#define EXC_TRACE ( (void)0 )

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
//    
//       .
//       m_dwError  DWORD.
/////////////////////////////////////////////////////////////////////////////
#define _ErrLastError(variable, value)		if(variable == value) { m_dwError = ::GetLastError(); return FALSE; }
#define _ErrTest(condition, error)			if(condition) { m_dwError = error; return FALSE; }



/////////////////////////////////////////////////////////////////////////////
#endif // __DEBUGROUTINES_H__
