// UpdatePatch.h : header file
//

#ifndef __UPDATEPATCH_H__
#define __UPDATEPATCH_H__

#pragma once
/////////////////////////////////////////////////////////////////////////////

#include "imagehlp.h"
#pragma comment(lib, "imagehlp.lib")


////////////////////////////////////////////////////////////////////////////
//  UpdatePatch.h
////////////////////////////////////////////////////////////////////////////
#define VERSION_UPDATEPATCH   "17.07.2020"



/////////////////////////////////////////////////////////////////////////////
 
#define IDS_ERRBADFILENAME    1000 
 
typedef struct tagDbgSection *LPSECTIONINFO;
typedef struct tagDbgSection 
{ 
	char	szSection[IMAGE_SIZEOF_SHORT_NAME]; 
	ULONG	uVirtualAddress; 
	ULONG	uSize; 
	LPSECTIONINFO	Next; 

} SECTIONINFO; 
 
 

typedef struct _IMAGE_IMPORT_MODULE_DIRECTORY
{
	DWORD    dwRVAFunctionNameList;
	DWORD    dwUseless1;
	DWORD    dwUseless2;
	DWORD    dwRVAModuleName;
	DWORD    dwRVAFunctionAddressList;

} IMAGE_IMPORT_MODULE_DIRECTORY, *PIMAGE_IMPORT_MODULE_DIRECTORY;
 
 
// global prototypes for functions 
PIMAGE_FILE_HEADER WINAPI FileHeaderOffset(LPVOID); 
PIMAGE_OPTIONAL_HEADER WINAPI OptionalHeaderOffset(LPVOID); 
PIMAGE_SECTION_HEADER WINAPI SectionHeaderOffset(LPVOID); 
PIMAGE_IMPORT_MODULE_DIRECTORY WINAPI ImportDirectoryOffset(LPVOID); 
int WINAPI NumOfSections(LPVOID); 
void WINAPI RetrieveModuleName(char*, HANDLE); 
void WINAPI RetrieveSectionNames(HANDLE, HANDLE, SECTIONINFO**); 
LPVOID WINAPI GetModuleEntryPoint(LPVOID); 
LPVOID WINAPI GetImageBase(LPVOID); 
int WINAPI GetImportModuleNames(LPVOID, HANDLE, char**); 
int WINAPI GetImportFunctionNamesByModule(LPVOID, HANDLE, char*, char**); 
int WINAPI GetExportFunctionNames(LPVOID, HANDLE, char**); 
 



/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
template <class TYPE>
inline void _set_data(BYTE** ppBuf, TYPE tData)
{
	*(TYPE*)(*ppBuf) = tData;
	*ppBuf += sizeof(TYPE);
}


/////////////////////////////////////////////////////////////////////////////
//  HEX  INT
/////////////////////////////////////////////////////////////////////////////
inline unsigned long _htoi(LPSTR& buf, int count)
{
	unsigned long n = 0;
	if(cmp2(buf, 0x5830) || cmp2(buf, 0x7830)) buf += 2;  // 0x  0X

	while(1)
	{
		char ch = *buf;
		if(!is_char(ch, CCM_HEX)) break;
		if(count == 0) break;
		buf++;
		ch = _EcxConvertHex(ch);
		n <<= 4;
		n += ch - '0';
		count--;
	}
	return n;
}


/////////////////////////////////////////////////////////////////////////////
//   HEX-  count
/////////////////////////////////////////////////////////////////////////////
inline BOOL _is_hex(LPSTR buf, int count)
{
	char* tmp = buf;
	BOOL isHex = FALSE;
	if(cmp2(tmp, 0x5830) || cmp2(tmp, 0x7830)) tmp += 2;  // 0x  0X

	while(1)
	{
		if(!is_char(*tmp, CCM_HEX)) break;
		if(count == 0) break;
		tmp++;
		count--;
		isHex = TRUE;
	}
	if(isHex)
	{
//		if(count < 0) isHex = FALSE;
		if(is_char(*tmp, CCM_SPACE|CCM_STOP|CCM_HEX) || *tmp == ':' || *tmp == '/') return TRUE;
	}
	return FALSE;
}


/////////////////////////////////////////////////////////////////////////////

//----------------------------- NEAR JUMP -----------------------------------

	// JMP offset;	in memory: EB xxxx
	// Jcc offset;	in memory: 7n xxxx


//----------------------------- FAR JUMP ------------------------------------
	
	// JMP short addr;	in memory: E9 xxxxxxxx
	// Jcc short addr;	in memory: 0F 8n xxxxxxxx


// xxxx -   
// xxxxxxxx -    ;
// n -   
// cc - 


/////////////////////////////////////////////////////////////////////////////

// string: 0-JMP;n-Jcc;...

static char _jumps[] = "0-JMP;4-JE;5-JNE;4-JZ;5-JNZ;2-JC;3-JNC;8-JS;9-JNS;F-JG;E-JNG;D-JGE;C-JNGE;C-JL;D-JNL;E-JLE;F-JNLE;7-JA;6-JNA;3-JAE;2-JNAE;2-JB;3-JNB;6-JBE;7-JNBE;";




/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
inline BYTE _get_jump_cc(LPSTR& string)
{
	char* begin_pos = string;
	skip_to_class(string, CCM_SPACE|CCM_STOP);

	int len = string - begin_pos;
	char* jump = new char[len + 1];
	memcpy(jump, begin_pos, len);
	jump[len] = '\0';
	::CharUpper(jump);

	char* dest = strstr(_jumps, jump);
	delete [] jump;

	if(dest == NULL) return 0xFF;		// error!!!
	return _EcxConvertHex(*(dest - 2)) - '0';
}






//  
#define FT_COMMIT	SEC_COMMIT
#define FT_IMAGE	SEC_IMAGE


//   
enum _LastErrors
{
	ERR_IMAGE_FILE_CORRUPT				= 0x20FF0000,
	ERR_FILE_NOT_OPENED					= 0x20FF0001,
	ERR_FILE_NOT_READED					= 0x20FF0002,
	ERR_NOT_FOUND_IMAGE_DOS_SIGNATURE	= 0x20FF0003,
	ERR_NOT_FOUND_IMAGE_NT_SIGNATURE	= 0x20FF0004,
	ERR_BAD_VIRTUAL_ADDRESS				= 0x20FF0005,
	ERR_BAD_OFFSET						= 0x20FF0006,
	ERR_BAD_SYNTAX						= 0x20FF0007,
	ERR_TO_BIG_LABEL_INDEX				= 0x20FF0008,
};


//    
static char* _UpdatePatchErrorStr[] =
{
	" , , .",
	"    EUpdatePatch::Open.",
	"    EUpdatePatch::Read.",
	"  DOS .",
	"  NT .",
	"   .",
	"  .",
	"  .",
	"   127.",
};


//  
enum _TheMessageTypes
{
	MSG_LAST_ERROR,
	MSG_USER_ERROR,
};


//  
enum _ImageDirectoryEntryes
{
	TAB_EXPORT			=0,		// Export Directory					.edata
	TAB_IMPORT			=1,		// Import Directory					.idata
	TAB_RESOURCE		=2,		// Resource Directory				.rsrc
	TAB_EXCEPTION		=3,		// Exception Directory				?
	TAB_SECURITY		=4,		// Security Directory				?
	TAB_BASERELOC		=5,		// Base Relocation Table			.reloc
	TAB_DEBUG			=6,		// Debug Directory					.rdata
	TAB_ARCHITECTURE	=7,		// Description String				?
	TAB_GLOBALPTR		=8,		// Relative Virtual Addr			?
	TAB_TLS				=9,		// Thread Local Storage Directory	.tls
	TAB_LOADCONFIG		=10,	// Load Config Directory			?
	TAB_BOUNDIMPORT		=11,	// Bound Import Directory			?
	TAB_IAT				=12,	// Import Address Table				?
	TAB_DELAYIMPORT		=13,	// Delay Import Table				?
	TAB_COMDESCR		=14,	// COM Descriptor Table				?
	TAB_RESERVED		=15		// Reserved							?
};

#ifdef USED_IMAGEDIRECTORYENTRYESSTR
	static char* _ImageDirectoryEntryesStr[][2] =
	{
		"Export Directory",			".edata",
		"Import Directory",			".idata",
		"Resource Directory",		".rsrc",
		"Exception Directory",		"",
		"Security Directory",		"",
		"Base Relocation Table",	".reloc",
		"Debug Directory",			".rdata",
		"Description String",		"",
		"Machine Value",			"",
		"TLS Directory",			".tls",
		"Load Config Directory",	"",
		"Bound Import Directory",	"",
		"Import Address Table",		"",
		"Delay Load Import",		"",
		"COM Runtime Descriptor",	"",
		"<reserved>",				""
	};
#endif // USED_IMAGEDIRECTORYENTRYESSTR







/////////////////////////////////////////////////////////////////////////////
// Class EUpdatePatch
/////////////////////////////////////////////////////////////////////////////

class EUpdatePatch		// up 
{
public:

	 EUpdatePatch();
	~EUpdatePatch();


	DWORD VAtoFA(DWORD dwVirtAddr, BOOL isShow = FALSE);
	BOOL VAPatchData(BYTE* buf, LPCSTR lpszPatch, BOOL isBuffer);
	EString PrintBuffer(BYTE* buf, int nSize);



public:
	
	LPAPI_VERSION GetImageHelpVersion(BOOL isShow = FALSE);


	BOOL Open(LPCTSTR lpszFileName); //=
	BOOL Read(); //=
	void Write(); //=
	void Close(); //=

	inline DWORD GetLength(); //=
	inline BYTE* GetBuffer(); //=

	inline DWORD GetLastError(); //=

	char* IMG_GetSectionStr(char* dest, BOOL isFullInfo, int nMode = 0, int nSection = 0xFFFFFFFF);
	char* IMG_GetTableStr(char* dest, BOOL isFullInfo, int nMode = 0, int nTable = 0xFFFFFFFF);







	DWORD GetTableVA(int nTable); //=



	void ShowMessages(BOOL isShowMsg);
	char* ErrorMessage(LPCTSTR lpszMessage, int nMessageType = MSG_LAST_ERROR);


	void DeletePacth();




public:

	char m_szFileName[_MAX_FNAME + _MAX_EXT];
	char m_szPatchName[_MAX_FNAME + _MAX_EXT];
	HANDLE	m_hFile; //=


//private:
	LPSTR m_lpszPatch;

	LPSTR m_lpszFileName;
	BOOL m_isTmpFile;

	BOOL	m_isShowMsg;
	BOOL	m_isOpen; //=
	BOOL	m_isRead; //=

	HANDLE  m_hMap; //=
	BYTE*	m_pBuff; // get
	DWORD	m_dwFileLength; // get
	DWORD	m_dwError; // get

	DWORD	m_dwOffsFileHeader;
	DWORD	m_dwOffsOptionalHeader;
	DWORD	m_dwOffsFirstSection;

	WORD	m_nNumberOfSections;
	DWORD	m_nImageBase;

	IMAGE_DOS_HEADER*		pImageDosHeader;
	IMAGE_FILE_HEADER*		pImageFileHeader;
	IMAGE_OPTIONAL_HEADER*	pImageOptionalHeader;

	IMAGE_SECTION_HEADER*	pImageSectionHeader;

	char m_error[128];

};



/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////

inline DWORD EUpdatePatch::GetLastError() { return m_dwError; };
inline DWORD EUpdatePatch::GetLength() { return m_dwFileLength; };
inline BYTE* EUpdatePatch::GetBuffer() { return (m_isRead) ? m_pBuff : NULL; };
inline DWORD EUpdatePatch::GetTableVA(int nTable) { return pImageOptionalHeader->DataDirectory[nTable].VirtualAddress; };
inline void EUpdatePatch::ShowMessages(BOOL isShowMsg) { m_isShowMsg = isShowMsg; };


/////////////////////////////////////////////////////////////////////////////
// Macro
/////////////////////////////////////////////////////////////////////////////


#define _WARNING(condition, errorcode) \
				if(condition) { \
				m_dwError = errorcode; \
				WARNING(errorcode & 0x000000FF, _UpdatePatchErrorStr); \
				return FALSE; }


//#define _ErrLastError(variable, value)		if(variable == value) { m_dwError = ::GetLastError(); return FALSE; }
//#define _ErrTest(condition, error)			if(condition) { m_dwError = error; return FALSE; }

/////////////////////////////////////////////////////////////////////////////

#define EGIN_ERROR_STR(name)				static char* _##name##_ErrorTable[][3] = {
#define END_ERROR_STR()						0, 0, 0 };
#define ERROR_STR(errcode, description)		(char*)errcode, #errcode, description,


enum _LastErrors1
{
	ERR_IMAGE_FILE_CORRUPT1			= 0x20FF0000,
	ERR_FILE_NOT_OPENED1			= 0x20FF0001,
};


EGIN_ERROR_STR(EUpdatePatch)
	ERROR_STR(ERR_FILE_NOT_OPENED1, "    EUpdatePatch::Open.")
END_ERROR_STR()


////////////////////////////////////////////////////////////////////////////

//    IMAGEHLP.DLL
//  isShow  TRUE,     
inline LPAPI_VERSION EUpdatePatch::GetImageHelpVersion(BOOL isShow)
{
	LPAPI_VERSION lpapiver = ::ImagehlpApiVersion();

	char str[256];
	if(isShow)
	{
		sprintf(str, "IMAGEHLP.DLL\n\n%d.%.2d.%.2d", lpapiver->MajorVersion, lpapiver->MinorVersion, lpapiver->Revision);
		::MessageBox(NULL, str, "PE Image Helper", MB_OK | MB_ICONINFORMATION);
	}

	return lpapiver;
}


/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////////////////////////////
inline EUpdatePatch::EUpdatePatch()
{
	m_isShowMsg = FALSE;
	m_isTmpFile = FALSE;
	m_isOpen = m_isRead = FALSE;
	m_lpszPatch = NULL;
	m_lpszFileName = new char[MAX_PATH];

	m_szFileName[0] = '\0';
	m_szPatchName[0] = '\0';
	m_dwError = 0;

	Close();
}


/////////////////////////////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////////////////////////////
inline EUpdatePatch::~EUpdatePatch()
{
	delete [] m_lpszFileName;

	DeletePacth();
	Close();
}


/////////////////////////////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////////////////////////////
inline void EUpdatePatch::DeletePacth()
{
	if(m_lpszPatch) delete [] m_lpszPatch;
	m_lpszPatch = NULL;
}


/////////////////////////////////////////////////////////////////////////////
//  
/////////////////////////////////////////////////////////////////////////////
inline BOOL EUpdatePatch::Open(LPCTSTR lpszFileName)
{
	//     -  
	Close();
	strcpy(m_lpszFileName, lpszFileName);

	//  
	if((m_hFile = ::CreateFile(lpszFileName, GENERIC_READ|GENERIC_WRITE, FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL|FILE_FLAG_WRITE_THROUGH, NULL)) == INVALID_HANDLE_VALUE)
//	if((m_hFile = ::CreateFile(lpszFileName, GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL)) == INVALID_HANDLE_VALUE)
	{
		m_dwError = ::GetLastError();

		//     
		if(m_dwError == ERROR_SHARING_VIOLATION)
		{
			//     
			::GetTempPath(MAX_PATH, m_lpszFileName);
			::GetTempFileName(m_lpszFileName, "~UP", 0, m_lpszFileName);

			//      
			if(::CopyFile(lpszFileName, m_lpszFileName, FALSE))
			{
				m_isTmpFile = TRUE;
				return Open(m_lpszFileName);
			}
		}

		return FALSE;
	}

	//  
	m_dwFileLength = ::GetFileSize(m_hFile, NULL);
	_ErrLastError(m_dwFileLength, 0xFFFFFFFF);

	//  
	m_dwError = ERROR_SUCCESS;
	m_isOpen = TRUE;
	return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
//         
/////////////////////////////////////////////////////////////////////////////
inline void EUpdatePatch::Close()
{
	//     
	if(m_isRead)
	{
		::UnmapViewOfFile((LPCVOID)m_pBuff);
		::CloseHandle(m_hMap);

		if(pImageSectionHeader != NULL)
		{
			delete [] pImageSectionHeader;
			pImageSectionHeader = NULL;
		}
	}

	//    
	if(m_isOpen)
	{
		::CloseHandle(m_hFile);

		if(m_isTmpFile)
		{
			::DeleteFile(m_lpszFileName);
			m_isTmpFile = FALSE;
		}
	}

	//  -
	m_pBuff = NULL;
	m_hMap = NULL;
	m_hFile = NULL;
	m_isOpen = FALSE;
	m_isRead = FALSE;
	
	m_dwFileLength = m_nImageBase = 0xFFFFFFFF;
	m_nNumberOfSections = 0;
	pImageSectionHeader = NULL;

//	LARGE_INTEGER Frequency;
//	BOOL res = ::QueryPerformanceFrequency(&Frequency);
}


/////////////////////////////////////////////////////////////////////////////
//         
/////////////////////////////////////////////////////////////////////////////
inline BOOL EUpdatePatch::Read()
{
	// ,    ;  
	m_dwError = ERROR_SUCCESS;

	// ,     
	if(m_isRead) return TRUE;

	//   
	if(m_isOpen)
	{
		//    
		m_hMap = ::CreateFileMapping(m_hFile, NULL, PAGE_READWRITE | FT_COMMIT, 0, 0, NULL);
		_ErrLastError(m_hMap, NULL);

		//     
		m_pBuff = (BYTE*)::MapViewOfFile(m_hMap, FILE_MAP_ALL_ACCESS, 0, 0, 0);
		_ErrLastError(m_pBuff, NULL);

		//   MZ-;   IMAGE_DOS_HEADER
		pImageDosHeader = (IMAGE_DOS_HEADER*)m_pBuff;
		_ErrTest(pImageDosHeader->e_magic != IMAGE_DOS_SIGNATURE, ERR_NOT_FOUND_IMAGE_DOS_SIGNATURE);

		//    IMAGE_FILE_HEADER;   
		m_dwOffsFileHeader = pImageDosHeader->e_lfanew + sizeof(DWORD);
		_ErrTest(m_dwOffsFileHeader > m_dwFileLength, ERR_IMAGE_FILE_CORRUPT);

		//  IMAGE_NT_SIGNATURE
		_ErrTest(*(DWORD*)(m_pBuff + m_dwOffsFileHeader - sizeof(DWORD)) != IMAGE_NT_SIGNATURE, ERR_NOT_FOUND_IMAGE_NT_SIGNATURE);

		//   IMAGE_FILE_HEADER
		pImageFileHeader = (IMAGE_FILE_HEADER*)(m_pBuff + m_dwOffsFileHeader);

		//    IMAGE_OPTIONAL_HEADER;   
		m_dwOffsOptionalHeader = m_dwOffsFileHeader + IMAGE_SIZEOF_FILE_HEADER;
		pImageOptionalHeader = (PIMAGE_OPTIONAL_HEADER)(m_pBuff + m_dwOffsOptionalHeader);

		//  ImageBase   
		m_nImageBase = pImageOptionalHeader->ImageBase;
		m_nNumberOfSections = pImageFileHeader->NumberOfSections;

		//     
		m_dwOffsFirstSection = m_dwOffsOptionalHeader + pImageFileHeader->SizeOfOptionalHeader;

		//     ;   
		pImageSectionHeader = new IMAGE_SECTION_HEADER[m_nNumberOfSections];
		memcpy(pImageSectionHeader, m_pBuff + m_dwOffsFirstSection, m_nNumberOfSections * IMAGE_SIZEOF_SECTION_HEADER);

		m_isRead = TRUE;
		return TRUE;
	}

	m_dwError = ERR_FILE_NOT_OPENED;
	return FALSE;
}


/////////////////////////////////////////////////////////////////////////////
//       
/////////////////////////////////////////////////////////////////////////////
inline void EUpdatePatch::Write()
{
	if(m_isOpen && m_isRead)
	{
		BOOL res = ::FlushViewOfFile(m_pBuff, 0);
	}
}


/////////////////////////////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////////////////////////////
inline char* EUpdatePatch::IMG_GetSectionStr(char* dest, BOOL isFullInfo, int nMode, int nSection)
{
	if(dest == NULL) return NULL;
	dest[0] = '\0';
	char tmp[256], name[IMAGE_SIZEOF_SHORT_NAME + 1];
	int n1 = 0, n2 = m_nNumberOfSections - 1;
	if(nSection != 0xFFFFFFFF) n1 = n2 = nSection;

	if(m_isOpen && m_isRead)
	{
		for(int i=n1; i<=n2; i++)
		{
			strncpy(name, (LPCTSTR)pImageSectionHeader[i].Name, IMAGE_SIZEOF_SHORT_NAME);
			name[IMAGE_SIZEOF_SHORT_NAME] = '\0';
			if(nMode == 0)
			{
				sprintf(tmp, "%2d %8s   va: 0x%.8X; vs: 0x%.8X; fa: 0x%.8X; fs: 0x%.8X\r\n",
					i, name,
					pImageSectionHeader[i].VirtualAddress,
					pImageSectionHeader[i].Misc.VirtualSize,
					pImageSectionHeader[i].PointerToRawData,
					pImageSectionHeader[i].SizeOfRawData);
			}
			if(nMode == 1)
			{
				sprintf(tmp, "%2d %8s   0x%.8X - 0x%.8X; 0x%.8X - 0x%.8X\r\n",
					i, name,
					pImageSectionHeader[i].VirtualAddress,
					pImageSectionHeader[i].VirtualAddress + pImageSectionHeader[i].Misc.VirtualSize - 1,
					pImageSectionHeader[i].PointerToRawData,
					pImageSectionHeader[i].PointerToRawData + pImageSectionHeader[i].SizeOfRawData - 1);
				strcat(dest, tmp);
			}
		}
	}
	return dest;
}


/////////////////////////////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////////////////////////////
inline char* EUpdatePatch::IMG_GetTableStr(char* dest, BOOL isFullInfo, int nMode, int nTable)
{
	if(dest == NULL) return NULL;
	dest[0] = '\0';
	char tmp[256], s1[48];
	int n1 = 0, n2 = IMAGE_NUMBEROF_DIRECTORY_ENTRIES - 1;
	if(nTable != 0xFFFFFFFF) n1 = n2 = nTable;

	if(m_isOpen && m_isRead)
	{
		for(int i=n1; i<=n2; i++)
		{
			IMAGE_DATA_DIRECTORY* pIdd = &pImageOptionalHeader->DataDirectory[i];
			s1[0] = '\0';
			if(pIdd->VirtualAddress != 0)
			{
				if(nMode == 0)  sprintf(s1, "(va: 0x%.8X; vs: 0x%.8X)", pIdd->VirtualAddress, pIdd->Size);
				if(nMode == 1)  sprintf(s1, "(0x%.8X - 0x%.8X)", pIdd->VirtualAddress, pIdd->VirtualAddress + pIdd->Size);
			}
#ifdef USED_IMAGEDIRECTORYENTRYESSTR
			if(isFullInfo)
				sprintf(tmp, "%2d %8s - %22s %s\r\n", i, _ImageDirectoryEntryesStr[i][1], _ImageDirectoryEntryesStr[i][0], s1);
			else
				sprintf(tmp, "%2d %s\r\n", i, s1);
#else
				sprintf(tmp, "%2d %s\r\n", i, s1);
#endif // USED_IMAGEDIRECTORYENTRYESSTR
			strcat(dest, tmp);
		}
	}
	return dest;
}




/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
inline char* EUpdatePatch::ErrorMessage(LPCTSTR lpszMessage, int nMessageType)
{
	UINT uIcon = nMessageType & 0x0FFFFFFF;
	LPVOID lpBuffer = NULL;
	char* lpszMsg;
	m_error[0] = '\0';

	// 
	if(nMessageType == MSG_LAST_ERROR)
	{
		//    
		if(m_dwError == ERROR_SUCCESS) uIcon = MB_ICONINFORMATION;
		
		//    
		if(m_dwError >= 0x20FF0000)
		{
			lpszMsg = _UpdatePatchErrorStr[m_dwError & 0x000000FF];
			strcpy(m_error, lpszMsg);
		}

		//    
		else
		{
			if(m_dwError == 0) return NULL;

			DWORD res = ::FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER|FORMAT_MESSAGE_FROM_SYSTEM|FORMAT_MESSAGE_IGNORE_INSERTS, NULL, m_dwError, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR)&lpBuffer, 0, NULL);
			lpszMsg = (char*)lpBuffer;
			strcpy(m_error, lpszMsg);
			if(*(WORD*)lpszMsg == 0x3125) // %1
			{
				strcpy(m_error, m_szFileName);
				strcpy(m_error + strlen(m_szFileName), lpszMsg + 2);
			}
			if(lpBuffer != NULL) ::LocalFree(lpBuffer);
			return m_error;
		}
	}
	else
	{
		if(lpszMessage != NULL) strcpy(m_error, lpszMessage);
	}
	return m_error;
}


/*
	MSG_LAST_ERROR
	MSG_USER_ERROR

*/









/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
inline DWORD EUpdatePatch::VAtoFA(DWORD dwVirtAddr, BOOL isShow)
{
	int i;
	DWORD dwFA, dwVA = dwVirtAddr;

	try
	{
		if(dwVA < m_nImageBase) throw (DWORD)0xFFFFFFFF;

		dwVA -= m_nImageBase;	// - ImageBase

		// sections
		for(i=0; i<m_nNumberOfSections; i++)
		{
			if(dwVA >= pImageSectionHeader[i].VirtualAddress && dwVA < pImageSectionHeader[i].VirtualAddress + pImageSectionHeader[i].SizeOfRawData)
			{
				throw pImageSectionHeader[i].PointerToRawData + dwVA - pImageSectionHeader[i].VirtualAddress;
			}
		}

		throw (DWORD)0xFFFFFFFF;
	}

	catch(DWORD dwAddr)
	{
		dwFA = dwAddr;
		
		if(isShow)
		{
			char str[256];
			char name[IMAGE_SIZEOF_SHORT_NAME + 1];

			if(dwFA < 0xFFFFFFFF)
			{
				strcpy(name, (LPCTSTR)pImageSectionHeader[i].Name);
				sprintf(str, ": %s\n : 0x%.8X\n :  0x%.8X", name, dwVirtAddr, dwFA);
			}
			else
			{
				sprintf(str, "  .\n : 0x%0.8X", dwVirtAddr);
			}
			
			::MessageBox(::GetForegroundWindow(), str, "VAtoFA", MB_OK | ((dwFA < 0xFFFFFFFF) ? MB_ICONINFORMATION : MB_ICONERROR));
		}
	}

	return dwFA;
}



/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
inline BOOL EUpdatePatch::VAPatchData(BYTE* buf, LPCSTR lpszPatch, BOOL isBuffer)
{
	char *szPatch, *tmp;
	BYTE *pos = NULL;
	BOOL isOffset = FALSE, isNear = FALSE, isLabel = FALSE,isImport = FALSE, isRepeat = FALSE;
	BOOL isJMP = FALSE, isCALL = FALSE, isREP = FALSE;
	BOOL isProc = FALSE;
	DWORD fa=0, addr1, addr2, xx, rep = 1, m;
	DWORD size = 0; //  
	BYTE cc;

	typedef struct
	{
		int nLabel;
		int nSize;
		DWORD dwJmpInstAddr;
		BYTE* pos;
	} JMPS;
	JMPS arrJmps[256];
	DWORD arrLabs[8192];
	int nJmps = 0;
	int nLabs = 0;

//	WARNING(ERR_FILE_NOT_OPENED, _EUpdatePatch_ErrorTable[0][1]);
//	_WARNING(m_isOpen == FALSE, ERR_FILE_NOT_OPENED);

//	_ErrTest(m_isOpen == FALSE, ERR_FILE_NOT_OPENED);
//	_ErrTest(m_isRead == FALSE, ERR_FILE_NOT_READED);

	if(isBuffer || (m_isOpen && m_isRead))
	{
		try
		{
			szPatch = new char[lstrlen(lpszPatch) + 1];
			strcpy(szPatch, lpszPatch);
			tmp = szPatch;
			if(isBuffer) pos = buf;

			while(1)
			{
				_EcxSkipNonInformation(&tmp);
				//   
				if(is_eot(tmp)) break;
				if(cmpi3(tmp, 0x00464F45)) break; // EOF
				
				//         hex-
				if(isProc == FALSE && is_char(*tmp, CCM_HEX))
				{
					if(!_is_hex(tmp, 8)) throw (DWORD)ERR_BAD_SYNTAX;
					addr1 = _htoi(tmp, 8);
					if(!isBuffer)
					{
						fa = VAtoFA(addr1);
						if(fa == 0xFFFFFFFF) throw (DWORD)ERR_BAD_VIRTUAL_ADDRESS;
						pos = buf + fa;
					}
					else
						pos = buf;
					isProc = TRUE;
					continue;
				}

				//   ( : )
				if(*tmp == 0x3A)
				{
					tmp++;
					continue;
				}

				//    ( END )
				if(cmpi3(tmp, 0x00444E45))
				{
					isProc = FALSE;
					tmp += 3;
					continue;
				}

				//  ( Mnnn [non_info] [:] )
				if(is_char(*tmp, CCM_ALPHA))
				{
					//   
					char* p0 = tmp;
					p0++;
					skip_digit(p0);
					_EcxSkipNonInformation(&p0, NULL, SKIP_BLANK|SKIP_NEWLINE_BREAK|SKIP_MULTYLINE_COMMENT|SKIP_ONELINE_COMMENT);

					//   
					if((is_char(*p0, CCM_EOL) && (isJMP || isCALL)) || *p0 == ':')
					{
						BYTE ch = (*(BYTE*)tmp & 0xDF) - 'A';
						tmp++;
						m = __strtoul(tmp);
						if(m < 0 || m > 127) throw (DWORD)ERR_TO_BIG_LABEL_INDEX;
						m = (m << 5) | ch;

						//     ( Mnnn )
						if(isJMP || isCALL)
						{
							isLabel = TRUE;
							arrJmps[nJmps].nLabel = m;
							arrJmps[nJmps].dwJmpInstAddr = addr1;
							arrJmps[nJmps].pos = pos;
							nJmps++;
							goto M1;
						}
						//   ( Mnnn: )
						else
						{
							arrLabs[m] = addr1;
							nLabs++;
						}
						continue;
					}
				}

				if(isJMP)
				{
					//   ( NEAR, SHORT )
					if(cmpi4(tmp, 0x5241454E))
					{
						isNear = TRUE;
						tmp += 4;
						continue;
					}
					if(cmpi5(tmp, 0x524F4853, 0x54))
					{
						isNear = TRUE;
						tmp += 5;
						continue;
					}

					//  ( $ )
					if(*tmp == '$')
					{
						isOffset = TRUE;
						tmp++;
						continue;
					}

					if(!isLabel)
					{
						//    
						if(!_is_hex(tmp, 8)) throw (DWORD)ERR_BAD_SYNTAX;
						addr2 = _htoi(tmp, 8);
						xx = addr2 - addr1;
						if(isOffset)
						{
							xx = addr2 + 2;
							isNear = TRUE;
							if((int)xx < -126 || xx > 129 || xx == 0 || xx == 1) throw (DWORD)ERR_BAD_OFFSET;
						}
					}
M1:
					//   - 
					if(isNear == FALSE)
					{
						size = 5;
						*(pos++) = (cc == 0x00) ? 0xE9 : 0x0F;
						if(cc != 0x00)
						{
							*(pos++) = 0x80 | cc;
							size++;
						}
						xx -= size;
						if(isLabel) xx = 0;
						_set_data(&pos, (DWORD)xx);
					}
					//   - 
					else
					{
						size = 2;
						*(pos++) = (cc == 0x00) ? 0xEB : 0x70 | cc;
						xx -= 2;
						if(isLabel) xx = 0;
						*(pos++) = (BYTE)xx;
					}
					
					if(isLabel) arrJmps[nJmps - 1].nSize = size;
					addr1 += size;
					isJMP = isOffset = isNear = isLabel = FALSE;
					continue;
				}

				if(isCALL)
				{
					//   ( IMP )
					if(cmpi3(tmp, 0x00504D49))
					{
						isImport = TRUE;
						tmp += 3;
						continue;
					}

					if(!isLabel)
					{
						//  
						if(!_is_hex(tmp, 8)) throw (DWORD)ERR_BAD_SYNTAX;
						addr2 = _htoi(tmp, 8);
					}

					//  
					if(isImport == FALSE)
					{
						size = 5;
						xx = addr2 - addr1 - 5;
						*(pos++) = 0xE8;
					}
					//  
					else
					{
						size = 6;
						xx = addr2;
						_set_data(&pos, (WORD)0x15FF);
					}
					_set_data(&pos, (DWORD)xx);

					if(isLabel) arrJmps[nJmps - 1].nSize = size;
					addr1 += size;
					isCALL = isImport = isLabel = FALSE;
					continue;
				}

				if(isREP)
				{
					if(!isRepeat)
					{
						rep = __strtoul(tmp);
						if(rep == 0) throw (DWORD)ERR_BAD_SYNTAX;
						isRepeat = TRUE;
						continue;
					}
					else
					{
						if(*tmp == ',')
						{
							tmp++;
							continue;
						}
						else
						{
							if(is_char(*tmp, CCM_HEX))
							{
								isREP = isRepeat = FALSE;
								goto LAB_RAWCODE;
							}
						}
						throw (DWORD)ERR_BAD_SYNTAX;
					}
					isREP = FALSE;
				}

				// JMP, Jcc
				if(cmpi1(tmp, 0x4A))
				{
					isJMP = TRUE;
					cc = _get_jump_cc(tmp);
					if(cc == 0xFF) throw (DWORD)ERR_BAD_SYNTAX;
					continue;
				}

				// CALL
				if(cmpi4(tmp, 0x4C4C4143))
				{
					isCALL = TRUE;
					tmp += 4;
					continue;
				}

				// REP
				if(cmpi3(tmp, 0x00504552))
				{
					isREP = TRUE;
					tmp += 3;
					continue;
				}

				// "String"
				if(*tmp == 0x22)
				{
					int n = _EcxGetString(&tmp, GSTR_LENGTH, FALSE, TRUE, TRUE);
					char* dest = new char[n + 1];
					_EcxGetString(&tmp, dest, FALSE, TRUE, TRUE);
					dest[n] = '\0';
					char* s1 = dest;

					for(int i=0; i<=n; i++)
					{
						cc = (BYTE)*(s1++);
						*pos++ = cc;
					}
					delete [] dest;
					addr1 += n;
					continue;
				}

				if(pos == NULL) throw (DWORD)ERR_BAD_SYNTAX;
				if(!is_char(*tmp, CCM_HEX) && !is_char(*(tmp + 1), CCM_HEX)) throw (DWORD)ERR_BAD_SYNTAX;

LAB_RAWCODE:
				// raw code
				if(!_is_hex(tmp, 2)) throw (DWORD)ERR_BAD_SYNTAX;
				cc = (BYTE)_htoi(tmp, 2);

				for(DWORD i=0; i<rep; i++)
				{
					*pos++ = cc;
					addr1++;
				}

				rep = 1;
			}

			if(nJmps)
			{
				
				for(int i=0; i<nJmps; i++)
				{
					m = arrJmps[i].nLabel;
					addr1 = arrJmps[i].dwJmpInstAddr;
					addr2 = arrLabs[m];
					size = arrJmps[i].nSize;
					pos = arrJmps[i].pos;
					xx = addr2 - addr1 - size;

					if(size == 2)
					{
						pos++;
						*pos = (BYTE)xx;
					}
					else
					{
						pos += size - 4;
						_set_data(&pos, (DWORD)xx);
					}
				}
			}

			throw (DWORD)ERROR_SUCCESS;
		}

		catch(DWORD dwEnd)
		{
			m_dwError = dwEnd;
			delete [] szPatch;
		}
	}

	return (m_dwError == ERROR_SUCCESS);
}

/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
inline EString EUpdatePatch::PrintBuffer(BYTE* buf, int nSize)
{
	EString s1, s2;

	for(int j=0; j<nSize / 16 + 1; j++)
	{
		for(int i=0; i<16; i++)
		{
			s2.Format("%0.2X ", buf[j * 16 + i]);
			s1 += s2;
			if(i == 7) s1 += " ";
		}
		s1 += "\r\n";
	}
	return s1;
}





/////////////////////////////////////////////////////////////////////////////

PIMAGE_SECTION_HEADER WINAPI IDSectionHeaderOffset(LPVOID); 




// return offset to file header 
inline PIMAGE_FILE_HEADER WINAPI FileHeaderOffset(LPVOID lpFile) 
{ 
	int ImageHdrOffset = 0; 

	// if DOS based file, skip DOS header and file signature 
	if(*((USHORT *)lpFile) == IMAGE_DOS_SIGNATURE) 
	{ 
		// file image header offset exists after DOS header and nt signature 
		ImageHdrOffset = (int)((ULONG *)lpFile)[15] + sizeof (ULONG); 
		if(*((ULONG *)((char *)lpFile + ImageHdrOffset - sizeof (ULONG))) != IMAGE_NT_SIGNATURE) 
		return NULL; 
	} 

	// optional header exists immediately after file header and image header 
	return (PIMAGE_FILE_HEADER)((int)lpFile + ImageHdrOffset); 
} 




/* return optional header data */ 
inline PIMAGE_OPTIONAL_HEADER WINAPI OptionalHeaderOffset ( 
LPVOID lpFile) 
{ 
int ImageHdrOffset = 0; 

/* if DOS based file, skip DOS header and file signature */ 
if(*((USHORT *)lpFile) == IMAGE_DOS_SIGNATURE) 
{ 
/* file image header offset exists after DOS header and nt signature */ 
ImageHdrOffset = (int)((ULONG *)lpFile)[15] + sizeof (ULONG); 
if(*((ULONG *)((char *)lpFile + ImageHdrOffset - sizeof (ULONG))) != IMAGE_NT_SIGNATURE) 
return NULL; 
} 

/* optional header exists immediately after file header and image header */ 
return (PIMAGE_OPTIONAL_HEADER)((char *)lpFile + ImageHdrOffset + sizeof (IMAGE_FILE_HEADER)); 
} 




/* return pointer to first section header */ 
inline PIMAGE_SECTION_HEADER WINAPI SectionHeaderOffset ( 
LPVOID lpFile) 
{ 
int ImageHdrOffset = 0; 

/* if DOS based file, skip DOS header and file signature */ 
if(*((USHORT *)lpFile) == IMAGE_DOS_SIGNATURE) 
{ 
/* file image header offset exists after DOS header and nt signature */ 
ImageHdrOffset = (int)((ULONG *)lpFile)[15] + sizeof (ULONG); 
if(*((ULONG *)((char *)lpFile + ImageHdrOffset - sizeof (ULONG))) != IMAGE_NT_SIGNATURE) 
return NULL; 
} 

/* optional header exists immediately after file header and image header */ 
return (PIMAGE_SECTION_HEADER)((int)OptionalHeaderOffset (lpFile) + 
(int)((PIMAGE_FILE_HEADER)((int)lpFile + ImageHdrOffset))->SizeOfOptionalHeader); 
} 





/* return offset to first IMAGE_IMPORT_MODULE_DIRECTORY entry */ 
inline PIMAGE_IMPORT_MODULE_DIRECTORY WINAPI ImportDirectoryOffset ( 
LPVOID lpFile) 
{ 
PIMAGE_OPTIONAL_HEADER poh = OptionalHeaderOffset (lpFile); 
PIMAGE_SECTION_HEADER psh = SectionHeaderOffset (lpFile); 
int nSections = NumOfSections (lpFile); 
int i = 0; 
LPVOID VAImportDir; 

VAImportDir = (LPVOID)poh->DataDirectory[IMAGE_DIRECTORY_ENTRY_IMPORT].VirtualAddress; 

/* locate section containing import directory */ 
while (i++<nSections) 
{ 
if(psh->VirtualAddress <= (DWORD)VAImportDir && 
psh->VirtualAddress + psh->SizeOfRawData > (DWORD)VAImportDir) 
break; 
psh++; 
} 

if(i > nSections) 
return 0; 

/* return image import directory offset */ 
return (PIMAGE_IMPORT_MODULE_DIRECTORY)(((int)lpFile + (int)VAImportDir - psh->VirtualAddress) + 
(int)psh->PointerToRawData); 
} 




/* return pointer to image directory section header */ 
inline PIMAGE_SECTION_HEADER WINAPI IDSectionHeaderOffset ( 
LPVOID lpFile) 
{ 
int ImageHdrOffset = 0; 
PIMAGE_OPTIONAL_HEADER poh = OptionalHeaderOffset (lpFile); 
PIMAGE_SECTION_HEADER psh = SectionHeaderOffset (lpFile); 
int nSections = NumOfSections (lpFile); 
int i = 0; 
LPVOID VAImportDir; 

VAImportDir = (LPVOID)poh->DataDirectory[IMAGE_DIRECTORY_ENTRY_IMPORT].VirtualAddress; 

/* locate section containing import directory */ 
while (i++<nSections) 
{ 
if(psh->VirtualAddress <= (DWORD)VAImportDir && 
psh->VirtualAddress + psh->SizeOfRawData > (DWORD)VAImportDir) 
break; 
psh++; 
} 

if(i > nSections) 
return 0; 
else 
return psh; 
} 





/* return the total number of sections in the module */ 
inline int WINAPI NumOfSections ( 
LPVOID lpFile) 
{ 
int ImageHdrOffset = 0; 

/* if DOS based file, skip DOS header and file signature */ 
if(*((USHORT *)lpFile) == IMAGE_DOS_SIGNATURE) 
{ 
/* file image header offset exists after DOS header and nt signature */ 
ImageHdrOffset = (int)((ULONG *)lpFile)[15] + sizeof (ULONG); 
if(*((ULONG *)((char *)lpFile + ImageHdrOffset - sizeof (ULONG))) != IMAGE_NT_SIGNATURE) 
return 0; 
} 

/* section total is found in the file header */ 
return (int)((PIMAGE_FILE_HEADER)((int)lpFile + ImageHdrOffset))->NumberOfSections; 
} 



/* retrieve name of module from module's open file handle */ 
inline void WINAPI RetrieveModuleName ( 
char *lpszModule, 
HANDLE hFile) 
{ 
HANDLE hMapFile; 
LPVOID lpFile; 
char *lpszName; 
int nSections; 
ULONG VAExportDir; 
int i=0; 
int ImageHdrOffset; 
PIMAGE_SECTION_HEADER psh; 
PIMAGE_FILE_HEADER pfh; 
PIMAGE_OPTIONAL_HEADER poh; 
PIMAGE_EXPORT_DIRECTORY ped; 


/* memory map handle to DLL for easy access */ 
hMapFile = CreateFileMapping (hFile, 
(LPSECURITY_ATTRIBUTES)NULL, 
PAGE_READONLY, 
0, 
0, 
NULL); 

/* map view of entire file */ 
lpFile = MapViewOfFile (hMapFile, FILE_MAP_READ, 0, 0, 0); 

/* if DOS based file */ 
if(*((USHORT *)lpFile) == IMAGE_DOS_SIGNATURE) 
{ 
/* file image header offset exists after DOS header and nt signature */ 
ImageHdrOffset = (int)((ULONG *)lpFile)[15] + sizeof (ULONG); 
if(*((ULONG *)((char *)lpFile + ImageHdrOffset - sizeof (ULONG))) != 
IMAGE_NT_SIGNATURE) 
{ 
strcpy (lpszModule, "Error, no IMAGE_NT_SIGNATURE"); 
goto EXIT; 
} 
} 

pfh = (PIMAGE_FILE_HEADER)((char *)lpFile + ImageHdrOffset); 

/* if optional header exists and exports directory exists proceed */ 
if(pfh->SizeOfOptionalHeader) 
{ 
/* locate virtual address for Export Image Directory in OptionalHeader */ 
poh = (PIMAGE_OPTIONAL_HEADER)((char *)pfh + sizeof (IMAGE_FILE_HEADER)); 
VAExportDir = poh->DataDirectory[IMAGE_DIRECTORY_ENTRY_EXPORT].VirtualAddress; 

/* locate section where export virtual address is located */ 
psh = (PIMAGE_SECTION_HEADER)((char *)poh + pfh->SizeOfOptionalHeader); 
nSections = pfh->NumberOfSections; 
while (i++<nSections) 
{ 
if(psh->VirtualAddress <= VAExportDir && 
psh->VirtualAddress + psh->SizeOfRawData > VAExportDir) 
break; 
psh++; 
} 

/* locate export image directory */ 
if(i < nSections) 
ped = (PIMAGE_EXPORT_DIRECTORY)((char *)lpFile + 
(VAExportDir - psh->VirtualAddress) + psh->PointerToRawData); 
else 
{ 
strcpy (lpszModule, "IMAGE_EXPORT_DIRECTORY not found"); 
goto EXIT; 
} 

/* read name from export directory */ 
lpszName = (char *)lpFile + ped->Name + (psh->PointerToRawData - psh->VirtualAddress); 
strcpy (lpszModule, lpszName); 
} 

else 
strcpy (lpszModule, "Error, no IMAGE_OPTIONAL_HEADER"); 

EXIT: 
/* clean up before exiting */ 
UnmapViewOfFile (lpFile); 
CloseHandle (hMapFile); 
} 




/* retieve section names from module file handle */ 
inline void WINAPI RetrieveSectionNames ( 
HANDLE hHeap, 
HANDLE hFile, 
SECTIONINFO **pSection) 
{ 
HANDLE hMapFile; 
LPVOID lpFile; 
int nSections; 
int i=0; 
int ImageHdrOffset; 
PIMAGE_SECTION_HEADER psh; 
PIMAGE_FILE_HEADER pfh; 
SECTIONINFO *ps; 


/* memory map handle to DLL for easy access */ 
hMapFile = CreateFileMapping (hFile, 
(LPSECURITY_ATTRIBUTES)NULL, 
PAGE_READONLY, 
0, 
0, 
NULL); 

/* map view of entire file */ 
lpFile = MapViewOfFile (hMapFile, FILE_MAP_READ, 0, 0, 0); 

/* if DOS based file */ 
if(*((USHORT *)lpFile) == IMAGE_DOS_SIGNATURE) 
{ 
/* file image header offset exists after DOS header and nt signature */ 
ImageHdrOffset = (int)((ULONG *)lpFile)[15] + sizeof (ULONG); 
if(*((ULONG *)((char *)lpFile + ImageHdrOffset - sizeof (ULONG))) != 
IMAGE_NT_SIGNATURE) 
goto EXIT; 
} 

pfh = (PIMAGE_FILE_HEADER)((char *)lpFile + ImageHdrOffset); 

/* if optional header exists, offset first section header */ 
psh = (PIMAGE_SECTION_HEADER)((char *)pfh + 
sizeof (IMAGE_FILE_HEADER) + pfh->SizeOfOptionalHeader); 

/* allocate one section header for each section */ 
ps = *pSection = (SECTIONINFO *)HeapAlloc (hHeap, 
HEAP_ZERO_MEMORY, 
sizeof (SECTIONINFO)); 
nSections = pfh->NumberOfSections; 
while (TRUE) 
{ 
strcpy (ps->szSection, (const char*)psh[i].Name); 
ps->uVirtualAddress = psh[i].VirtualAddress; 
ps->uSize = psh[i].SizeOfRawData; 

if(++i >= nSections) 
break; 

/* allocate heap memory for sections */ 
ps->Next = (LPSECTIONINFO)HeapAlloc (hHeap, 
HEAP_ZERO_MEMORY, 
sizeof (SECTIONINFO)); 
ps = (SECTIONINFO *)ps->Next; 
} 

EXIT: 
/* clean up before exiting */ 
UnmapViewOfFile (lpFile); 
CloseHandle (hMapFile); 
} 




/* function returns the entry point for an exe module lpFile must 
be a memory mapped file pointer to the beginning of the image file */ 
inline LPVOID WINAPI GetModuleEntryPoint ( 
LPVOID lpFile) 
{ 
PIMAGE_OPTIONAL_HEADER poh = OptionalHeaderOffset (lpFile); 

if(poh != NULL) 
return (LPVOID)(poh->AddressOfEntryPoint); 
else 
return NULL; 
} 




/* retrieve entry point */ 
inline LPVOID WINAPI GetImageBase ( 
LPVOID lpFile) 
{ 
PIMAGE_OPTIONAL_HEADER poh = OptionalHeaderOffset (lpFile); 

if(poh != NULL) 
return (LPVOID)(poh->ImageBase); 
else 
return NULL; 
} 




/* get import modules names separated by null terminators, return module count */ 
inline int WINAPI GetImportModuleNames ( 
LPVOID lpFile, 
HANDLE hHeap, 
char **pszModules) 
{ 
PIMAGE_IMPORT_MODULE_DIRECTORY pid = ImportDirectoryOffset (lpFile); 
PIMAGE_SECTION_HEADER pidsh = IDSectionHeaderOffset (lpFile); 
BYTE *pData = (BYTE *)pid; 
int nCnt = 0, nSize = 0, i; 
char *pModule[1024]; /* hardcoded maximum number of modules?? */ 
char *psz; 

/* extract all import modules */ 
while (pid->dwRVAModuleName) 
{ 
/* allocate temporary buffer for absolute string offsets */ 
pModule[nCnt] = (char *)(pData + (pid->dwRVAModuleName - pidsh->VirtualAddress)); 
nSize += strlen (pModule[nCnt]) + 1; 

/* increment to the next import directory entry */ 
pid++; 
nCnt++; 
} 

/* copy all strings to one chunk of heap memory */ 
*pszModules = (char*)HeapAlloc (hHeap, HEAP_ZERO_MEMORY, nSize); 
psz = *pszModules; 
for (i=0; i<nCnt; i++) 
{ 
strcpy (psz, pModule[i]); 
psz += strlen (psz) + 1; 
} 

return nCnt; 
} 




/* get import module function names separated by null terminators, return function count */ 
inline int WINAPI GetImportFunctionNamesByModule ( 
LPVOID lpFile, 
HANDLE hHeap, 
char *pszModule, 
char **pszFunctions) 
{ 
PIMAGE_IMPORT_MODULE_DIRECTORY pid = ImportDirectoryOffset (lpFile); 
PIMAGE_SECTION_HEADER pidsh = IDSectionHeaderOffset (lpFile); 
DWORD dwBase = ((DWORD)pid - pidsh->VirtualAddress); 
int nCnt = 0, nSize = 0; 
DWORD dwFunction; 
char *psz; 

/* find module's pid */ 
while (pid->dwRVAModuleName && 
strcmp (pszModule, (char *)(pid->dwRVAModuleName + dwBase))) 
pid++; 

/* count functions and total space required for them */ 
nSize += strlen ((char *)((*(DWORD *)(pid->dwRVAFunctionAddressList + dwBase)) + dwBase+2)) + 1; 

/* last image directory does not have a separate function list, so improvise */ 
if(!(dwFunction = pid->dwRVAFunctionNameList)) 
dwFunction = pid->dwRVAFunctionAddressList + 4; 

while (dwFunction && 
*(char *)((*(DWORD *)(dwFunction + dwBase)) + dwBase+2)) 
{ 
nSize += strlen ((char *)((*(DWORD *)(dwFunction + dwBase)) + dwBase+2)) + 1; 
dwFunction += 4; 
nCnt++; 
} 

/* allocate memory off heap for function names */ 
*pszFunctions = (char*)HeapAlloc (hHeap, HEAP_ZERO_MEMORY, nSize); 
psz = *pszFunctions; 
strcpy (psz, (char *)((*(DWORD *)(pid->dwRVAFunctionAddressList + dwBase)) + dwBase+2)); 
psz += strlen ((char *)((*(DWORD *)(pid->dwRVAFunctionAddressList + dwBase)) + dwBase+2)) + 1; 

/* last image directory does not have a separate function list, so improvise */ 
if(!(dwFunction = pid->dwRVAFunctionNameList)) 
dwFunction = pid->dwRVAFunctionAddressList + 4; 

while (dwFunction && 
*((char *)((*(DWORD *)(dwFunction + dwBase)) + dwBase+2))) 
{ 
strcpy (psz, (char *)((*(DWORD *)(dwFunction + dwBase)) + dwBase+2)); 
psz += strlen ((char *)((*(DWORD *)(dwFunction + dwBase)) + dwBase+2)) + 1; 
dwFunction += 4; 
} 

return nCnt; 
} 




/* get exported function names separated by null terminators, return count of functions */ 
inline int WINAPI GetExportFunctionNames ( 
LPVOID lpFile, 
HANDLE hHeap, 
char **pszFunctions) 
{ 

return 0; 
} 











/////////////////////////////////////////////////////////////////////////////

#endif // __UPDATEPATCH_H__
