// Dbf.h : header file
//

#ifndef __DBF_H__
#define __DBF_H__
#pragma once
////////////////////////////////////////////////////////////////////////////



/////////////////////////////////////////////////////////////////////////////
//  EPath.h
/////////////////////////////////////////////////////////////////////////////
#define VERSION_DBF   "13.12.2012"


#pragma pack(1)

////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////
typedef struct _DBF_DATE  // dt
{
	BYTE nYear;
	BYTE nMonth;
	BYTE nDay;

} DBF_DATE;


////////////////////////////////////////////////////////////////////////////
//  dbf-
////////////////////////////////////////////////////////////////////////////
typedef struct _DBF_HEADER  // hdr
{
	BYTE nVersion;			// Version number
	DBF_DATE dt;			// Date of last update YYMMDD
	DWORD dwRecordsCount;	// Number of records in the table
	WORD wHeaderSize;		// Length of header in bytes
	WORD wRecordSize;		// Length of each record in bytes
	BYTE res1[2];			// Reserved (=0)
	BYTE nHoldTrans;		// Incomplete transaction
	BYTE isEncoded;			// Encryption flag
	BYTE nFree[4];			// Free record thread (reserved for LAN only)
	BYTE nMulty[8];			// Reserved for multi-user processing
	BYTE nTableFlags;		// Flags
	BYTE nCodePageMark;		// Language driver ID
	BYTE res2[2];			// Reserved (=0)

} DBF_HEADER;


////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////
typedef struct _DBF_FIELD  // field
{
	char strFieldName[11];		// Field name in ASCII terminated by 0x00
	BYTE nFieldType;			// Field type (ASCII)
	DWORD dwFieldDataAddr;		// Field data address in memory (Displacement of field in record)
	BYTE nFieldLength;			// Field length
	BYTE nNumberDecimalPlaces;	// Decimal count
	BYTE nFieldFlags;			// Field flags
	DWORD dwAutoincrementNext;	// Value of autoincrement Next value
	BYTE nAutoincrementStept;	// Value of autoincrement Step value
	BYTE res[8];				// Reserved

	_DBF_FIELD() { memset(this, 0, sizeof(DBF_FIELD)); }

} DBF_FIELD;



/*
dwFieldDataAddr
Field data address 
 Note that this field has two very different interpretations. 

Version  Offset  Description  
dBASE  12-15  Address in memory.  
FoxPro  12-13  Offset of field from beginning of record.  

Field flags:
0x01   System Column (not visible to user)
0x02   Column can store null values
0x04   Binary column (for CHAR and MEMO only) 
0x06   (0x02+0x04) When a field is NULL and binary (Integer, Currency, and Character/Memo fields)
0x0C   Column is autoincrementing


*/

#pragma pack(8)



////////////////////////////////////////////////////////////////////////////
//       19702069
////////////////////////////////////////////////////////////////////////////
inline DWORD GetYear(BYTE nYear)
{
	return ((nYear < 70) ? 2000 : 1900) + nYear;
}


////////////////////////////////////////////////////////////////////////////
//        
////////////////////////////////////////////////////////////////////////////
inline EString GetStr_Date(DBF_DATE dt)
{
	EString str;
	str.Format("%.2d.%.2d.%d", dt.nDay, dt.nMonth, GetYear(dt.nYear));
	return str;
}


////////////////////////////////////////////////////////////////////////////
//      
////////////////////////////////////////////////////////////////////////////
// FS = FlagShip
// Fb = FoxBASE
// Fp = FoxPro 2.x
// Vf = Visual FoxPro
// CL = Clipper
// D2 = dBASE II
// D3 = dBASE III PLUS
// D4 = dBASE IV
// D5 = dBASE V
// SI = Successware Database Engine
// VO = Visual Objects for dBASE III
////////////////////////////////////////////////////////////////////////////
inline EString FormatVersion(DBF_HEADER hdr, int nMode = 0)
{
	EString str;
	switch(nMode)
	{
		case 0:
		{
			switch(hdr.nVersion)
			{
				case 0x02: str = "D2, Fb"; break;
				case 0x03: str = "FS, Fb, Fp, CL, D3, D4, D5"; break;
				case 0x04: str = "FS, D4, D5"; break;
				case 0x05: str = "FS, Fp, D5"; break;
				case 0x07: str = "VO"; break;
				case 0x30: str = "Vf"; break;
				case 0x31: str = "Vf (autoincrement enabled)"; break;
				case 0x43: str = "D4 (SQL); FS (memo .dbv)"; break;
				case 0x63: str = "D4 (SQL system)"; break;
				case 0x7B: str = "D4 (memo)"; break;
				case 0x83: str = "FS, Fb, Fp, CL, D3, D4, D5 (memo .dbt)"; break;
				case 0x87: str = "VO (memo)"; break;
				case 0x8B: str = "D4, D5 (memo .dbt)"; break;
				case 0x8E: str = "D4 (SQL)"; break;
				case 0xB3: str = "FS (memo .dbt and .dbv)"; break;
				case 0xCB: str = "D4 (SQL memo)"; break;
				case 0xE5: str = "CL, SI (memo .smt)"; break;
				case 0xF5: str = "Fp (memo .fpt)"; break;
				case 0xFB: str = "Fb"; break;
				default: str = "<unknown>"; break;
			}
			break;
		}

		case 1:
		{
			str.Format("0x%.2X", hdr.nVersion);
			break;
		}
	}
	return str;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
static char* g_szDbfCodePageMark[][3] =
{
	(char*)0x01, "437",   "US MS-DOS",
	(char*)0x02, "850",   "International MS-DOS",
	(char*)0x03, "1252",  "Windows ANSI Latin I",
	(char*)0x04, "10000", "Standard Macintosh",
	(char*)0x08, "865",   "Danish OEM",
	(char*)0x09, "437",   "Dutch OEM",
	(char*)0x0A, "850",   "Dutch OEM*",
	(char*)0x0B, "437",   "Finnish OEM",
	(char*)0x0D, "437",   "French OEM",
	(char*)0x0E, "850",   "French OEM*",
	(char*)0x0F, "437",   "German OEM",
	(char*)0x10, "850",   "German OEM*",
	(char*)0x11, "437",   "Italian OEM",
	(char*)0x12, "850",   "Italian OEM*",
	(char*)0x13, "932",   "Japanese Shift-JIS",
	(char*)0x14, "850",   "Spanish OEM*",
	(char*)0x15, "437",   "Swedish OEM",
	(char*)0x16, "850",   "Swedish OEM*",
	(char*)0x17, "865",   "Norwegian OEM",
	(char*)0x18, "437",   "Spanish OEM",
	(char*)0x19, "437",   "English OEM (Great Britain)",
	(char*)0x1A, "850",   "English OEM (Great Britain)*",
	(char*)0x1B, "437",   "English OEM (US)",
	(char*)0x1C, "863",   "French OEM (Canada)",
	(char*)0x1D, "850",   "French OEM*",
	(char*)0x1F, "852",   "Czech OEM",
	(char*)0x22, "852",   "Hungarian OEM",
	(char*)0x23, "852",   "Polish OEM",
	(char*)0x24, "860",   "Portuguese OEM",
	(char*)0x25, "850",   "Portuguese OEM*",
	(char*)0x26, "866",   "Russian OEM",
	(char*)0x37, "850",   "English OEM (US)*",
	(char*)0x40, "852",   "Romanian OEM",
	(char*)0x4D, "936",   "Chinese GBK (PRC)",
	(char*)0x4E, "949",   "Korean (ANSI/OEM)",
	(char*)0x4F, "950",   "Chinese Big5 (Taiwan)",
	(char*)0x50, "874",   "Thai (ANSI/OEM)",
	(char*)0x57, "",      "Current ANSI",
	(char*)0x58, "1252",  "Western European ANSI",
	(char*)0x59, "1252",  "Spanish ANSI",
	(char*)0x64, "852",   "Eastern European MS-DOS",
	(char*)0x65, "866",   "Russian MS-DOS",
	(char*)0x66, "865",   "Nordic MS-DOS",
	(char*)0x67, "861",   "Icelandic MS-DOS",
	(char*)0x68, "895",   "Kamenicky (Czech) MS-DOS",
	(char*)0x69, "620",   "Mazovia (Polish) MS-DOS",
	(char*)0x6A, "737",   "Greek MS-DOS (437G)",
	(char*)0x6B, "857",   "Turkish MS-DOS",
	(char*)0x6C, "863",   "French-Canadian MS-DOS",
	(char*)0x78, "950",   "Taiwan Big 5",
	(char*)0x79, "949",   "Hangul (Wansung)",
	(char*)0x7A, "936",   "PRC GBK",
	(char*)0x7B, "932",   "Japanese Shift-JIS",
	(char*)0x7C, "874",   "Thai Windows/MSDOS",
	(char*)0x86, "737",   "Greek OEM",
	(char*)0x87, "852",   "Slovenian OEM",
	(char*)0x88, "857",   "Turkish OEM",
	(char*)0x96, "10007", "Russian Macintosh",
	(char*)0x97, "10029", "Eastern European Macintosh",
	(char*)0x98, "10006", "Greek Macintosh",
	(char*)0xC8, "1250",  "Eastern European Windows",
	(char*)0xC9, "1251",  "Russian Windows",
	(char*)0xCA, "1254",  "Turkish Windows",
	(char*)0xCB, "1253",  "Greek Windows",
	(char*)0xCC, "1257",  "Baltic Windows"
};


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline int _Dbf_FindCodePage(BYTE nCodePageMark)
{
	for(int i=0; i < sizeof(g_szDbfCodePageMark) / sizeof(char*) / 3; i++)
	{
		if((BYTE)g_szDbfCodePageMark[i][0] == nCodePageMark) return i;
	}
	return -1;

}


////////////////////////////////////////////////////////////////////////////
//       
////////////////////////////////////////////////////////////////////////////
inline EString GetStr_CodePage(DBF_HEADER hdr)
{
	EString str;
	int n = _Dbf_FindCodePage(hdr.nCodePageMark);
	if(n >= 0)
	{
		str.Format("%s, %s", g_szDbfCodePageMark[n][1], g_szDbfCodePageMark[n][2]);
	}
	return str;
}


////////////////////////////////////////////////////////////////////////////
//     
////////////////////////////////////////////////////////////////////////////
inline EString GetStr_TableFlags(DBF_HEADER hdr)
{
	EString str;
	if(hdr.nVersion == 0x30 || hdr.nVersion == 0x31)
	{
		if(hdr.nTableFlags > 0)
		{
			char strTableFlags[256];
			strTableFlags[0] = '\0';
			char s1[32];
			char* g_str30[] = { "TF_CDX","TF_MEMO","TF_DBC",NULL,NULL,NULL,NULL,NULL };

			BYTE n = hdr.nTableFlags;
			for(int i=0; i<8; i++)
			{
				if(n & 0x01)
				{
					if(g_str30[i] != NULL)
						sprintf(s1, "%s, ", g_str30[i]);
					else
						sprintf(s1, "bit(%d), ", i);
					strcat(strTableFlags, s1);
				}
				n >>= 1;
			}
			strTableFlags[strlen(strTableFlags) - 2] = '\0';
			str = strTableFlags;
		}
	}
	return str;
}

////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////
inline EString GetStr_HeaderDbf(DBF_HEADER hdr)
{
	EStringFormat s1;
	EString str;
	str.Format
	(
		"struct _DBF_HEADER\r\n" \
		"{\r\n" \
		"    BYTE nVersion;         0x%.2X \"%s\"\r\n" \
		"    DBF_DATE dt;           %s\r\n" \
		"    DWORD dwRecordsCount;  %d\r\n" \
		"    WORD wHeaderSize;      %d\r\n" \
		"    WORD wRecordSize;      %d\r\n" \
		"    BYTE res1[2];          hex(%s)\r\n" \
		"    BYTE nHoldTrans;       %d\r\n" \
		"    BYTE isEncoded;        %d\r\n" \
		"    BYTE nFree[4];         hex(%s)\r\n" \
		"    BYTE nMulty[8];        hex(%s)\r\n" \
		"    BYTE nTableFlags;      0x%.2X %s\r\n" \
		"    BYTE nCodePageMark;    0x%.2X \"%s\"\r\n" \
		"    BYTE res2[2];          hex(%s)\r\n" \
		"}\r\n",
		
		hdr.nVersion, FormatVersion(hdr, 0),
		GetStr_Date(hdr.dt),
		hdr.dwRecordsCount,
		hdr.wHeaderSize,
		hdr.wRecordSize,
		s1.HexSequence((BYTE*)&hdr + 12, 2),
		hdr.nHoldTrans,
		hdr.isEncoded,
		s1.HexSequence((BYTE*)&hdr + 16, 4),
		s1.HexSequence((BYTE*)&hdr + 20, 8),
		hdr.nTableFlags, GetStr_TableFlags(hdr),
		hdr.nCodePageMark, GetStr_CodePage(hdr),
		s1.HexSequence((BYTE*)&hdr + 30, 2)
	);

//	s1.HexSequence((BYTE*)&hdr, sizeof(DBF_HEADER));
//	str += "\r\n";
//	str += s1;
//	str += "\r\n";

	return str;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline EString GetStr_FieldTypeFormat(DBF_FIELD* pField)
{
	EString str;
	char s[16];
	sprintf(s, "%s(%c", pField->strFieldName, pField->nFieldType);
	
	int n = pField->nFieldLength;
	int m = pField->nNumberDecimalPlaces;
	
	BYTE nFieldType = pField->nFieldType;
	if(nFieldType == 'N' && pField->nNumberDecimalPlaces == 0) nFieldType = 'n';
	if(nFieldType == 'F' && pField->nNumberDecimalPlaces == 0) nFieldType = 'n';

	if(_EcxIsCharInSet(nFieldType, "NFB")) str.Format("%s%d,%d)", s, n, m);
	else
		if(_EcxIsCharInSet(nFieldType, "CLDTIYMGP+O@Vn")) str.Format("%s%d)", s, n);
		else
			str.Format("%s%d,%d) ???", s, n, m);

	return str;
}


////////////////////////////////////////////////////////////////////////////
//        
////////////////////////////////////////////////////////////////////////////
inline char* GetField(DBF_FIELD field, char* str)
{
	sprintf(str, "%c  ''%s''", field.nFieldType, field.strFieldName);
	return str;
}


////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////
inline int GetFieldsCount(BYTE* data)
{
	BYTE* tmp = data + sizeof(DBF_HEADER);
	int count = 0;
	while(*tmp != 0x0D)
	{
		tmp += sizeof(DBF_FIELD);
		count++;
	}
	return count;

//	BYTE* p1 = data + ((DBF_HEADER*)data)->wHeaderSize - 1;
//	while(p1 != data)
//	{
//		if(*p1 == 0x0D) break;
//		p1--;
//	}
//
//	return (((DBF_HEADER*)data)->wHeaderSize - 33) / 32; // ok
}





////////////////////////////////////////////////////////////////////////////
//      
//  : L[1], D[8], Cn, Nn[,m], Fn,m
////////////////////////////////////////////////////////////////////////////
inline BOOL ParseDataType(char** ppBuf, DBF_FIELD* pField)
{
	//    
	#define PCS_WAITING_NAME		0	// w0
	#define PCS_WAITING_BT			1	// w1
	#define PCS_WAITING_ET			2	// w2
	#define PCS_WAITING_TYPE		3	// w3
	#define PCS_WAITING_COMMA		4	// w4
	#define PCS_WAITING_LENGTH		5	// w5
	#define PCS_WAITING_DECIMAL		6	// w6

	//  
	#define PCS_CHAR_UNKNOWN		0	// c0
	#define PCS_CHAR_NMTP			1	// c1
	#define PCS_CHAR_NUMBER			2	// c2
	#define PCS_CHAR_BT				3	// c3
	#define PCS_CHAR_COMMA			4	// c4
	#define PCS_CHAR_ET				5	// c5

	//  
	#define PCS_ACTION_ERROR		0	// a0
	#define PCS_ACTION_GET_NAME		1	// a1
	#define PCS_ACTION_GET_TYPE		2	// a2
	#define PCS_ACTION_GET_LENGTH	3	// a3
	#define PCS_ACTION_GET_DECIMAL	4	// a4
	#define PCS_ACTION_SKIP_CHAR	5	// a5

	//   +  
	static BYTE nCodeAW[] =
	{
	//  w0    w1    w2    w3    w4    w5    w6
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,  // c0
		0x11, 0x00, 0x00, 0x25, 0x00, 0x00, 0x00,  // c1
		0x00, 0x00, 0x00, 0x00, 0x00, 0x34, 0x42,  // c2
		0x00, 0x53, 0x00, 0x00, 0x00, 0x00, 0x00,  // c3
		0x00, 0x00, 0x42, 0x00, 0x56, 0x00, 0x00,  // c4
		0x00, 0x00, 0x50, 0x00, 0x50, 0x50, 0x50,  // c5
	};

	char *p0, *tmp = *ppBuf;
	int i, def, res = TRUE;
	int nCharType, nWaiting = PCS_WAITING_NAME, nAction;
	char ch;


	//  
	while(1)
	{
		skip_spaces(tmp);
		if(is_eot(tmp)) break;

		ch = *tmp;
		nCharType = PCS_CHAR_UNKNOWN;
		if(is_char(ch, CCM_WORD) || (ch == '_') || _EcxIsCharInSet(ch, "CNFLDTIBYMGP+O@V")) nCharType = PCS_CHAR_NMTP;
		if(is_char(ch, CCM_DIGIT)) nCharType = PCS_CHAR_NUMBER;
		if(ch == '(') nCharType = PCS_CHAR_BT;
		if(ch == ',') nCharType = PCS_CHAR_COMMA;
		if(ch == ')') nCharType = PCS_CHAR_ET;
		
		//  
		i = nCodeAW[7 * nCharType + nWaiting];
//		if(i == 0x32 && _EcxIsCharInSet(pField->nFieldType, "NF")) i = 0x34;

		//     
		nAction = i >> 4;
		nWaiting = i & 0x0F;

		//  
		switch(nAction)
		{
			// 
			case PCS_ACTION_ERROR:
				goto RET_FALSE;

			//   
			case PCS_ACTION_GET_NAME:
			{
				p0 = tmp;
				skip_word(tmp);
				i = tmp - p0;
				if(i == 0 || i > 10) goto RET_FALSE;
				if(!_EcxIsIdentifierSequence(&p0, i)) goto RET_FALSE;
				memcpy(&pField->strFieldName, p0, i);
				continue;
			}

			//   
			case PCS_ACTION_GET_LENGTH:
			{
				i = __strtoul(tmp);
				if(def > 0) i = def;
				else
					if(i == 0) goto RET_FALSE;
				pField->nFieldLength = i;
				continue;
			}

			//    
			case PCS_ACTION_GET_DECIMAL:
			{
				i = __strtoul(tmp);
				if(def == 0)
				{
					if(i > 0 && pField->nFieldLength - i - 1 < 1) goto RET_FALSE;
					pField->nNumberDecimalPlaces = i;
				}
				continue;
			}

			//   
			case PCS_ACTION_GET_TYPE:
			{
				tmp++;
				if(!_EcxIsCharInSet(ch, "CNFLDTIBYMGP+O@V")) goto RET_FALSE;
				pField->nFieldType = ch;
				def = 0;
				if(ch == 'L') def = 1;
				if(ch == 'D') def = 8;
				continue;
			}

			//   
			case PCS_ACTION_SKIP_CHAR:
			{
				tmp++;
				if(nCharType == PCS_CHAR_ET)
				{
//					if(n > 0) pField->nFieldLength = n;
					res = TRUE;
					goto RET_TRUE;
				}
				continue;
			}
		}

		tmp++;
	}

RET_FALSE:
	res = FALSE;

RET_TRUE:
	*ppBuf = tmp;
	return res;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////







////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////
class EFileDBF  // dbf
{
public:

	EFileDBF();
	~EFileDBF();

	void Create(BYTE nVersion);
	BOOL AddField(LPCSTR szName, LPCSTR szType, BYTE nLength = 0, BYTE nDecimal = 0);
	BOOL AddField(char** ppBuf);
	BOOL AddField(DBF_FIELD* pField);
	void EndAddFields();

	BOOL AddFields(BYTE nVersion, LPCSTR szCreationString);



	DBF_FIELD* GetField(int nIndex);

	EString GetCreationString();



//	DWORD GetYear(BYTE nYear);
//	char* GetDate(DBF_DATE dt, char* str);
//	char* GetVersion(DBF_HEADER hdr, char* str);
//	char* FieldTypeFormat(DBF_FIELD* pField, char* dest);
//	int GetFieldsCount(BYTE* data);
//	int ParseCreationString(LPSTR lpszCreationString, DBF_HEADER* pHdr, DBF_FIELD* pField);


	DBF_HEADER m_Hdr;

	EMemory m_Fields;
	DBF_FIELD* m_pFields;
	int m_nFieldsCount;
	
	EMemory m_Records;
	int m_nRecordsCount;
	WORD m_wRecordSize;




	void _remove_header()
		{ memset(&m_Hdr, 0, sizeof(DBF_HEADER)); }
	void _remove_fields()
		{ m_Fields.MemAlloc(0); m_nFieldsCount = 0; m_pFields = (DBF_FIELD*)m_Fields.m_pBuffer; }
	void _remove_records()
		{ m_Records.MemAlloc(0); m_nRecordsCount = 0; m_wRecordSize = 0; }
	void _remove()
		{ _remove_header(); _remove_fields(); _remove_records(); }
};


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline EFileDBF::EFileDBF()
{
	_remove();
}

inline EFileDBF::~EFileDBF()
{
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline void EFileDBF::Create(BYTE nVersion)
{
	_remove();
	m_Hdr.nVersion = nVersion;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline BOOL EFileDBF::AddField(LPCSTR szName, LPCSTR szType, BYTE nLength, BYTE nDecimal)
{
	if(m_Hdr.nVersion > 0)
	{
		DBF_FIELD field;
		EString s;
		s.Format("%s(%s%d,%d)", szName, szType, nLength, nDecimal);
		char* tmp = s.m_pBuffer;
		if(ParseDataType(&tmp, &field))
		{
			return AddField(&field);
		}
	}
	return FALSE;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline BOOL EFileDBF::AddField(char** ppBuf)
{
	DBF_FIELD field;
	if(ParseDataType(ppBuf, &field))
	{
		return AddField(&field);
	}
	return FALSE;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline BOOL EFileDBF::AddField(DBF_FIELD* pField)
{
	if(pField)
	{
		m_Fields.MemAppend(sizeof(DBF_FIELD));
		m_pFields = (DBF_FIELD*)m_Fields.m_pBuffer;
		memcpy(m_pFields + m_nFieldsCount, pField, sizeof(DBF_FIELD));
		m_nFieldsCount++;
		m_wRecordSize += pField->nFieldLength;
		return TRUE;
	}
	return FALSE;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline void EFileDBF::EndAddFields()
{
	m_Fields.MemAppend(1);
	m_Fields[~m_Fields] = 0x0D;
	m_Hdr.wHeaderSize = sizeof(DBF_HEADER) + m_nFieldsCount * sizeof(DBF_FIELD) + 1;
	m_Hdr.wRecordSize = m_wRecordSize;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline BOOL EFileDBF::AddFields(BYTE nVersion, LPCSTR szCreationString)
{
	Create(nVersion);
	char* tmp = (char*)szCreationString;

	while(*tmp)
	{
		if(!AddField(&tmp)) return FALSE;
	}

	EndAddFields();
	return TRUE;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline DBF_FIELD* EFileDBF::GetField(int nIndex)
{
	if(__in10(nIndex, 0, m_nFieldsCount))
	{
		return m_pFields + nIndex;
	}
	return NULL;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline EString EFileDBF::GetCreationString()
{
	EString strCreationString;
	for(int i=0; i<m_nFieldsCount; i++)
	{
		strCreationString += GetStr_FieldTypeFormat(GetField(i));
		strCreationString += " ";
	}
	strCreationString.TrimRight();
	return strCreationString;
}















////////////////////////////////////////////////////////////////////////////
#endif // __DBF_H__
