// EPath.h : header file
//

#ifndef __EPATH_H__
#define __EPATH_H__
#pragma once
/////////////////////////////////////////////////////////////////////////////

#include "..\TextParsing\TextParsingMacro.h"


/////////////////////////////////////////////////////////////////////////////
//  EPath.h
/////////////////////////////////////////////////////////////////////////////
#define VERSION_EPATH   "19.11.2013"


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
#define DELIM_ENTRY		0
#define DELIM_ATTR		1
#define MODE_FILE		0
#define MODE_JSON		1


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
class EPath  // path
{
public:

	EPath();
	~EPath();

	char SetDelimiter(char nDelim, BOOL isAttrDelim = FALSE);
	BOOL Split(LPCSTR szPath);
	void Clear();
	EString Print();

	int m_nMode;			// 0 = MODE_FILE; 1 = MODE_JSON

	char* m_szPath;			//  
	char* m_szAttr;			// 
	char m_nDelim;			//   
	char m_nAttrDelim;		//  

	LPSTR* m_ppEntry;		//  
	int m_nCount;			//   

	LPSTR* m_ppAttrEntry;	//  
	int m_nAttrCount;		//   

	EString m_str;

};


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
inline EPath::EPath()
{
	m_nMode = MODE_FILE;
	m_szPath = m_szAttr = NULL;
	m_nDelim = '\\';
	m_nAttrDelim = '.';
	m_ppEntry = m_ppAttrEntry = NULL;
	m_nCount = m_nAttrCount = 0;
}


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
inline EPath::~EPath()
{
	Clear();
}


/////////////////////////////////////////////////////////////////////////////
//      
/////////////////////////////////////////////////////////////////////////////
inline char EPath::SetDelimiter(char nDelim, int nDelimType)
{
	char tmp;

	if(nDelimType == DELIM_ENTRY)
	{
		tmp = m_nDelim;
		m_nDelim = nDelim;
	}

	if(nDelimType == DELIM_ATTR)
	{
		tmp = m_nAttrDelim;
		m_nAttrDelim = nDelim;
	}

	return tmp;
}


/////////////////////////////////////////////////////////////////////////////
//    
/////////////////////////////////////////////////////////////////////////////
inline BOOL EPath::Split(LPCSTR szPath)
{
	//   
	Clear();
	if(szPath == NULL) return FALSE;

	// 
	int len = strlen(szPath);
	if(len == 0) return FALSE;
	m_szPath = new char[len + 1];
	memcpy(m_szPath, szPath, len);
	m_szPath[len] = '\0';
	char* tmp = m_szPath;

	//   
	if(tmp[len - 1] == m_nDelim) tmp[len - 1] = '\0';
	if(tmp[len - 1] == m_nAttrDelim) tmp[len - 1] = '\0';

	int n, i, j, count = 1;

	if(m_nMode == MODE_FILE)
	{
		//     
		while(*tmp) 
		{
			if(*tmp == m_nDelim) count++;
			tmp++;
		}

		//   
		m_nCount = count;
		m_ppEntry = new LPSTR[count];


		char *p0, *p1;
		p1 = p0 = m_szPath;
		i = 0;

		//    
		while(1)
		{
			skip_to_char(p1, m_nDelim);
			len = p1 - p0;

			//   
			m_ppEntry[i] = new char[len + 1];
			memcpy(m_ppEntry[i], p0, len);
			m_ppEntry[i][len] = '\0';

			if(*p1 == '\0') break;

			i++;
			p0 = ++p1;
		}

		//  
		p1 = p0 = m_ppEntry[m_nCount - 1];
		skip_to_char(p1, m_nAttrDelim);
		if(*p1 == '\0') return TRUE;
		len = p1 - p0;

		//  
		if(len != 0)
		{
			i = strlen(m_ppEntry[m_nCount - 1]);
			m_ppEntry[m_nCount - 1][len] = '\0';
			len = i - len - 1;

			//  
			m_szAttr = new char[len + 1];
			memcpy(m_szAttr, p1 + 1, len);
			m_szAttr[len] = '\0';

			char* tmp = m_szAttr;
			count = 1;

			//     
			while(*tmp) 
			{
				if(*tmp == m_nAttrDelim) count++;
				tmp++;
			}

			//   
			m_nAttrCount = count;
			m_ppAttrEntry = new LPSTR[count];


			char *p0, *p1;
			p1 = p0 = m_szAttr;
			i = 0;

			//    
			while(1)
			{
				skip_to_char(p1, m_nAttrDelim);
				len = p1 - p0;

				//   
				m_ppAttrEntry[i] = new char[len + 1];
				memcpy(m_ppAttrEntry[i], p0, len);
				m_ppAttrEntry[i][len] = '\0';

				if(*p1 == '\0') break;

				i++;
				p0 = ++p1;
			}
		}
	}
	else
	{
		//     
		while(*tmp) 
		{
			if(*tmp == '.') count++;
			if(*tmp == '[') count++;
			tmp++;
		}

		//   
		m_nCount = count;
		m_ppEntry = new LPSTR[m_nCount];

		char *p0, *p1, *e0, *e1;
		p1 = p0 = m_szPath;
		i = 0;

		//    
		while(1)
		{
			skip_to_char(p1, '.');
			len = p1 - p0;

			//   
			m_ppEntry[i] = new char[len + 1];
			memcpy(m_ppEntry[i], p0, len);
			m_ppEntry[i][len] = '\0';

			if(*p1 == '\0') break;


			//     
			e0 = e1 = m_ppEntry[i];
			j = i;

			//    
			while(*e1 != '[' && !is_char(*e1, CCM_SPACE) && !is_char(*e1, CCM_EOT)) e1++;
			len = e1 - e0;

			//    
			while(*e1)
			{
				skip_spaces(e1);

				if(*e1 == ']')
				{
					e1++;
					continue;
				}

				if(*e1 == '[')
				{
					e1++;
					n = __strtoul(e1);
					i++;
					m_ppEntry[i] = (char*)((n + 1) | 0xDEF00000);
				}
			}
			m_ppEntry[j][len] = '\0';

			i++;
			p0 = ++p1;
		}

	}

	return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
//   
/////////////////////////////////////////////////////////////////////////////
inline void EPath::Clear()
{
	char* tmp;

	//  
	if(m_szPath != NULL)
	{
		delete [] m_szPath;
		m_szPath = NULL;
	}

	//   
	if(m_ppEntry != NULL)
	{
		for(int i=0; i<m_nCount; i++)
		{
			tmp = m_ppEntry[i];
			if(tmp != NULL && (((int)tmp & 0xDEF00000) != 0xDEF00000)) delete [] tmp;
		}

		delete [] m_ppEntry;
		m_ppEntry = NULL;
		m_nCount = 0;
	}

	//  
	if(m_szAttr != NULL)
	{
		delete [] m_szAttr;
		m_szAttr = NULL;
	}

	//   
	if(m_ppAttrEntry != NULL)
	{
		if(m_nMode == MODE_FILE)
		{
			for(int i=0; i<m_nAttrCount; i++)
			{
				tmp = m_ppAttrEntry[i];
				if(tmp != NULL) delete [] tmp;
			}
		}

		delete [] m_ppAttrEntry;
		m_ppAttrEntry = NULL;
		m_nAttrCount = 0;
	}
}


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
inline EString EPath::Print()
{
	int i;
	m_str.Format("m_szPath = \"%s\"\r\n\r\n", m_szPath);
	EString s1, s2;

	for(i = 0; i < m_nCount; i++)
	{
		if(((int)m_ppEntry[i] & 0xDEF00000) == 0xDEF00000)
		{
			s2.Format("[%d]", ((int)m_ppEntry[i] & 0x000FFFFF) - 1);
		}
		else
		{
			s2 = m_ppEntry[i];
		}
		s1.Format("m_ppEntry[%d] = %s\r\n", i,  s2);
		m_str += s1;
	}

	m_str += "\r\n";

	if(m_nMode == MODE_FILE)
	{
		for(i = 0; i < m_nAttrCount; i++)
		{
			s1.Format("m_ppAttrEntry[%d] = %s\r\n", i, m_ppAttrEntry[i]);
			m_str += s1;
		}
	}
	return m_str;
}



/////////////////////////////////////////////////////////////////////////////
#endif // __EPATH_H__
