// FileInputOutputFunc.h : header file
//

#ifndef __FILEINPUTOUTPUTFUNC_H__
#define __FILEINPUTOUTPUTFUNC_H__
#pragma once
////////////////////////////////////////////////////////////////////////////

#include "FileInputOutput.h"


/////////////////////////////////////////////////////////////////////////////
//  FileInputOutputFunc.h
/////////////////////////////////////////////////////////////////////////////
#define VERSION_FILEINPUTOUTPUTFUNC   "09.02.2018"






////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline BOOL _EcxCopyFile(LPCTSTR szDirTo, LPCTSTR szDirFrom, LPCTSTR szFileName)
{
	EString to = szDirTo, from = szDirFrom;
	to.TrimRight("/\\");
	from.TrimRight("/\\");
	to += "\\"; to += szFileName;
	from += "\\"; from += szFileName;
	return ::CopyFile(to, from, FALSE);
}


/////////////////////////////////////////////////////////////////////////////
//    
/////////////////////////////////////////////////////////////////////////////
inline BOOL _EcxIsValidDirectoryName(LPCTSTR lpszPath)
{
	char szOldPath[MAX_PATH];
	BOOL res;

	::GetCurrentDirectory(MAX_PATH, szOldPath);
	res = ::SetCurrentDirectory(lpszPath);

	::SetCurrentDirectory(szOldPath);

	return res;
}


/////////////////////////////////////////////////////////////////////////////
//   
/////////////////////////////////////////////////////////////////////////////
inline BOOL _EcxIsFileExist(LPCTSTR lpszFullPath)
{
	return (::GetFileAttributes(lpszFullPath) != 0xFFFFFFFF);
}


/////////////////////////////////////////////////////////////////////////////
//   
/////////////////////////////////////////////////////////////////////////////
inline BOOL _EcxIsDirectoryExist(LPCTSTR lpszPath)
{
	DWORD res = ::GetFileAttributes(lpszPath);
	if(res == 0xFFFFFFFF) return FALSE;
	return (res & FILE_ATTRIBUTE_DIRECTORY) == FILE_ATTRIBUTE_DIRECTORY;
}


/////////////////////////////////////////////////////////////////////////////
// TRUE -    
/////////////////////////////////////////////////////////////////////////////
inline BOOL _EcxIsDirectoryNotEmpty(LPCTSTR lpszPath, int nPresentFlag)
{
	if(!_EcxIsDirectoryExist(lpszPath)) return FALSE;

	WIN32_FIND_DATA wfd;
	HANDLE hFile;
	BOOL res = FALSE;

	//    
	int len = strlen(lpszPath);
	char* tmp = new char[len + 5];
	strcpy(tmp, lpszPath);

	//  "\*.*"
	if(tmp[len - 1] == '\\') len--;
	tmp[len] = '\0';
	strcat(tmp, "\\*.*");

	//    
	if((hFile = ::FindFirstFile(tmp, &wfd)) != INVALID_HANDLE_VALUE)
	{
		do
		{
			if(wfd.cFileName[0] == '.') continue;

			int det = nPresentFlag * 3 + ((wfd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) == FILE_ATTRIBUTE_DIRECTORY);
			if(det == 4 || det == 6) continue;

			res = TRUE;
			break;
		}
		while(::FindNextFile(hFile, &wfd));
	}

	::FindClose(hFile);

	delete [] tmp;
	return res;
}


/////////////////////////////////////////////////////////////////////////////
//   
/////////////////////////////////////////////////////////////////////////////
inline BOOL _EcxCreateDirectory(LPCTSTR lpszPath)
{
	//   
	BOOL res = ::CreateDirectory(lpszPath, NULL);

	//    
	if(res == FALSE)
	{
		char* tmp = (char*)lpszPath;

		//   
		int i = strlen(tmp);
		if(tmp[i - 1] == '\\') tmp[i - 1] = '\0';
		i = 0;

		//  Universal Naming Code (UNC)
		// \\server_name\share_name
		if(*(WORD*)tmp == 0x5C5C) i = 2;
		

		//    
		while(1)
		{
			//   
			while(tmp[i] != '\\' && tmp[i]) i++;

			//  
			char ch = tmp[i];
			tmp[i] = '\0';

			//   
			res = ::CreateDirectory(tmp, NULL);
			
			//  
			tmp[i] = ch;
			
			//    
			if(tmp[i] == '\0') break;
			
			//  
			i++;
		}
	}

	return res;
}


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
inline void _EcxSplitPath(LPCSTR szPath, PATHELEMENTS* pPes)
{
	_splitpath(szPath, pPes->szDrive, pPes->szDir, pPes->szFile, pPes->szExt);
}


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
inline void _EcxMakePath(PATHELEMENTS* pPes, LPSTR szPath)
{
	_splitpath(szPath, pPes->szDrive, pPes->szDir, pPes->szFile, pPes->szExt);
}


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
inline BOOL _EcxCompareExt(LPCSTR szExt, char** ppExt, int nCount)
{
	for(int i=0; i<nCount; i++)
	{
		if(_stricmp(szExt, ppExt[i]) == 0) return TRUE;
	}

	return FALSE;
}


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
#define EXT_PARAM(arr)		(char**)&arr, _array_count(arr, char*)









////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline DWORD _EcxGetSectorSize(LPCSTR lpFileName)
{
	char tmp[MAX_PATH];
	LPSTR ptmp;
	if(::GetFullPathName(lpFileName, MAX_PATH, tmp, &ptmp))
	{
		PATHELEMENTS pes;
		_EcxSplitPath(tmp, &pes);
	
		DWORD dwFreeClusters, dwBytesPerSector, dwSectorsPerCluster, dwClusters;
		if(::GetDiskFreeSpace(pes.szDrive, &dwSectorsPerCluster, &dwBytesPerSector, &dwFreeClusters, &dwClusters))
		{
			return dwSectorsPerCluster * dwBytesPerSector;
		}
	}
	return 0;
}


////////////////////////////////////////////////////////////////////////////
//   szFileName      pBuf.
////////////////////////////////////////////////////////////////////////////
inline BOOL _EcxLoadFile(LPCTSTR szFileName, void* pBuf, DWORD dwLength)
{
	if(pBuf)
	{
		HANDLE hFile = ::CreateFile(szFileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
		if(hFile != INVALID_HANDLE_VALUE)
		{
			DWORD dw, dwLen = ::GetFileSize(hFile, NULL);
			if(dwLength < dwLen) dwLen = dwLength;
			::ReadFile(hFile, pBuf, dwLen, &dw, NULL);
			::CloseHandle(hFile);
			return TRUE;
		}
	}
	return FALSE;
}


////////////////////////////////////////////////////////////////////////////
//   szFileName      pBuf.
////////////////////////////////////////////////////////////////////////////
inline BOOL _EcxSaveFile(LPCTSTR szFileName, void* pBuf, DWORD dwLength)
{
	if(pBuf)
	{
		HANDLE hFile = ::CreateFile(szFileName, GENERIC_WRITE, FILE_SHARE_WRITE, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
		if(hFile != INVALID_HANDLE_VALUE)
		{
			DWORD dw;
			::WriteFile(hFile, pBuf, dwLength, &dw, NULL);
			::CloseHandle(hFile);
			return TRUE;
		}
	}
	return FALSE;
}




////////////////////////////////////////////////////////////////////////////
#endif // __FILEINPUTOUTPUTFUNC_H__
