// Bitmap.h : header file
//

#ifndef __BITMAP_H__
#define __BITMAP_H__
#pragma once
/////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////
//  Bitmap.h
/////////////////////////////////////////////////////////////////////////////
#define VERSION_BITMAP   "10.06.2009"



/////////////////////////////////////////////////////////////////////////////
/*
#define RED		0x000000ff
#define GREEN	0x0000ff00
#define BLUE	0x00ff0000
*/

//---------------------------------------


#define SIZE_H1		( sizeof(BITMAPFILEHEADER) )
#define SIZE_H2		( sizeof(BITMAPINFOHEADER) )
#define SIZE_3		( sizeof(RGBQUAD) )


class EBitmap  // bmp
{
public:


	EBitmap() {};
	EBitmap(HWND hWnd);
	EBitmap(HDC hDC);
	~EBitmap();

///
	HBITMAP CreateBitmap(int nWidth, int nHeight);
	HBITMAP CreateBitmap(SIZE& size);

	HDC BeginDraw(HBITMAP hBitmap);
	void EndDraw();

	void DeleteBitmap(HBITMAP hBitmap);

	void ShowBitmap(HBITMAP hBitmap, int x, int y);
	void ShowBitmap(HBITMAP hBitmap, int x, int y, int cx, int cy);

	SIZE GetBitmapSize(HBITMAP hBitmap);


	void FillBitmapBrush(HBITMAP hBitmap, HBRUSH hBrush);
	void FillBitmapColor(HBITMAP hBitmap, COLORREF crColor);
	void FillBitmap(HBITMAP hBitmap, BOOL bMode = TRUE);
	void InvertBitmap(HBITMAP hBitmap);


	HDC  GetBitmapDC()			{ return m_memDC; };
	LONG GetBitmapWidth()		{ return m_Width; };
	LONG GetBitmapHeight()		{ return m_Height; };
	SIZE GetBitmapSize()		{ return m_Size; };
	WORD GetBitmapBitCount()	{ return m_BitCount; };


///

	BOOL OpenFileBMP(LPCSTR lpszFileName);

	void DrawBitmap1(HDC hDC, int x, int y);
	void DrawBitmap(HDC hDC, int x, int y, double k = 1.0);
	void StretchBitmap(HDC hDC, RECT* pRect);


	inline LONG GetWidth() { return m_Width; };
	inline LONG GetHeight() { return m_Height; }

	double BitmapToRect(LONG nWidth, LONG nHeight, RECT* pRect);
	
	BOOL IsPalette();
	DWORD GetPaletteSize(WORD wBitCount);



	void CreateBitmap24(LONG nWidth, LONG nHeight, BOOL bMode = TRUE);
	void CreateBitmap8(LONG nWidth, LONG nHeight, BOOL bMode = TRUE);

	void PALETTEENTRY_to_RGBQUAD(int nCount, PALETTEENTRY* pPALETTEENTRY, RGBQUAD* pRGBQUAD);
	void RGBQUAD_to_PALETTEENTRY(int nCount, PALETTEENTRY* pPALETTEENTRY, RGBQUAD* pRGBQUAD);

//private:

	void _zero();
	void _free();

	void set_bitmap_info(DWORD dwOffset);
	void get_bitmap_info(HBITMAP hBitmap);

	void alloc_and_set_ptr();
	void set_variables(LONG nWidth, LONG nHeight, WORD wBitCount);
	void set_BITMAPFILEHEADER();
	void set_BITMAPINFOHEADER();
	void put_headers();
	void alloc_and_set_BITMAPINFO();


//	int Rotate90(CBMP& bmpIn,int K=0);
//	int DecartToPolar(CBMP& bmpIn,POINT center,DWORD RWidth,double dAstep);

//private:

	DWORD m_dwFileSize;			//  
	DWORD m_dwPaletteSize;		//  
	DWORD m_dwBitmapSize;		//   
	DWORD m_dwBitmapInfoSize;	// size of BITMAPINFO

	BITMAPFILEHEADER m_bmfh;	// size BITMAPFILEHEADER is SIZE_H1
	BITMAPINFOHEADER m_bmih;	// size BITMAPINFOHEADER is SIZE_H2
	BITMAPINFO* m_pbmi;			// pointer to BITMAPINFO

	BYTE* dest_file;			//   BMP-
	BYTE* dest_pal;				//    
	BYTE* dest_bits;			//    

	BITMAP m_bm;				// Bitmap information
	LONG m_Width;				// 
	LONG m_Height;				// 
	SIZE m_Size;				//   
	WORD m_BitCount;			//    


	HDC m_hDC;					//   
	HDC m_memDC;				//    
	HBITMAP m_hBmpCurr;			//   
	HBITMAP m_hBitmap;			//   




};





/////////////////////////////////////////////////////////////////////////////

inline EBitmap::EBitmap(HWND hWnd)
{
	_zero();
	m_hDC = ::GetDC(hWnd);
	m_memDC = ::CreateCompatibleDC(m_hDC);
}

inline EBitmap::EBitmap(HDC hDC)
{
	_zero();
	m_hDC = hDC;
	m_memDC = ::CreateCompatibleDC(m_hDC);
}

inline EBitmap::~EBitmap()
{
	_free();
	::DeleteDC(m_memDC);
}

inline void EBitmap::_zero()
{
	dest_file = NULL;
	
	m_pbmi = NULL;
	m_memDC = NULL;
//	m_hBitmap = NULL;
}

inline void EBitmap::_free()
{
	if(dest_file != NULL) delete [] dest_file;

	if(m_pbmi != NULL) delete [] m_pbmi;
//	if(m_hBitmap != NULL) ::DeleteObject(m_hBitmap);
	
}

//  ,     
//--------------------------------------------------------------------
inline HBITMAP EBitmap::CreateBitmap(int nWidth, int nHeight)
{
	return ::CreateCompatibleBitmap(m_hDC, nWidth, nHeight);
}

inline HBITMAP EBitmap::CreateBitmap(SIZE& size)
{
	return ::CreateCompatibleBitmap(m_hDC, size.cx, size.cy);
}

//     
//--------------------------------------------------------------------
inline HDC EBitmap::BeginDraw(HBITMAP hBitmap)
{
	m_hBmpCurr = (HBITMAP)::SelectObject(m_memDC, hBitmap);

	::GetObject(hBitmap, sizeof(BITMAP), &m_bm);

	m_Width = m_bm.bmWidth;
	m_Height = m_bm.bmHeight;
	
	m_Size.cx = m_Width;
	m_Size.cy = m_Height;
	
	m_BitCount = m_bm.bmBitsPixel;

	return m_memDC;
}

//     
//--------------------------------------------------------------------
inline void EBitmap::EndDraw()
{
	::SelectObject(m_memDC, m_hBmpCurr);
}

// e 
//--------------------------------------------------------------------
inline void EBitmap::DeleteBitmap(HBITMAP hBitmap)
{
	::DeleteObject(hBitmap);
}

//
//--------------------------------------------------------------------

//
//--------------------------------------------------------------------
inline void EBitmap::ShowBitmap(HBITMAP hBitmap, int x, int y)
{
	HBITMAP m_hBitmap = CreateBitmap(m_Width, m_Height);
	int a = ::SetDIBits(m_hDC, m_hBitmap, 0, m_Height,
		dest_bits, m_pbmi, DIB_RGB_COLORS);

//	m_hBitmap = ::CreateDIBitmap(m_hDC, (LPBITMAPINFOHEADER)m_pbmi,
//		CBM_INIT, dest_bits, m_pbmi, DIB_RGB_COLORS);

	HDC m_memDC = BeginDraw(m_hBitmap);

	BOOL r=::BitBlt(m_hDC, 0, 0, m_Width, m_Height, m_memDC, 0, 0, SRCCOPY);

//	get_bitmap_info(hBitmap);
//	DrawBitmap1(m_hDC,x,y);
	EndDraw();
	
	DeleteBitmap(m_hBitmap);
}

inline void EBitmap::ShowBitmap(HBITMAP hBitmap, int x, int y, int cx, int cy)
{
	BeginDraw(hBitmap);
	::StretchBlt(m_hDC, x, y, cx, cy, m_memDC, 0, 0, m_Width, m_Height, SRCCOPY);
	EndDraw();
}

inline void EBitmap::FillBitmapColor(HBITMAP hBitmap, COLORREF crColor)
{
	HBRUSH hBrush = ::CreateSolidBrush(crColor);
	HBRUSH hBrushOld = (HBRUSH)::SelectObject(m_memDC, hBrush);
	::PatBlt(m_memDC, 0, 0, m_Width, m_Height, PATCOPY);
	::SelectObject(m_memDC, hBrushOld);
	::DeleteObject(hBrush);

}

inline void EBitmap::FillBitmapBrush(HBITMAP hBitmap, HBRUSH hBrush)
{
	HBRUSH hBrushOld = (HBRUSH)::SelectObject(m_memDC, hBrush);
	::PatBlt(m_memDC, 0, 0, m_Width, m_Height, PATCOPY);
	::SelectObject(m_memDC, hBrushOld);
}

inline void EBitmap::FillBitmap(HBITMAP hBitmap, BOOL bMode)
{
	::PatBlt(m_memDC, 0, 0, m_Width, m_Height, (bMode) ? WHITENESS : BLACKNESS);
}

inline void EBitmap::InvertBitmap(HBITMAP hBitmap)
{
	::PatBlt(m_memDC, 0, 0, m_Width, m_Height, DSTINVERT);
}


inline BOOL EBitmap::OpenFileBMP(LPCSTR lpszFileName)
{
	_free();
/*
	if(!FOpen(lpszFileName)) return FALSE;
	m_dwFileSize = FSize();

	dest_file = (BYTE*)new BYTE[m_dwFileSize];
	FRead(dest_file, m_dwFileSize);

	FClose();


	m_bmfh = *((BITMAPFILEHEADER*)dest_file);
	m_bmih = *((BITMAPINFOHEADER*)(dest_file + SIZE_H1));

	m_Width = m_bmih.biWidth;
	m_Height = m_bmih.biHeight;
	m_BitCount = m_bmih.biBitCount;

	dest_bits = dest_file + m_bmfh.bfOffBits;
	m_dwBitmapSize = m_dwFileSize - m_bmfh.bfOffBits;
	set_bitmap_info(SIZE_H1);
*/	
	return TRUE;
}

inline void EBitmap::DrawBitmap1(HDC hDC, int x, int y)
{
	::SetDIBitsToDevice(hDC, x, y, m_Width, m_Height,
		0, 0, 0, m_Height, dest_bits, m_pbmi, DIB_RGB_COLORS);
}

inline void EBitmap::DrawBitmap(HDC hDC, int x, int y, double k)
{
	::StretchDIBits(hDC, x, y, (DWORD)(m_Width * k), (DWORD)(m_Height * k),
		0, 0, m_Width, m_Height, dest_bits, m_pbmi, DIB_RGB_COLORS, SRCCOPY);
}

inline void EBitmap::StretchBitmap(HDC hDC, RECT* pRect)
{
	::StretchDIBits(hDC, pRect->left, pRect->top,
		pRect->right - pRect->left, pRect->bottom - pRect->top,
		0, 0, m_Width, m_Height, dest_bits, m_pbmi, DIB_RGB_COLORS, SRCCOPY);
}

inline double EBitmap::BitmapToRect(LONG nWidth, LONG nHeight, RECT* pRect)
{
	double	kx = (double)nWidth / (double)(pRect->right - pRect->left);
	double	ky = (double)nHeight / (double)(pRect->bottom - pRect->top);

	double k = __max(kx, ky);

	pRect->right = pRect->left + (LONG)(nWidth / k);
	pRect->bottom = pRect->top + (LONG)(nHeight / k);

	return k;
}

inline BOOL EBitmap::IsPalette()
{
	return (BOOL)GetPaletteSize(m_BitCount);
}

inline DWORD EBitmap::GetPaletteSize(WORD wBitCount)
{
	DWORD dwSize;
	switch(wBitCount)
	{
		case 24: dwSize = 0; break;
		
		case 16:
		case 32: dwSize = sizeof(DWORD) * 3; break;
		
		default: dwSize = sizeof(RGBQUAD) * (1 << wBitCount); break;
	}
	return dwSize;
}

// DWORD set_bitmap_info(DWORD dwOffset)
// -------------------------------------------------------------------

inline void EBitmap::set_bitmap_info(DWORD dwOffset)
{
	m_dwPaletteSize = GetPaletteSize(m_BitCount);
	m_dwBitmapInfoSize = SIZE_H2 + m_dwPaletteSize;

	alloc_and_set_BITMAPINFO();
}

   
inline void EBitmap::get_bitmap_info(HBITMAP hBitmap)
{
	_free();

	m_dwPaletteSize = GetPaletteSize(m_BitCount);
	m_dwBitmapInfoSize = SIZE_H2 + m_dwPaletteSize;

	m_pbmi = (BITMAPINFO*)new BYTE[m_dwBitmapInfoSize];
	memset(m_pbmi, 0, m_dwBitmapInfoSize);
	m_pbmi->bmiHeader.biSize = SIZE_H2;

	int nScanLine = ::GetDIBits(m_hDC, hBitmap, 0, 0, (LPVOID)NULL, m_pbmi, DIB_RGB_COLORS);
	m_dwBitmapSize = m_pbmi->bmiHeader.biSizeImage;
	m_dwFileSize = SIZE_H1 + m_dwBitmapInfoSize + m_dwBitmapSize;

	alloc_and_set_ptr();

	::GetDIBits(m_hDC, hBitmap, 0, nScanLine, (LPVOID)dest_bits, m_pbmi, DIB_RGB_COLORS);
}

inline void EBitmap::CreateBitmap24(LONG nWidth, LONG nHeight, BOOL bMode)
{
	// step 01
	m_dwPaletteSize = 0;
	m_dwBitmapInfoSize = SIZE_H2;
	m_dwBitmapSize = __even4(nWidth * sizeof(RGBTRIPLE)) * nHeight;
	m_dwFileSize = __even4(SIZE_H1 + m_dwBitmapInfoSize + m_dwBitmapSize);

	// step 02, 03
//	alloc_and_set_ptr();

	dest_file = (BYTE*)new BYTE[m_dwFileSize];
	dest_pal = dest_file + SIZE_H1 + SIZE_H2;
	dest_bits = dest_file + SIZE_H1 + m_dwBitmapInfoSize;

//	set_variables(nWidth, nHeight, 24);

	m_Width = nWidth;
	m_Height = nHeight;
	m_Size.cx = m_Width;
	m_Size.cy = m_Height;
	m_BitCount = 24;

	// step 04, 05
//	set_BITMAPFILEHEADER();
//	set_BITMAPINFOHEADER();

	m_bmfh.bfType = 0x4D42;
	m_bmfh.bfSize = m_dwFileSize;
	m_bmfh.bfReserved1 = 0;
	m_bmfh.bfReserved2 = 0;
	m_bmfh.bfOffBits = SIZE_H1 + SIZE_H2 + m_dwPaletteSize;

	::ZeroMemory(&m_bmih, SIZE_H2);
	m_bmih.biSize = SIZE_H2;
	m_bmih.biWidth = m_Width;
	m_bmih.biHeight = m_Height;
	m_bmih.biPlanes = 1;
	m_bmih.biBitCount = m_BitCount;
	m_bmih.biCompression = BI_RGB;

	// step 06, 07
//	put_headers();

	 *((BITMAPFILEHEADER*)dest_file) = m_bmfh;
	 *((BITMAPINFOHEADER*)(dest_file + SIZE_H1)) = m_bmih;


	memset(dest_bits, 255, m_dwBitmapSize);
//	for(DWORD i=0; i<m_dwBitmapSize; i += 3)
//	{
//		dest_bits[i] = (BYTE)(2*i);
//		dest_bits[i+1] = (BYTE)(i*i/8);
//	}

//	alloc_and_set_BITMAPINFO();

	m_pbmi = (BITMAPINFO*)new BYTE[m_dwBitmapInfoSize + m_dwBitmapSize];
	::CopyMemory((BYTE*)m_pbmi, dest_file + SIZE_H1, m_dwBitmapInfoSize + m_dwBitmapSize);
}




inline void EBitmap::CreateBitmap8(LONG nWidth, LONG nHeight, BOOL bMode)
{
	// step 01
	m_dwPaletteSize = 256 * sizeof(RGBQUAD);
	m_dwBitmapInfoSize = SIZE_H2 + m_dwPaletteSize;
	m_dwBitmapSize = __even4(nWidth) * nHeight ;
	m_dwFileSize = __even4(SIZE_H1 + m_dwBitmapInfoSize + m_dwBitmapSize);

	// step 02, 03
	alloc_and_set_ptr();
	set_variables(nWidth, nHeight, 8);

	// step 04, 05
	set_BITMAPFILEHEADER();
	set_BITMAPINFOHEADER();

	// step 06, 
	put_headers();

//	int entries = ::GetSystemPaletteEntries(m_hDC, 0, 256, 0);
	PALETTEENTRY* pPal = (PALETTEENTRY*)new BYTE[256 * sizeof(PALETTEENTRY)];
//	::GetSystemPaletteEntries(m_hDC, 0, entries, pPal);
	int	entries = ::GetPaletteEntries((HPALETTE)::GetStockObject(DEFAULT_PALETTE), 0, 256, pPal);

/*	BYTE* tmp = dest_pal;
	for(int i=0; i<256; i++)
	{
		*(tmp++) = pPal[i].peBlue;
		*(tmp++) = pPal[i].peGreen;
		*(tmp++) = pPal[i].peRed;
		*(tmp++) = 0;
	}
*/
	PALETTEENTRY_to_RGBQUAD(255, pPal, (RGBQUAD*)dest_pal);
/*
00403F21   mov         ecx,dword ptr [ebp-10h]
00403F24   mov         edx,dword ptr [ebp-18h]
00403F27   mov         dword ptr [ecx],edx
*/
	delete [] pPal;

	alloc_and_set_BITMAPINFO();

	if(bMode)
	{
		::FillMemory(dest_bits, m_dwBitmapSize, 9);
		
		int line_di = nWidth + __supp4(nWidth);
		int x, y = 0;
		while(y < nHeight)
		{
			x = 0;
			while(x < line_di)
			{
				if(x >= nWidth) *(dest_bits + x + line_di * y) = 0;
				x++;
			}
			y++;
		}
	
	}
/**/
}

// RGBQUAD:			00 RR GG BB ...
// PALETTEENTRY:	ff BB GG RR ..., ff = 0x02
//
//  PALETTEENTRY -> RGBQUAD
inline void EBitmap::PALETTEENTRY_to_RGBQUAD(int nCount, PALETTEENTRY* pPALETTEENTRY, RGBQUAD* pRGBQUAD)
{
	DWORD* pDest = (DWORD*)pRGBQUAD;
	DWORD* pSrc = (DWORD*)pPALETTEENTRY;

	for(int i=0; i<nCount; i++)
	{
		__asm
		{
			mov		ecx, dword ptr[pSrc];
			mov     eax, dword ptr[ecx];	//	eax = *((DWORD*)pSrc)
											//	eax = ff BB GG RR
			mov		ebx, eax;					
			rol		eax, 16;				//	eax	= GG RR ff BB
			and		eax, 0x00FF00FF;		//	eax	= 00 RR 00 BB
			and		ebx, 0x0000FF00;		//	ebx = 00 00 GG 00
			or		eax, ebx;				//	eax	= 00 RR GG BB

			mov     ecx, dword ptr[pDest];
			mov     dword ptr[ecx], eax;	//	*pDest = eax
		}
		pDest++;
		pSrc++;
	}
}

//  RGBQUAD -> PALETTEENTRY
inline void EBitmap::RGBQUAD_to_PALETTEENTRY(int nCount, PALETTEENTRY* pPALETTEENTRY, RGBQUAD* pRGBQUAD)
{
	DWORD* pDest = (DWORD*)pPALETTEENTRY;
	DWORD* pSrc = (DWORD*)pRGBQUAD;

	for(int i=0; i<nCount; i++)
	{
		__asm
		{
			mov		ecx, dword ptr[pSrc];
			mov     eax, dword ptr[ecx];	//	eax = *((DWORD*)pSrc)
											//	eax = 00 RR GG BB
			mov		ebx, eax;					
			rol		eax, 16;				//	eax	= GG BB 00 RR
			and		eax, 0x00FF00FF;		//	eax	= 00 BB 00 PP
			and		ebx, 0x0000FF00;		//	ebx = 00 00 GG 00
			or		eax, ebx;				//	eax	= 00 BB GG RR
			or		eax, 0x02000000;		//	eax = ff BB GG RR

			mov     ecx, dword ptr[pDest];
			mov     dword ptr[ecx], eax;	//	*pDest = eax
		}
		pDest++;
		pSrc++;
	}
}


// step 02
inline void EBitmap::alloc_and_set_ptr()
{
	dest_file = (BYTE*)new BYTE[m_dwFileSize];
	dest_pal = dest_file + SIZE_H1 + SIZE_H2;
	dest_bits = dest_file + SIZE_H1 + m_dwBitmapInfoSize;
}

// step 03
inline void EBitmap::set_variables(LONG nWidth, LONG nHeight, WORD wBitCount)
{
	m_Width = nWidth;
	m_Height = nHeight;
	m_Size.cx = m_Width; m_Size.cy = m_Height;
	m_BitCount = wBitCount;
}

// step 04
inline void EBitmap::set_BITMAPFILEHEADER()
{
	m_bmfh.bfType = 0x4D42;
	m_bmfh.bfSize = m_dwFileSize;
	m_bmfh.bfReserved1 = 0;
	m_bmfh.bfReserved2 = 0;
	m_bmfh.bfOffBits = SIZE_H1 + SIZE_H2 + m_dwPaletteSize;
}

// step 05
inline void EBitmap::set_BITMAPINFOHEADER()
{
	::ZeroMemory(&m_bmih, SIZE_H2);
	m_bmih.biSize = SIZE_H2;
	m_bmih.biWidth = m_Width;
	m_bmih.biHeight = m_Height;
	m_bmih.biPlanes = 1;
	m_bmih.biBitCount = m_BitCount;
	m_bmih.biCompression = BI_RGB;
}

// step 06
inline void EBitmap::put_headers()
{
	 *((BITMAPFILEHEADER*)dest_file) = m_bmfh;
	 *((BITMAPINFOHEADER*)(dest_file + SIZE_H1)) = m_bmih;
}

// step 07
inline void EBitmap::alloc_and_set_BITMAPINFO()
{
	m_pbmi = (BITMAPINFO*)new BYTE[m_dwBitmapInfoSize];
	::CopyMemory((BYTE*)m_pbmi, dest_file + SIZE_H1, m_dwBitmapInfoSize);
}

/*
inline int EBitmap::Rotate90(CBMP& bmpIn,int K)
{
	FreeBuf(pBuf);
	bmfh=bmpIn.bmfh;
	bmih=bmpIn.bmih;
	pBuf=(RGBQUAD*)malloc(sizeof(RGBQUAD) *
		bmpIn.bmih.biWidth * bmpIn.bmih.biHeight);
	if (pBuf==NULL)
		return IDS_NOTENOUGHMEMORY;

	if (K&1)
	{
		bmih.biWidth = bmpIn.bmih.biHeight;
		bmih.biHeight = bmpIn.bmih.biWidth;
	}

	long x,y,xx,yy,ii;

	for (y=0;y<bmpIn.bmih.biHeight;y++)
		for (x=0;x<bmpIn.bmih.biWidth;x++)
		{
			xx=x;yy=y;
			if (K&2)
			{
				xx=bmpIn.bmih.biWidth-x-1;
				yy=bmpIn.bmih.biHeight-y-1;
			}
			if (K&1)
			{
				ii=yy;
				yy=bmpIn.bmih.biWidth-xx-1;
				xx=ii;
			}
			*(pBuf + xx + yy * bmih.biWidth) =
				*(bmpIn.pBuf + x + y * bmpIn.bmih.biWidth);
		}
	return IDOK;
}
///////////////////////////////////////////////////////////////////////////
inline int EBitmap::DecartToPolar(CBMP& bmpIn,POINT center,DWORD RWidth,double dAstep)
{
	DWORD Rmax=__min(
		__min(center.x,bmpIn.bmih.biWidth-center.x),
		__min(center.y,bmpIn.bmih.biHeight-center.y));

	DWORD x,y,R,iA=0;
	RGBQUAD *pD;
	double iR,A=0;

	FreeBuf(pBuf);
	pBuf=(RGBQUAD*)malloc(RWidth * (10+DWORD(PI2/dAstep)) * sizeof(RGBQUAD));
	if (pBuf==NULL)
		return IDS_NOTENOUGHMEMORY;

	while (A<PI2)
	{

		for (R=0;R<RWidth;R++)
		{
			iR = Rmax * R / RWidth;
			x=(DWORD)(iR*sin(A))+center.x;
			y=(DWORD)(iR*cos(A))+center.y;
			pD=bmpIn.pBuf+x+y*bmpIn.bmih.biWidth;
			*(pBuf+R+iA*RWidth)=*pD;
		}
		A+=dAstep;
		iA++;
	}

	bmih=bmpIn.bmih;
	bmih.biWidth=RWidth;
	bmih.biHeight=iA;
	pBuf=(RGBQUAD*)realloc(pBuf,RWidth * iA * sizeof(RGBQUAD));
	return IDOK;
}
*/








inline void FlipDIB(BYTE* pDIB)
{
	BITMAPINFOHEADER* pBmih = (BITMAPINFOHEADER*)pDIB;

	int nRowSize = 4 * ((16 * pBmih->biWidth + 31) / 32);
	BYTE* pRow = new BYTE[nRowSize];

	int i, nSwap = pBmih->biHeight / 2;
	int nPalSize = 0;
	BYTE* pData = pDIB + sizeof(BITMAPINFOHEADER) + nPalSize;

	for(int j = 0; j < nSwap; j++)
	{
		i = pBmih->biHeight - j - 1;
		memcpy(pRow, pData + j * nRowSize, nRowSize);
		memcpy(pData + j * nRowSize, pData + i * nRowSize, nRowSize);
		memcpy(pData + i * nRowSize, pRow, nRowSize);
	}

	delete [] pRow;
}


inline DWORD _EcxGetPaletteSize(BYTE* pDIB)
{
	BITMAPINFOHEADER* pBmih = (BITMAPINFOHEADER*)pDIB;
	DWORD dwSize = 0;
	int nBitFields = (pBmih->biSize == 56) ? 4 : 3;

	if(pBmih->biBitCount < 16)
	{
		dwSize = sizeof(RGBQUAD) * (1 << pBmih->biBitCount);
	}
	else
	{
		if(pBmih->biClrUsed != 0)
			dwSize = sizeof(RGBQUAD) * pBmih->biClrUsed;

		if(pBmih->biCompression == BI_BITFIELDS)
			dwSize += sizeof(DWORD) * nBitFields;
	}

	return dwSize;
}






/////////////////////////////////////////////////////////////////////////////
#endif // __BITMAP_H__