// ECanvas.h : header file
//

#ifndef __ECANVAS_H__
#define __ECANVAS_H__
#pragma once
/////////////////////////////////////////////////////////////////////////////



/////////////////////////////////////////////////////////////////////////////
//  ECanvas.h
/////////////////////////////////////////////////////////////////////////////
#define VERSION_ECANVAS   "11.02.2013"



/////////////////////////////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////////////////////////////
#define CDM_BITBLT				0
#define CDM_STRETCHBLT			1
#define CDM_TRANSPARENTBLT		2
#define CDM_ALPHABLEND			3


/////////////////////////////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////////////////////////////
typedef struct _CanvasDrawParams  // cdp
{
	int nMode;
	double dKx;
	double dKy;
	COLORREF crTransparent;
	int nAlpha;

	_CanvasDrawParams()
	{
		init();
	}

	_CanvasDrawParams(_CanvasDrawParams* pCdp)
	{
		init();
		if(is_ptrr(pCdp, _CanvasDrawParams)) *this = *pCdp;
	}

	void init()
	{
		nMode = CDM_BITBLT;
		dKx = dKy = 1.0;
		crTransparent = RGB(255,255,0);
		nAlpha = 1000;
	}


} CANVASDRAWPARAMS;




/////////////////////////////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////////////////////////////
#define FF_NORMAL		0x00000000
#define FF_BOLD			0x00000001
#define FF_ITALIC		0x00000002
#define FF_UNDERLINE	0x00000004
#define FF_STRIKEOUT	0x00000008
#define FF_SHADOW		0x00000010



typedef struct _FontParams  // fp
{
	int nID;
	int nHeight;
	int nWidth;
	int nAngle;
	int nFormat;
//	BYTE lfCharSet;
	char szFaceName[LF_FACESIZE];

} FONTPARAMS; 




/////////////////////////////////////////////////////////////////////////////
//      Bullet
/////////////////////////////////////////////////////////////////////////////
#define BULLET_CROSS		0
#define BULLET_SQUARE		1
#define BULLET_CIRCLE		2
#define BULLET_TRIANGLE		3




/////////////////////////////////////////////////////////////////////////////
//   ECanvas
/////////////////////////////////////////////////////////////////////////////
class ECanvas  // cnv
{
public:

	////////////////////////////////////////////////////////////////
	// 
	////////////////////////////////////////////////////////////////
	ECanvas()
	{
		_init();
	}

	ECanvas(HDC hDC, int cx, int cy)
	{
		_init();
		Create(hDC, cx, cy);
	}

	ECanvas(ECanvas* pCnv, int cx, int cy)
	{
		_init();
		Create(pCnv->m_hDC, cx, cy);
	}


	////////////////////////////////////////////////////////////////
	// 
	////////////////////////////////////////////////////////////////
	~ECanvas()
	{
		Delete();
	}


	////////////////////////////////////////////////////////////////
	//  
	////////////////////////////////////////////////////////////////
	BOOL Create(HDC hDC, int cx, int cy)
	{
		Delete();
		_init();

		m_hParentDC = hDC;
		m_cx = cx; m_cy = cy;
		m_hDC = ::CreateCompatibleDC(m_hParentDC);
		if(m_hDC)
		{
			m_hBitmap = ::CreateCompatibleBitmap(m_hParentDC, cx, cy);
			if(m_hBitmap)
			{
				m_hBitmapOld = (HBITMAP)::SelectObject(m_hDC, m_hBitmap);
				if(m_hBitmapOld) m_isCreated = TRUE;
			}
		}
		return m_isCreated;
	}


	////////////////////////////////////////////////////////////////
	//  
	////////////////////////////////////////////////////////////////
	void Delete()
	{
		if(m_isCreated)
		{
			m_isCreated = FALSE;
		
			::SelectObject(m_hDC, m_hBitmapOld);
			::DeleteDC(m_hDC);
			::DeleteObject(m_hBitmap);
			
			DeleteDIB();
		}
		
		if(m_hFont) ::DeleteObject(m_hFont);
	}



	////////////////////////////////////////////////////////////////
	//  
	////////////////////////////////////////////////////////////////
	void Draw(int x, int y, CANVASDRAWPARAMS* pCdp = NULL)
	{
		CANVASDRAWPARAMS cdp(pCdp);

		if(cdp.nMode == CDM_BITBLT)
		{
			::BitBlt(m_hParentDC, x, y, m_cx, m_cy, m_hDC, 0, 0, SRCCOPY);
		}
		else
		{
			::SetStretchBltMode(m_hParentDC, COLORONCOLOR);

			int cx = (int)((double)m_cx * cdp.dKx);
			int cy = (int)((double)m_cy * cdp.dKy);

			switch(cdp.nMode)
			{
				case CDM_STRETCHBLT:
					::StretchBlt(m_hParentDC, x, y, cx, cy, m_hDC, 0, 0, m_cx, m_cy, SRCCOPY);
					break;
				case CDM_TRANSPARENTBLT:
					::TransparentBlt(m_hParentDC, x, y, cx, cy, m_hDC, 0, 0, m_cx, m_cy, _EcxGetSysColor(cdp.crTransparent));
					break;
				case CDM_ALPHABLEND:
				{
					BLENDFUNCTION blend;
					blend.BlendOp = AC_SRC_OVER;
					blend.BlendFlags = 0;
					blend.AlphaFormat = AC_SRC_OVER;
					blend.SourceConstantAlpha = (int)((1000.0 - (double)cdp.nAlpha) / 3.921568627450980);
					::AlphaBlend(m_hParentDC, x, y, cx, cy, m_hDC, 0, 0, m_cx, m_cy, blend);
					break;
				}
			}
		}
	}
	

	////////////////////////////////////////////////////////////////
	//     
	////////////////////////////////////////////////////////////////
	void Fill(COLORREF crBack, UINT nBorderSize = 0, COLORREF crBorder = 0)
	{
		if(nBorderSize != 0) _EcxFastColorRect(m_hDC, 0, 0, m_cx, m_cy, _EcxGetSysColor(crBorder));
		_EcxFastColorRect(m_hDC, nBorderSize, nBorderSize, m_cx - 2 * nBorderSize, m_cy - 2 * nBorderSize, _EcxGetSysColor(crBack));
	}


	////////////////////////////////////////////////////////////////
	//    
	////////////////////////////////////////////////////////////////
	void FastColorRect(RECT* prc, COLORREF cr)
		{ _EcxFastColorRect(m_hDC, prc, _EcxGetSysColor(cr)); }
	void FastColorRect(int x, int y, int cx, int cy, COLORREF cr)
		{ _EcxFastColorRect(m_hDC, x, y, cx, cy, _EcxGetSysColor(cr)); }
	void FastColorRect(POINT pt, SIZE siz, COLORREF cr)
		{ _EcxFastColorRect(m_hDC, pt.x, pt.y, siz.cx, siz.cy, _EcxGetSysColor(cr)); }

	////////////////////////////////////////////////////////////////
	//  
	////////////////////////////////////////////////////////////////
	void Bullet(int x, int y, COLORREF cr, int nType = BULLET_CIRCLE)
	{
		switch(nType)
		{
			case BULLET_CROSS:
			{
				_EcxFastColorRect(m_hDC, x - 4, y, 9, 1, cr);
				_EcxFastColorRect(m_hDC, x, y - 4, 1, 9, cr);
				break;
			}
			case BULLET_SQUARE:
			{
				_EcxFastColorRect(m_hDC, x - 3, y - 3, 7, 7, cr);
				break;
			}
			case BULLET_CIRCLE:
			{
				_EcxFastColorRect(m_hDC, x - 1, y - 3, 3, 7, cr);
				_EcxFastColorRect(m_hDC, x - 3, y - 1, 7, 3, cr);
				_EcxFastColorRect(m_hDC, x - 2, y - 2, 5, 5, cr);
				break;
			}
			case BULLET_TRIANGLE:
			{
				_EcxFastColorRect(m_hDC, x - 3, y + 1, 7, 2, cr);
				_EcxFastColorRect(m_hDC, x - 2, y - 1, 5, 2, cr);
				_EcxFastColorRect(m_hDC, x - 1, y - 3, 3, 2, cr);
				_EcxFastColorRect(m_hDC, x, y - 5, 1, 2, cr);
				break;
			}
		}
	}



	////////////////////////////////////////////////////////////////
	//
	////////////////////////////////////////////////////////////////

	void SetShadowSize(int cx, int cy)
	{
		m_sizShadow.cx = cx;
		m_sizShadow.cy = cy;

	}


	////////////////////////////////////////////////////////////////
	//
	////////////////////////////////////////////////////////////////
	void CreateFont(LONG lHeight, LPSTR lpszFaceName, DWORD dwFormat)
	{
		LOGFONT lf;
		if(m_hFont) ::DeleteObject(m_hFont);
		lf.lfHeight = -::MulDiv(lHeight, ::GetDeviceCaps(m_hDC, LOGPIXELSY), 72);
		lf.lfWidth = 0; 
		lf.lfEscapement = 0; 
		lf.lfOrientation = 0; 
		lf.lfWeight = _is_flag(dwFormat, FF_BOLD) ? FW_BOLD : FW_NORMAL; 
		lf.lfItalic = _is_flag(dwFormat, FF_ITALIC); 
		lf.lfUnderline = _is_flag(dwFormat, FF_UNDERLINE); 
		lf.lfStrikeOut = _is_flag(dwFormat, FF_STRIKEOUT); 
		lf.lfCharSet = RUSSIAN_CHARSET; 
		lf.lfOutPrecision = 0; 
		lf.lfClipPrecision = 0; 
		lf.lfQuality = PROOF_QUALITY; 
		lf.lfPitchAndFamily = 0; 
		strcpy(lf.lfFaceName, lpszFaceName);
		m_hFont = ::CreateFontIndirect(&lf);
	}



	////////////////////////////////////////////////////////////////
	//
	////////////////////////////////////////////////////////////////
	void TextOut(int x, int y, LPCTSTR lpString, LONG lHeight = 8, LPSTR lpszFaceName = "MS Sans Serif", DWORD dwFormat = FF_NORMAL, COLORREF cr = 0)
	{
		CreateFont(lHeight, lpszFaceName, dwFormat);
		HFONT hFontOld = (HFONT)::SelectObject(m_hDC, m_hFont);
		::SetBkMode(m_hDC, TRANSPARENT);
		if(_is_flag(dwFormat, FF_SHADOW))
		{
			::SetTextColor(m_hDC, _EcxGetSysColor(0));
			::TextOut(m_hDC, x + m_sizShadow.cx, y + m_sizShadow.cy, lpString, strlen(lpString));
		}
		::SetTextColor(m_hDC, _EcxGetSysColor(cr));
		::TextOut(m_hDC, x, y, lpString, strlen(lpString));
		::SetTextColor(m_hDC, 0);
		::SelectObject(m_hDC, hFontOld);
	}

	////////////////////////////////////////////////////////////////
	//
	////////////////////////////////////////////////////////////////
	void DrawText(int x, int y, int cx, int cy, UINT uFormat, LPCTSTR lpString, LONG lHeight = 8, LPSTR lpszFaceName = "MS Sans Serif", DWORD dwFormat = FF_NORMAL, COLORREF cr = 0)
	{
		RECT rc = { x, y, x + cx, y + cy };
		CreateFont(lHeight, lpszFaceName, dwFormat);
		HFONT hFontOld = (HFONT)::SelectObject(m_hDC, m_hFont);
		::SetBkMode(m_hDC, TRANSPARENT);
		if(_is_flag(dwFormat, FF_SHADOW))
		{
			RECT rc1 = rc;
			::OffsetRect(&rc1, m_sizShadow.cx, m_sizShadow.cy);
			::SetTextColor(m_hDC, _EcxGetSysColor(0));
			::DrawText(m_hDC, lpString, strlen(lpString), &rc1, uFormat);
		}
		::SetTextColor(m_hDC, _EcxGetSysColor(cr));
		::DrawText(m_hDC, lpString, strlen(lpString), &rc, uFormat);
		::SetTextColor(m_hDC, 0);
		::SelectObject(m_hDC, hFontOld);
	}


	////////////////////////////////////////////////////////////////
	//  DDB  DIB
	////////////////////////////////////////////////////////////////
	BITMAPINFO* GetDIB()
	{
		DeleteDIB();

		m_bm.bmType = 0;
		::GetObject(m_hBitmap, sizeof(BITMAP),(LPSTR)&m_bm);

		DWORD dwBitmapSize = _even4(m_bm.bmHeight * sizeof(RGBTRIPLE)) * m_bm.bmWidth;
		m_pBmi = (BITMAPINFO*)new BYTE[sizeof(BITMAPINFOHEADER) + dwBitmapSize];
		memset(m_pBmi, 0, sizeof(BITMAPINFOHEADER) + dwBitmapSize);
		BYTE* pDest = (BYTE*)m_pBmi + sizeof(BITMAPINFOHEADER);

		m_pBmi->bmiHeader.biSize			= sizeof(BITMAPINFOHEADER);
		m_pBmi->bmiHeader.biHeight 			= m_bm.bmHeight;
		m_pBmi->bmiHeader.biWidth			= m_bm.bmWidth;
		m_pBmi->bmiHeader.biPlanes 			= 1;
		m_pBmi->bmiHeader.biBitCount		= m_bm.bmBitsPixel;
		m_pBmi->bmiHeader.biCompression		= BI_RGB;
		m_pBmi->bmiHeader.biSizeImage		= 0;
		m_pBmi->bmiHeader.biXPelsPerMeter	= 0;
		m_pBmi->bmiHeader.biYPelsPerMeter	= 0;
		m_pBmi->bmiHeader.biClrUsed			= 0;
		m_pBmi->bmiHeader.biClrImportant	= 0;
	
		int nScanLine = ::GetDIBits(m_hParentDC, m_hBitmap, 0, 120, pDest, m_pBmi, DIB_RGB_COLORS);
		return m_pBmi;
	}


	////////////////////////////////////////////////////////////////
	//  DIB
	////////////////////////////////////////////////////////////////
	void DeleteDIB()
	{
		if(m_pBmi != NULL)
		{
			delete [] m_pBmi;
			m_pBmi = NULL;
			memset(&m_bm, 0, sizeof(BITMAP));
		}
	}




	////////////////////////////////////////////////////////////////
	//     
	////////////////////////////////////////////////////////////////
	operator HDC()
	{
		return m_hDC;
	}


	////////////////////////////////////////////////////////////////
	//  
	////////////////////////////////////////////////////////////////
	HDC m_hDC;
	BOOL m_isCreated;



	void _init()
	{
		memset(this, 0, sizeof(ECanvas));
		m_sizShadow.cx = 1;
		m_sizShadow.cy = 1;
	}






	HDC m_hParentDC;
	HBITMAP m_hBitmap, m_hBitmapOld;
	BITMAP m_bm;
	BITMAPINFO* m_pBmi;
	int m_cx, m_cy;

	HFONT m_hFont;
	SIZE m_sizShadow;
};


/////////////////////////////////////////////////////////////////////////////
#endif // __ECANVAS_H__