// ColorDefinition.h : header file
//

#ifndef __COLORDEFINITION_H__
#define __COLORDEFINITION_H__
#pragma once
////////////////////////////////////////////////////////////////////////////

#include <math.h>


/////////////////////////////////////////////////////////////////////////////
//  ColorDefinition.h
/////////////////////////////////////////////////////////////////////////////
#define VERSION_COLORDEFINITION   "30.07.2020"







/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
enum _ColorSpacesID
{
	CS_GRAY,
	CS_RGB,
	CS_HSV,
	CS_CMYK,

	CS_LAB,
	CS_XYZ,
	CS_HLS,
	CS_YUV,
};


/*
The GRAYCOLOR Structure
struct GRAYCOLOR {
    WORD    gray;
};
 
The RGBCOLOR Structure
struct RGBCOLOR {
    WORD    red;
    WORD    green;
    WORD    blue;
};
 
The CMYKCOLOR Structure
struct CMYKCOLOR {
    WORD    cyan;
    WORD    magenta;
    WORD    yellow;
    WORD    black;
};
 
The XYZCOLOR Structure
struct XYZCOLOR {
    WORD    X;
    WORD    Y;
    WORD    Z;
};
 
The YxyCOLOR Structure
struct YxyCOLOR {
    WORD    Y;
    WORD    x;
    WORD    y;
};
 
The L*a*bCOLOR Structure
struct LabCOLOR {
    WORD    L;
    WORD    a;
    WORD    b;
};
 
The GENERIC3CHANNEL Structure
struct GENERIC3CHANNEL {
    WORD    ch1;
    WORD    ch2;
    WORD    ch3;
};
 
The NAMEDCOLOR Structure
struct NAMEDCOLOR {
    DWORD   dwIndex;
};
 
The HiFiCOLOR Structure
struct HiFiCOLOR {
    BYTE    channel[MAX_COLOR_CHANNELS];
};
 
The COLOR Structure
typedef union tagCOLOR {
    struct GRAYCOLOR        gray;
    struct RGBCOLOR         rgb;
    struct CMYKCOLOR        cmyk;
    struct XYZCOLOR         XYZ;
    struct YxyCOLOR         Yxy;
    struct LabCOLOR         Lab;
    struct GENERIC3CHANNEL  gen3ch;
    struct NAMEDCOLOR       named;
    struct HiFiCOLOR        hifi;
} COLOR;

*/


/////////////////////////////////////////////////////////////////////////////
// graY
/////////////////////////////////////////////////////////////////////////////
typedef struct tagGRAY  // gray
{
	int y;			//   (0-255)

	int nType;		//     RGB
	double dGamma;	//  

} _GRAY;


/////////////////////////////////////////////////////////////////////////////
// Red, Green, Blue
/////////////////////////////////////////////////////////////////////////////
typedef struct tagRGB  // rgb
{
	int r;
	int g;
	int b;

	BOOL operator==(tagRGB& rgb) { return RGB(r, g, b) == RGB(rgb.r, rgb.g, rgb.b); }
	BOOL operator!=(tagRGB& rgb) { return RGB(r, g, b) != RGB(rgb.r, rgb.g, rgb.b); }
	DWORD toCOLORREF() { return RGB(r, g, b); }

} _RGB;


/////////////////////////////////////////////////////////////////////////////
// Cyan, Magenta, Yellow, blacK
/////////////////////////////////////////////////////////////////////////////
typedef struct tagCMYK  // cmyk
{
	int c;
	int m;
	int y;
	int k;

	BOOL isCMYK;

} _CMYK;


/////////////////////////////////////////////////////////////////////////////
// Hue, Lightness, Saturation
/////////////////////////////////////////////////////////////////////////////
typedef struct tagHLS  // hls
{
	int h;
	int l;
	int s;

} _HLS;


/////////////////////////////////////////////////////////////////////////////
// Hue, Saturation, Value
/////////////////////////////////////////////////////////////////////////////
typedef struct tagHSV  // hsv
{
	int h;
	int s;
	int v;

} _HSV;


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
typedef struct tagYUV  // yuv
{
	int y;
	int u;
	int v;

} _YUV;


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
typedef struct tagXYZ  // xyz
{
	double x;
	double y;
	double z;

} _XYZ;


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
typedef struct tagLab  // lab
{
	int L;
	int a;
	int b;

} _LAB;






/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
inline void RGBtoH(_RGB rgb, int* pH)
{
	BYTE cMax, cMin, mMaxMin;

	cMax = max3(rgb.r, rgb.g, rgb.b);
	cMin = min3(rgb.r, rgb.g, rgb.b);
	mMaxMin = cMax - cMin;
	*pH = 0;

	if(cMax != cMin)
	{
		/*
		if(cMax == rgb.r)
		{
			*pH = (int)(60.0 * (double)(rgb.g - rgb.b) / (double)mMaxMin + 0.5);
			if(rgb.g < rgb.b) *pH += 360;
		}
		if(cMax == rgb.g) *pH = (int)(60.0 * (double)(rgb.b - rgb.r) / (double)mMaxMin + 120.5);// + 120;
		if(cMax == rgb.b) *pH = (int)(60.0 * (double)(rgb.r - rgb.g) / (double)mMaxMin + 0.5) + 240;
		*/

		if(cMax == rgb.r)
		{
			*pH = (int)_round(60.0 * (double)(rgb.g - rgb.b) / (double)mMaxMin);
			if(rgb.g < rgb.b) *pH += 360;
		}
		
		if(cMax == rgb.g)
		{
			*pH = (int)_round(60.0 * (double)(rgb.b - rgb.r) / (double)mMaxMin) + 120;
		}

		if(cMax == rgb.b)
		{
			*pH = (int)_round(60.0 * (double)(rgb.r - rgb.g) / (double)mMaxMin) + 240;
		}
	}
}


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
inline double F(double s)
{
	if(s >= 0.008856)
		return pow(s, 0.333333);
	else
		return 7.787000 * s + 0.137931;
}


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
inline double S(double f)
{
	if(f >= 0.206893)
		return pow(f, 3.000000);
	else
		return -0.017713 + 0.128419 * f;
}



/////////////////////////////////////////////////////////////////////////////
//   SMPTE
/////////////////////////////////////////////////////////////////////////////
inline void RGBtoXYZ(_RGB rgb, _XYZ* pxyz)
{
//	pxyz->x = (0.431 * (double)rgb.r + 0.342 * (double)rgb.g + 0.178 * (double)rgb.b) / 255.0;
//	pxyz->y = (0.222 * (double)rgb.r + 0.707 * (double)rgb.g + 0.071 * (double)rgb.b) / 255.0;
//	pxyz->z = (0.020 * (double)rgb.r + 0.130 * (double)rgb.g + 0.939 * (double)rgb.b) / 255.0;

	double d = 4.0 - (double)(rgb.r + rgb.g + rgb.b) / 255.0;
	pxyz->x = d * (0.210000 * (double)rgb.r + 0.103333 * (double)rgb.g + 0.051667 * (double)rgb.b) / 255.0;
	pxyz->y = d * (0.113333 * (double)rgb.r + 0.198333 * (double)rgb.g + 0.023333 * (double)rgb.b) / 255.0;
	pxyz->z = d * (0.010000 * (double)rgb.r + 0.031667 * (double)rgb.g + 0.258333 * (double)rgb.b) / 255.0;
}

inline void XYZtoRGB(_XYZ xyz, _RGB* prgb) ///?????
{
	double d = (xyz.x + xyz.y + xyz.z);
	prgb->r = (BYTE)(d * 255.0*( 6.569931*xyz.x - 3.260210*xyz.y - 1.019516*xyz.z));
	prgb->g = (BYTE)(d * 255.0*(-3.778822*xyz.x + 6.990965*xyz.y + 0.124322*xyz.z));
	prgb->b = (BYTE)(d * 255.0*( 0.208890*xyz.x - 0.730755*xyz.y + 3.895193*xyz.z));
}

inline void XYZtoLAB(_XYZ xyz, _LAB* plab)
{
	plab->L = (int)(116.0 * F(xyz.y / 0.335) - 16.0);
	plab->a = (int)(500.0 * (F(xyz.x / 0.365) - F(xyz.y / 0.335)));
	plab->b = (int)(200.0 * (F(xyz.y / 0.335) - F(xyz.z / 0.300)));
}

inline void LABtoXYZ(_LAB lab, _XYZ* pxyz)
{
	double d = 0.137931 + 0.008621 * (double)lab.L;

	pxyz->x = 0.365 * S(d + 0.002000 * (double)lab.a);
	pxyz->y = 0.335 * S(d);
	pxyz->z = 0.300 * S(d - 0.005000 * (double)lab.b);
}


inline void RGBtoLAB(_RGB rgb, _LAB* plab)
{
	_XYZ xyz;
	RGBtoXYZ(rgb, &xyz);
	XYZtoLAB(xyz, plab);
}


inline void LABtoRGB(_LAB lab, _RGB* prgb)
{
	_XYZ xyz;
	LABtoXYZ(lab, &xyz);
	XYZtoRGB(xyz, prgb);
}


/////////////////////////////////////////////////////////////////////////////
// U==Cb, V==Cr
/////////////////////////////////////////////////////////////////////////////
inline void RGBtoYUV(_RGB rgb, _YUV* pyuv)
{
	pyuv->y = (BYTE)( 0.299*(double)rgb.r + 0.587*(double)rgb.g + 0.114*(double)rgb.b);
	pyuv->u = (BYTE)(-0.147*(double)rgb.r - 0.289*(double)rgb.g + 0.436*(double)rgb.b) + 128;
	pyuv->v = (BYTE)( 0.615*(double)rgb.r - 0.515*(double)rgb.g - 0.100*(double)rgb.b) + 128;
}


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
inline void YUVtoRGB(_YUV yuv, _RGB* prgb)
{
	prgb->r = (BYTE)(0.772*(double)yuv.y + 1.140*(double)(yuv.v - 128) - 0.463*(double)(yuv.u - 128));
	prgb->g = (BYTE)(1.116*(double)yuv.y - 0.581*(double)(yuv.v - 128) - 0.159*(double)(yuv.u - 128));
	prgb->b = (BYTE)(1.000*(double)yuv.y + 0.000*(double)(yuv.v - 128) + 2.032*(double)(yuv.u - 128));
}


/////////////////////////////////////////////////////////////////////////////        OK
// 
/////////////////////////////////////////////////////////////////////////////
inline void RGBtoCMYK(_RGB rgb, _CMYK* pcmyk)
{
	pcmyk->c = 255 - rgb.r;
	pcmyk->m = 255 - rgb.g;
	pcmyk->y = 255 - rgb.b;

	if(pcmyk->isCMYK)
	{
		pcmyk->k = min3(pcmyk->c, pcmyk->m, pcmyk->y);
		pcmyk->c = pcmyk->c - pcmyk->k;
		pcmyk->m = pcmyk->m - pcmyk->k;
		pcmyk->y = pcmyk->y - pcmyk->k;
	}
}


/////////////////////////////////////////////////////////////////////////////        OK
// 
/////////////////////////////////////////////////////////////////////////////
inline void CMYKtoRGB(_CMYK cmyk, _RGB* prgb)
{
	prgb->r = 255 - cmyk.c;
	prgb->g = 255 - cmyk.m;
	prgb->b = 255 - cmyk.y;

	if(cmyk.isCMYK)
	{
		prgb->r -= cmyk.k;
		prgb->g -= cmyk.k;
		prgb->b -= cmyk.k;
	}

	if(prgb->r < 0) prgb->r = 0;
	if(prgb->g < 0) prgb->g = 0;
	if(prgb->b < 0) prgb->b = 0;
}

/*
/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
inline void YUVtoRGB(_YUV yuv, _RGB* prgb)
{
	prgb->r = (BYTE)(0.772*(double)yuv.y + 1.140*(double)(yuv.v - 128) - 0.463*(double)(yuv.u - 128));
	prgb->g = (BYTE)(1.116*(double)yuv.y - 0.581*(double)(yuv.v - 128) - 0.159*(double)(yuv.u - 128));
	prgb->b = (BYTE)(1.000*(double)yuv.y + 0.000*(double)(yuv.v - 128) + 2.032*(double)(yuv.u - 128));
}
*/



/////////////////////////////////////////////////////////////////////////////        OK
//  HSV (Hue, Saturation, Value) -> RGB (Red, Green, Blue)
/////////////////////////////////////////////////////////////////////////////
inline void HSVtoRGB(_HSV hsv, _RGB* prgb)
{
	if(hsv.h == 360) hsv.h = 0;

	double r,g,b;
	double h = (double)hsv.h;
	double s = (double)hsv.s / 255.0;
	double v = (double)hsv.v / 255.0;//V

	int nHZone = hsv.h / 60;
	double f = (double)hsv.h / 60.0 - (double)nHZone;//Hi
	double p = v * (1.0 - s);//Vmin
	double q = v * (1.0 - f * s);//Vdec
	double t = v * (1.0 - (1.0 - f) * s);//Vinc

	switch(nHZone)
	{
		case 0: r = v; g = t; b = p; break;
		case 1: r = q; g = v; b = p; break;
		case 2: r = p; g = v; b = t; break;
		case 3: r = p; g = q; b = v; break;
		case 4: r = t; g = p; b = v; break;
		case 5: r = v; g = p; b = q; break;
	}

	prgb->r = (BYTE)_round(r * 255.0);
	prgb->g = (BYTE)_round(g * 255.0);
	prgb->b = (BYTE)_round(b * 255.0);
}


/////////////////////////////////////////////////////////////////////////////        OK
//  H (Hue) -> RGB (Red, Green, Blue)
/////////////////////////////////////////////////////////////////////////////
inline void HtoRGB(int H, _RGB* prgb)
{
	_HSV hsv = { H, 255, 255 };
	HSVtoRGB(hsv, prgb);
}


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
inline void RGBtoHSV(_RGB rgb, _HSV* phsv)
{
	BYTE cMax = max3(rgb.r, rgb.g, rgb.b);
	BYTE cMin = min3(rgb.r, rgb.g, rgb.b);
	
	RGBtoH(rgb, &phsv->h);
	phsv->s = 0;
	phsv->v = cMax;

	if(cMax == 0) return;
	phsv->s = 255 - 255 * cMin / cMax;
}


inline void rgb2hsv(_RGB rgb, _HSV* phsv)
{
	double h, s, v;
	double r = (double)rgb.r / 255.0;
	double g = (double)rgb.g / 255.0;
	double b = (double)rgb.b / 255.0;

	double min = min3(r, g, b);
	double max = max3(r, g, b);
	double del = max - min;
	v = max;
	if(del == 0.0)
		h = s = 0.0;
	else
	{
		s = del / max;

		if(max == r)
		{
			h = ((g - b) / del) / 6.0;
			if(g < b) h += 1.0;
		}
		
		if(max == g)
		{
			h = ((b - r) / del + 2.0) / 6.0;
		}

		if(max == b)
		{
			h = ((r - g) / del + 3.0) / 6.0;
		}
/*
		double dr = ((max - r) / 6.0 + del / 2.0) / del;
		double dg = ((max - g) / 6.0 + del / 2.0) / del;
		double db = ((max - b) / 6.0 + del / 2.0) / del;
		if(max == r)
		{
			h = dg - db;
		}
		else
		{
			if(max == g)
			{
				h = 1/3 + dr - db;
			}
			else
			{
				if(max == b)
					h = 2/3 + dg - dr;
			}
		}

		if(h < 0.0)
			h += 1.0;
		if(h > 1)
			h -= 1.0;
*/	}

	phsv->h = (int)_round(h * 360.0);
	phsv->s = (int)_round(s * 255.0);
	phsv->v = (int)_round(v * 255.0);
}




/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
inline void RGBtoHLS(_RGB rgb, _HLS* phls)
{
	BYTE cMax = max3(rgb.r, rgb.g, rgb.b);
	BYTE cMin = min3(rgb.r, rgb.g, rgb.b);
	BYTE mMaxMin = cMax - cMin;
	BYTE sMaxMin = cMax + cMin;
	
	RGBtoH(rgb, &phls->h);
	phls->l = sMaxMin / 2;
	phls->s = 0;


	if(phls->l == 0 || cMax == cMin) return;

	if(phls->l < 128)
		phls->s = 255 * mMaxMin / sMaxMin;
	else
		phls->s = 255 * mMaxMin / (510 - sMaxMin);
}


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
inline void HLStoRGB(_HLS hls, _RGB* prgb)
{
	int i;
	int Q = (hls.l < 128) ? hls.l * (255 + hls.s) : hls.l + hls.s - hls.l * hls.s;
	int P = 510 * hls.l - Q;

	int T[3];

	T[0] = hls.h + 120;
	T[1] = hls.h;
	T[2] = hls.h - 120;

	for(i=0; i<3; i++)
	{
		if(T[i] < 0) T[i] += 360;
		if(T[i] > 360) T[i] -= 360;
	}

	for(i=0; i<3; i++)
	{
		if(T[i] < 60)
			T[i] = P + ((Q - P) * T[i]);
		else
			if(T[i] >= 60 && T[i] < 180)
				T[i] = Q;
			else
				if(T[i] >= 180 && T[i] < 240)
					T[i] = P + ((Q - P) * (240 - T[i]));
				else
					T[i] = P;

		T[i] /= 255 * 6;
	}

	prgb->r = T[0];
	prgb->g = T[1];
	prgb->b = T[2];
}





/////////////////////////////////////////////////////////////////////////////        OK
//  RGB (Red, Green, Blue) -> GRAY (Gray)
/////////////////////////////////////////////////////////////////////////////
inline void RGBtoGRAY(_RGB rgb, _GRAY* pGray)
{
	double r, g, b, gray = 0.0;
	r = (double)rgb.r / 255.0;
	g = (double)rgb.g / 255.0;
	b = (double)rgb.b / 255.0;

	switch(pGray->nType)
	{
		default:
		case 0:
			gray = r * 0.2126 + g * 0.7152 + b * 0.0722;
			break;

		case 1:
			gray = r * 0.299 + g * 0.587 + b * 0.114;
			break;

		case 2:
			gray = (r + g + b) / 3.0;
			break;
	}

	if(pGray->dGamma <= 0.0) pGray->dGamma = 1.0;
	gray = pow(gray, 1 / pGray->dGamma);
	pGray->y = _round((1.0 - gray) * 255.0);
}


/////////////////////////////////////////////////////////////////////////////        OK
//  GRAY (Gray) -> RGB (Red, Green, Blue)
/////////////////////////////////////////////////////////////////////////////
inline void GRAYtoRGB(_GRAY gray, _RGB* prgb)
{
	prgb->r = prgb->g = prgb->b = 255 - gray.y;
}





/*
rgb2hsv:function(rgb)
	{var h,s,v;
	var r=rgb[0]/255;
	var g=rgb[1]/255;
	var b=rgb[2]/255;
	var min=Math.min(r,g,b);
	var max=Math.max(r,g,b);
	var del=max-min;v=max;
	if(del==0)
		h=s=0;
	else
	{
		s=del/max;
		var dr=((max-r)/6+del/2)/del;
		var dg=((max-g)/6+del/2)/del;
		var db=((max-b)/6+del/2)/del;
		if(r==max)
		{
			h=db-dg;
		}
		else
		{
			if(g==max)
			{
				h=(1/3)+dr-db;
			}
			else
			{
				if(b==max)
					h=(2/3)+dg-dr;
			}
		}

		if(h<0)
			h+=1;
		if(h>1)
			h-=1;
	}
	return[h*360,s*100,v*100];
	}
	
	rgb2xyz:function(rgb)
	{
		var r=rgb[0]/255;
		var g=rgb[1]/255;
		var b=rgb[2]/255;
		r=(r>0.04045)?Math.pow((r+0.055)/1.055,2.4):r/12.92;
		g=(g>0.04045)?Math.pow((g+0.055)/1.055,2.4):g/12.92;
		b=(b>0.04045)?Math.pow((b+0.055)/1.055,2.4):b/12.92;
		r*=100;
		g*=100;
		b*=100;
		var x=r*0.4124+g*0.3576+b*0.1805;
		var y=r*0.2126+g*0.7152+b*0.0722;
		var z=r*0.0193+g*0.1192+b*0.9505;
		return[x,y,z];
	}
		
		xyz2lab:function(xyz)
		{
			var x=xyz[0]/95.047;
			var y=xyz[1]/100;
			var z=xyz[2]/108.883;
			x=(x>0.008856)?Math.pow(x,1/3):(7.787*x)+(16/116);
			y=(y>0.008856)?Math.pow(y,1/3):(7.787*y)+(16/116);
			z=(z>0.008856)?Math.pow(z,1/3):(7.787*z)+(16/116);
			var l=116*y-16;
			var a=500*(x-y);
			var b=200*(y-z);
			return[l,a,b];
		}
		


*/



/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
#define CSF_GRAY	0x00000001
#define CSF_RGB		0x00000002
#define CSF_HSV		0x00000004
#define CSF_CMYK	0x00000008
#define CSF_LAB		0x00000040
#define CSF_XYZ		0x00000080
#define CSF_HLS		0x00000010
#define CSF_YUV		0x00000020


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
typedef struct tagColorSpace  // csp
{
	int nColorSpaceFlags;

	_GRAY gray;
	_RGB rgb;
	_HSV hsv;
	_CMYK cmyk;

	_LAB lab;
	_XYZ xyz;
	_HLS hls;
	_YUV yuv;


	void RGBtoSP()
	{
		if(nColorSpaceFlags & CS_GRAY) RGBtoGRAY(rgb, &gray);
		if(nColorSpaceFlags & CS_HSV)  RGBtoHSV(rgb, &hsv);
		if(nColorSpaceFlags & CS_CMYK) RGBtoCMYK(rgb, &cmyk);
	}

} _CSP;






/*///////////////////////////////////////////////////////////////////////////

COLOR_RGB == 0x00BBGGRR
COLOR_CMYK == 0xKKYYMMCC


COLORREF == 0x00BBGGRR

COLORREF RGB(BYTE bRed, BYTE bGreen, BYTE bBlue);
#define RGB(r, g ,b)  ((DWORD) (((BYTE) (r) | ((WORD) (g) << 8)) | (((DWORD) (BYTE) (b)) << 16))) 

BYTE GetBValue(DWORD rgb);
#define GetBValue(rgb)   ((BYTE) ((rgb) >> 16))

BYTE GetGValue(DWORD rgb);
#define GetGValue(rgb)   ((BYTE) (((WORD) (rgb)) >> 8))

BYTE GetRValue(DWORD rgb);
#define GetRValue(rgb)   ((BYTE) (rgb))

COLORREF PALETTEINDEX(WORD wPaletteIndex);
#define PALETTEINDEX(i) ((COLORREF) (0x01000000 | (DWORD) (WORD) (i)))

COLORREF PALETTERGB(BYTE bRed, BYTE bGreen, BYTE bBlue);
#define PALETTERGB(r, g, b)  (0x02000000 | RGB(r, g, b))
   


///////////////////////////////////////////////////////////////////////////*/




////////////////////////////////////////////////////////////////////////////
#endif // __COLORDEFINITION_H__






































