// EIndicatorLCD.h : header file  (10.11.2011)
//

#ifndef __EINDICATORLCD_H__
#define __EINDICATORLCD_H__
#pragma once
//////////////////////////////////////////////////////////////////////////////

#include "..\GDI\FilledShapes\FilledShapesFunc.h"
//////////////////////////////////////////////////////////////////////////////

// 06.08.2012

class EIndicatorLCD  // lcd
{
public:
	void SetText(char* szText);
	void Clear();

	void SetIndCount(int nIndCount);
	void SetColor(COLORREF cr, int nType = 0);
	void SetXY(int x, int y);
	void SetWHDB(int w, int h, int d, int b);
	void UpdateData(HDC hdc);

	EIndicatorLCD();
	virtual ~EIndicatorLCD();

//private:

	char* m_szText;
	int m_nIndCount;
	int m_nIX, m_nIY, m_nIW, m_nIH, m_nBd, m_nDx, m_nW, m_nH;
	COLORREF m_cr[4];
	BOOL m_isError;

	void _init();
	void _free();
	void _calc_wh();

	void _display(HDC hDC, char *cData);
	
	void _drav_cell(HDC hDC, int nCode, int nPoints, int iPos, COLORREF cr);
	void _drav_points(HDC hDC, int nPoints, int iPos, COLORREF cr);

	void _drav_segment(HDC hDC, int nSeg, int Xc, int Yc, int Wc, int Hc, COLORREF cr);
	void _drav_element(HDC hDC, int nType, int x, int y, int cx, int cy, COLORREF cr);

	int _get_length(char* szText);

};



//////////////////////////////////////////////////////////////////////////////
// 
//////////////////////////////////////////////////////////////////////////////
inline EIndicatorLCD::EIndicatorLCD()
{
	_init();
}


//////////////////////////////////////////////////////////////////////////////
// 
//////////////////////////////////////////////////////////////////////////////
inline EIndicatorLCD::~EIndicatorLCD()
{
	_free();
}


//////////////////////////////////////////////////////////////////////////////
// 
//////////////////////////////////////////////////////////////////////////////
inline void EIndicatorLCD::_init()
{
	m_nIndCount = 10;
	m_nIX = m_nIY = 0;
	m_nBd = 8;
	m_nDx = 13;
	m_nW = 17;
	m_nH = 27;
	_calc_wh();
	
	m_cr[0] = RGB(255,0,0);
	m_cr[1] = RGB(84,84,84);
	m_cr[2] = RGB(0,0,0);
	m_isError = FALSE;

	m_szText = new char[48];
	memset(m_szText, '\0', 48);
}


//////////////////////////////////////////////////////////////////////////////
// 
//////////////////////////////////////////////////////////////////////////////
inline void EIndicatorLCD::_calc_wh()
{
	m_nIW = (m_nW + m_nDx) * m_nIndCount + 2 * m_nBd - (m_nDx - 6);
	m_nIH = m_nH + 2 * m_nBd;
}


//////////////////////////////////////////////////////////////////////////////
// 
//////////////////////////////////////////////////////////////////////////////
inline void EIndicatorLCD::_free()
{
	if(m_szText)
	{
		delete [] m_szText;
		m_szText = NULL;
	}
}


//////////////////////////////////////////////////////////////////////////////
// 
//////////////////////////////////////////////////////////////////////////////
inline void EIndicatorLCD::SetText(char* szText)
{
	if(szText)
	{
		int len = strlen(szText);
		Clear();
		memcpy(m_szText, szText, len);
	}
}


//////////////////////////////////////////////////////////////////////////////
// 
//////////////////////////////////////////////////////////////////////////////
inline void EIndicatorLCD::Clear()
{
	if(m_szText)
		memset(m_szText, 0, 48);
}


//////////////////////////////////////////////////////////////////////////////
// 
//////////////////////////////////////////////////////////////////////////////
inline void EIndicatorLCD::SetIndCount(int nIndCount)
{
	if(nIndCount <= 32)
	{
		m_nIndCount = nIndCount;
		m_isError = FALSE;
		return;
	}

	m_nIndCount = 5;
	m_isError = TRUE;
}


//////////////////////////////////////////////////////////////////////////////
// 
//////////////////////////////////////////////////////////////////////////////
inline void EIndicatorLCD::SetColor(COLORREF cr, int nType)
{
	m_cr[nType & 3] = cr;
}


//////////////////////////////////////////////////////////////////////////////
// 
//////////////////////////////////////////////////////////////////////////////
inline void EIndicatorLCD::SetXY(int x, int y)
{
	m_nIX = x;
	m_nIY = y;
}


//////////////////////////////////////////////////////////////////////////////
// 
//////////////////////////////////////////////////////////////////////////////
inline void EIndicatorLCD::SetWHDB(int w, int h, int d, int b)
{
	m_nW = w;
	if(_is_even(m_nW)) m_nW++;
	if(m_nW < 13) m_nW = 13;

	m_nH = h;
	if(_is_even(m_nH)) m_nH++;
	if(m_nH < 23) m_nH = 23;

	m_nDx = d;
	if(_is_even(m_nDx)) m_nDx++;
	if(m_nDx < 9) m_nDx = 9;

	m_nBd = b;
}


//////////////////////////////////////////////////////////////////////////////
// 
//////////////////////////////////////////////////////////////////////////////
inline void EIndicatorLCD::UpdateData(HDC hDC)
{
	if(m_szText)
		_display(hDC,m_szText);
}


//////////////////////////////////////////////////////////////////////////////
// 
// H -  
// # -  
// Q -  
//////////////////////////////////////////////////////////////////////////////
//
//  ...
//                                                            |
//                                                            |
//                                                            | m_nBd
//               c=2                          c=2             |
//            HHHHHHHH                     HHHHHHHH          -+
//           HHHHHHHHHH                   HHHHHHHHHH          |
//          # HHHHHHHH #                 # HHHHHHHH #         |
//         ###        ###               ###        ###        |
//         ###  i=1   ###               ###  i=1   ###        |
//      a=0###   QQ   ###f=5         a=0###   QQ   ###f=5     |
//         ###   QQ   ###               ###   QQ   ###        |
//         ###        ###               ###        ###        |
//         ###  d=3   ###               ###  d=3   ###        |
//          # HHHHHHHH #                 # HHHHHHHH #         |
//           HHHHHHHHHH                   HHHHHHHHHH          | m_nIH
//          # HHHHHHHH #                 # HHHHHHHH #         |
//         ###        ###               ###        ###        |
//         ###  j=2   ###               ###  j=2   ###        |
//      b=1###   QQ   ###g=6         b=1###   QQ   ###g=6     |
//         ###   QQ   ###               ###   QQ   ###        |
//         ###        ###               ###        ###        |
//         ###  e=4   ###   h=0         ###  e=4   ###   h=0  |
//          # HHHHHHHH #    QQQ          # HHHHHHHH #    QQQ  |
//           HHHHHHHHHH     QQQ           HHHHHHHHHH     QQQ  |
//            HHHHHHHH      QQQ            HHHHHHHH      QQQ -+
//         |            |               |
//  -------+------------+---------------+
//   m_nBd     m_nIW          m_nDx
//  
//  ...


//////////////////////////////////////////////////////////////////////////////
// 
//////////////////////////////////////////////////////////////////////////////
inline void EIndicatorLCD::_drav_element(HDC hDC, int nType, int x, int y, int cx, int cy, COLORREF cr)
{
	int x1, y1, cx1, cy1;
	int x2, y2, cx2, cy2;

	//   
	switch(nType)
	{
		//  
		case 0:
			x1 = x + 1; y1 = y; cx1 = cx - 2; cy1 = 3;
			x2 = x; y2 = y + 1; cx2 = cx; cy2 = 1;
			break;

		//  
		case 1:
			x1 = x; y1 = y + 1; cx1 = 3; cy1 = cy - 2;
			x2 = x + 1; y2 = y; cx2 = 1; cy2 = cy;
			break;

		//  
		case 2:
			x1 = x; y1 = y; cx1 = 3; cy1 = 3;
			break;
	}

	_EcxFastColorRect(hDC, x1, y1, cx1, cy1, cr);
	if(nType < 2) _EcxFastColorRect(hDC, x2, y2, cx2, cy2, cr);
}


//////////////////////////////////////////////////////////////////////////////
// 
//////////////////////////////////////////////////////////////////////////////
inline void EIndicatorLCD::_drav_segment(HDC hDC, int nSeg, int Xc, int Yc, int Wc, int Hc, COLORREF cr)
{
	int x, y, cx, cy, nType;
	cx = Wc - 4;
	cy = (Hc - 5) / 2;

	switch(nSeg)
	{
		case 0: nType = 0;  x = 2;       y = 0;           break;
		case 1: nType = 1;  x = cx + 1;  y = 2;           break;
		case 2: nType = 1;  x = cx + 1;  y = cy + 3;      break;
		case 3: nType = 0;  x = 2;       y = 2 * cy + 2;  break;
		case 4: nType = 1;  x = 0;       y = cy + 3;      break;
		case 5: nType = 1;  x = 0;       y = 2;           break;
		case 6: nType = 0;  x = 2;       y = cy + 1;      break;

		case 7: nType = 2;  x = Wc + 3;        y = Hc - 3;                 break;
		case 8: nType = 2;  x = (Wc - 3) / 2;  y = (cy - 3) / 2 + 2;       break;
		case 9: nType = 2;  x = (Wc - 3) / 2;  y = (cy - 3) / 2 + 3 + cy;  break;
	}

	x += Xc; y += Yc;
	_drav_element(hDC, nType, x, y, cx, cy, cr);
}


//////////////////////////////////////////////////////////////////////////////
// 
//////////////////////////////////////////////////////////////////////////////
static BYTE _gn1251[] =
{
//      0    1    2    3    4    5    6    7     8    9    A    B    C    D    E    F
/*00*/  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
/*10*/  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
/*20*/  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
/*30*/  0x3F,0x06,0x5B,0x4F,0x66,0x6D,0x7D,0x07, 0x7F,0x6F,0x00,0x00,0x00,0x00,0x00,0x00,

/*40*/  0x00,0x77,0x7C,0x39,0x5E,0x79,0x71,0x00, 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
/*50*/  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
/*60*/  0x00,0x77,0x7C,0x39,0x5E,0x79,0x71,0x00, 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
/*70*/  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,

/*80*/  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
/*90*/  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
/*A0*/  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
/*B0*/  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,

/*C0*/  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
/*D0*/  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
/*E0*/  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
/*F0*/  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
};


//////////////////////////////////////////////////////////////////////////////
// 
//////////////////////////////////////////////////////////////////////////////
inline void EIndicatorLCD::_drav_cell(HDC hDC, int nCode, int nPoints, int iPos, COLORREF cr)
{
	int offset = m_nW + m_nDx;
	int x = iPos * offset + m_nBd;
//	_EcxFastColorRect(hDC, x, m_nBd, m_nW, m_nH, RGB(180,180,128));
//	_EcxFastColorRect(hDC, x + m_nW, m_nBd, m_nDx, m_nH, RGB(128,180,180));

	int tmp = nCode;

	for(int i=0; i<7; i++)
	{
		if(tmp & 1)
		{
			_drav_segment(hDC, i, x, m_nBd, m_nW, m_nH, cr);
		}
		tmp >>= 1;
	}
	
	if(nCode & 0x80) nPoints |= 1;

	_drav_points(hDC, nPoints, iPos, cr);
}


//////////////////////////////////////////////////////////////////////////////
// 
//////////////////////////////////////////////////////////////////////////////
inline void EIndicatorLCD::_drav_points(HDC hDC, int nPoints, int iPos, COLORREF cr)
{
	int offset = m_nW + m_nDx;
	int x = iPos * offset + m_nBd;

	if(nPoints & 1) _drav_segment(hDC, 7, x, m_nBd, m_nW, m_nH, cr);
	if(nPoints & 2) _drav_segment(hDC, 8, x, m_nBd, m_nW, m_nH, cr);
	if(nPoints & 4) _drav_segment(hDC, 9, x, m_nBd, m_nW, m_nH, cr);
}


//////////////////////////////////////////////////////////////////////////////
// 
//////////////////////////////////////////////////////////////////////////////
inline int EIndicatorLCD::_get_length(char* szText)
{
	int len = strlen(szText);
	if(len == 0) return 0;

	int nLength = 0;
	for(int i=0; i<len; i++)
	{
		if(_EcxIsCharInSet(szText[i], ".,':")) continue;
		if(szText[i] == '@' && is_char(szText[i + 1], CCM_OCT))
		{
			i++;
			continue;
		}
		if(szText[i] == '#' && is_char(szText[i + 1], CCM_HEX) && is_char(szText[i + 2], CCM_HEX))
		{
			i += 2;
		}
		nLength++;
	}

	return nLength;
}


//////////////////////////////////////////////////////////////////////////////
// 
//////////////////////////////////////////////////////////////////////////////
inline void EIndicatorLCD::_display(HDC hDC, char* cData)
{
	int len = strlen(cData);
	if(len == 0) return;

	int nLength = _get_length(cData);
	if(nLength == 0) return;
	if(nLength > m_nIndCount) nLength = m_nIndCount;
	int dx = m_nIndCount - nLength;


	ECanvas cnv(hDC, m_nIW, m_nIH);
	{
		int i;
		_EcxFastColorRect(cnv.m_hDC, 0, 0, m_nIW, m_nIH, m_cr[2]);

		for(i=0; i<m_nIndCount; i++)
		{
			_drav_cell(cnv.m_hDC, 0x7F, 7, i, m_cr[1]);
		}



		BYTE ch0, ch1, ch2;
		int nPoints = 0;
		int nSymNum = 0;
		int pos = 0;

		for(i=0; i<len; i++)
		{
			ch0 = cData[i];
			ch1 = cData[i + 1];
			ch2 = cData[i + 2];

			if(_EcxIsCharInSet(ch0, ".,':")) continue;

			nSymNum = _gn1251[ch0];

			if(ch0 == '#' && is_char(ch1, CCM_HEX) && is_char(ch2, CCM_HEX))
			{
				nSymNum = ((_EcxConvertHex(ch1) - '0') << 4) | (_EcxConvertHex(ch2) - '0');
				i += 2;
			}

			ch1 = cData[i + 1];
			ch2 = cData[i + 2];

			nPoints = 0;
			if(ch1 == 0x2E) nPoints += 1; // .
			if(ch1 == 0x27) nPoints += 2; // '
			if(ch1 == 0x2C) nPoints += 4; // ,
			if(ch1 == 0x3A) nPoints += 6; // :
			if(ch1 == '@' && is_char(ch2, CCM_OCT))
			{
				nPoints = ch2 - '0';
				i += 2;
			}
						
			_drav_cell(cnv.m_hDC, nSymNum, nPoints, pos + dx, m_cr[0]);
			pos++;
		}

	}
	cnv.Draw(m_nIX, m_nIY);

}






//////////////////////////////////////////////////////////////////////////////
#endif // __EINDICATORLCD_H__

