// rectx.h : header file
//

#ifndef __RECTX_H__
#define __RECTX_H__
#pragma once
////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////
//  rectx.h
/////////////////////////////////////////////////////////////////////////////
#define VERSION_RECTX   "06.02.2013"




////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////
template <class TYPE>
struct POINTX
{
	TYPE x;
	TYPE y;
};

////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////
template <class TYPE>
struct SIZEX
{
	TYPE cx;
	TYPE cy;

};

////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////
template <class TYPE>
struct RECTX
{
	TYPE left;
	TYPE top;
	TYPE right;
	TYPE bottom;
};


////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////
typedef POINTX<__int64> POINT64;
typedef POINTX<double>  POINTD;

typedef SIZEX<__int64> SIZE64;
typedef SIZEX<double>  SIZED;

typedef RECTX<__int64> RECT64;
typedef RECTX<double>  RECTD;


/////////////////////////////////////////////////////////////////////////////
//   
/////////////////////////////////////////////////////////////////////////////
template <class TYPE> BOOL _EcxCopyRect      (RECTX<TYPE>* rc, RECTX<TYPE>* rc1);
template <class TYPE> BOOL _EcxEqualRect     (RECTX<TYPE>* rc1, RECTX<TYPE>* rc2);
template <class TYPE> BOOL _EcxInflateRect   (RECTX<TYPE>* rc, TYPE dx, TYPE dy);
template <class TYPE> BOOL _EcxIntersectRect (RECTX<TYPE>* rc, RECTX<TYPE>* rc1, RECTX<TYPE>* rc2);
template <class TYPE> BOOL _EcxIsRectEmpty   (RECTX<TYPE>* rc);
template <class TYPE> BOOL _EcxOffsetRect    (RECTX<TYPE>* rc, TYPE dx, TYPE dy);
template <class TYPE> BOOL _EcxPtInRect      (RECTX<TYPE>* rc, POINTX<TYPE> pt);
template <class TYPE> BOOL _EcxSetRect       (RECTX<TYPE>* rc, TYPE left, TYPE top, TYPE right, TYPE bottom);
template <class TYPE> BOOL _EcxSetRect       (RECTX<TYPE>* rc, POINTX<TYPE> pt, SIZEX<TYPE> siz);
template <class TYPE> BOOL _EcxSetRectEmpty  (RECTX<TYPE>* rc);
template <class TYPE> BOOL _EcxSubtractRect  (RECTX<TYPE>* rc, RECTX<TYPE>* rc1, RECTX<TYPE>* rc2);
template <class TYPE> BOOL _EcxUnionRect     (RECTX<TYPE>* rc, RECTX<TYPE>* rc1, RECTX<TYPE>* rc2);
template <class TYPE> BOOL _EcxToRect        (RECT* rc, RECTX<TYPE>* rc1);


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
template <class TYPE>
inline BOOL _EcxCopyRect(RECTX<TYPE>* rc, RECTX<TYPE>* rc1)
{
	if(rc == NULL || rc1 == NULL) return FALSE;
	*rc = *rc1;
	return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
template <class TYPE>
inline BOOL _EcxEqualRect(RECTX<TYPE>* rc1, RECTX<TYPE>* rc2)
{
	if(rc1 == NULL || rc2 == NULL) return FALSE;
	return (rc1->left == rc2->left) && (rc1->top == rc2->top) &&
		(rc1->right == rc2->right) && (rc1->bottom == rc2->bottom);
}


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
template <class TYPE>
inline BOOL _EcxInflateRect(RECTX<TYPE>* rc, TYPE dx, TYPE dy) 
{
	rc->left -= dx;
	rc->top -= dy;
	rc->right += dx;
	rc->bottom += dy;
	return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
template <class TYPE>
inline BOOL _EcxIntersectRect(RECTX<TYPE>* rc, RECTX<TYPE>* rc1, RECTX<TYPE>* rc2)
{
	if(_EcxIsRectEmpty(rc1) || _EcxIsRectEmpty(rc2) ||
		rc1->left >= rc2->right || rc2->left >= rc1->right ||
		rc1->top >= rc2->bottom || rc2->top >= rc1->bottom)
	{
		_EcxSetRectEmpty(rc);
		return FALSE;
	}
	rc->left = max(rc1->left, rc2->left);
	rc->right = min(rc1->right, rc2->right);
	rc->top = max(rc1->top, rc2->top);
	rc->bottom = min(rc1->bottom, rc2->bottom);
	return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
template <class TYPE>
inline BOOL _EcxIsRectEmpty(RECTX<TYPE>* rc)
{
	return (rc->left >= rc->right) || (rc->top >= rc->bottom);
}


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
template <class TYPE>
inline BOOL _EcxOffsetRect(RECTX<TYPE>* rc, TYPE dx, TYPE dy)
{
	if(rc == NULL) return FALSE;
	rc->left += dx;
	rc->top += dy;
	rc->right += dx;
	rc->bottom += dy;
	return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
template <class TYPE>
inline BOOL _EcxPtInRect(RECTX<TYPE>* rc, POINTX<TYPE> pt)
{
	return((pt.x >= rc->left) && (pt.x < rc->right) &&
	(pt.y >= rc->top) && (pt.y < rc->bottom));
}


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
template <class TYPE>
inline BOOL _EcxSetRect(RECTX<TYPE>* rc, TYPE left, TYPE top, TYPE right, TYPE bottom)
{
	rc->left = left;
	rc->top = top;
	rc->right = right;
	rc->bottom = bottom;
	return TRUE;
}

template <class TYPE>
BOOL _EcxSetRect(RECTX<TYPE>* rc, POINTX<TYPE> pt, SIZEX<TYPE> siz)
{
	rc->left = pt.x;
	rc->top = pt.y;
	rc->right = pt.x + siz.cx;
	rc->bottom = pt.y + siz.cy;
	return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
template <class TYPE>
inline BOOL _EcxSetRectEmpty(RECTX<TYPE>* rc)
{
	rc->left = rc->right = rc->top = rc->bottom = 0;
	return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
template <class TYPE>
inline BOOL _EcxSubtractRect(RECTX<TYPE>* rc, RECTX<TYPE>* rc1, RECTX<TYPE>* rc2)
{
	RECTX<TYPE> tmp;
	if(rc == NULL || rc1 == NULL || rc2 == NULL) return FALSE;
	_EcxCopyRect(rc, rc1);
	if(!_EcxIntersectRect(&tmp, rc1, rc2)) return TRUE;
	if(_EcxEqualRect(&tmp, rc))
	{
		_EcxSetRectEmpty(rc);
		return FALSE;
	}
	if(rc->top == tmp.top && rc->bottom == tmp.bottom)
	{
		if(rc->left == tmp.left)
			rc->left = tmp.right;
		else
			if(rc->right == tmp.right)
				rc->right = tmp.left;
	}
	else
		if(rc->left == tmp.left && rc->right == tmp.right)
		{
			if(rc->top == tmp.top)
				rc->top = tmp.bottom;
			else
				if(rc->right == tmp.right)
					rc->right = tmp.left;
		}
	return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
template <class TYPE>
inline BOOL _EcxUnionRect(RECTX<TYPE>* rc, RECTX<TYPE>* rc1, RECTX<TYPE>* rc2)
{
	if(_EcxIsRectEmpty(rc1))
	{
		if(_EcxIsRectEmpty(rc2))
		{
			_EcxSetRectEmpty(rc);
			return FALSE;
		}
		else
		{
			*rc = *rc2;
		}
	}
	else
	{
		if(_EcxIsRectEmpty(rc2))
		{
			*rc = *rc1;
		}
		else
		{
			rc->left = min(rc1->left, rc2->left);
			rc->top = min(rc1->top, rc2->top);
			rc->right = max(rc1->right, rc2->right);
			rc->bottom = max(rc1->bottom, rc2->bottom);
		}
	}
	return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
template <class TYPE>
BOOL _EcxToRect(RECT* rc, RECTX<TYPE>* rc1)
{
	if(rc == NULL || rc1 == NULL) return FALSE;
	rc->left = (long)rc1->left;
	rc->top = (long)rc1->top;
	rc->right = (long)rc1->right;
	rc->bottom = (long)rc1->bottom;
	return TRUE;
}













/////////////////////////////////////////////////////////////////////////////
#endif // __RECTX_H__