// Internet.h : header file
//

#ifndef __INTERNET_H__
#define __INTERNET_H__
#pragma once
////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////
//  Internet.h
/////////////////////////////////////////////////////////////////////////////
#define VERSION_INTERNET   "08.12.2012"


/////////////////////////////////////////////////////////////////////////////
#define HTTP_QUERY_EXPECT					68
#define HTTP_QUERY_PROXY_CONNECTION			69
// 70
#define HTTP_QUERY_UNLESS_MODIFIED_SINCE	70
#define HTTP_QUERY_ECHO_REQUEST				71
#define HTTP_QUERY_ECHO_REPLY				72
#define HTTP_QUERY_ECHO_HEADERS				73
#define HTTP_QUERY_ECHO_HEADERS_CRLF		74
#define HTTP_QUERY_PROXY_SUPPORT			75
#define HTTP_QUERY_AUTHENTICATION_INFO		76
#define HTTP_QUERY_PASSPORT_URLS			77
#define HTTP_QUERY_PASSPORT_CONFIG			78
#define HTTP_QUERY_X_CONTENT_TYPE_OPTIONS	79
// 80
#define HTTP_QUERY_P3P						80
#define HTTP_QUERY_X_P2P_PEERDIST			81
#define HTTP_QUERY_TRANSLATE				82
#define HTTP_QUERY_X_UA_COMPATIBLE			83
#define HTTP_QUERY_DEFAULT_STYLE			84
#define HTTP_QUERY_X_FRAME_OPTIONS			85
#define HTTP_QUERY_X_XSS_PROTECTION			86


////////////////////////////////////////////////////////////////////////////
//    dwInfoLevel  HttpQueryInfo
////////////////////////////////////////////////////////////////////////////
#ifdef USED_HTTPQUERYINFO_NAMES

	static char* g_szHttpQueryInfoNames[][2] =
	{
		// 00
		"HTTP_QUERY_MIME_VERSION",              "Receives the version of the MIME protocol that was used to construct the message.",
		"HTTP_QUERY_CONTENT_TYPE",              "Receives the content type of the resource (such as text/html).",
		"HTTP_QUERY_CONTENT_TRANSFER_ENCODING", "Receives the additional content coding that has been applied to the resource.",
		"HTTP_QUERY_CONTENT_ID",                "Retrieves the content identification.",
		"HTTP_QUERY_CONTENT_DESCRIPTION",       "Obsolete. Maintained for legacy application compatibility only.",
		"HTTP_QUERY_CONTENT_LENGTH",            "Retrieves the size of the resource, in bytes.",
		"HTTP_QUERY_CONTENT_LANGUAGE",          "Retrieves the language that the content is in.",
		"HTTP_QUERY_ALLOW",                     "Receives the HTTP verbs supported by the server.",
		"HTTP_QUERY_PUBLIC",                    "Receives methods available at this server.",
		"HTTP_QUERY_DATE",                      "Receives the date and time at which the message was originated.",
		// 10
		"HTTP_QUERY_EXPIRES",                   "Receives the date and time after which the resource should be considered outdated.",
		"HTTP_QUERY_LAST_MODIFIED",             "Receives the date and time at which the server believes the resource was last modified.",
		"HTTP_QUERY_MESSAGE_ID",                "No longer supported.",
		"HTTP_QUERY_UR",                        "Receives some or all of the Uniform Resource Identifiers (URIs) by which the Request-URI resource can be identified.",
		"HTTP_QUERY_DERIVED_FROM",              "No longer supported.",
		"HTTP_QUERY_COST",                      "No longer supported.",
		"HTTP_QUERY_LINK",                      "Obsolete. Maintained for legacy application compatibility only.",
		"HTTP_QUERY_PRAGMA",                    "Receives the implementation-specific directives that might apply to any recipient along the request/response chain.",
		"HTTP_QUERY_VERSION",                   "Receives the last response code returned by the server.",
		"HTTP_QUERY_STATUS_CODE",               "Receives the status code returned by the server. For more information and a list of possible values, see HTTP Status Codes.",
		// 20
		"HTTP_QUERY_STATUS_TEXT",               "Receives any additional text returned by the server on the response line.",
		"HTTP_QUERY_RAW_HEADERS",               "Receives all the headers returned by the server. Each header is terminated by \"\\0\". An additional \"\\0\" terminates the list of headers.",
		"HTTP_QUERY_RAW_HEADERS_CRLF",          "Receives all the headers returned by the server. Each header is separated by a carriage return/line feed (CR/LF) sequence.",
		"HTTP_QUERY_CONNECTION",                "Retrieves any options that are specified for a particular connection and must not be communicated by proxies over further connections.",
		"HTTP_QUERY_ACCEPT",                    "Retrieves the acceptable media types for the response.",
		"HTTP_QUERY_ACCEPT_CHARSET",            "Retrieves the acceptable character sets for the response.",
		"HTTP_QUERY_ACCEPT_ENCODING",           "Retrieves the acceptable content-coding values for the response.",
		"HTTP_QUERY_ACCEPT_LANGUAGE",           "Retrieves the acceptable natural languages for the response.",
		"HTTP_QUERY_AUTHORIZATION",             "Retrieves the authorization credentials used for a request.",
		"HTTP_QUERY_CONTENT_ENCODING",          "Retrieves any additional content codings that have been applied to the entire resource.",
		// 30
		"HTTP_QUERY_FORWARDED",                 "Obsolete. Maintained for legacy application compatibility only.",
		"HTTP_QUERY_FROM",                      "Retrieves the email address for the human user who controls the requesting user agent if the From header is given.",
		"HTTP_QUERY_IF_MODIFIED_SINCE",         "Retrieves the contents of the If-Modified-Since header.",
		"HTTP_QUERY_LOCATION",                  "Retrieves the absolute Uniform Resource Identifier (URI) used in a Location response-header.",
		"HTTP_QUERY_ORIG_URI",                  "Obsolete. Maintained for legacy application compatibility only.",
		"HTTP_QUERY_REFERER",                   "Receives the Uniform Resource Identifier (URI) of the resource where the requested URI was obtained.",
		"HTTP_QUERY_RETRY_AFTER",               "Retrieves the amount of time the service is expected to be unavailable.",
		"HTTP_QUERY_SERVER",                    "Retrieves data about the software used by the origin server to handle the request.",
		"HTTP_QUERY_TITLE",                     "Obsolete. Maintained for legacy application compatibility only.",
		"HTTP_QUERY_USER_AGENT",                "Retrieves data about the user agent that made the request.",
		// 40
		"HTTP_QUERY_WWW_AUTHENTICATE",          "Retrieves the authentication scheme and realm returned by the server.",
		"HTTP_QUERY_PROXY_AUTHENTICATE",        "Retrieves the authentication scheme and realm returned by the proxy.",
		"HTTP_QUERY_ACCEPT_RANGES",             "Retrieves the types of range requests that are accepted for a resource.",
		"HTTP_QUERY_SET_COOKIE",                "Receives the value of the cookie set for the request.",
		"HTTP_QUERY_COOKIE",                    "Retrieves any cookies associated with the request.",
		"HTTP_QUERY_REQUEST_METHOD",            "Receives the HTTP verb that is being used in the request, typically GET or POST.",
		"HTTP_QUERY_REFRESH",                   "Obsolete. Maintained for legacy application compatibility only.",
		"HTTP_QUERY_CONTENT_DISPOSITION",       "Obsolete. Maintained for legacy application compatibility only.",
		"HTTP_QUERY_AGE",                       "Retrieves the Age response-header field, which contains the sender's estimate of the amount of time since the response was generated at the origin server.",
		"HTTP_QUERY_CACHE_CONTROL",             "Retrieves the cache control directives.",
		// 50
		"HTTP_QUERY_CONTENT_BASE",              "Retrieves the base URI (Uniform Resource Identifier) for resolving relative URLs within the entity.",
		"HTTP_QUERY_CONTENT_LOCATION",          "Retrieves the resource location for the entity enclosed in the message.",
		"HTTP_QUERY_CONTENT_MD5",               "Retrieves an MD5 digest of the entity-body for the purpose of providing an end-to-end message integrity check (MIC) for the entity-body (RFC1864).",
		"HTTP_QUERY_CONTENT_RANGE",             "Retrieves the location in the full entity-body where the partial entity-body should be inserted and the total size of the full entity-body.",
		"HTTP_QUERY_ETAG",                      "Retrieves the entity tag for the associated entity.",
		"HTTP_QUERY_HOST",                      "Retrieves the Internet host and port number of the resource being requested.",
		"HTTP_QUERY_IF_MATCH",                  "Retrieves the contents of the If-Match request-header field.",
		"HTTP_QUERY_IF_NONE_MATCH",             "Retrieves the contents of the If-None-Match request-header field.",
		"HTTP_QUERY_IF_RANGE",                  "Retrieves the contents of the If-Range request-header field. This header enables the client application to verify that the entity related to a partial copy of the entity in the client application cache has not been updated. If the entity has not been updated, send the parts that the client application is missing. If the entity has been updated, send the entire updated entity.",
		"HTTP_QUERY_IF_UNMODIFIED_SINCE",       "Retrieves the contents of the If-Unmodified-Since request-header field.",
		// 60
		"HTTP_QUERY_MAX_FORWARDS",              "Retrieves the number of proxies or gateways that can forward the request to the next inbound server.",
		"HTTP_QUERY_PROXY_AUTHORIZATION",       "Retrieves the header that is used to identify the user to a proxy that requires authentication. This header can only be retrieved before the request is sent to the server.",
		"HTTP_QUERY_RANGE",                     "Retrieves the byte range of an entity.",
		"HTTP_QUERY_TRANSFER_ENCODING",         "Retrieves the type of transformation that has been applied to the message body so it can be safely transferred between the sender and recipient.",
		"HTTP_QUERY_UPGRADE",                   "Retrieves the additional communication protocols that are supported by the server.",
		"HTTP_QUERY_VARY",                      "Retrieves the header that indicates that the entity was selected from a number of available representations of the response using server-driven negotiation.",
		"HTTP_QUERY_VIA",                       "Retrieves the intermediate protocols and recipients between the user agent and the server on requests, and between the origin server and the client on responses.",
		"HTTP_QUERY_WARNING",                   "Retrieves additional data about the status of a response that might not be reflected by the response status code.",
		"HTTP_QUERY_EXPECT",                    "Retrieves the Expect header, which indicates whether the client application should expect 100 series responses.",
		"HTTP_QUERY_PROXY_CONNECTION",          "Retrieves the Proxy-Connection header.",
		// 70
		"HTTP_QUERY_UNLESS_MODIFIED_SINCE",     "Retrieves the Unless-Modified-Since header.",
		"HTTP_QUERY_ECHO_REQUEST",              "Not currently implemented.",
		"HTTP_QUERY_ECHO_REPLY",                "Not currently implemented.",
		"HTTP_QUERY_ECHO_HEADERS",              "Not currently implemented.",
		"HTTP_QUERY_ECHO_HEADERS_CRLF",         "Not currently implemented.",
		"HTTP_QUERY_PROXY_SUPPORT",             "",
		"HTTP_QUERY_AUTHENTICATION_INFO",       "",
		"HTTP_QUERY_PASSPORT_URLS",             "",
		"HTTP_QUERY_PASSPORT_CONFIG",           "",
		"HTTP_QUERY_X_CONTENT_TYPE_OPTIONS",    "Retrieves the X-Content-Type-Options header value.",
		// 80
		"HTTP_QUERY_P3P",                       "Retrieves the P3P header value.",
		"HTTP_QUERY_X_P2P_PEERDIST",            "Retrieves the X-P2P-PeerDist header value.",
		"HTTP_QUERY_TRANSLATE",                 "Retrieves the translate header value.",
		"HTTP_QUERY_X_UA_COMPATIBLE",           "Retrieves the X-UA-Compatible header value.",
		"HTTP_QUERY_DEFAULT_STYLE",             "Retrieves the Default-Style header value.",
		"HTTP_QUERY_X_FRAME_OPTIONS",           "Retrieves the X-Frame-Options header value.",
		"HTTP_QUERY_X_XSS_PROTECTION",          "Retrieves the X-XSS-Protection header value.",
	};

#endif // USED_HTTPQUERYINFO_NAMES














////////////////////////////////////////////////////////////////////////////
#endif // __INTERNET_H__
