// TreeViewControlsFunc.h : header file
//

#ifndef __TREEVIEWCONTROLSFUNC_H__
#define __TREEVIEWCONTROLSFUNC_H__
#pragma once
////////////////////////////////////////////////////////////////////////////


#include "commctrl.h"


/////////////////////////////////////////////////////////////////////////////
//  TreeViewControlsFunc.h
/////////////////////////////////////////////////////////////////////////////
#define VERSION_TREEVIEWCONTROLSFUNC   "09.10.2012"



////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline BOOL _EcxTreeView_GetCheck(HWND hTreeCtrl, HTREEITEM hItem)
{
	TVITEM item;
	item.mask = TVIF_HANDLE | TVIF_STATE;
	item.hItem = hItem;
	item.stateMask = TVIS_STATEIMAGEMASK;
	::SendMessage(hTreeCtrl, TVM_GETITEM, 0, (LPARAM)&item);
	// Return zero if it's not checked, or nonzero otherwise.
	return ((BOOL)(item.state >> 12) -1);
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline DWORD _EcxTreeView_GetItemData(HWND hTreeCtrl, HTREEITEM hItem)
{
	TVITEM item;
	item.hItem = hItem;
	item.mask = TVIF_PARAM;
	::SendMessage(hTreeCtrl, TVM_GETITEM, 0, (LPARAM)&item);
	return (DWORD)item.lParam;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline BOOL _EcxTreeView_GetItemImage(HWND hTreeCtrl, HTREEITEM hItem, int& nImage, int& nSelectedImage)
{
	TVITEM item;
	item.hItem = hItem;
	item.mask = TVIF_IMAGE|TVIF_SELECTEDIMAGE;
	BOOL bRes = (BOOL)::SendMessage(hTreeCtrl, TVM_GETITEM, 0, (LPARAM)&item);
	if(bRes)
	{
		nImage = item.iImage;
		nSelectedImage = item.iSelectedImage;
	}
	return bRes;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline HTREEITEM _EcxTreeView_InsertItem(HWND hTreeCtrl, UINT nMask, LPCTSTR lpszItem, int nImage, int nSelectedImage, UINT nState, UINT nStateMask, LPARAM lParam, HTREEITEM hParent, HTREEITEM hInsertAfter)
{
	TVINSERTSTRUCT tvis;
	tvis.hParent = hParent;
	tvis.hInsertAfter = hInsertAfter;
	tvis.item.mask = nMask;
	tvis.item.pszText = (LPTSTR)lpszItem;
	tvis.item.iImage = nImage;
	tvis.item.iSelectedImage = nSelectedImage;
	tvis.item.state = nState;
	tvis.item.stateMask = nStateMask;
	tvis.item.lParam = lParam;
	return (HTREEITEM)::SendMessage(hTreeCtrl, TVM_INSERTITEM, 0, (LPARAM)&tvis);
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline HTREEITEM _EcxTreeView_InsertItem(HWND hTreeCtrl, LPCTSTR lpszItem, HTREEITEM hParent = TVI_ROOT, HTREEITEM hInsertAfter = TVI_LAST)
	{ return _EcxTreeView_InsertItem(hTreeCtrl, TVIF_TEXT, lpszItem, 0, 0, 0, 0, 0, hParent, hInsertAfter); }

inline HTREEITEM _EcxTreeView_InsertItem(HWND hTreeCtrl, LPCTSTR lpszItem, int nImage, int nSelectedImage, HTREEITEM hParent = TVI_ROOT, HTREEITEM hInsertAfter = TVI_LAST)
	{ return _EcxTreeView_InsertItem(hTreeCtrl, TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE, lpszItem, nImage, nSelectedImage, 0, 0, 0, hParent, hInsertAfter); }



////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline BOOL _EcxTreeView_SetItem(HWND hTreeCtrl, HTREEITEM hItem, UINT nMask, LPCTSTR lpszItem, int nImage, int nSelectedImage, UINT nState, UINT nStateMask, LPARAM lParam)
{
	TVITEM item;
	item.hItem = hItem;
	item.mask = nMask;
	item.pszText = (LPTSTR) lpszItem;
	item.iImage = nImage;
	item.iSelectedImage = nSelectedImage;
	item.state = nState;
	item.stateMask = nStateMask;
	item.lParam = lParam;
	return (BOOL)::SendMessage(hTreeCtrl, TVM_SETITEM, 0, (LPARAM)&item);
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline BOOL _EcxTreeView_SetItemData(HWND hTreeCtrl, HTREEITEM hItem, DWORD dwData)
	{ return _EcxTreeView_SetItem(hTreeCtrl, hItem, TVIF_PARAM, NULL, 0, 0, 0, 0, (LPARAM)dwData); }

inline BOOL _EcxTreeView_SetItemImage(HWND hTreeCtrl, HTREEITEM hItem, int nImage, int nSelectedImage)
	{ return _EcxTreeView_SetItem(hTreeCtrl, hItem, TVIF_IMAGE|TVIF_SELECTEDIMAGE, NULL, nImage, nSelectedImage, 0, 0, NULL); }

inline BOOL _EcxTreeView_SetItemState(HWND hTreeCtrl, HTREEITEM hItem, UINT nState, UINT nStateMask)
	{ return _EcxTreeView_SetItem(hTreeCtrl, hItem, TVIF_STATE, NULL, 0, 0, nState, nStateMask, NULL); }

inline BOOL _EcxTreeView_SetItemText(HWND hTreeCtrl, HTREEITEM hItem, LPCTSTR lpszItem)
	{ return _EcxTreeView_SetItem(hTreeCtrl, hItem, TVIF_TEXT, lpszItem, 0, 0, 0, 0, NULL); }


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline BOOL _EcxTreeView_ItemHasChildren(HWND hTreeCtrl, HTREEITEM hItem)
{
	TVITEM item;
	item.hItem = hItem;
	item.mask = TVIF_CHILDREN;
	::SendMessage(hTreeCtrl, TVM_GETITEM, 0, (LPARAM)&item);
	return item.cChildren;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline UINT _EcxTreeView_GetItemState(HWND hTreeCtrl, HTREEITEM hItem, UINT nStateMask)
{
	TVITEM item;
	item.hItem = hItem;
	item.mask = TVIF_STATE;
	item.stateMask = nStateMask;
	item.state = 0;
	::SendMessage(hTreeCtrl, TVM_GETITEM, 0, (LPARAM)&item);
	return item.state;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline EString _EcxTreeView_GetItemText(HWND hTreeCtrl, HTREEITEM hItem)
{
	TVITEM item;
	item.hItem = hItem;
	item.mask = TVIF_TEXT;
	EString str;
	int nLen = 128;
	int nRes;
	do
	{
		nLen *= 2;
		item.pszText = (char*)str.ToBuffer(nLen);
		item.cchTextMax = nLen;
		::SendMessage(hTreeCtrl, TVM_GETITEM, 0, (LPARAM)&item);
		nRes = lstrlen(item.pszText);
	} while (nRes == nLen - 1);
	str.Release(nRes);
	return str;
}

/*

+BOOL GetCheck( HTREEITEM hItem ) const;
+DWORD GetItemData( HTREEITEM hItem ) const;
+BOOL GetItemImage( HTREEITEM hItem, int& nImage, int& nSelectedImage ) const;
+UINT GetItemState( HTREEITEM hItem, UINT nStateMask ) const;
+CString GetItemText( HTREEITEM hItem ) const;
=HTREEITEM HitTest( CPoint pt, UINT* pFlags );
+HTREEITEM InsertItem(UINT nMask, LPCTSTR lpszItem, int nImage, int nSelectedImage, UINT nState, UINT nStateMask, LPARAM lParam, HTREEITEM hParent, HTREEITEM hInsertAfter );
+HTREEITEM InsertItem( LPCTSTR lpszItem, HTREEITEM hParent = TVI_ROOT, HTREEITEM hInsertAfter = TVI_LAST );
+HTREEITEM InsertItem( LPCTSTR lpszItem, int nImage, int nSelectedImage, HTREEITEM hParent = TVI_ROOT, HTREEITEM hInsertAfter = TVI_LAST);
+BOOL ItemHasChildren( HTREEITEM hItem );
=BOOL SetCheck( HTREEITEM hItem, BOOL fCheck = TRUE );
+BOOL SetItem( HTREEITEM hItem, UINT nMask, LPCTSTR lpszItem, int nImage, int nSelectedImage, UINT nState, UINT nStateMask, LPARAM lParam );
+BOOL SetItemData( HTREEITEM hItem, DWORD dwData );
+BOOL SetItemImage( HTREEITEM hItem, int nImage, int nSelectedImage );
+BOOL SetItemState( HTREEITEM hItem, UINT nState, UINT nStateMask );
+BOOL SetItemText( HTREEITEM hItem, LPCTSTR lpszItem );

HIMAGELIST TreeView_CreateDragImage(HWND hwndTV, HTREEITEM hitem);
BOOL TreeView_DeleteAllItems(HWND hwndTV);
BOOL TreeView_DeleteItem(HWND hwndTV, HTREEITEM hitem);
HWND TreeView_EditLabel(HWND hwndTV, HTREEITEM hitem);
BOOL TreeView_EndEditLabelNow(HWND hwndTV, BOOL fCancel);
BOOL TreeView_EnsureVisible(HWND hwndTV, HTREEITEM hitem);
BOOL TreeView_Expand(HWND hwndTV, HTREEITEM hItem, UINT flag);
COLORREF TreeView_GetBkColor(HWND hwndTV);
HTREEITEM TreeView_GetChild(HWND hwndTV, HTREEITEM hitem);
UINT TreeView_GetCount(HWND hwndTV);
HTREEITEM TreeView_GetDropHilight(HWND hwndTV);
HWND TreeView_GetEditControl(HWND hwndTV);
HTREEITEM TreeView_GetFirstVisible(HWND hwndTV);
HIMAGELIST TreeView_GetImageList(HWND hwndTV, INT iImage);
UINT TreeView_GetIndent(HWND hwndTV);
COLORREF TreeView_GetInsertMarkColor(HWND hwndTV);
BOOL TreeView_GetISearchString(HWND hwndTV, LPTSTR lpsz);
BOOL TreeView_GetItem(HWND hwndTV, LPTVITEM pitem);
int TreeView_GetItemHeight(HWND hwndTV);
BOOL TreeView_GetItemRect(HWND hwndTV, HTREEITEM hitem, LPRECT prc, BOOL fItemRect);
HTREEITEM TreeView_GetLastVisible(HWND hwndTV);
HTREEITEM TreeView_GetNextItem(HWND hwndTV, HTREEITEM hitem, UINT flag);
HTREEITEM TreeView_GetNextSibling(HWND hwndTV, HTREEITEM hitem);
HTREEITEM TreeView_GetNextVisible(HWND hwndTV, HTREEITEM hitem);
HTREEITEM TreeView_GetParent(HWND hwndTV, HTREEITEM hitem);
HTREEITEM TreeView_GetPrevSibling(HWND hwndTV, HTREEITEM hitem);
HTREEITEM TreeView_GetPrevVisible(HWND hwndTV, HTREEITEM hitem);
HTREEITEM TreeView_GetRoot(HWND hwndTV);
UINT TreeView_GetScrollTime(HWND hwndTV);
HTREEITEM TreeView_GetSelection(HWND hwndTV);
COLORREF TreeView_GetTextColor(HWND hwndTV);
HWND TreeView_GetToolTips(HWND hwndTV);
BOOL TreeView_GetUnicodeFormat(HWND hwnd);
UINT TreeView_GetVisibleCount(HWND hwndTV);
HTREEITEM TreeView_HitTest(HWND hwndTV, LPTVHITTESTINFO lpht);
HTREEITEM TreeView_InsertItem(HWND hwndTV, LPTVINSERTSTRUCT lpis);
BOOL TreeView_Select(HWND hwndTV, HTREEITEM hitem, UINT flag);
BOOL TreeView_SelectDropTarget(HWND hwndTV, HTREEITEM hitem);
BOOL TreeView_SelectItem(HWND hwndTV, HTREEITEM hitem);
COLORREF TreeView_SetBkColor(HWND hwndTV, COLORREF clrBk);
BOOL TreeView_SelectSetFirstVisible(HWND hwndTV, HTREEITEM hitem);
HIMAGELIST TreeView_SetImageList(HWND hwndTV, HIMAGELIST himl, INT iImage);
BOOL TreeView_SetIndent(HWND hwndTV, INT indent);
BOOL TreeView_SetInsertMark(HWND hwndTV, HTREEITEM htiInsert, BOOL fAfter);
COLORREF TreeView_SetInsertMarkColor(HWND hwndTV, COLORREF clrInsertMark);
BOOL TreeView_SetItem(HWND hwndTV, LPTVITEM pitem);
int TreeView_SetItemHeight(HWND hwndTV, SHORT cyItem);
UINT TreeView_SetScrollTime(HWND hwndTV, UINT uMaxScrollTime);
COLORREF TreeView_SetTextColor(HWND hwndTV, COLORREF clrText);
HWND TreeView_SetToolTips(HWND hwndTV, HWND hwndTooltip);
BOOL TreeView_SetUnicodeFormat(HWND hwnd, BOOL fUnicode);
BOOL TreeView_SortChildren(HWND hwndTV, HTREEITEM hitem, BOOL fRecurse);
BOOL TreeView_SortChildrenCB(HWND hwndTV, LPTVSORTCB psort, BOOL fRecurse);

*/








////////////////////////////////////////////////////////////////////////////
#endif // __TREEVIEWCONTROLSFUNC_H__
