// ShellFunc.h : header file
//

#ifndef __SHELLFUNC_H__
#define __SHELLFUNC_H__
#pragma once
////////////////////////////////////////////////////////////////////////////

#include "shell.h"


/////////////////////////////////////////////////////////////////////////////
//  ShellFunc.h
/////////////////////////////////////////////////////////////////////////////
#define VERSION_SHELLFUNC   "09.03.2011"



///////////////////////////////////////////////////////////////////////////
// Value specifying the folder for which to retrieve the location
// Shellcc.chm::/inet401/help/itt/Shell/Functions/SHGetSpecialFolderLocation.htm
///////////////////////////////////////////////////////////////////////////
#ifdef USED_SPECIALFOLDERLOCATIONINFO

	static char* g_arrSpecialFolderLocationInfo[][3] =
	{
		{ (char*)CSIDL_ALTSTARTUP,              "CSIDL_ALTSTARTUP",              "File system directory that corresponds to the user's nonlocalized Startup program group." },
		{ (char*)CSIDL_APPDATA,                 "CSIDL_APPDATA",                 "File system directory that serves as a common repository for application-specific data." },
		{ (char*)CSIDL_BITBUCKET,               "CSIDL_BITBUCKET",               "File system directory containing file objects in the user's Recycle Bin. The location of this directory is not in the registry; it is marked with the hidden and system attributes to prevent the user from moving or deleting it." },
		{ (char*)CSIDL_COMMON_ALTSTARTUP,       "CSIDL_COMMON_ALTSTARTUP",       "File system directory that corresponds to the nonlocalized Startup program group for all users." },
		{ (char*)CSIDL_COMMON_DESKTOPDIRECTORY, "CSIDL_COMMON_DESKTOPDIRECTORY", "File system directory that contains files and folders that appear on the desktop for all users." },
		{ (char*)CSIDL_COMMON_FAVORITES,        "CSIDL_COMMON_FAVORITES",        "File system directory that serves as a common repository for all users' favorite items." },
		{ (char*)CSIDL_COMMON_PROGRAMS,         "CSIDL_COMMON_PROGRAMS",         "File system directory that contains the directories for the common program groups that appear on the Start menu for all users." },
		{ (char*)CSIDL_COMMON_STARTMENU,        "CSIDL_COMMON_STARTMENU",        "File system directory that contains the programs and folders that appear on the Start menu for all users." },
		{ (char*)CSIDL_COMMON_STARTUP,          "CSIDL_COMMON_STARTUP",          "File system directory that contains the programs that appear in the Startup folder for all users. " },
		{ (char*)CSIDL_CONTROLS,                "CSIDL_CONTROLS",                "Virtual folder containing icons for the Control Panel applications." },
		{ (char*)CSIDL_COOKIES,                 "CSIDL_COOKIES",                 "File system directory that serves as a common repository for Internet cookies." },
		{ (char*)CSIDL_DESKTOP,                 "CSIDL_DESKTOP",                 "Windows Desktop  virtual folder at the root of the namespace." },
		{ (char*)CSIDL_DESKTOPDIRECTORY,        "CSIDL_DESKTOPDIRECTORY",        "File system directory used to physically store file objects on the desktop (not to be confused with the desktop folder itself)." },
		{ (char*)CSIDL_DRIVES,                  "CSIDL_DRIVES",                  "My Computer  virtual folder containing everything on the local computer: storage devices, printers, and Control Panel. The folder may also contain mapped network drives." },
		{ (char*)CSIDL_FAVORITES,               "CSIDL_FAVORITES",               "File system directory that serves as a common repository for the user's favorite items." },
		{ (char*)CSIDL_FONTS,                   "CSIDL_FONTS",                   "Virtual folder containing fonts." },
		{ (char*)CSIDL_HISTORY,                 "CSIDL_HISTORY",                 "File system directory that serves as a common repository for Internet history items." },
		{ (char*)CSIDL_INTERNET,                "CSIDL_INTERNET",                "Virtual folder representing the Internet." },
		{ (char*)CSIDL_INTERNET_CACHE,          "CSIDL_INTERNET_CACHE",          "File system directory that serves as a common repository for temporary Internet files." },
		{ (char*)CSIDL_NETHOOD,                 "CSIDL_NETHOOD",                 "File system directory containing objects that appear in the network neighborhood." },
		{ (char*)CSIDL_NETWORK,                 "CSIDL_NETWORK",                 "Network Neighborhood Folder  virtual folder representing the top level of the network hierarchy." },
		{ (char*)CSIDL_PERSONAL,                "CSIDL_PERSONAL",                "File system directory that serves as a common repository for documents." },
		{ (char*)CSIDL_PRINTERS,                "CSIDL_PRINTERS",                "Virtual folder containing installed printers." },
		{ (char*)CSIDL_PRINTHOOD,               "CSIDL_PRINTHOOD",               "File system directory that serves as a common repository for printer links." },
		{ (char*)CSIDL_PROGRAMS,                "CSIDL_PROGRAMS",                "File system directory that contains the user's program groups (which are also file system directories)." },
		{ (char*)CSIDL_RECENT,                  "CSIDL_RECENT",                  "File system directory that contains the user's most recently used documents." },
		{ (char*)CSIDL_SENDTO,                  "CSIDL_SENDTO",                  "File system directory that contains Send To menu items." },
		{ (char*)CSIDL_STARTMENU,               "CSIDL_STARTMENU",               "File system directory containing Start menu items." },
		{ (char*)CSIDL_STARTUP,                 "CSIDL_STARTUP",                 "File system directory that corresponds to the user's Startup program group. The system starts these programs whenever any user logs onto Windows NT or starts Windows 95." },
		{ (char*)CSIDL_TEMPLATES,               "CSIDL_TEMPLATES",               "File system directory that serves as a common repository for document templates." },
	};

#endif // USED_SPECIALFOLDERLOCATIONINFO


///////////////////////////////////////////////////////////////////////////
//
///////////////////////////////////////////////////////////////////////////
inline BOOL _EcxGetSpecialFolderLocation(int nFolder, char* lpszPath)
{
	lpszPath[0] = '\0';

	// Get a pointer to an item ID list that
	// represents the path of a special folder
	// Shellcc.chm::/inet401/help/itt/Shell/Functions/SHGetSpecialFolderLocation.htm
	LPITEMIDLIST pidl;
	if(::SHGetSpecialFolderLocation(NULL, nFolder, &pidl) != NOERROR) return FALSE;

	// Convert the item ID list's binary
	// representation into a file system path
	// Shellcc.chm::/inet401/help/itt/Shell/Functions/SHGetPathFromIDList.htm
	BOOL res = ::SHGetPathFromIDList(pidl, lpszPath);

	// Allocate a pointer to an IMalloc interface
	// Get the address of our task allocator's IMalloc interface
	// Shellcc.chm::/inet401/help/itt/Shell/Functions/SHGetMalloc.htm
	LPMALLOC pMalloc;
	if(::SHGetMalloc(&pMalloc) == NOERROR)
	{
		// Free the item ID list allocated by SHGetSpecialFolderLocation
		pMalloc->Free(pidl);

		// Free our task allocator
		pMalloc->Release();
	}

	return res;
}


///////////////////////////////////////////////////////////////////////////
//
///////////////////////////////////////////////////////////////////////////
inline BOOL _EcxGetFileInfo(LPCTSTR lpszFilePath, SHFILEINFO* pSfi, HIMAGELIST* phImgList16, HIMAGELIST* phImgList32)
{
	if(pSfi == NULL) return FALSE;
	ZeroMemory(pSfi, sizeof(SHFILEINFO));

	if(phImgList16 != NULL) *phImgList16 = (HIMAGELIST)::SHGetFileInfo(lpszFilePath, 0, pSfi, sizeof(SHFILEINFO), SHGFI_SYSICONINDEX|SHGFI_SMALLICON);
	if(phImgList32 != NULL) *phImgList32 = (HIMAGELIST)::SHGetFileInfo(lpszFilePath, 0, pSfi, sizeof(SHFILEINFO), SHGFI_SYSICONINDEX|SHGFI_LARGEICON);
  
	// Shellcc.chm::/inet401/help/itt/Shell/Functions/SHGetFileInfo.htm
	return ::SHGetFileInfo(lpszFilePath, 0, pSfi, sizeof(SHFILEINFO), SHGFI_DISPLAYNAME|SHGFI_TYPENAME|SHGFI_ATTRIBUTES);
}







////////////////////////////////////////////////////////////////////////////
#endif // __SHELLFUNC_H__
