// StringManipulationFunc.h : header file
//

#ifndef __STRINGMANIPULATIONFUNC_H__
#define __STRINGMANIPULATIONFUNC_H__
#pragma once
////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////
//  StringManipulationFunc.h
/////////////////////////////////////////////////////////////////////////////
#define VERSION_STRINGMANIPULATIONFUNC   "08.03.2019"



/////////////////////////////////////////////////////////////////////////////
//   
/////////////////////////////////////////////////////////////////////////////
#include "..\StringManipulation\StringManipulationMacro.h"
#include "..\TextParsing\TextParsingMacro.h"



////////////////////////////////////////////////////////////////////////////
//    szFileName,   ppBuf  
//    .  ppBuf  .
////////////////////////////////////////////////////////////////////////////
inline DWORD _EcxReadTextFile(LPCTSTR szFileName, char** ppBuf)
{
	if(ppBuf == NULL) return INVALID_FILE_SIZE;
	HANDLE hFile = ::CreateFile(szFileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if(hFile != INVALID_HANDLE_VALUE)
	{
		DWORD dw, dwLen = ::GetFileSize(hFile, NULL);
		*ppBuf = new char[dwLen + 1];
		::ReadFile(hFile, *ppBuf, dwLen, &dw, NULL);
		::CloseHandle(hFile);
		(*ppBuf)[dwLen] = '\0';
		return dwLen;
	}
	return INVALID_FILE_SIZE;
}


////////////////////////////////////////////////////////////////////////////
//       pBuf.      nLength,
//  ,       .
////////////////////////////////////////////////////////////////////////////
inline DWORD _EcxWriteBufferToFile(LPCTSTR szFileName, BYTE* pBuf, int nLength = -1)
{
	if(pBuf == NULL) return INVALID_FILE_SIZE;
	HANDLE hFile = ::CreateFile(szFileName, GENERIC_WRITE, FILE_SHARE_WRITE, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
	if(hFile != INVALID_HANDLE_VALUE)
	{
		DWORD dw, dwLen = (nLength == -1) ? strlen((char*)pBuf) : nLength;
		::WriteFile(hFile, pBuf, dwLen, &dw, NULL);
		::CloseHandle(hFile);
		return dwLen;
	}
	return INVALID_FILE_SIZE;
}




////////////////////////////////////////////////////////////////////////////
//     ch   set
////////////////////////////////////////////////////////////////////////////
inline BOOL _EcxIsCharInSet(char ch, LPCSTR set)
{
	if(set == NULL) return FALSE;
	char str[] = { ch, '\0' };
	return !!strpbrk(str, set);
}

////////////////////////////////////////////////////////////////////////////
//    (  )  ch   set
//     ,    
////////////////////////////////////////////////////////////////////////////
inline int _EcxGetCharInSet(char ch, LPCSTR set)
{
	if(set == NULL) return 0;
	for(int i=0; i<(int)strlen(set); i++)
	{
		if(set[i] == ch) return i + 1;
	}
	return 0;
}

/////////////////////////////////////////////////////////////////////////////
//   ppBuf   OCT-  nCount.
//  isSeek  TRUE,      
//  OCT-.  pnCount   NULL,    
//    OCT-.
/////////////////////////////////////////////////////////////////////////////
inline BOOL _EcxIsOctSequence(char** ppBuf, int nCount, BOOL isSeek = FALSE, int* pnCount = NULL)
{
	char* tmp = *ppBuf;
	BOOL isPrefix = FALSE;

	//    OCT-
	if(*tmp == 0x30)	// 0
	{
		isPrefix = TRUE;
		tmp++;
	}

	//     OCT-
	while(*tmp)
	{
		if(nCount == 0) break;
		if(!is_char(*tmp, CCM_OCT)) break;
		tmp++;
		nCount--;
	}
	
	if(pnCount) *pnCount = tmp - *ppBuf - ((isPrefix) ? 1 : 0);
	if(isSeek) *ppBuf = tmp;

	if(nCount <= 0)
	{
		if(is_char(*tmp, CCM_SPACE|CCM_STOP|CCM_OCT|CCM_PUNCT|CCM_OPERAT) ||
			(*tmp == 'o')) return TRUE;
	}
	return FALSE;
}


/////////////////////////////////////////////////////////////////////////////
//   ppBuf   HEX-  nCount.
//  isSeek  TRUE,      
//  HEX-.  pDelim   
//    HEX-.  pnCount   NULL, 
//       HEX-.
/////////////////////////////////////////////////////////////////////////////
inline BOOL _EcxIsHexSequence(char** ppBuf, int nCount, BOOL isSeek = FALSE, char* pDelim = NULL, int* pnCount = NULL)
{
	char* tmp = *ppBuf;
	BOOL isPrefix = FALSE;

	//    HEX-
	if(cmp2(tmp, 0x5830) || cmp2(tmp, 0x7830))	// 0x  0X
	{
		isPrefix = TRUE;
		tmp += 2;
	}

	//     HEX-
	while(*tmp)
	{
		if(nCount == 0) break;
		if(!is_char(*tmp, CCM_HEX)) break;
		tmp++;
		nCount--;
	}
	
	if(pnCount) *pnCount = tmp - *ppBuf - ((isPrefix) ? 2 : 0);
	if(isSeek) *ppBuf = tmp;

	if(nCount == 0)
	{
		if(is_char(*tmp, CCM_SPACE|CCM_STOP|CCM_HEX|CCM_PUNCT|CCM_OPERAT) ||
			(*tmp == 'h') || _EcxIsCharInSet(*tmp, pDelim)) return TRUE;
	}
	return FALSE;
}


/////////////////////////////////////////////////////////////////////////////
//   ppBuf      nCount.
//  isSeek  TRUE,      
//   .  pFirstm   
//    ,   pNext   
//    .  pnCount   NULL, 
//        .
/////////////////////////////////////////////////////////////////////////////
inline BOOL _EcxIsIdentifierSequence(char** ppBuf, int nCount, BOOL isSeek = FALSE, char* pFirst = NULL, char* pNext = NULL, int* pnCount = NULL)
{
	char* tmp = *ppBuf;
	BOOL res1 = TRUE, res2 = TRUE, isPrefix = FALSE;

	//    
	res1 = is_char(*tmp, CCM_ALPHA) || (*tmp == '_') || _EcxIsCharInSet(*tmp, pFirst);

	//    
	if(res1)
	{
		tmp++;
		nCount--;

		//       
		while(*tmp)
		{
			if(nCount == 0) break;
			res2 = is_char(*tmp, CCM_WORD) || _EcxIsCharInSet(*tmp, pNext);
			if(!res2) break;
			tmp++;
			nCount--;
		}
	}
	
	if(pnCount) *pnCount = tmp - *ppBuf;
	if(isSeek) *ppBuf = tmp;
	return res1 && res2;
}


/////////////////////////////////////////////////////////////////////////////
//  BIN  TYPE
/////////////////////////////////////////////////////////////////////////////
template <class TYPE>
inline TYPE _bin2type(char** ppBuf, int nCount, TYPE)
{
	TYPE n = 0;
	int count = nCount;
	if(cmp2(*ppBuf, 0x6230) || cmp2(*ppBuf, 0x4230)) (*ppBuf) += 2;  // 0b  0B

	while(1)
	{
		int ch = **ppBuf;
		if(!is_char(ch, CCM_BIN)) break;
		if(count == 0) break;

		ch -= '0';
		n = (n << 1) + ch;
		
		(*ppBuf)++;
		count--;
	}
	return n;
}

/////////////////////////////////////////////////////////////////////////////
//  OCT  TYPE
/////////////////////////////////////////////////////////////////////////////
template <class TYPE>
inline TYPE _oct2type(char** ppBuf, int nCount, TYPE)
{
	TYPE n = 0;
	int count = nCount;
	if(**ppBuf == 0x30) (*ppBuf)++;  // 0

	while(1)
	{
		int ch = **ppBuf;
		if(!is_char(ch, CCM_OCT)) break;
		if(count == 0) break;

		ch -= '0';
		n = (n << 3) + ch;
		
		(*ppBuf)++;
		count--;
	}
	return n;
}

/////////////////////////////////////////////////////////////////////////////
//  DEC  TYPE
/////////////////////////////////////////////////////////////////////////////
template <class TYPE>
inline TYPE _dec2type(char** ppBuf, int nCount, TYPE)
{
	TYPE n = 0;
	int count = nCount;
	if(cmp2(*ppBuf, 0x6430) || cmp2(*ppBuf, 0x4430)) (*ppBuf) += 2;  // 0d  0D

	while(1)
	{
		int ch = **ppBuf;
		if(!is_char(ch, CCM_DEC)) break;
		if(count == 0) break;

		ch -= '0';
		n = (n * 10) + ch;
		
		(*ppBuf)++;
		count--;
	}
	return n;
}

/////////////////////////////////////////////////////////////////////////////
//  HEX  TYPE
/////////////////////////////////////////////////////////////////////////////
template <class TYPE>
inline TYPE _hex2type(char** ppBuf, int nCount, TYPE)
{
	TYPE n = 0;
	int count = nCount;
	if(cmp2(*ppBuf, 0x5830) || cmp2(*ppBuf, 0x7830)) (*ppBuf) += 2;  // 0x  0X

	while(1)
	{
		int ch = **ppBuf;
		if(!is_char(ch, CCM_HEX)) break;
		if(count == 0) break;

		ch = _EcxHex2Int(ch);
		n = (n << 4) + ch;
		
		(*ppBuf)++;
		count--;
	}
	return n;
}

/////////////////////////////////////////////////////////////////////////////
//  HEX   BYTE, WORD, DWORD
/////////////////////////////////////////////////////////////////////////////
#define _hex2byte(ppbuf)	_hex2type(ppbuf, 2 * sizeof(BYTE), (BYTE)0)
#define _hex2word(ppbuf)	_hex2type(ppbuf, 2 * sizeof(WORD), (WORD)0)
#define _hex2dword(ppbuf)	_hex2type(ppbuf, 2 * sizeof(DWORD), (DWORD)0)


////////////////////////////////////////////////////////////////////////////
//  Esc- \esc  
// error C2022: '511' : too big for character
////////////////////////////////////////////////////////////////////////////
inline int _EcxEscapeToChar(char** esc, BOOL* pIsOverflow = NULL)
{
	int n, ch = **esc;
	(*esc)++;

	if(pIsOverflow) *pIsOverflow = FALSE;

	if(ch == '\\')
	{
		ch = **esc;
		(*esc)++;

		switch(ch)
		{
			// 1 - smpl-esc-seq
			case 'a': return 0x07;  // \a
			case 'b': return 0x08;	// \b
			case 't': return 0x09;	// \t
			case 'n': return 0x0A;	// \n
			case 'v': return 0x0B;	// \v
			case 'f': return 0x0C;	// \f
			case 'r': return 0x0D;	// \r
			case 's': return 0x20;	// \s

			// 2 - oct-esc-seq
			case '0':
			{
				(*esc)--;
				// \0[0-7]+
				n = strtoul(*esc, esc, 8);
				goto LabReturnN;
			}

			// 3 - hex-esc-seq
			case 'x':
			{
				// \x[0-9a-fA-F]+
				if(is_char(**esc, CCM_HEX))
				{
					n = strtoul(*esc, esc, 16);
					goto LabReturnN;
				}
				// \x
				break;
			}

			// 4 - ctrl-esc-seq
			case 'c':
			{
				ch = **esc;
				// \c[\x41-\x60]
				if(__in11(ch, 0x41, 0x60))
				{
					(*esc)++;
					return ch - 0x41;
				}
				break;
			}
		}

		// 5 - dec-esc-seq
		if(is_char(ch, CCM_DEC))
		{
			(*esc)--;
			n = strtoul(*esc, esc, 10);
			goto LabReturnN;
		}
	}

	return ch;

LabReturnN:
	
	if(n > 255 && pIsOverflow) *pIsOverflow = TRUE;
	return (BYTE)n;
}

////////////////////////////////////////////////////////////////////////////
//  Trigraph- ??trigraph  
// ISO 646-1983 
////////////////////////////////////////////////////////////////////////////
inline int _EcxTrigraphToChar(char** trigraph)
{
	int ch = **trigraph;

	if(*(WORD*)*trigraph == 0x3F3F)
	{
		(*trigraph) += 2;
		ch = _EcxEscapeToChar(trigraph);

		switch(ch)
		{
			case '=': ch = '#'; break;
			case '(': ch = '['; break;
			case '/': ch = '\\'; break;
			case ')': ch = ']'; break;
			case '': ch = '^'; break;
			case '<': ch = '{'; break;
			case '!': ch = '|'; break;
			case '>': ch = '}'; break;
			case '-': ch = '~'; break;
		}
		return ch;
	}

	(*trigraph)++;
	return ch;
}


////////////////////////////////////////////////////////////////////////////
//    Esc-
////////////////////////////////////////////////////////////////////////////
inline char* _EcxByteToEscape(char* dest, BYTE ch)
{
	dest[0] = 0x5C;
	dest[2] = 0x00;

	switch((int)ch)
	{
		case 0x00: dest[1] = '0'; break;
		case 0x07: dest[1] = 'a'; break;
		case 0x08: dest[1] = 'b'; break;
		case 0x09: dest[1] = 't'; break;
		case 0x0A: dest[1] = 'n'; break;
		case 0x0B: dest[1] = 'v'; break;
		case 0x0C: dest[1] = 'f'; break;
		case 0x0D: dest[1] = 'r'; break;

		case 0x20: dest[1] = 's'; break;
		case 0x22: dest[1] = 0x22; break;
		case 0x23: dest[1] = '#'; break;
		case 0x27: dest[1] = 0x27; break;
		case 0x2A: dest[1] = '*'; break;
		case 0x3F: dest[1] = '?'; break;
		case 0x5C: dest[1] = 0x5C; break;

		default:
		{
			if(is_char(ch, CCM_CNTRL))
				sprintf(dest, "\\x%.2X", ch);
			else
			{
				dest[0] = ch;
				dest[1] = 0x00;
			}
		}
	}

	return dest;
}



////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline char* _EcxStrAsterisk(char* str)
{
	if(str == NULL)
	{
	}
		
	return str;
}


////////////////////////////////////////////////////////////////////////////
//  TRUE,  pBuf -      
////////////////////////////////////////////////////////////////////////////
inline BOOL _EcxStrIsNull(LPCSTR pBuf)
{
	return (pBuf == NULL || pBuf[0] == '\0');
}



////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////
inline char* _EcxPrtToStr(char* str, void* pPtr, BOOL isNull = TRUE)
{
	if(isNull && pPtr == NULL)
	{
		strcpy(str, "NULL");
		return str;
	}

	sprintf(str, "0x%.8X", (DWORD)pPtr);
	return str;
}


////////////////////////////////////////////////////////////////////////////
//    (-)     nLength.
//     ppString   
//         szSrc.
////////////////////////////////////////////////////////////////////////////
inline int _EcxStrDup(char** ppString, LPCSTR szSrc, int nLength = 0xFFFFFFFF)
{
	//     
	if(szSrc == NULL || ppString == NULL) return 0;

	//     
	int nLen = (nLength == 0xFFFFFFFF) ? strlen(szSrc) : nLength;
	*ppString = new char[nLen + 1];
	memcpy(*ppString, szSrc, nLen);
	(*ppString)[nLen] = '\0';
	return nLen;
}


////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////
inline int _EcxStrCompare(LPCSTR pBuf1, LPCSTR pBuf2, BOOL isCase = FALSE)
{
	if(pBuf1 == NULL && pBuf2 == NULL) return 0;
	if(pBuf1 != NULL && pBuf2 == NULL) return 1;
	if(pBuf1 == NULL && pBuf2 != NULL) return -1;
	return ((isCase) ? _stricmp(pBuf1, pBuf2) : strcmp(pBuf1, pBuf2));
}


////////////////////////////////////////////////////////////////////////////
//    set   src
//    dest    
////////////////////////////////////////////////////////////////////////////
inline int _EcxStrTrim(char* dest, char* src, char* set)
{
	int nLength = 0, nSetCount = strlen(set);
	char* tmp = dest;
	BOOL isTrim;

	while(1)
	{
		//   
		isTrim = FALSE;

		//   
		for(int i=0; i<nSetCount; i++)
		{
			//    
			if(*src == set[i])
			{
				src++;
				isTrim = TRUE;
				break;
			}
		}

		//   
		if(isTrim) continue;

		//    
		if(*src == '\0') break;

		//  
		*tmp++ = *src++;
		nLength++;
	}
	dest[nLength] = '\0';

	return nLength;
}





////////////////////////////////////////////////////////////////////////////
//  buffer   dest  count 
////////////////////////////////////////////////////////////////////////////
inline char* _EcxRepBuffer(char* dest, char* buffer, int count)
{
	int n, m = strlen(buffer);
	for(int j=0; j < count; j++)
	{
		n = strlen(dest) + 1;
		memmove(dest + m, dest, n);
		memcpy(dest, buffer, m);
	}
	return dest;
}


////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////
inline char* _EcxStrClass(char* string, DWORD dwMask)
{
	while(*string)
	{
		if(is_char(*string, dwMask)) return string;
		string++;
	}
	return NULL;
}



////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
typedef   BOOL (CALLBACK* PFN_STRITERATORPROC)	(char, void**);

inline BOOL CALLBACK IteratorProc(char ch, void** ppData)
{
	return TRUE;
}

inline BOOL _EcxStrIterator(char* szStr, BOOL isRev, PFN_STRITERATORPROC lpIteratorProc = NULL, void** ppData = NULL)
{
	char* tmp;
	BOOL res;
	if(lpIteratorProc == NULL) lpIteratorProc = IteratorProc;

	if(isRev)
	{
		int len = strlen(szStr);
		tmp = szStr + len - 1;
		while(len)
		{
			res = lpIteratorProc(*tmp, ppData);
			if(!res) break;
			tmp--;
			len--;
		}
	}
	else
	{
		tmp = szStr;
		while(*tmp)
		{
			res = lpIteratorProc(*tmp, ppData);
			if(!res) break;
			tmp++;
		}
	}
	return res;
}










////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////
inline int _EcxStrTrimRight(char** ppBuf)
{
	if(ppBuf == NULL) return 0;

	char* tmp = *ppBuf;
	int nLength = strlen(tmp);
	if(nLength == 0) return 0;
	nLength--;
	tmp += nLength;

	while(1)
	{
		if(nLength == 0) break;
		if(!is_char(*tmp, CCM_SPACE)) break;

		tmp--;
		nLength--;
	}
	nLength++;
	
	(*ppBuf)[nLength] = '\0';
	return nLength;
}



////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline int _EcxStrGetSkeleton(char** ppBuf, char* str, BOOL isUpper = FALSE)
{
	int p = 0;
	int len = _EcxStrDup(ppBuf, str);
	char* tmp = *ppBuf;
	for(int i=0; i<len; i++)
	{
		if(is_char(tmp[i], CCM_CH_CONSN))
		{
			tmp[p] = tmp[i];
			p++;
		}
	}
	tmp[p] = '\0';
	len = strlen(tmp);

	if(isUpper)
		::CharUpperBuff(tmp, len);
	else
		::CharLowerBuff(tmp, len);

	return len;
}


////////////////////////////////////////////////////////////////////////////
//      - Longest Common
// Subsequence (LCS)   str1  str2.   
////////////////////////////////////////////////////////////////////////////

static char gL[4096];
static int ga1, ga2, ga3;
static int gb1, gb2, gb3;

inline int _EcxStrFindLCS(char** dest, char* str1, char* str2)
{
	int i, j, k, n, m;

	ga1 = ga2 = ga3 = 0;

	//   str1  str2
	n = strlen(str1);
	m = strlen(str2);

	//    n + 1  m + 1
	int** L = _EcxAllocArray2D(n + 1, m + 1, (int)0);

	//  
	for(i = n; i >= 0; i--)
	{
		for(j = m; j >= 0; j--)
		{
			if(str1[i] == '\0' || str2[j] == '\0')
			{
				L[i][j] = 0;
				ga1++;
			}
			else
			{
				if(str1[i] == str2[j])
				{
					L[i][j] = L[i + 1][j + 1] + 1;
					ga2++;
				}
				else
				{
					L[i][j] = max(L[i + 1][j], L[i][j + 1]);
					ga3++;
				}
			}

		}
	}
	// m * n = a2 + a3
	// n + m + 1 = a1

	//   
	k = L[0][0];
	*dest = new char[k + 1];
	(*dest)[k] = '\0';

	i = j = k = 0;

	gb1 = gb2 = gb3 = 0;

	//    
	while(i < n && j < m)
	{
		if(str1[i] == str2[j])
		{
			(*dest)[k++] = str1[i];
			i++;
			j++;
			gb1++;
		}
		else
		{
			if(L[i + 1][j] >= L[i][j + 1])
			{
				i++;
				gb2++;
			}
			else
			{
				j++;
				gb3++;
			}
		}
	}


	gL[0] = '\0';
	char s[16];
	
	for(i = 0; i <= n; i++)
	{
		for(j = 0; j <= m; j++)
		{
			int t = L[i][j];
			sprintf(s, "%02d ", t);
			strcat(gL, s);
		}
		strcat(gL, "\r\n");
	}


	//   
	k = L[0][0];

	//  
	_EcxFreeArray2D(L, n + 1, m + 1);

#define Q1  10.0

	double k1 = Q1 / (Q1 + (double)abs(n - m));
//	k1 = 1;

	double t1 = (double)gb1 / (double)(gb1 + gb2 + gb3) * k1;
//	double t1 = (double)b1 / (double)(max(n, m) + b2 + b3);
//	double t1 = (double)b1 / (double)(b1 + 2 * (b2 + b3) + abs(b2 - b3));

	k = (int)(t1 * 100.0);

	return k;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline int _EcxStrGetLine(char** ppBuf, char* str)
{
	char* tmp = str;
	skip_to_class(tmp, CCM_STOP);
	int len = tmp - str;
	*ppBuf = new char[len + 1];
	memcpy(*ppBuf, str, len);
	(*ppBuf)[len] = '\0';
	return len;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline int _EcxStrQuote(char** ppBuf, BOOL isQuote = TRUE)
{
	int len = strlen(*ppBuf);
	char* tmp = new char[len + 3];
	memcpy(tmp + 1, *ppBuf, len);
	tmp[0] = 0x22;
	tmp[len + 1] = 0x22;
	tmp[len + 2] = 0x00;
	delete [] *ppBuf;
	*ppBuf = tmp;
	return len + 2;
}


////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////
inline char* _EcxStrDeleteSlash(char** ppBuf)
{
	char* tmp = *ppBuf;
	while(1)
	{
		int i = strlen(tmp);
		if(i == 0) break;
		char ch = tmp[i - 1];
		if(ch == '\\' || ch == '/')
		{
			tmp[i - 1] = '\0';
			continue;
		}
		break;
	}
	return tmp;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline LPSTR _EcxHexSequence(LPSTR szDest, BYTE* pBuf, int nCount, BOOL isChars, LPCSTR szAdder, BOOL (CALLBACK *pAsHex)(BYTE ch) = FALSE)
{
	szDest[0] = 0;
	BOOL isHexPrev = FALSE, isHexCurr = FALSE, isSpace = FALSE;
	char s1[32];
	s1[0] = 0x20;

	for(int i=0; i<nCount; i++)
	{
		BYTE ch = *(pBuf + i);
		isHexCurr = (pAsHex) ? pAsHex(ch) : (ch < 0x20);
		if(isChars)
		{
			if(isHexCurr)
				sprintf(s1 + 1, "%02X", ch);
			else
				if(szAdder && (isHexPrev == TRUE || i == 0) && isHexCurr == FALSE)
					sprintf(s1 + 1, "%s%c", szAdder, ch);
				else
					sprintf(s1 + 1, "%c", ch);
			isSpace = ((isHexPrev | isHexCurr) && i > 0);
		}
		else
		{
			if(szAdder)
				sprintf(s1 + 1, "%s%02X", szAdder, ch);
			else
				sprintf(s1 + 1, "%02X", ch);
		}

		strcat(szDest, s1 + !isSpace);
		isHexPrev = isHexCurr;
	}
	return szDest;
}



////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
#endif // __STRINGMANIPULATIONFUNC_H__


