// EConfig.h : header file
//

#ifndef __ECONFIG_H__
#define __ECONFIG_H__
#pragma once

#include "crt\Timers\EHighResolutionTimer.h"

/////////////////////////////////////////////////////////////////////////////
//  EConfig.h
/////////////////////////////////////////////////////////////////////////////
#define VERSION_ECONFIG   "28.03.2013"


// +---------------+-------+----------+-------------------------------------------------------------------------+
// |                                  |                             _GetNameData                                |
// +---------------+-------+----------+------------------------------------+------------------------------------+
// |                 |          szEntryName               |          szEntryData               |
// +---------------+-------+----------+------------------------------------+------------------------------------+
// | CET_WORD_WORD | ww    |  |  STATUS_GET_NAME, ENTRY_TYPE_WORD  |  STATUS_GET_DATA, ENTRY_TYPE_WORD  |
// +---------------+-------+----------+------------------------------------+------------------------------------+
// | CET_SECTION   | w{}   |    |  STATUS_GET_NAME, ENTRY_TYPE_WORD  |                                    |
// +---------------+-------+----------+------------------------------------+------------------------------------+
// | CET_LIST_WORD | {}w   |  |  STATUS_GET_NAME, ENTRY_TYPE_LIST  |  STATUS_GET_DATA, ENTRY_TYPE_WORD  |
// +---------------+-------+----------+------------------------------------+------------------------------------+
// | CET_LIST_LIST | {}{}  |  |  STATUS_GET_NAME, ENTRY_TYPE_LIST  |  STATUS_GET_DATA, ENTRY_TYPE_LIST  |
// +---------------+-------+----------+------------------------------------+------------------------------------+
//
// +---------------+-------+----------+------------------------------------+------------------------------------+
// |                 |          szEntryName               |          szEntryData               |
// +---------------+-------+----------+------------------------------------+------------------------------------+
// | CET_WORD_WORD | w=w   |  |  STATUS_GET_NAME, ENTRY_TYPE_WORD  |  STATUS_GET_DATA, ENTRY_TYPE_WORD  |
// +---------------+-------+----------+------------------------------------+------------------------------------+
// | CET_WORD_LIST | w={}  |  |  STATUS_GET_NAME, ENTRY_TYPE_WORD  |  STATUS_GET_DATA, ENTRY_TYPE_LIST  |
// +---------------+-------+----------+------------------------------------+------------------------------------+
// | CET_LIST_WORD | {}=w  |  |  STATUS_GET_NAME, ENTRY_TYPE_LIST  |  STATUS_GET_DATA, ENTRY_TYPE_WORD  |
// +---------------+-------+----------+------------------------------------+------------------------------------+
// | CET_LIST_LIST | {}={} |  |  STATUS_GET_NAME, ENTRY_TYPE_LIST  |  STATUS_GET_DATA, ENTRY_TYPE_LIST  |
// +---------------+-------+----------+------------------------------------+------------------------------------+
//




/////////////////////////////////////////////////////////////////////////////
//   (CET_ - Config Entry Type)
/////////////////////////////////////////////////////////////////////////////
enum _ConfigEntryType
{
	CET_SECTION,	// w{}    w=w{}

	CET_WORD_WORD,	// ww,    w=w	WW
	CET_WORD_LIST,	//        w={}	WL
	CET_LIST_WORD,	// {}w,  {}=w	LW
	CET_LIST_LIST,	// {}{}, {}={}	LL
	
	CET_PARAM,		//     WW, WL, LW, LL
	CET_WORDNAME,	//     WW, WL
	CET_LISTNAME,	//     LW, LL
	CET_WORDDATA,	//     WW, LW
	CET_LISTDATA,	//     WL, LL

	CET_NULL = -1
};

static char* _ConfigEntryTypeStr[] =
{
	"CET_SECTION",

	"CET_WORD_WORD",
	"CET_WORD_LIST",
	"CET_LIST_WORD",
	"CET_LIST_LIST",
	
	"CET_PARAM",
	"CET_WORDNAME",
	"CET_LISTNAME",
	"CET_WORDDATA",
	"CET_LISTDATA",
};


/////////////////////////////////////////////////////////////////////////////
//   (TYPE_)
/////////////////////////////////////////////////////////////////////////////
enum _ConfigType
{
	TYPE_NULL,
	TYPE_UNKNOWN,

	TYPE_INTEGER,
	TYPE_BOOLEAN,
	TYPE_STRING,

	TYPE_CHAR,
	TYPE_LONG,
	TYPE_DWORD,
	TYPE_FLOAT,
	TYPE_BOOL,
	TYPE_LPSTR,
	TYPE_LPWSTR,

	TYPE_HEXSEQUENCE,
	TYPE_POINT,
	TYPE_RECT,
	TYPE_WINDOWPLACEMENT,

	TYPE_ARRAY_INTEGER,
	TYPE_ARRAY_BOOLEAN,
	TYPE_ARRAY_DOUBLE,
	TYPE_ARRAY_STRING,

	TYPE_FUNCTION,
	TYPE_ARGUMENT,
};

static char* _ConfigTypeStr[] =
{
	"TYPE_NULL",
	"TYPE_UNKNOWN",

	"TYPE_INTEGER",
	"TYPE_BOOLEAN",
	"TYPE_STRING",

	"TYPE_CHAR",
	"TYPE_LONG",
	"TYPE_DWORD",
	"TYPE_FLOAT",
	"TYPE_BOOL",
	"TYPE_LPSTR",
	"TYPE_LPWSTR",

	"TYPE_HEXSEQUENCE",
	"TYPE_POINT",
	"TYPE_RECT",
	"TYPE_WINDOWPLACEMENT",

	"TYPE_ARRAY_INTEGER",
	"TYPE_ARRAY_BOOLEAN",
	"TYPE_ARRAY_DOUBLE",
	"TYPE_ARRAY_STRING",

	"TYPE_FUNCTION",
	"TYPE_ARGUMENT",
};

inline BOOL typeNull(int nType)		{ return (nType == TYPE_NULL || nType == TYPE_UNKNOWN); }
inline BOOL typeNumber(int nType)	{ return (nType == TYPE_INTEGER || nType == TYPE_LONG || nType == TYPE_DWORD); }
inline BOOL typeSigned(int nType)	{ return (nType == TYPE_INTEGER || nType == TYPE_LONG); }
inline BOOL typeUnsigned(int nType)	{ return (nType == TYPE_DWORD); }
inline BOOL typeCharacter(int nType){ return (nType == TYPE_CHAR); }
inline BOOL typeReal(int nType)		{ return (nType == TYPE_FLOAT); }
inline BOOL typeBoolean(int nType)	{ return (nType == TYPE_BOOLEAN || nType == TYPE_BOOL); }
inline BOOL typeString(int nType)	{ return (nType == TYPE_STRING || nType == TYPE_LPSTR || nType == TYPE_LPWSTR); }
inline BOOL typeStringA(int nType)	{ return (nType == TYPE_STRING || nType == TYPE_LPSTR); }
inline BOOL typeStringW(int nType)	{ return (nType == TYPE_LPWSTR); }


////////////////////////////////////////////////////////////////////////////
//     
////////////////////////////////////////////////////////////////////////////
#define FIELD_NAME	0
#define FIELD_DATA	1


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
#define STATUS_GET_NAME	0
#define STATUS_GET_DATA	1
#define STATUS_GET_SECT	2

#define ENTRY_TYPE_WORD	3
#define ENTRY_TYPE_LIST	4


////////////////////////////////////////////////////////////////////////////
//   nMode  InsertEntry
////////////////////////////////////////////////////////////////////////////
#define INSERT_BEFOR	0
#define INSERT_AFTER	1


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
#define CFG_ROOT			(CONFIGENTRY*)0x00636667
#define CFG_MAX_LENGTH		MAX_PATH * 100


////////////////////////////////////////////////////////////////////////////
//    (PCT_ - Print Cfg Type)
////////////////////////////////////////////////////////////////////////////
#define PCT_CLEARED_CONFIG		0
#define PCT_CONFIG_STRUCT		1
#define PCT_ELEMENT_ADDICTION	2




//#########################################################################//
/////////////////////////////////////////////////////////////////////////////
//  
/////////////////////////////////////////////////////////////////////////////
typedef struct _ListEntry  // le
{
	int nType;			//  
	DWORD dwSize;		//  
//	char* lpData;		// 

	union
	{
		char	chVal;
		long	nVal;
		DWORD	dwVal;
		float	fVal;
		BOOL	isVal;
		char*	lpData;
		wchar_t* wpData;
	};
	int nComm;


	_ListEntry* next;
	_ListEntry* prev;

	// 
	_ListEntry()
	{
		memset(this, 0, sizeof(_ListEntry));
	}

	// 
	~_ListEntry()
	{
		if(nType == TYPE_NULL || nType == TYPE_UNKNOWN || nType == TYPE_LPSTR || nType == TYPE_LPWSTR)
		{
			if(lpData != NULL) delete [] lpData;
		}
	}

} LISTENTRY;


/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
typedef struct _List  // lst
{
	_ListEntry* pLe;	// 
	int nCount;			//   
	_ListEntry** hash;	//   


	// 
	_List()
	{
		memset(this, 0, sizeof(_List));
	}

	// 
	~_List()
	{
		ListClear();
		DeleteHash();
	}

	//  
	void ListClear()
	{
		if(pLe == NULL) return;
		_ListEntry* tmp = pLe;
		do
		{
			_ListEntry* next = tmp->next;
			delete tmp;
			tmp = next;

		} while(tmp);
	}

	//  
	void DeleteHash()
	{
		if(hash != NULL) delete [] hash;
	}

	//  
	void CreateHash()
	{
		//     
		if(hash) delete [] hash;

		//       
		hash = new _ListEntry*[nCount];
		int i = 0;
		_ListEntry* tmp = pLe;
		do
		{
			hash[i++] = tmp;
			tmp = tmp->next;

		} while(tmp);
	}

} _LIST;



/////////////////////////////////////////////////////////////////////////////
//     
/////////////////////////////////////////////////////////////////////////////
struct _ConfigEntry;



//#########################################################################//
/////////////////////////////////////////////////////////////////////////////
//   
/////////////////////////////////////////////////////////////////////////////
typedef struct _ConfigAltEntry  // ae
{
	_ConfigEntry* first;	//   
	_ConfigEntry* last;		//   
	int nCount;				//    
	int nNode;				//   
	_ConfigEntry** hash;	//   

} CONFIGALTENTRY;




//#########################################################################//
/////////////////////////////////////////////////////////////////////////////
//  
/////////////////////////////////////////////////////////////////////////////
typedef struct _ConfigEntry  // ce
{
	int nType;			//   (CET_)
	int nLevel;			//  

	_ConfigEntry* prev;		//  
	_ConfigEntry* next;		//  
	_ConfigEntry* parent;	//   
	_ConfigEntry** hash;	//   

	CONFIGALTENTRY child;	//   

	char* szEntryName;		//  
	char* szEntryData;		//  
	int nEntryNameLength;	//   
	int nEntryDataLength;	//   
	int nNameType;			//   
	int nDataType;			//   

	_LIST* pLstName;		//    (  - )
	_LIST* pLstData;		//    (  - )

	//
	LISTENTRY* GetListName(int nChild, int nData)
	{
		return child.hash[nChild]->pLstName->hash[nData];
	}
	//
	LISTENTRY* GetListData(int nChild, int nData)
	{
		return child.hash[nChild]->pLstData->hash[nData];
	}


	// 
	_ConfigEntry()
	{
		memset(this, 0, sizeof(_ConfigEntry));
	}

	// 
	~_ConfigEntry()
	{
		if(szEntryName != NULL) delete [] szEntryName;
		if(szEntryData != NULL) delete [] szEntryData;
		if(hash != NULL) delete [] hash;
		if(child.hash != NULL) delete [] child.hash;
		if(pLstName != NULL) delete pLstName;
		if(pLstData != NULL) delete pLstData;
	}



	//    
	BOOL IsChild()
		{ return parent != NULL; }

	//     
	BOOL IsChildFirst()
		{ return (parent != NULL) && (prev == NULL); }
	
	//     
	BOOL IsChildLast()
		{ return (parent != NULL) && (next == NULL); }

	//   
	BOOL IsNode()
		{ return (child.first != NULL); }

	//      szName
	BOOL IsName(LPCSTR szName)
		{ return _EcxStrCompare(szEntryName, szName) == 0; }

	//      nCmpType
	BOOL IsType(int nCmpType)
	{
		BOOL res = (nType == nCmpType);

		switch(nCmpType)
		{
			case CET_PARAM:
				res = (nType != CET_SECTION);
				break;
			case CET_WORDNAME:
				res = (nType == CET_WORD_WORD) || (nType == CET_WORD_LIST);
				break;
			case CET_LISTNAME:
				res = (nType == CET_LIST_WORD) || (nType == CET_LIST_LIST);
				break;
			case CET_WORDDATA:
				res = (nType == CET_WORD_WORD) || (nType == CET_LIST_WORD);
				break;
			case CET_LISTDATA:
				res = (nType == CET_WORD_LIST) || (nType == CET_LIST_LIST);
				break;
		}

		return res;
	}

	//     (CET_)
	EString GetEntryTypeStr(BOOL isShort)
	{
		EString s1 = _ConfigEntryTypeStr[nType];
		if(isShort) s1 = s1.Mid(4);
		return s1;
	}

	//     (TYPE_)
	EString GetDataTypeStr(BOOL isShort, int nField)
	{
		EString s1 = _ConfigTypeStr[(nField == FIELD_NAME) ? nNameType : nDataType];
		if(isShort) s1 = s1.Mid(5);
		return s1;
	}

	//       
	EString GetEntryTypeDataStr()
	{
		EString s1;
		s1.Format("%s (%s,%s)", GetEntryTypeStr(TRUE), GetDataTypeStr(TRUE, FIELD_NAME), GetDataTypeStr(TRUE, FIELD_DATA));
		return s1;
	}


} CONFIGENTRY;


////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////
#define CFG_ERROR  0xFFFFFFFF

enum _CfgErrorCode
{
	ERROR_NOERROR,
	ERROR_INCORRECT_ENTRY_TYPE,
	ERROR_INCORRECT_NAME,
	ERROR_EQUALS_SIGN,
	ERROR_TAG_UNKNOWN,
	ERROR_NULL_POINTER,
	ERROR_MAX_LENGTH,
};

static char* _CfgErrorCodeStr[] =
{
	"ERROR_NOERROR\r\n ",
	"ERROR_INCORRECT_ENTRY_TYPE\r\n  ",
	"ERROR_INCORRECT_NAME\r\n      ",
	"ERROR_EQUALS_SIGN\r\n   =",
	"ERROR_TAG_UNKNOWN\r\n  ",
	"ERROR_NULL_POINTER\r\n ",
	"ERROR_MAX_LENGTH\r\n  ",
};


#ifdef _DEBUG
	#define CfgError(errorcode)  EXC_ERROR(errorcode, FALSE, (char**)&_CfgErrorCodeStr)
#else
	#define CfgError(errorcode) ( (void)0 )
#endif



////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
typedef struct _CfgError // err
{
	int nError;
	char szFunction[32];

	////////////////////////////////////////////////////////////////////////
	// 
	////////////////////////////////////////////////////////////////////////
	_CfgError()
	{
		memset(this, 0, sizeof(_CfgError));
	}

} CFGERROR;




////////////////////////////////////////////////////////////////////////////
//    :
// +-----------------------------+----------------------------------------+
// |          nGetStatus         |                                   |
// +-----------------------------+----------+-----------------------------+
// |        STATUS_GET_NAME      |  ppName  |  CONFIGENTRY.szEntryName    |
// |                             |  pnType  |  CONFIGENTRY.nNameType      |
// +-----------------------------+----------+-----------------------------+
// |        STATUS_GET_DATA      |  ppName  |  CONFIGENTRY.szEntryData    |
// |                             |  pnType  |  CONFIGENTRY.nDataType      |
// +-----------------------------+----------+-----------------------------+
//    _GetNameTypePtr.
////////////////////////////////////////////////////////////////////////////
typedef struct _CfgNameTypePtr  // ntp
{
	int nGetStatus;
	char** ppName;
	int* pnType;

} CFGNAMETYPEPTR;



/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
DECLARE_HANDLE(HCFG);




/////////////////////////////////////////////////////////////////////////////
//   
/////////////////////////////////////////////////////////////////////////////
class EConfig  // cfg
{
// Construction
public:
	EConfig();
	~EConfig();

	LPSTR Load(LPCSTR szCfgFileName);
	void Save(LPCSTR szCfgFileName, int nPrintCfgType);


	int Parse(char** ppData = NULL, CONFIGENTRY* pCe = NULL, BOOL isRecurse = FALSE);
	void _NewEntry(CONFIGENTRY** ppCe, CONFIGENTRY* parent, int& nCount, int& nLevel, BOOL& isAlt, int nType);


	CONFIGENTRY* AddEntry(CONFIGENTRY* pCe, int nType, LPCSTR szName = NULL);


	HCFG AddSection(LPCSTR szName, HCFG hParent = NULL)
	{
		return (HCFG)AddEntry((CONFIGENTRY*)hParent, CET_SECTION, szName);
	}





	void SetData(CONFIGENTRY* pCe, LPCSTR szData, int nDataType);
	BOOL DeleteData(CONFIGENTRY* pCe);

	void _WriteData(CONFIGENTRY* pCe, void* pData = NULL, int nDataType = TYPE_NULL, DWORD dw = 4);
	CONFIGENTRY* _WriteInteger(CONFIGENTRY* pCe, LPCSTR szName, int val);
	CONFIGENTRY* _WriteBoolean(CONFIGENTRY* pCe, LPCSTR szName, BOOL val);
	CONFIGENTRY* _WriteDouble(CONFIGENTRY* pCe, LPCSTR szName, double val, int nPrec = 4);
	CONFIGENTRY* _WriteString(CONFIGENTRY* pCe, LPCSTR szName, LPCSTR val);
	CONFIGENTRY* _WriteHexSequence(CONFIGENTRY* pCe, LPCSTR szName, BYTE* pBuf, int nCount);
	CONFIGENTRY* _WritePoint(CONFIGENTRY* pCe, LPCSTR szName, POINT pt);
	CONFIGENTRY* _WriteRect(CONFIGENTRY* pCe, LPCSTR szName, RECT rc);
	CONFIGENTRY* _WriteWindowPlacement(CONFIGENTRY* pCe, LPCSTR szName, WINDOWPLACEMENT* wp);

	//     szName   szPath
	HCFG WriteInteger(LPCSTR szPath, LPCSTR szName, int val);
	HCFG WriteBoolean(LPCSTR szPath, LPCSTR szName, BOOL val);
	HCFG WriteDouble(LPCSTR szPath, LPCSTR szName, double val, int nPrec = 4);
	HCFG WriteString(LPCSTR szPath, LPCSTR szName, LPCSTR val);
	HCFG WriteHexSequence(LPCSTR szPath, LPCSTR szName, BYTE* pBuf, int nCount);
	HCFG WritePoint(LPCSTR szPath, LPCSTR szName, POINT pt);
	HCFG WriteRect(LPCSTR szPath, LPCSTR szName, RECT rc);
	HCFG WriteWindowPlacement(LPCSTR szPath, LPCSTR szName, WINDOWPLACEMENT* wp);

	CONFIGENTRY* _WriteArray(CONFIGENTRY* pCe, LPCSTR szName, void** ppData, int nCount, int nDataType, DWORD dw = 4);
//	void WriteArrayInteger(CONFIGENTRY* pCe, LPCSTR szName, void** ppData, int nCount, int nDataType);
//	void WriteArrayBoolean(CONFIGENTRY* pCe, LPCSTR szName, void** ppData, int nCount, int nDataType);
//	void WriteArrayDouble(CONFIGENTRY* pCe, LPCSTR szName, void** ppData, int nCount, int nDataType, DWORD dw = 4);
//	void WriteArrayString(CONFIGENTRY* pCe, LPCSTR szName, void** ppData, int nCount, int nDataType);
//	void WriteArrayHexSequence(CONFIGENTRY* pCe, LPCSTR szName, void** ppData, int nCount, int nDataType);
//	void WriteArrayPoint(CONFIGENTRY* pCe, LPCSTR szName, void** ppData, int nCount, int nDataType);


	BOOL ReadInteger(LPCSTR szPath, LPCSTR szName, int* val);
	BOOL ReadBoolean(LPCSTR szPath, LPCSTR szName);
	double ReadDouble(LPCSTR szPath, LPCSTR szName);
	char* ReadString(LPCSTR szPath, LPCSTR szName);
	BOOL ReadWindowPlacement(LPCSTR szPath, LPCSTR szName, WINDOWPLACEMENT* wp);



	void Remove();
	void Clear(CONFIGENTRY* pCe);
	BOOL Delete(CONFIGENTRY* pCe);

	void Print(CONFIGENTRY* pCe, int nPrintCfgType);
	void _Print(CONFIGENTRY* pCe, int nPrintCfgType);
	void SetTree(CONFIGENTRY* pCe, HWND hTreeCtrl);
	void _SetTree(CONFIGENTRY* pCe, HWND hTreeCtrl, HTREEITEM hParent);
	void Run();
	void _Run(CONFIGENTRY* pCe);

	void _AddCounts(CONFIGENTRY* pCe, int nType);
	void _SubCounts(CONFIGENTRY* pCe, int nType);

	
	
	BOOL IsIncorrectEntryType(int nType);


	EString GetEntryStr(CONFIGENTRY* pCe, int nMode);
	EString GetEntryNameStr(CONFIGENTRY* pCe);
	EString GetEntryDataStr(CONFIGENTRY* pCe, int nMode);
	EString GetEntryPtrStr(CONFIGENTRY* pCe);
	EString GetSysVarStr();


	CONFIGENTRY* FindEntry(LPCSTR szPath, LPCSTR szName, int nEntryType);
	CONFIGENTRY* GetEntry(LPCSTR szPath, int nNumber);

	char* GetEntryName(LPCSTR szPath, int nNumber);
	char* GetEntryData(LPCSTR szPath, int nNumber);
	char* GetEntryString(LPCSTR szPath, int nNumber, int nStringType);


	CONFIGENTRY* FindPath(LPCSTR szPath, CONFIGENTRY** ppFndLast = NULL);
	CONFIGENTRY* CreatePath(LPCSTR szPath);
	CONFIGENTRY* _FindPath(CONFIGENTRY* pCe, CONFIGENTRY** ppFndLast = NULL);

	CONFIGENTRY* _FindEntry(LPCSTR szName, CONFIGENTRY* pCe, int nEntryType);
	CONFIGENTRY* _FindEntryUp(LPCSTR szName, CONFIGENTRY* pCe, int nEntryType);
	CONFIGENTRY* _GetEntry(int nNumber, CONFIGENTRY* pCe);


	void _GetNameData(char** ppBuf, CONFIGENTRY* pCe, int nGetStatus, int nGetEntryType);
	int _ExpandEnvironment(char** ppBuf, char* dest, CONFIGENTRY* pCe, PFN_EXPANDENVIRONMENTPROC ExpandEnvironmentProc = NULL);
	friend DWORD CALLBACK _ExpandEnvironmentProc(LPCTSTR lpEnv, LPTSTR lpExpEnv, BOOL* pExpanded, void* pData);
	void _GetNameTypePtr(CONFIGENTRY* pCe, CFGNAMETYPEPTR* pNtp);
	void _SetChildHash(CONFIGENTRY* pCe);
	int _GetChildHash(CONFIGENTRY* pCe);
	int SetNeighborsHasht(CONFIGENTRY* pCe, BOOL isHash = FALSE);




	int CreateList(CONFIGENTRY* pCe, int nListType, int nConfigEntryType = CET_NULL);




	char* m_pCfgBuffer;
	int m_nLength;				//   

	CONFIGALTENTRY m_base;		//  ()

	EString m_strCfgName;
	EString m_strCfgPath;

	BOOL m_isDeleteNullEntry;	// 
	BOOL m_isSetEqualSign;		// 
	BOOL m_isWideChar;
//	BOOL m_nVersion;

	CFGERROR m_err;

	EPath m_path;
	EString m_strOutput;

	EHighResolutionTimer hrt;

};


////////////////////////////////////////////////////////////////////////////
//    _ExpandEnvironmentProc
////////////////////////////////////////////////////////////////////////////
typedef struct _CfgDatas  // cds
{
	CONFIGENTRY* pCe;
	EConfig* pConfig;

} CFGDATA;



////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline EConfig::EConfig()
{
	m_pCfgBuffer = NULL;
	memset(&m_base, 0, sizeof(CONFIGALTENTRY));
	memset(&m_err, 0, sizeof(CFGERROR));
	m_isDeleteNullEntry = TRUE;
	m_isSetEqualSign = TRUE;
	m_isWideChar = FALSE;

	m_path.SetDelimiter('\\');

	//	m_nVersion = 2;
	memset(&m_err, 0, sizeof(CFGERROR));
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline EConfig::~EConfig()
{
	Remove();
}


////////////////////////////////////////////////////////////////////////////
//     szCfgFileName
////////////////////////////////////////////////////////////////////////////
inline LPSTR EConfig::Load(LPCSTR szCfgFileName)
{
	Remove();
	m_nLength = _EcxReadTextFile(szCfgFileName, &m_pCfgBuffer);
	return m_pCfgBuffer;
}


////////////////////////////////////////////////////////////////////////////
//    nPrintCfgType   szCfgFileName
////////////////////////////////////////////////////////////////////////////
inline void EConfig::Save(LPCSTR szCfgFileName, int nPrintCfgType)
{
	Print(NULL, nPrintCfgType);
	_EcxWriteBufferToFile(szCfgFileName, (BYTE*)(LPCSTR)m_strOutput);
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline int EConfig::CreateList(CONFIGENTRY* pCe, int nListType, int nConfigEntryType)
{
	int count = 0;
	BOOL isEnd;
	LISTENTRY* pLe = NULL;
	char* tmp = NULL;
	
	_LIST** ppLst = (nListType == FIELD_NAME) ? &pCe->pLstName : &pCe->pLstData;
	if(*ppLst)
	{
		(*ppLst)->ListClear();
		delete *ppLst;
	}
	*ppLst = new _LIST;

	//    
	if(pCe->nType == CET_LIST_WORD || nConfigEntryType == CET_LISTNAME) tmp = pCe->szEntryName; // LW, QW
	if(pCe->nType == CET_WORD_LIST || nConfigEntryType == CET_LISTDATA) tmp = pCe->szEntryData; // WL, WQ
	if(pCe->nType == CET_LIST_LIST) tmp = (nListType == FIELD_NAME) ? pCe->szEntryName : pCe->szEntryData; // LL

	//   
	if(tmp)
	{
		char* p0 = tmp;
		if(is_eot(tmp)) return 0;

		while(1)
		{
			if(!_EcxSkipString(&tmp, TRUE))
			{
				while(*tmp != ',' && *tmp != '}' && !is_eot(tmp)) tmp++;
			}
			isEnd = is_eot(tmp);

			//      
			if(*tmp == ',' || isEnd)
			{
				LISTENTRY* ple = new LISTENTRY;
				if(pLe == NULL)
				{
					(*ppLst)->pLe = pLe = ple;
				}
				else
				{
					ple->prev = pLe;
					pLe->next = ple;
				}

				pLe = ple;
				pLe->dwSize = _EcxStrDup(&pLe->lpData, p0, tmp - p0);
				switch(pCe->nDataType)
				{
//					case TYPE_POINT:
//					case TYPE_ARRAY_INTEGER: pLe->nType = TYPE_INTEGER; break;
//					case TYPE_ARRAY_BOOLEAN: pLe->nType = TYPE_BOOLEAN; break;
//					case TYPE_ARRAY_DOUBLE: pLe->nType = TYPE_DOUBLE; break;
//					case TYPE_ARRAY_STRING: pLe->nType = TYPE_STRING; break;

					case TYPE_NULL:
					case TYPE_UNKNOWN:

						char* p0 = pLe->lpData;
						_EcxSkipNonInformation(&p0);
						char* p1 = p0;

						int rtype;
						char ch = _EcxTestReal(p1, &rtype, TRUE);

						if(ch == 'R' || ch == 'E')
						{
							double n = __strtod(p1);
							if(*p1 == '*')
							{
								p0 = p1;
								while(*p1 == '*') p1++;
								pLe->nComm = p1 - p0;
							}
							if(*p1 == ',' || is_eot(p1))
							{
								delete [] pLe->lpData;
								pLe->fVal = (float)n;
								pLe->nType = TYPE_FLOAT;
								pLe->dwSize = 4;
								break;
							}
						}

						// Number
						if(is_char(*p1, CCM_DIGIT) || *p1 == '-' || *p1 == '+')
						{
							int m; DWORD n; BOOL isUnsigned = FALSE;
							if(is_char(*p1, CCM_DIGIT))
							{
								isUnsigned = TRUE;
								n = __strtoul(p1);
							}
							else
							{
								m = __strtol(p1);
							}
							if(*p1 == '*')
							{
								p0 = p1;
								while(*p1 == '*') p1++;
								pLe->nComm = p1 - p0;
							}
							if(*p1 == ',' || is_eot(p1))
							{
								delete [] pLe->lpData;
								if(isUnsigned)
								{
									pLe->dwVal = n;
									pLe->nType = TYPE_DWORD;
								}
								else
								{
									pLe->nVal = m;
									pLe->nType = TYPE_LONG;
								}
								pLe->dwSize = 4;
								break;
							}
						}

						// String
						if(*p1 == '\x22')
						{
LabString:					p0 = p1;
							_EcxSkipString(&p1, TRUE);
							p0++;
							int size = p1 - p0 - 1;
							char* p2 = p0;
							if(*p1 == '*')
							{
								p0 = p1;
								while(*p1 == '*') p1++;
								pLe->nComm = p1 - p0;
							}
							if(*p1 == ',' || is_eot(p1))
							{
								memmove(pLe->lpData, p2, size);
								pLe->lpData[size] = 0;
								pLe->nType = TYPE_LPSTR;
								pLe->dwSize = size;

								if(m_isWideChar)
								{
									int len = ::MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, pLe->lpData, size, NULL, 0);
									wchar_t* wszBuffer = new wchar_t[len + 1];
									len = ::MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, pLe->lpData, size, wszBuffer, size);
									wszBuffer[len] = 0x0000;
									delete [] pLe->lpData;
									pLe->wpData = new wchar_t[len + 1];
									memmove(pLe->wpData, wszBuffer, 2*(len + 1));
									delete [] wszBuffer;
									pLe->nType = TYPE_LPWSTR;
									pLe->dwSize = len;
								}
								break;
							}
						}

						// Char
						if(*p1 == '\x27')
						{
							p0 = p1;
							p1++;
							BOOL isOverflow;
							int n = _EcxEscapeToChar(&p1, &isOverflow);
							if(*p1 == '\x27') p1++;
							else
							{
								p1 = p0;
								goto LabString;
							}
							if(*p1 == '*')
							{
								p0 = p1;
								while(*p1 == '*') p1++;
								pLe->nComm = p1 - p0;
							}
							if(*p1 == ',' || is_eot(p1))
							{
								delete [] pLe->lpData;
								pLe->chVal = n;
								pLe->nType = TYPE_CHAR;
								pLe->dwSize = 1;
								break;
							}
						}

						p0 = pLe->lpData;
						p1 = p0;
						int nBool = 4;
						if(strcmp(p0, "false") == 0 || strcmp(p0, "FALSE") == 0) nBool = 0;
						if(strcmp(p0, "true") == 0 || strcmp(p0, "TRUE") == 0) nBool = 1;
						if(strcmp(p0, "no") == 0 || strcmp(p0, "NO") == 0) nBool = 2;
						if(strcmp(p0, "yes") == 0 || strcmp(p0, "YES") == 0) nBool = 3;
						if(nBool < 4)
						{
							if(nBool == 0) p1 += 5;
							if(nBool == 1) p1 += 4;
							if(nBool > 1)
							{
								nBool -= 2;
								if(nBool == 0) p1 += 2;
								if(nBool == 1) p1 += 3;
							}

							if(*p1 == '*')
							{
								p0 = p1;
								while(*p1 == '*') p1++;
								pLe->nComm = p1 - p0;
							}
							if(*p1 == ',' || is_eot(p1))
							{
								delete [] pLe->lpData;
								pLe->isVal = nBool;
								pLe->nType = TYPE_BOOL;
								pLe->dwSize = 4;
								break;
							}
						}

/*
						//   
						while(!is_char(*p1, CCM_SPACE|CCM_SYMBOL|CCM_EOT)) p1++;
						char* p2 = p0;
						skip_class(p2, CCM_DEC);
						if(p1 == p2)
						{
							char* tmp;
							pLe->dwSize = _EcxStrDup(&tmp, p0, p1 - p0);
							delete [] pLe->lpData;
							pLe->lpData = tmp;
							pLe->nType = TYPE_INTEGER;
						}
*/
						break;

				}

				p0 = ++tmp;
				count++;
				(*ppLst)->nCount = count;

				if(isEnd) break;
			}
		}
		if(count > 0)
			(*ppLst)->CreateHash();
	}
	return count;
}


////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////
inline void EConfig::_NewEntry(CONFIGENTRY** ppCe, CONFIGENTRY* parent, int& nCount, int& nLevel, BOOL& isAlt, int nType)
{
	//  
	CONFIGENTRY* pCeNew  = new CONFIGENTRY;
	nCount++;

	//   
	if(*ppCe == NULL)
	{
		m_base.first = pCeNew;
	}
	else
	{
		//   
		if(isAlt)
			(*ppCe)->child.first = pCeNew;
		else
		{
			pCeNew->prev = *ppCe;
			(*ppCe)->next = pCeNew;
		}
	}

	//  
	*ppCe = pCeNew;
	if(nLevel == 0) m_base.last = pCeNew;
	(*ppCe)->parent = parent;
	if(parent != NULL) parent->child.last = pCeNew;
	isAlt = FALSE;

	//   
	(*ppCe)->nType = nType;
	(*ppCe)->nLevel = nLevel;
}


////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////
inline int EConfig::Parse(char** ppData, CONFIGENTRY* pCe, BOOL isRecurse)
{
	BOOL isAlt = FALSE, isEq = FALSE, isFunc = FALSE, isSequence = FALSE;
	BOOL isOnelineComment = FALSE, isEol = FALSE;
	char* tmp;
	hrt.Start();


	//    ( )
	if(!isRecurse)
	{
		//    
		if(ppData)
		{
			Remove();
			m_nLength = _EcxStrDup(&m_pCfgBuffer, *ppData);
		}
		tmp = m_pCfgBuffer;
	}
	else
	{
		if(ppData)
			tmp = *ppData;
		else
			return 0;
	}

	//     
	int nStatus = STATUS_GET_NAME;

	CONFIGENTRY* parent = pCe;
	int nCount = 0, nLevel = 0;

//	static int number = 0;


	//     
	if(pCe != NULL)
	{
		isAlt = TRUE;
		nLevel = pCe->nLevel + 1;
	}


	//   
	while(1)
	{
		//    
		while(is_char(*tmp, CCM_BLANK)) tmp++;
		isOnelineComment = is_oneline_comment(tmp);
		isEol = is_eol_eot(tmp);

		//   
		if(_EcxSkipNonInformation(&tmp)) break;


		//    
		if(*tmp == ';')
		{
			nStatus = STATUS_GET_NAME;
			tmp++;
			continue;
		}

		//       -  
		if(nStatus == STATUS_GET_DATA && (isOnelineComment || isEol))
		{
			if(!is_char(*tmp, CCM_BRACKET_L)) nStatus = STATUS_GET_NAME;
		}

		//   
		if(nStatus == STATUS_GET_DATA && *tmp == '=')
		{
			isEq = TRUE;
			tmp++;
			continue;
		}

		//    
		if(nStatus == STATUS_GET_DATA && *tmp == '(')
		{
			if(pCe->nType == CET_WORD_WORD)
			{
				isFunc = TRUE;
				pCe->nType++;
				goto LABEL_GETDATA;
			}
		}

		//  Hex Sequence
		if(nStatus == STATUS_GET_DATA && *tmp == '[')
		{
			if(pCe->nType == CET_WORD_WORD)
			{
				isSequence = TRUE;
				goto LABEL_GETDATA;
			}
		}

		//
		if(*tmp == '}' && isRecurse)
		{
			tmp++;
			goto LABEL_END;
		}


		//////////////////////////////////////////////////////////////////////
		//      - W?
		//////////////////////////////////////////////////////////////////////
		if(*tmp != '{')
		{
			// W?
			//   (   )
			if(nStatus == STATUS_GET_NAME)
			{
				//  
				_NewEntry(&pCe, parent, nCount, nLevel, isAlt, CET_WORD_WORD);

				//  
				_GetNameData(&tmp, pCe, STATUS_GET_NAME, ENTRY_TYPE_WORD);
				pCe->nEntryNameLength = strlen(pCe->szEntryName);

				CreateList(pCe, FIELD_NAME, CET_LISTNAME);

				//    
				nStatus = STATUS_GET_DATA;
				isEq = FALSE;
				continue;
			}

			// xW
			//   (     )
			if(nStatus == STATUS_GET_DATA)
			{
				//  
				_GetNameData(&tmp, pCe, STATUS_GET_DATA, ENTRY_TYPE_WORD);
				pCe->nEntryDataLength = strlen(pCe->szEntryData);

				CreateList(pCe, FIELD_DATA, CET_LISTDATA);

				if(m_isDeleteNullEntry && pCe->nNameType == TYPE_NULL && pCe->nDataType == TYPE_NULL)
				{
					CONFIGENTRY* pCeTmp = pCe->prev;
					Delete(pCe);
					pCe = pCeTmp;
					nCount--;
					if(parent != NULL && pCe == NULL)
					{
						pCe = parent;
						isAlt = TRUE;
					}
				}

				//      
				nStatus = (pCe->nType == CET_WORD_WORD) ? STATUS_GET_SECT : STATUS_GET_NAME;
				continue;
			}

			//     
			if(nStatus == STATUS_GET_SECT)
			{
				//     
				nStatus = STATUS_GET_NAME;
				continue;
			}
		}


		//////////////////////////////////////////////////////////////////////
		//     - L?
		//////////////////////////////////////////////////////////////////////
		if(*tmp == '{')
		{
			// L?
			//     
			if(nStatus == STATUS_GET_NAME)
			{
				//  
				_NewEntry(&pCe, parent, nCount, nLevel, isAlt, CET_LIST_WORD);

				//  
				_GetNameData(&tmp, pCe, STATUS_GET_NAME, ENTRY_TYPE_LIST);
				pCe->nEntryNameLength = strlen(pCe->szEntryName);

				CreateList(pCe, FIELD_NAME);

				//    
				nStatus = STATUS_GET_DATA;
				isEq = FALSE;
				continue;
			}

			// xL
			//       
			if(nStatus == STATUS_GET_DATA || nStatus == STATUS_GET_SECT)
			{
				pCe->nType++;

				//  
				if(!isEq && pCe->nType == CET_WORD_LIST) pCe->nType = CET_SECTION;
				if(nStatus == STATUS_GET_SECT) pCe->nType = CET_SECTION;


				//   
				if(pCe->nType == CET_SECTION)
				{
					//   
					if(parent != NULL) parent->child.nNode++;

					tmp++;
					pCe->child.nCount = Parse(&tmp, pCe, TRUE);

					if(pCe->child.nCount > 0)
					{
						_SetChildHash(pCe);
					}
				}
				else
				{
LABEL_GETDATA:
					//  
					_GetNameData(&tmp, pCe, STATUS_GET_DATA, ENTRY_TYPE_LIST);
					pCe->nEntryDataLength = strlen(pCe->szEntryData);

					if(pCe->IsType(CET_LISTDATA))
					{
						CreateList(pCe, FIELD_DATA);
					}

					if(isFunc)
					{
						if(pCe->nNameType != TYPE_NULL) pCe->nNameType = TYPE_FUNCTION;
						pCe->nDataType = TYPE_ARGUMENT;
						isFunc = FALSE;
					}

					if(isSequence)
					{
						pCe->nDataType = TYPE_HEXSEQUENCE;
						isSequence = FALSE;
					}
				}


				//     
				nStatus = STATUS_GET_NAME;
//				nStatus = STATUS_GET_SECT;
				continue;
			}
		}


		//////////////////////////////////////////////////////////////////////
		// 
		//////////////////////////////////////////////////////////////////////
		if(nStatus == STATUS_GET_SECT)
		{
			nStatus = STATUS_GET_NAME;
			continue;
		}


		//  
		tmp++;
	}

LABEL_END:

	if(isRecurse)
	{
		*ppData = tmp;
	}
	hrt.Stop();

	
	//   
	m_base.nCount += nCount;
	return nCount;
}


////////////////////////////////////////////////////////////////////////////
//   (szEntryName, nNameType)  (szEntryData, nDataType)
//      STATUS_GET_NAME  STATUS_GET_DATA
////////////////////////////////////////////////////////////////////////////
inline void EConfig::_GetNameTypePtr(CONFIGENTRY* pCe, CFGNAMETYPEPTR* pNtp)
{
	if(pNtp->nGetStatus == STATUS_GET_NAME)
	{
		pNtp->ppName = &pCe->szEntryName;
		pNtp->pnType = &pCe->nNameType;
	}
	else
	{
		pNtp->ppName = &pCe->szEntryData;
		pNtp->pnType = &pCe->nDataType;
	}
}


////////////////////////////////////////////////////////////////////////////
//        
//       
////////////////////////////////////////////////////////////////////////////
inline void EConfig::_SetChildHash(CONFIGENTRY* pCe)
{
	//     
	if(pCe->child.hash) delete [] pCe->child.hash;

	//       
	pCe->child.hash = new CONFIGENTRY*[pCe->child.nCount];
	int i = 0;
	CONFIGENTRY* tmp = pCe->child.first;
	do
	{
		pCe->child.hash[i++] = tmp;
		tmp = tmp->next;

	} while(tmp);
}


////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////
inline int EConfig::_GetChildHash(CONFIGENTRY* pCe)
{
	//    
	if(pCe->IsChild())
	{
		for(int i=0; i<pCe->parent->child.nCount; i++)
		{
			if(pCe->parent->child.hash[i] == pCe) return i;
		}
	}
	return -1;
}


////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////
inline void EConfig::_GetNameData(char** ppBuf, CONFIGENTRY* pCe, int nGetStatus, int nGetEntryType)
{
	int nType = TYPE_UNKNOWN;
	int nLength, len;
	char *src, *dest, *buf = NULL;
	char *p0, *tmp;
	p0 = tmp = *ppBuf;

	//     
	CFGDATA cds = { pCe, this };

	//   (szEntryName, nNameType)  (szEntryData, nDataType)
	CFGNAMETYPEPTR ntp = { nGetStatus, NULL, NULL };
	_GetNameTypePtr(pCe, &ntp);


	//  WORD-
	if(nGetEntryType == ENTRY_TYPE_WORD)
	{
		//    
		while(!is_eot(tmp))
		{
			if(nGetStatus == STATUS_GET_NAME)
			{
				if(is_char(*tmp, CCM_SPACE)) break;
				if(*tmp == '=') break;
				if(is_comment(tmp)) break;
			}

			if(nGetStatus == STATUS_GET_DATA)
			{
				if(is_char(*tmp, CCM_EOL)) break;
				if(*tmp == ';') break;
				if(is_oneline_comment(tmp)) break;
				if(is_newline_break(tmp)) _EcxSkipNonInformation(&tmp, NULL, SKIP_NEWLINE_BREAK);
			}

			if(is_char(*tmp, CCM_BRACKET)) break;
			if(*tmp == 0x22)
			{
				_EcxSkipString(&tmp, TRUE);
				continue;
			}
			tmp++;
		}
		nLength = tmp - p0;

		if(nGetStatus == STATUS_GET_DATA)
		{
			_EcxStrDup(&src, p0, nLength);
			buf = new char[nLength + 1];
			nLength = _EcxClearNonInformation(buf, src, NULL, SKIP_NEWLINE_BREAK|SKIP_MULTYLINE_COMMENT|SKIP_SPACES, TRUE);
			delete [] src;
			p0 = buf;
		}
	}

	//  LIST-
	else
	{
		//     
		BLOCK_INFO binf;
		binf.pBuff = tmp;

		//   {...}  (...)
		_EcxFindBlock(&binf);
		p0 = binf.pBeg + 1;
		nLength = binf.nLength - 2;
		tmp += binf.nLength;
	}


	//    
	
	_EcxStrDup(&src, p0, nLength);
	dest = new char[CFG_MAX_LENGTH];
	p0 = dest;

	nLength = 0;
	char *dest1 = dest, *src1 = src;


//	strcpy(dest, src);



	while(1)
	{
		//   
		if(_EcxSkipNonInformation(&src1, NULL, (nGetEntryType == ENTRY_TYPE_WORD) ? 0 : SKIP_BASE)) break;

		if(*src1 == '%')
		{
			len = _ExpandEnvironment(&src1, NULL, pCe);
			if(nLength + len >= CFG_MAX_LENGTH) CfgError(ERROR_MAX_LENGTH);

			len = _ExpandEnvironment(&src1, dest1, pCe);
			nLength += len;
			dest1 += len;
			continue;
		}

		if(*src1 == 0x22)
		{
			len = _EcxGetString(&src1, GSTR_LENGTH, TRUE, TRUE, TRUE, _ExpandEnvironmentProc, &cds);
			if(nLength + len + 2 >= CFG_MAX_LENGTH) CfgError(ERROR_MAX_LENGTH);

			len = _EcxGetString(&src1, dest1, TRUE, TRUE, TRUE, _ExpandEnvironmentProc, &cds, TRUE);
			dest1 += len;
			nLength += len;
			continue;
		}

		*dest1++ = *src1++;
		nLength++;

		if(nLength >= CFG_MAX_LENGTH) CfgError(ERROR_MAX_LENGTH);
	}
	dest[nLength] = '\0';
/**/

	//    
	_EcxStrDup(ntp.ppName, p0, nLength);
	_EcxStrTrimRight(ntp.ppName);


	if(nGetEntryType == ENTRY_TYPE_WORD)
	{
		// 
		if(nLength == 0) nType = TYPE_NULL;
		else
			if(strcmp(p0, "null") == 0) nType = TYPE_NULL;
	}


	//   
	delete [] src;
	delete [] dest;
	if(buf) delete [] buf;

	*ppBuf = tmp;
	*ntp.pnType = nType;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline DWORD CALLBACK _ExpandEnvironmentProc(LPCTSTR lpEnv, LPTSTR lpExpEnv, BOOL* pExpanded, void* pData)
{
	DWORD dwLength = 0;
	BOOL isExpanded = FALSE;
	
	CONFIGENTRY* pCe = ((CFGDATA*)pData)->pCe;
	EConfig* pConfig = ((CFGDATA*)pData)->pConfig;

	char* exp_env = new char[CFG_MAX_LENGTH / 10 + 1];
	exp_env[0] = '\0';

	ifstricmp(lpEnv, "%cfg_curr%") strcpy(exp_env, pConfig->GetEntryNameStr(pCe));
	ifstricmp(lpEnv, "%cfg_prev%") strcpy(exp_env, pConfig->GetEntryNameStr(pCe->prev));
	ifstricmp(lpEnv, "%cfg_parent%") strcpy(exp_env, pConfig->GetEntryNameStr(pCe->parent));
	ifstricmp(lpEnv, "%cfg_name%") strcpy(exp_env, pConfig->m_strCfgName);
	ifstricmp(lpEnv, "%cfg_path%") strcpy(exp_env, pConfig->m_strCfgPath);


	//    
	if(!isExpanded)
	{
		int exp_env_len = strlen(lpEnv);

		//    
		char* var = new char[exp_env_len];
		memcpy(var, lpEnv + 1, exp_env_len - 2);
		var[exp_env_len - 2] = '\0';

		//     
		CONFIGENTRY* pCeFnd = pConfig->_FindEntryUp(var, pCe, CET_WORDNAME);
		delete [] var;

		//   
		if(pCeFnd)
		{
			//   
			delete [] exp_env;
			exp_env = new char[pCeFnd->nEntryDataLength + 1];
			strcpy(exp_env, pCeFnd->szEntryData);
			exp_env_len = pCeFnd->nEntryDataLength;
			if(pCeFnd->nDataType == TYPE_STRING) exp_env_len = _EcxStrQuote(&exp_env);
		}
	}

	dwLength = strlen(exp_env);
	isExpanded = (dwLength > 0);

	if(isExpanded && lpExpEnv != NULL) strcpy(lpExpEnv, exp_env);
	if(pExpanded) *pExpanded = isExpanded;

	delete [] exp_env;
	return dwLength;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline int EConfig::_ExpandEnvironment(char** ppBuf, char* dest, CONFIGENTRY* pCe, PFN_EXPANDENVIRONMENTPROC ExpandEnvironmentProc)
{
	char* tmp = *ppBuf;
	int exp_env_len = 0, env_len;
	BOOL isExpanded;

	CFGDATA cds = { pCe, this };

	//    
	char* v0 = tmp++;
	skip_to_char(tmp, '%');
	
	//    
	if(*tmp == '%')
	{
		//   %
		tmp++;

		//   
		char* env =  new char[MAX_PATH];
		exp_env_len, env_len = tmp - v0;
		memcpy(env, v0, env_len);
		env[env_len] = '\0';

		//    
		exp_env_len = _EcxExpandEnvironment(env, NULL, NULL, _ExpandEnvironmentProc, &cds);

		//   
		char* exp_env = new char[exp_env_len + 1];
		exp_env_len = _EcxExpandEnvironment(env, exp_env, &isExpanded, _ExpandEnvironmentProc, &cds);

		//    
		if(dest != NULL)
		{
			//  :   
			memcpy(dest, exp_env, exp_env_len);
			*ppBuf = tmp;
		}

		delete [] exp_env;
		delete [] env;
	}

	return exp_env_len;
}


////////////////////////////////////////////////////////////////////////////
//     
////////////////////////////////////////////////////////////////////////////
inline void EConfig::Run()
{
	m_strOutput = "";
	_Run(m_base.first);
}


////////////////////////////////////////////////////////////////////////////
//     ,    pCe
////////////////////////////////////////////////////////////////////////////
inline void EConfig::_Run(CONFIGENTRY* pCe)
{
	EString s1, s2, s11;
	CONFIGENTRY* tmp = pCe;

	do
	{
		if(tmp->nType == CET_WORD_LIST)
		{
			if(tmp->nNameType == TYPE_FUNCTION && tmp->nDataType == TYPE_ARGUMENT)
			{
//				s1.Format("%s(%s)\r\n", tmp->szEntryName, tmp->szEntryData);
				s1.Format("%s\r\n", tmp->szEntryData);
				m_strOutput += s1;
			}
		}

		if(tmp->child.first != NULL) _Run(tmp->child.first);

		tmp = tmp->next;

	} while(tmp);
}


////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////
inline void EConfig::Remove()
{
	if(m_base.first)
		Clear(m_base.first);
	memset(&m_base, 0, sizeof(CONFIGALTENTRY));
	memset(&m_err, 0, sizeof(CFGERROR));
	m_path.Clear();
	if(m_pCfgBuffer)
	{
		delete [] m_pCfgBuffer;
		m_pCfgBuffer = NULL;
	}
}


////////////////////////////////////////////////////////////////////////////
//   ,    pCe
////////////////////////////////////////////////////////////////////////////
inline void EConfig::Clear(CONFIGENTRY* pCe)
{
	if(pCe == NULL) return;
	CONFIGENTRY* tmp = pCe;
	do
	{
		CONFIGENTRY* next = tmp->next;
		if(tmp->child.first != NULL) Clear(tmp->child.first);
		delete tmp;
		m_base.nCount--;
		tmp = next;

	} while(tmp);
}


////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////
inline BOOL EConfig::Delete(CONFIGENTRY* pCe)
{
	if(pCe == NULL) return FALSE;

	if(pCe->next != NULL) pCe->next->prev = pCe->prev;
	if(pCe->prev != NULL) pCe->prev->next = pCe->next;

	if(pCe == m_base.last) m_base.last = pCe->prev;
	if(pCe == m_base.first) m_base.first = pCe->next;
	
	//   
	Clear(pCe->child.first);
	
	if(pCe->parent != NULL)
	{
		if(pCe->prev == NULL) pCe->parent->child.first = pCe->next;
		if(pCe->next == NULL) pCe->parent->child.last = pCe->prev;

		//  
		_SubCounts(pCe->parent, pCe->nType);
	}
	delete pCe;

	m_base.nCount--;
	return TRUE;
}


////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////
inline CONFIGENTRY* EConfig::AddEntry(CONFIGENTRY* pCe, int nType, LPCSTR szName)
{
	//  
	#define ADD_ENTRY 0
	#define ADD_CHILD 1


	//     
	if(IsIncorrectEntryType(nType)) CfgError(ERROR_INCORRECT_ENTRY_TYPE);

	//   
	int nMode = ADD_ENTRY;

	//     
	if(pCe != NULL)
	{
		//   
		if(pCe->nType == CET_SECTION)
		{
			//   
			nMode = ADD_CHILD;
		}

		//   
		else
		{
			//    
			if(pCe->IsChild())
			{
				//      -  (.. )
				nMode = ADD_CHILD;
				pCe = pCe->parent;
			}
		}
	}

	//   
	CONFIGENTRY* pCeAdd = new CONFIGENTRY;
	pCeAdd->nType = nType;
	m_base.nCount++;

	//  
	if(szName != NULL)
	{
		pCeAdd->nEntryNameLength = _EcxStrDup(&pCeAdd->szEntryName, szName);
		pCeAdd->nNameType = TYPE_UNKNOWN;
	}


	//   
	if(nMode == ADD_ENTRY)
	{
		//   
		if(m_base.first == NULL)
		{
			m_base.first = pCeAdd;
		}
		else
		{
			//    
			pCeAdd->prev = m_base.last;
			m_base.last->next = pCeAdd;
		}

		//   
		m_base.last = pCeAdd;
	}

	//   
	else
	{
		//   alt-
		if(pCe->child.first == NULL)
		{
			pCe->child.first = pCeAdd;
		}
		else
		{
			//    
			pCeAdd->prev = pCe->child.last;
			pCe->child.last->next = pCeAdd;
		}

		//   
		pCe->child.last = pCeAdd;
		
		//   
		pCeAdd->parent = pCe;
		pCeAdd->nLevel = pCe->nLevel + 1;

		//  
		_AddCounts(pCe, nType);
	}

	//  
	return pCeAdd;
}


////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////
inline BOOL EConfig::DeleteData(CONFIGENTRY* pCe)
{
	if(pCe)
	{
		if(pCe->szEntryData)
		{
			delete [] pCe->szEntryData;
			pCe->nEntryDataLength = 0;
			pCe->nDataType = TYPE_NULL;
			return TRUE;
		}
	}
	return FALSE;
}


////////////////////////////////////////////////////////////////////////////
//     szData  nDataType.
//   .
////////////////////////////////////////////////////////////////////////////
inline void EConfig::SetData(CONFIGENTRY* pCe, LPCSTR szData, int nDataType)
{
	DeleteData(pCe);
	
	if(pCe)
	{
		pCe->nEntryDataLength = _EcxStrDup(&pCe->szEntryData, szData);
		pCe->nDataType = nDataType;
	}

}


////////////////////////////////////////////////////////////////////////////
//   pData  nDataType  pCe
////////////////////////////////////////////////////////////////////////////
inline void EConfig::_WriteData(CONFIGENTRY* pCe, void* pData, int nDataType, DWORD dw)
{
	if(pCe)
	{
		EString str;
		if(pData == NULL) nDataType = TYPE_NULL;

		switch(nDataType)
		{
			case TYPE_NULL:
				DeleteData(pCe);
				break;
			case TYPE_UNKNOWN:
			case TYPE_FUNCTION:
			case TYPE_ARGUMENT:
				break;
			case TYPE_INTEGER:
				str.Format("%d", *(int*)pData);
				break;
			case TYPE_BOOLEAN:
				str.Format("%s", (*(int*)pData) ? "true" : "false");
				break;
			case TYPE_FLOAT:
			{
				EString format;
				format.Format("%%.%df", dw);
				str.Format(format, *(double*)pData);
				break;
			}
			case TYPE_STRING:
				str = (char*)pData;
				break;
		}
		SetData(pCe, str, nDataType);
	}
}

////////////////////////////////////////////////////////////////////////////
//   val  TYPE_INTEGER  pCe
////////////////////////////////////////////////////////////////////////////
inline CONFIGENTRY* EConfig::_WriteInteger(CONFIGENTRY* pCe, LPCSTR szName, int val)
{
	CONFIGENTRY* tmp = AddEntry(pCe, CET_WORD_WORD, szName);
	_WriteData(tmp, &val, TYPE_INTEGER);
	return tmp;
}

////////////////////////////////////////////////////////////////////////////
//   val  TYPE_BOOLEAN  pCe
////////////////////////////////////////////////////////////////////////////
inline CONFIGENTRY* EConfig::_WriteBoolean(CONFIGENTRY* pCe, LPCSTR szName, BOOL val)
{
	CONFIGENTRY* tmp = AddEntry(pCe, CET_WORD_WORD, szName);
	_WriteData(tmp, &val, TYPE_BOOLEAN);
	return tmp;
}

////////////////////////////////////////////////////////////////////////////
//   val  TYPE_FLOAT  pCe
////////////////////////////////////////////////////////////////////////////
inline CONFIGENTRY* EConfig::_WriteDouble(CONFIGENTRY* pCe, LPCSTR szName, double val, int nPrec)
{
	CONFIGENTRY* tmp = AddEntry(pCe, CET_WORD_WORD, szName);
	_WriteData(tmp, &val, TYPE_FLOAT, nPrec);
	return tmp;
}

////////////////////////////////////////////////////////////////////////////
//   val  TYPE_STRING  pCe
////////////////////////////////////////////////////////////////////////////
inline CONFIGENTRY* EConfig::_WriteString(CONFIGENTRY* pCe, LPCSTR szName, LPCSTR val)
{
	CONFIGENTRY* tmp = AddEntry(pCe, CET_WORD_WORD, szName);
	_WriteData(tmp, (void*)val, TYPE_STRING);
	return tmp;
}

////////////////////////////////////////////////////////////////////////////
//   pBuf  nCount  TYPE_HEXSEQUENCE  pCe
////////////////////////////////////////////////////////////////////////////
inline CONFIGENTRY* EConfig::_WriteHexSequence(CONFIGENTRY* pCe, LPCSTR szName, BYTE* pBuf, int nCount)
{
	CONFIGENTRY* tmp = AddEntry(pCe, CET_WORD_WORD, szName);
	EStringFormat strf;
	EString str = strf.HexSequence(pBuf, nCount);
	str.Quote(TRUE, '[', ']');
	SetData(tmp, str, TYPE_HEXSEQUENCE);
	return tmp;
}

////////////////////////////////////////////////////////////////////////////
//   pt  TYPE_POINT  pCe
////////////////////////////////////////////////////////////////////////////
inline CONFIGENTRY* EConfig::_WritePoint(CONFIGENTRY* pCe, LPCSTR szName, POINT pt)
{
	CONFIGENTRY* tmp = AddEntry(pCe, CET_WORD_LIST, szName);
	EString str;
	str.Format("%d,%d", pt.x, pt.y);
	SetData(tmp, str, TYPE_POINT);
	return tmp;
}

////////////////////////////////////////////////////////////////////////////
//   rc  TYPE_RECT  pCe
////////////////////////////////////////////////////////////////////////////
inline CONFIGENTRY* EConfig::_WriteRect(CONFIGENTRY* pCe, LPCSTR szName, RECT rc)
{
	CONFIGENTRY* tmp = AddEntry(pCe, CET_WORD_LIST, szName);
	EString str;
	str.Format("%d,%d,%d,%d", rc.left, rc.top, rc.right, rc.bottom);
	SetData(tmp, str, TYPE_RECT);
	return tmp;
}

////////////////////////////////////////////////////////////////////////////
//   wp  TYPE_WINDOWPLACEMENT  pCe
////////////////////////////////////////////////////////////////////////////
inline CONFIGENTRY* EConfig::_WriteWindowPlacement(CONFIGENTRY* pCe, LPCSTR szName, WINDOWPLACEMENT* wp)
{
	CONFIGENTRY* tmp = AddEntry(pCe, CET_WORD_LIST, szName);
	EString str;
	str.Format("%d,%d,%d,%d,%d,%d", wp->flags, wp->showCmd, wp->rcNormalPosition.left, wp->rcNormalPosition.top, wp->rcNormalPosition.right, wp->rcNormalPosition.bottom);
	SetData(tmp, str, TYPE_WINDOWPLACEMENT);
	return tmp;
}


////////////////////////////////////////////////////////////////////////////
//     szName   szPath
////////////////////////////////////////////////////////////////////////////
inline HCFG EConfig::WriteInteger(LPCSTR szPath, LPCSTR szName, int val)
	{ return (HCFG)_WriteInteger(CreatePath(szPath), szName, val); }
inline HCFG EConfig::WriteBoolean(LPCSTR szPath, LPCSTR szName, BOOL val)
	{ return (HCFG)_WriteBoolean(CreatePath(szPath), szName, val); }
inline HCFG EConfig::WriteDouble(LPCSTR szPath, LPCSTR szName, double val, int nPrec)
	{ return (HCFG)_WriteDouble(CreatePath(szPath), szName, val, nPrec); }
inline HCFG EConfig::WriteString(LPCSTR szPath, LPCSTR szName, LPCSTR val)
	{ return (HCFG)_WriteString(CreatePath(szPath), szName, val); }
inline HCFG EConfig::WriteHexSequence(LPCSTR szPath, LPCSTR szName, BYTE* pBuf, int nCount)
	{ return (HCFG)_WriteHexSequence(CreatePath(szPath), szName, pBuf, nCount); }
inline HCFG EConfig::WritePoint(LPCSTR szPath, LPCSTR szName, POINT pt)
	{ return (HCFG)_WritePoint(CreatePath(szPath), szName, pt); }
inline HCFG EConfig::WriteRect(LPCSTR szPath, LPCSTR szName, RECT rc)
	{ return (HCFG)_WriteRect(CreatePath(szPath), szName, rc); }
inline HCFG EConfig::WriteWindowPlacement(LPCSTR szPath, LPCSTR szName, WINDOWPLACEMENT* wp)
	{ return (HCFG)_WriteWindowPlacement(CreatePath(szPath), szName, wp); }










////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline CONFIGENTRY* EConfig::_WriteArray(CONFIGENTRY* pCe, LPCSTR szName, void** ppData, int nCount, int nDataType, DWORD dw)
{
	CONFIGENTRY* tmp = AddEntry(pCe, CET_WORD_LIST, szName);

	EString str, s1;

	for(int i=0; i<nCount; i++)
	{
		switch(nDataType)
		{
			case TYPE_NULL:
				DeleteData(pCe);
				break;
			case TYPE_UNKNOWN:
			case TYPE_FUNCTION:
			case TYPE_ARGUMENT:
				break;
			case TYPE_ARRAY_INTEGER:
				s1.Format("%d", *((int*)ppData + i));
				break;
			case TYPE_ARRAY_BOOLEAN:
				s1.Format("%s", (*((BOOL*)ppData + i)) ? "true" : "false");
				break;
			case TYPE_ARRAY_DOUBLE:
			{
				EString format;
				format.Format("%%.%df", dw);
				s1.Format(format, *((double*)ppData + i));
				break;
			}
			case TYPE_ARRAY_STRING:
			{
				char** pp = (char**)ppData;
				char* p = pp[i];
				s1.Format("\"%s\"", p);
				break;
			}
		}
		str += s1 + ",";
	}
	str.TrimRight(",");
	SetData(tmp, str, nDataType);
	return tmp;
}




////////////////////////////////////////////////////////////////////////////
//       
////////////////////////////////////////////////////////////////////////////
inline void EConfig::_AddCounts(CONFIGENTRY* pCe, int nType)
{
	if(pCe == NULL) return;
	pCe->child.nCount++;
	if(nType == CET_SECTION) pCe->child.nNode++;
}


////////////////////////////////////////////////////////////////////////////
//       
////////////////////////////////////////////////////////////////////////////
inline void EConfig::_SubCounts(CONFIGENTRY* pCe, int nType)
{
	if(pCe == NULL) return;
	pCe->child.nCount--;
	if(nType == CET_SECTION) pCe->child.nNode--;
}




////////////////////////////////////////////////////////////////////////////
//    ,   
////////////////////////////////////////////////////////////////////////////
inline CONFIGENTRY* EConfig::FindEntry(LPCSTR szPath, LPCSTR szName, int nEntryType)
{
	CONFIGENTRY* fnd = NULL;

	//    
	BOOL isSplited = m_path.Split(szPath);

	// 
	CONFIGENTRY* pCe = m_base.first;
	if(pCe == NULL) return NULL;

	//   
	if(szPath == NULL)
	{
		return _FindEntry(szName, pCe, nEntryType);
	}

	//    
	if(isSplited)
	{
		//    
		fnd = _FindPath(pCe);
		
		//      
		if(szName == NULL) return fnd;
		if(szName[0] == '\0') return fnd;

		//  
		if(fnd != NULL) fnd = _FindEntry(szName, fnd->child.first, nEntryType);
	}

	return fnd;
}


////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////
inline CONFIGENTRY* EConfig::FindPath(LPCSTR szPath, CONFIGENTRY** ppFndLast)
{
	CONFIGENTRY* fnd = NULL;
	if(ppFndLast) *ppFndLast = NULL;

	//    
	BOOL isSplited = m_path.Split(szPath);

	// 
	CONFIGENTRY* pCe = m_base.first;
	if(pCe == NULL) return NULL;

	//  
	if(szPath == NULL) return NULL;

	//    
	if(isSplited)
	{
		//    
		fnd = _FindPath(pCe, ppFndLast);
	}

	return fnd;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline CONFIGENTRY* EConfig::CreatePath(LPCSTR szPath)
{
	CONFIGENTRY* tmp;
	CONFIGENTRY* path = FindPath(szPath, &tmp);

	//  szPath  (   )
	if(path && !tmp) return path;

	int level = 0;

	//    ( tmp -    )
	if(!path && tmp)
	{
		level = tmp->nLevel + 1;
	}

	//   
	// 1- :     (level == 0, tmp == NULL)
	// 1- :     (level ==     + 1, tmp ==   )
	for(int i = level; i < m_path.m_nCount; i++)
	{
		char* name = m_path.m_ppEntry[i];
		tmp = AddEntry(tmp, CET_SECTION, name);
	}
	return tmp;
}


////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////
inline CONFIGENTRY* EConfig::_FindPath(CONFIGENTRY* pCe, CONFIGENTRY** ppFndLast)
{
	CONFIGENTRY* fnd = NULL;

	// 
	CONFIGENTRY* tmp = pCe;
	if(pCe == NULL) return NULL;

	//       .  
	if(tmp->nLevel >= m_path.m_nCount) return NULL;

	//  
	char* name = m_path.m_ppEntry[tmp->nLevel];

	//   
	do
	{
		CONFIGENTRY* next = tmp->next;

		//        
		if(tmp->IsName(name))
		{
			//   - 
			if(tmp->nType == CET_SECTION)
			{
				//   
				CONFIGENTRY* alt = tmp->child.first;
				if(alt != NULL)
				{
					//    
					if(alt->nLevel == m_path.m_nCount)
					{
						fnd = tmp;
						break;
					}

					//  
					fnd = _FindPath(tmp->child.first, ppFndLast);
					if(fnd != NULL) break;
					else
						if(ppFndLast && *ppFndLast == NULL) *ppFndLast = tmp;
				}
			}
		}

		tmp = next;

	} while(tmp);

	return fnd;
}


////////////////////////////////////////////////////////////////////////////
//     
////////////////////////////////////////////////////////////////////////////
inline CONFIGENTRY* EConfig::_FindEntry(LPCSTR szName, CONFIGENTRY* pCe, int nEntryType)
{
	CONFIGENTRY* fnd = NULL;

	//   
	CONFIGENTRY *next, *tmp = pCe;
	if(tmp == NULL) return NULL;

	if(szName == NULL) return NULL;
	if(szName[0] == '\0') return NULL;

	//   
	do
	{
		next = tmp->next;

		//       
		if(tmp->IsName(szName))
		{
			//       
			if(tmp->IsType(nEntryType))
			{
				fnd = tmp;
				break;
			}
		}

		tmp = next;

	} while(tmp);

	return fnd;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline CONFIGENTRY* EConfig::_FindEntryUp(LPCSTR szName, CONFIGENTRY* pCe, int nEntryType)
{
	CONFIGENTRY* fnd = NULL;

	//   
	CONFIGENTRY *prev, *tmp = pCe;
	if(tmp == NULL) return NULL;
	if(_EcxStrIsNull(szName)) return NULL;

	//   
	while(1)
	{
		prev = tmp->prev;
		if(prev == NULL) prev = tmp->parent;
		if(prev == NULL) break;

		//       
		if(prev->IsName(szName))
		{
			//       
			if(prev->IsType(nEntryType))
			{
				fnd = prev;
				break;
			}
		}

		tmp = prev;
	}

	return fnd;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline CONFIGENTRY* EConfig::_GetEntry(int nNumber, CONFIGENTRY* pCe)
{
	CONFIGENTRY* tmp = pCe->child.first;
	if(tmp == NULL) return NULL;
	if(nNumber >= pCe->child.nCount) return NULL;

	int i = 0;
	CONFIGENTRY* fnd = NULL;

	do
	{
		CONFIGENTRY* next = tmp->next;

		if(i == nNumber)
		{
			fnd = tmp;
			break;
		}

		tmp = next;
		i++;

	} while(tmp);

	return fnd;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline CONFIGENTRY* EConfig::GetEntry(LPCSTR szPath, int nNumber)
{
	CONFIGENTRY *pce0, *pce1;

	pce0 = FindEntry(szPath, NULL, CET_SECTION);
	if(pce0 == NULL) return NULL;
	pce1 = _GetEntry(nNumber, pce0);
	return pce1;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline char* EConfig::GetEntryName(LPCSTR szPath, int nNumber)
{
	CONFIGENTRY* pce = GetEntry(szPath, nNumber);
	if(pce == NULL) return NULL;

	return pce->szEntryName;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline char* EConfig::GetEntryData(LPCSTR szPath, int nNumber)
{
	CONFIGENTRY* pce = GetEntry(szPath, nNumber);
	if(pce == NULL) return NULL;

	return pce->szEntryData;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline char* EConfig::GetEntryString(LPCSTR szPath, int nNumber, int nStringType)
{
	switch(nStringType)
	{
		case FIELD_NAME:
			return GetEntryName(szPath, nNumber);
			break;

		case FIELD_DATA:
			return GetEntryData(szPath, nNumber);
			break;
	}

	return NULL;
}


////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////
inline BOOL EConfig::ReadBoolean(LPCSTR szPath, LPCSTR szName)
{
	BOOL value = FALSE;
	CONFIGENTRY* pce = FindEntry(szPath, szName, 1);
	if(pce != NULL)
	{
		char* tmp = pce->szEntryData;
		if(is_char(*tmp, CCM_ALPHA))
		{
			int len = strlen(tmp);
			char* psz = new char[len + 1];
			strcpy(psz, tmp);
			::CharUpperBuff(psz, len);

			if(strcmp(psz, "TRUE") == 0) value = TRUE;
			if(strcmp(psz, "YES") == 0) value = TRUE;
			if(strcmp(psz, "FALSE") == 0) value = FALSE;
			if(strcmp(psz, "NO") == 0) value = FALSE;

			delete [] psz;
		}
		else
		{
			int i = __strtol(tmp);
			value = (i == 0) ? FALSE : TRUE;
		}
	}
	return value;
}


////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////
inline BOOL EConfig::ReadInteger(LPCSTR szPath, LPCSTR szName, int* val)
{
	if(!val) return FALSE;

	*val = 0;
	CONFIGENTRY* pce = FindEntry(szPath, szName, 1);
	if(pce != NULL)
	{
		char* tmp = pce->szEntryData;
		*val = __strtol(tmp);
		return TRUE;
	}
	return FALSE;
}


////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////
inline double EConfig::ReadDouble(LPCSTR szPath, LPCSTR szName)
{
	double value = 0.0;
	CONFIGENTRY* pce = FindEntry(szPath, szName, 1);
	if(pce != NULL)
	{
		char* tmp = pce->szEntryData;
		value = __strtod(tmp);
	}
	return value;
}


////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////
inline char* EConfig::ReadString(LPCSTR szPath, LPCSTR szName)
{
	char* value = NULL;
	CONFIGENTRY* pce = FindEntry(szPath, szName, CET_PARAM);
	if(pce != NULL)
	{
		value = pce->szEntryData;
	}
	return value;
}


////////////////////////////////////////////////////////////////////////////
//  WINDOWPLACEMENT
////////////////////////////////////////////////////////////////////////////
inline BOOL EConfig::ReadWindowPlacement(LPCSTR szPath, LPCSTR szName, WINDOWPLACEMENT* wp)
{
	CONFIGENTRY* pce = FindEntry(szPath, szName, CET_PARAM);
	if(pce != NULL)
	{
		wp->length = sizeof(WINDOWPLACEMENT);
		wp->flags = atoi(pce->pLstData->hash[0]->lpData);
		wp->showCmd = atoi(pce->pLstData->hash[1]->lpData);
		wp->rcNormalPosition.left = atoi(pce->pLstData->hash[2]->lpData);
		wp->rcNormalPosition.top = atoi(pce->pLstData->hash[3]->lpData);
		wp->rcNormalPosition.right = atoi(pce->pLstData->hash[4]->lpData);
		wp->rcNormalPosition.bottom = atoi(pce->pLstData->hash[5]->lpData);
		return TRUE;
	}
	return FALSE;
}


////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////
inline void EConfig::Print(CONFIGENTRY* pCe, int nPrintCfgType)
{
	m_strOutput = "";
	_Print(pCe, nPrintCfgType);
}


////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////
inline EString GetListStr(CONFIGENTRY* pCe, BOOL isName)
{
	EString h4, h8, s11 = "";
	_LIST* pList = (isName) ? pCe->pLstName : pCe->pLstData;
	if(pList)
	{
		int count = pList->nCount;
		EString spaces(' ', 4 * pCe->nLevel);
		h8.Format("%s%s[%d]%s\r\n", spaces, (isName) ? "LIST_NAME" : "LIST_DATA", count, (count) ? ":" : "");
		s11 += h8;
		for(int i=0; i<count; i++)
		{
			_ListEntry* lst = pList->hash[i];
			EString comm('*', lst->nComm);

			if(typeNull(lst->nType)) h4 = lst->lpData;
			if(typeBoolean(lst->nType)) h4.Format("%s", _EcxBoolToStrSmall(lst->isVal));
			if(typeCharacter(lst->nType)) h4.Format("'\\x%02X'", lst->chVal);
			if(typeSigned(lst->nType)) h4.Format("%d", lst->nVal);
			if(typeUnsigned(lst->nType)) h4.Format("%u", lst->dwVal);
			if(typeReal(lst->nType)) h4.Format("%f", lst->fVal);
			if(typeStringA(lst->nType)) h4.Format("%s", lst->lpData);
			
			h8.Format("   %s %s[%d]%s:  %s\r\n", spaces, _ConfigTypeStr[lst->nType], lst->dwSize, comm, h4);
			s11 += h8;
		}
	}
	return s11;
}

inline void EConfig::_Print(CONFIGENTRY* pCe, int nPrintCfgType)
{
	if(pCe == NULL) pCe = m_base.first;
	if(pCe == NULL) return;

	EString s1, s2, s11;
	CONFIGENTRY* tmp = pCe;

	do
	{
		EString spaces(' ', 4 * tmp->nLevel);
		s2 = "";

		EString strEntryName = GetEntryNameStr(tmp);
		EString strEntryData = GetEntryDataStr(tmp, 1);

		//   cfg-
		switch(nPrintCfgType)
		{
			////////////////////////////////////////////////
			//  
			////////////////////////////////////////////////
			case PCT_CLEARED_CONFIG:
			{
				EString strEqual = (m_isSetEqualSign) ? "= " : "";

				switch(tmp->nType)
				{
					case CET_WORD_WORD:
						if(tmp->nDataType == TYPE_HEXSEQUENCE) strEntryData.Quote(TRUE, '[', ']');
						s1.Format("%s %s%s;\r\n", strEntryName, strEqual, strEntryData);
						break;

					case CET_WORD_LIST:
						if(tmp->nDataType == TYPE_ARGUMENT)
							s1.Format("%s(%s)\r\n", strEntryName, strEntryData);
						else
							s1.Format("%s %s{%s}\r\n", strEntryName, strEqual, strEntryData);
						break;

					case CET_LIST_WORD:
						s1.Format("{%s} %s%s;\r\n", strEntryName, strEqual, strEntryData);
						break;

					case CET_LIST_LIST:
						s1.Format("{%s} %s{%s}\r\n", strEntryName, strEqual, strEntryData);
						break;

					case CET_SECTION:
						if(tmp->szEntryData == NULL)
						{
							if(tmp->prev != NULL)
								s1.Format("\r\n%s%s\r\n%s{\r\n", spaces, strEntryName, spaces);
							else
								s1.Format("%s\r\n%s{\r\n", strEntryName, spaces);
							s2 = spaces + "}\r\n";
						}
						else
						{
							if(tmp->prev != NULL)
								s1.Format("\r\n%s%s %s%s\r\n%s{\r\n", spaces, strEntryName, strEqual, strEntryData, spaces);
							else
								s1.Format("%s %s%s\r\n%s{\r\n", strEntryName, strEqual, strEntryData, spaces);
							s2 = spaces + "}\r\n";
						}
						break;
				}
				break;
			}

			////////////////////////////////////////////////
			// 
			////////////////////////////////////////////////
			case PCT_CONFIG_STRUCT:
			{
				switch(tmp->nType)
				{
					case CET_WORD_WORD:
					case CET_LIST_WORD:
					case CET_LIST_LIST:
					case CET_WORD_LIST:
						s1.Format("%s\r\n", tmp->GetEntryTypeDataStr());
						break;

					case CET_SECTION:
						if(tmp->prev != NULL)
							s1.Format("\r\n%s%s : %d,%d\r\n%s{\r\n", spaces, tmp->GetEntryTypeDataStr(), tmp->child.nCount, tmp->child.nNode, spaces);
						else
						{
							EString sss = "qwery";
							s1.Format("%s : %d,%d\r\n%s{\r\n", tmp->GetEntryTypeDataStr(), tmp->child.nCount, tmp->child.nNode, spaces);
						}

						s2 = spaces + "}\r\n";
					break;
				}
				break;
			}

			////////////////////////////////////////////////
			// 
			////////////////////////////////////////////////
			case PCT_ELEMENT_ADDICTION:
			{
				s11 = "";
				EString h1, h2, h3, h4, h5, h6, h7, h8;
				EString hNameData;

				switch(tmp->nType)
				{
					case CET_WORD_WORD:
					{
						if(tmp->nDataType == TYPE_HEXSEQUENCE) strEntryData.Quote(TRUE, '[', ']');
						hNameData.Format("%s = %s\r\n", strEntryName, strEntryData);
						break;
					}

					case CET_WORD_LIST:
					{
						if(tmp->nDataType == TYPE_ARGUMENT)
							hNameData.Format("%s(%s)\r\n", strEntryName, strEntryData);
						else
							hNameData.Format("%s = {%s}\r\n", strEntryName, strEntryData);
						break;
					}

					case CET_LIST_WORD:
					{
						hNameData.Format("{%s} = %s\r\n", strEntryName, strEntryData);
						break;
					}

					case CET_LIST_LIST:
					{
						hNameData.Format("{%s} = {%s}\r\n", strEntryName, strEntryData);
						break;
					}

					case CET_SECTION:
					{
						if(tmp->szEntryData == NULL)
						{
							if(tmp->prev != NULL)
								hNameData.Format("\r\n%s%s\r\n", spaces, strEntryName);
							else
								hNameData.Format("%s\r\n", strEntryName);
						}
						else
						{
							if(tmp->prev != NULL)
								hNameData.Format("\r\n%s%s = %s\r\n", spaces, strEntryName, strEntryData);
							else
								hNameData.Format("%s = %s\r\n", strEntryName, strEntryData);
						}
						break;
					}
				}

				h1.Format("%s%s\r\n", spaces, _ConfigEntryTypeStr[tmp->nType]);
				h2.Format("%s%s\r\n", spaces, tmp->GetEntryTypeDataStr());
				h4 = GetListStr(tmp, TRUE);
				h5 = GetListStr(tmp, FALSE);
				s11 += hNameData + h1 + h2 + h4 + h5;


/*
				s11 = "";

				EString h1, h2, h3, h4, h5, h6, h7, h8, h9;
				EString a5, a6, a7;
				char str[12];
				
				h1.Format("0x%.8X, %s, level %d\r\n", tmp, _ConfigEntryTypeStr[tmp->nType], tmp->nLevel);
				h2.Format("%spCePrev     = %s\r\n", spaces, _EcxPrtToStr(str, tmp->prev));
				h3.Format("%spCeNext     = %s\r\n", spaces, _EcxPrtToStr(str, tmp->next));
				h4.Format("%spCeAlt      = %s\r\n", spaces, _EcxPrtToStr(str, tmp->child.first));
				h5.Format("%spCeLast     = %s\r\n", spaces, _EcxPrtToStr(str, tmp->child.last));
				h6.Format("%spCeParent   = %s\r\n", spaces, _EcxPrtToStr(str, tmp->parent));
//				h7.Format("%spLstData    = %s\r\n", spaces, _EcxPrtToStr(str, tmp->pLstData));
				if(tmp->pLstData) h8.Format("%sLIST        = %d\r\n", spaces, tmp->pLstData->nCount);
				a6.Format("%sszEntryName = %s\r\n", spaces, strEntryName);
				a7.Format("%sszEntryData = %s\r\n", spaces, strEntryData);

				s11 += h1 + h2 + h3 + h4 + h5 + h6 + h7 + h8;
				s11 += a6 + a7;
				s11 += "\r\n";
*/
				s11 += "\r\n";
				s1 = s11;
				break;
			}

		}

		m_strOutput += spaces + s1;
		if(tmp->child.first != NULL) _Print(tmp->child.first, nPrintCfgType);
		m_strOutput += s2;

		tmp = tmp->next;

	} while(tmp);
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline void EConfig::SetTree(CONFIGENTRY* pCe, HWND hTreeCtrl)
{	
	if(pCe == NULL) pCe = m_base.first;
	if(pCe == NULL) return;

	_EcxTreeView_DeleteAllItems(hTreeCtrl);
	_SetTree(pCe, hTreeCtrl, TVI_ROOT);
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline void EConfig::_SetTree(CONFIGENTRY* pCe, HWND hTreeCtrl, HTREEITEM hParent)
{
	CONFIGENTRY* tmp = pCe;
	HTREEITEM hCurr;

	do
	{
		hCurr = _EcxTreeView_InsertItem(hTreeCtrl, GetEntryStr(tmp, 1), hParent);
		_EcxTreeView_SetItemImage(hTreeCtrl, hCurr, tmp->nType, tmp->nType);
		_EcxTreeView_SetItemData(hTreeCtrl, hCurr, (DWORD)tmp);

		if(tmp->child.first != NULL) _SetTree(tmp->child.first, hTreeCtrl, hCurr);
		tmp = tmp->next;

	} while(tmp);
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline EString EConfig::GetEntryStr(CONFIGENTRY* pCe, int nMode)
{
	if(pCe == NULL) pCe = m_base.first;
	if(pCe == NULL) return "";

	EString str;
	CONFIGENTRY* tmp = pCe;

	EString strEntryName = GetEntryNameStr(tmp);
	EString strEntryData = GetEntryDataStr(tmp, nMode);

	switch(nMode)
	{
		case 0:
		{
			EString p1, p2, p3, p4, p5, p6, br = "\r\n";
			EString a1, a2, a3, a4, a5, a6, a7, a8, n1, n2;

			p1.Format("0x%.8X\r\n", tmp);
			p2.Format("prev     = %s\r\n", GetEntryPtrStr(tmp->prev));
			p3.Format("next     = %s\r\n", GetEntryPtrStr(tmp->next));
			p4.Format("child.first      = %s\r\n", GetEntryPtrStr(tmp->child.first));
			p5.Format("child.last     = %s\r\n", GetEntryPtrStr(tmp->child.last));
			p6.Format("parent   = %s\r\n", GetEntryPtrStr(tmp->parent));

			a1.Format("nType       = %s\r\n", tmp->GetEntryTypeStr(FALSE));
			a2.Format("nLevel      = %d\r\n", tmp->nLevel);
			a3.Format("nCount      = %d\r\n", tmp->child.nCount);
			a4.Format("nSecCount   = %d\r\n", tmp->child.nNode);

			a5.Format("szEntryName      = %s\r\n", strEntryName);
			a6.Format("szEntryData      = %s\r\n", strEntryData);
			a7.Format("nNameType        = %s\r\n", tmp->GetDataTypeStr(FALSE, FIELD_NAME));
			a8.Format("nDataType        = %s\r\n", tmp->GetDataTypeStr(FALSE, FIELD_DATA));
			n1.Format("nEntryNameLength = %d\r\n", tmp->nEntryNameLength);
			n2.Format("nEntryDataLength = %d\r\n", tmp->nEntryDataLength);

			EString b1, b2;
			b1.Format("%s:  ", tmp->GetEntryTypeDataStr());
			b2.Format("%s\r\n\r\n", GetEntryStr(tmp, 1));

			str = p1 + b1 + b2 + a1 + a2 + a3 + a4 + br + p2 + p3 + p4 + p5 + p6 + br + a5 + a6 + a7 + a8 + n1 + n2;
			break;
		}

		case 1:
		{
			EString strEqual = (m_isSetEqualSign) ? "= " : "";

			switch(tmp->nType)
			{
				case CET_WORD_WORD:
					if(tmp->nDataType == TYPE_HEXSEQUENCE) strEntryData.Quote(TRUE, '[', ']');
					str.Format("%s %s%s", strEntryName, strEqual, strEntryData);
					break;

				case CET_WORD_LIST:
					if(tmp->nDataType == TYPE_ARGUMENT)
						str.Format("%s(%s)", strEntryName, strEntryData);
					else
						str.Format("%s %s{%s}", strEntryName, strEqual, strEntryData);
					break;

				case CET_LIST_WORD:
					str.Format("{%s} %s%s", strEntryName, strEqual, strEntryData);
					break;

				case CET_LIST_LIST:
					str.Format("{%s} %s{%s}", strEntryName, strEqual, strEntryData);
					break;

				case CET_SECTION:
					if(tmp->szEntryData == NULL)
						str.Format("%s", strEntryName);
					else
						str.Format("%s %s%s", strEntryName, strEqual, strEntryData);
					break;
			}
			break;
		}
	}


	return str;

}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline EString EConfig::GetEntryNameStr(CONFIGENTRY* pCe)
{
	if(pCe == NULL) return "";

	EString str = pCe->szEntryName;
	if(pCe->nNameType == TYPE_STRING) str.Quote();
	if(pCe->nNameType == TYPE_NULL) str = "null";

	return str;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline EString EConfig::GetEntryDataStr(CONFIGENTRY* pCe, int nMode)
{
	if(pCe == NULL) return "";

//	EString str = pCe->szEntryData;
//	if(pCe->nDataType == TYPE_STRING) str.Quote();
//	if(pCe->nDataType == TYPE_NULL) str = "null";

	EString str;
	if(pCe->szEntryData == NULL)
		str = (nMode == 1) ? "null" : "NULL";
	else
	{
		str = pCe->szEntryData;
		if(pCe->nDataType == TYPE_STRING) str.Quote();
		if(pCe->nDataType == TYPE_NULL) str = "null";
	}

	return str;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline EString EConfig::GetEntryPtrStr(CONFIGENTRY* pCe)
{
	EString str = "NULL";
	if(pCe != NULL)
	{
		str.Format("0x%.8X:  %s", (DWORD)pCe, GetEntryStr(pCe, 1));
	}

	return str;
}


////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////
inline EString EConfig::GetSysVarStr()
{
	EString p1, p2;
	EString a1, a2, a3, a4, a5, a6, a7, a8;


	a7.Format("Parse at: %f ms\r\n", hrt.GetTime() * 1000.0);
//	p1.Format("m_base\r\n   first  = %s\r\n", GetEntryPtrStr(m_base.first));
//	p2.Format("   last   = %s\r\n", GetEntryPtrStr(m_base.last));
	a1.Format("count = %d\r\n", m_base.nCount);
	a2.Format("m_nLength = %d\r\n", m_nLength);
	a3.Format("m_isDeleteNullEntry = %s\r\n", _EcxBoolToStr(m_isDeleteNullEntry));
	a4.Format("m_isSetEqualSign = %s\r\n", _EcxBoolToStr(m_isSetEqualSign));
	a5.Format("m_isWideChar = %s\r\n", _EcxBoolToStr(m_isWideChar));

	EString str = a7 + p1 + p2 + a1 + a2 + a3 + a4 + a5;
	return str;
}


////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////
inline BOOL EConfig::IsIncorrectEntryType(int nType)
{
	return (nType < 0) || (nType > CET_LIST_LIST);
}


////////////////////////////////////////////////////////////////////////////
//       
////////////////////////////////////////////////////////////////////////////
inline int EConfig::SetNeighborsHasht(CONFIGENTRY* pCe, BOOL isHash)
{
	int count = 0;
	if(pCe)
	{
		CONFIGENTRY* tmp = pCe;
		while(tmp)
		{
			count++;
			tmp = tmp->next;
		} 

		if(isHash)
		{
			pCe->hash = new _ConfigEntry*[count];

			count = 0;
			CONFIGENTRY* tmp = pCe;
			do
			{
				pCe->hash[count++] = tmp;
				tmp = tmp->next;

			} while(tmp);
		}
	}
	return count;
}





/////////////////////////////////////////////////////////////////////////////
#endif // __ECONFIG_H__

