

// TextParsingFunc.h : header file
//

#ifndef __TEXTPARSINGFUNC_H__
#define __TEXTPARSINGFUNC_H__
#pragma once
////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
//  TextParsingFunc.h
/////////////////////////////////////////////////////////////////////////////
#define VERSION_TEXTPARSINGFUNC   "26.11.2016"



/////////////////////////////////////////////////////////////////////////////
//   
/////////////////////////////////////////////////////////////////////////////
#include "..\BufferManipulation\BufferManipulationFunc.h"
#include "..\SystemInformation\SystemInformationFunc.h"
#include "..\CharacterClassification\EBitArray.h"
#include "TextParsing.h"
#include "TextParsingMacro.h"



/////////////////////////////////////////////////////////////////////////////
//  
/////////////////////////////////////////////////////////////////////////////
BOOL _EcxSkipNonInformation(char**, PFN_SKIPNONINFOPROC, int nSkip);
BOOL _EcxSkipString(char** ppBuf, BOOL isCat);
int _EcxGetString(char** ppBuf, char* dest, BOOL isCat, BOOL isExpEsc, BOOL isExpEnv, PFN_EXPANDENVIRONMENTPROC ExpandEnvironmentProc = NULL, void* pData = NULL, BOOL isQuote = FALSE);




////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////
inline int _EcxBrCompare(char* pBuf, char* pBr, int nCount, BLOCK_INFO* pBinf)
{
	BOOL is = TRUE;

	return _EcxMemCompare(pBuf, pBr, nCount, pBinf->isCase) && is;
}


////////////////////////////////////////////////////////////////////////////
//   pBinf->lpBr1...lpBinf->pBr2
////////////////////////////////////////////////////////////////////////////
inline BOOL _EcxFindBlock(BLOCK_INFO* pBinf)
{
	pBinf->pBeg = NULL;
	char* tmp = pBinf->pBuff;

	int k, br = 0;


	//    (...), [...], {...}, <...>
	if(pBinf->lpBr1 == NULL && pBinf->lpBr2 == NULL)
	{
		char br1 = *tmp, br2;

		//        
		switch(br1)
		{
			case '(': br2 = ')'; break;
			case '[': br2 = ']'; break;
			case '{': br2 = '}'; break;
			case '<': br2 = '>'; break;
			default: return FALSE;
		}

		//    
		while(*tmp)
		{
			if(*tmp == br1) br++;
			if(*tmp == br2) br--;
			tmp++;

			//  
			if(br == 0) break;
		}
	}

	//    lpBr1...lpBr2 (: begin...end)
	else
	{
		int len1 = strlen(pBinf->lpBr1);
		int len2 = strlen(pBinf->lpBr2);
		if(len1 == 0 || len2 == 0) return FALSE;
		
		//    
		if(_EcxBrCompare(tmp, pBinf->lpBr1, len1, pBinf) != 0) return FALSE;

		//    
		while(*tmp)
		{
			k = 1;
			if(_EcxBrCompare(tmp, pBinf->lpBr1, len1, pBinf) == 0) { br++; k = len1; }
			if(_EcxBrCompare(tmp, pBinf->lpBr2, len2, pBinf) == 0) { br--; k = len2; }
			tmp += k;

			//  
			if(br == 0) break;
		}
	}


	//  
	if(br == 0)
	{
		pBinf->pBeg = pBinf->pBuff;
		pBinf->nLength = tmp - pBinf->pBeg;
		return TRUE;
	}

	return FALSE;
}


////////////////////////////////////////////////////////////////////////////
//    pBinf->lpBr1...lpBinf->pBr2
//   TRUE,   
////////////////////////////////////////////////////////////////////////////
inline BOOL _EcxSkipBlock(char** ppBuf, BLOCK_INFO* pBinf)
{
	char* tmp = *ppBuf;
	pBinf->pBuff = tmp;

	//  
	BOOL res = _EcxFindBlock(pBinf);
	tmp += pBinf->nLength;

	*ppBuf = tmp;
	return res;
}


////////////////////////////////////////////////////////////////////////////
//     BR1 COMMENT BR2
//   TRUE,   
////////////////////////////////////////////////////////////////////////////
inline BOOL _EcxSkipBlockComment(char** ppBuf, int nType, char* pBr1 = NULL, char* pBr2 = NULL)
{
	BOOL isPredefinedComment = FALSE;
	BLOCK_INFO binf;
	binf.nType = nType;

	//   
	switch(nType)
	{
		case SKIP_MULTYLINE_COMMENT:
		case SKIP_HTML_COMMENT:
		case SKIP_PASCAL_COMMENT:
		case SKIP_COMMENT3:
		case SKIP_COMMENT4:
		{
			//  
			isPredefinedComment = TRUE;
			break;
		}

		default:
		{
			if((pBr1 == NULL) || (pBr2 == NULL)) return FALSE;
			binf.lpBr1 = pBr1;
			binf.lpBr2 = pBr2;
			binf.nType = 0;
			break;
		}
	}

	//    
	if(isPredefinedComment)
	{
		binf.isDelete = TRUE;
		binf.lpBr1 = new char[16];
		binf.lpBr2 = new char[16];

		//   
		switch(nType)
		{
			case SKIP_MULTYLINE_COMMENT:	{ strcpy(binf.lpBr1, "/*");		strcpy(binf.lpBr2, "*/");	break; }
			case SKIP_HTML_COMMENT:			{ strcpy(binf.lpBr1, "<!--");	strcpy(binf.lpBr2, "-->");	break; }
			case SKIP_PASCAL_COMMENT:		{ strcpy(binf.lpBr1, "(*");		strcpy(binf.lpBr2, "*)");	break; }
			case SKIP_COMMENT3:				{ strcpy(binf.lpBr1, "{");		strcpy(binf.lpBr2, "}");	break; }
			case SKIP_COMMENT4:				{ strcpy(binf.lpBr1, "<-");		strcpy(binf.lpBr2, "->");	break; }
		}
	}

	//   
	return _EcxSkipBlock(ppBuf, &binf);
}


////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////
inline BOOL _EcxSkipLine(char** ppBuf, BOOL isSkipEOL = TRUE)
{
	BOOL res = FALSE;
	char* tmp = *ppBuf;

	//   
	while(1)
	{
		//    
		if(is_eot(tmp))
		{
			res = TRUE;
			break;
		}

		//   [\x00\x0A\x0D\x1A]
		while(!is_char(*tmp, CCM_STOP)) tmp++;

		//    -  
		if(*(tmp - 1) == 0x5C)
		{
			tmp--;

			//      ( \ BR )
			_EcxSkipNonInformation(&tmp, NULL, SKIP_NEWLINE_BREAK);
			continue;
		}
		else
		{
			if(isSkipEOL) skip_eol(tmp);
			break;
		}
	}

	*ppBuf = tmp;
	return res;
}



////////////////////////////////////////////////////////////////////////////
//     
//   TRUE,      \EOT
////////////////////////////////////////////////////////////////////////////
inline BOOL _EcxSkipNewlineBreak(char** ppBuf)
{
	//   \EOL  \EOT
	while(1)
	{
		WORD word = *(WORD*)*ppBuf;

		//      - \EOL
		if(word == 0x0D5C || word == 0x0A5C)
		{
			(*ppBuf)++;
			skip_eol(*ppBuf);
			continue;
		}

		//      - \EOT
		if(word == 0x005C || word == 0x1A5C)
		{
			(*ppBuf)++;
			return TRUE;
		}
		return FALSE;
	}

}



////////////////////////////////////////////////////////////////////////////
//     
////////////////////////////////////////////////////////////////////////////
static int g_nLastSkipNonInfoType;
static BOOL g_isNewLineBreakPresent;



////////////////////////////////////////////////////////////////////////////
//     
//   TRUE,   
////////////////////////////////////////////////////////////////////////////
inline BOOL CALLBACK _EcxSkipNonInfoProc(char** ppBuf, int nSkip)
{
	char* tmp = *ppBuf;
	BOOL res = FALSE;



/*
	char br1[] = "QwertY";
	char br2[] = "Ass";
	BLOCK_INFO binf = { 0 };
	binf.lpBr1 = br1;
	binf.lpBr2 = br2;
	binf.isCase = TRUE;


	BOOL is = _EcxSkipBlock(&tmp, &binf);
	if(is)
	{
		res = TRUE;
	}
*/


	*ppBuf = tmp;
	return res;
}


////////////////////////////////////////////////////////////////////////////
//    -   
//   TRUE,    
////////////////////////////////////////////////////////////////////////////
inline BOOL _EcxSkipNonInformation(char** ppBuf, PFN_SKIPNONINFOPROC pfnSkip = NULL, int nSkip = SKIP_BASE)
{
	BOOL res = FALSE;
	WORD word;
	char* tmp = *ppBuf;
	g_isNewLineBreakPresent = FALSE;


	//   
	while(1)
	{
		/////////////////////////////////////
		//    :   [\x00\x1A]
		/////////////////////////////////////
		if(is_eot(tmp))
		{
			res = TRUE;
			break;
		}

		/////////////////////////////////////
		//  :    [\v\f\t\s\xA0]  [\x0B\x0C\x09\x20\xA0]
		/////////////////////////////////////
		if(nSkip & SKIP_BLANK)
		{
			if(is_char(*tmp, CCM_BLANK))
			{
				skip_blank(tmp);
				g_nLastSkipNonInfoType = SKIP_BLANK;
				continue;
			}
		}

		/////////////////////////////////////
		//   :    [\n\r] [\x0A\x0D]
		/////////////////////////////////////
		if(nSkip & SKIP_EOL)
		{
			if(is_eol(tmp))
			{
				int n;
				while(n = is_eol(tmp)) tmp += n;
				g_nLastSkipNonInfoType = SKIP_EOL;
				continue;
			}
		}


		//   
		word = *(WORD*)tmp;


		/////////////////////////////////////
		//   :    // COMMENT BR
		/////////////////////////////////////
		if(nSkip & SKIP_ONELINE_COMMENT)
		{
			if(word == 0x2F2F)
			{
				_EcxSkipLine(&tmp, _is_flag(nSkip, SKIP_EOL));
				g_nLastSkipNonInfoType = SKIP_ONELINE_COMMENT;
				continue;
			}
		}

		/////////////////////////////////////
		//   :    ' COMMENT BR
		// Basic, VB script
		/////////////////////////////////////
		if(nSkip & SKIP_VBSCRIPT_COMMENT)
		{
			if(*tmp == 0x27)
			{
				skip_line(tmp);
				g_nLastSkipNonInfoType = SKIP_VBSCRIPT_COMMENT;
				continue;
			}
		}

		/////////////////////////////////////
		//   :    REM CH COMMENT BR
		// DOS Batch Files (.bat .cmd)
		/////////////////////////////////////
		if(nSkip & SKIP_BATCH_COMMENT)
		{
			DWORD dword = (*(DWORD*)tmp) & 0x00FFFFFF | 0x00202020;

			if(dword == 0x006D6572)
			{
				char CH = *(tmp + 3);
				BOOL res1 = is_char(CH, CCM_WORD);
				BOOL res2 = _EcxIsCharInSet(CH, "`~@$%^*)-\\}\"'?");

				if(res1 == FALSE && res2 == FALSE)
				{
					skip_line(tmp);
					g_nLastSkipNonInfoType = SKIP_BATCH_COMMENT;
					continue;
				}
			}
		}

		/////////////////////////////////////
		//   :    # COMMENT BR
		// Mathematica Text Resources (.tr)
		// Code-Genie Syntax Files (.cgsyn)
		// Code-Genie Configuration File (cgenie.conf)
		// Unicode
		/////////////////////////////////////
		if(nSkip & SKIP_COMMENT1)
		{
			if(*tmp == 0x23)
			{
				skip_line(tmp);
				g_nLastSkipNonInfoType = SKIP_COMMENT1;
				continue;
			}
		}

		/////////////////////////////////////
		//   :    ; COMMENT BR
		// Windows 9x Themes (.theme)
		// Windows Registry Files (.reg)
		// Windows INI Files (.ini)
		// Unicode
		/////////////////////////////////////
		if(nSkip & SKIP_COMMENT2)
		{
			if(*tmp == 0x3B)
			{
				skip_line(tmp);
				g_nLastSkipNonInfoType = SKIP_COMMENT2;
				continue;
			}
		}

		/////////////////////////////////////
		//   :    /* COMMENT */
		/////////////////////////////////////
		if(nSkip & SKIP_MULTYLINE_COMMENT)
		{
			if(word == 0x2A2F)
			{
				_EcxSkipBlockComment(&tmp, SKIP_MULTYLINE_COMMENT);
				g_nLastSkipNonInfoType = SKIP_MULTYLINE_COMMENT;
				continue;
			}
		}

		/////////////////////////////////////
		//   :    <!-- COMMENT -->
		// HTML, XML
		/////////////////////////////////////
		if(nSkip & SKIP_HTML_COMMENT)
		{
			if(*(DWORD*)tmp == 0x2D2D213C)
			{
				_EcxSkipBlockComment(&tmp, SKIP_HTML_COMMENT);
				g_nLastSkipNonInfoType = SKIP_HTML_COMMENT;
				continue;
			}
		}

		/////////////////////////////////////
		//   :    (* COMMENT *)
		// Mathematica Files (.m .nb)
		// Pascal
		/////////////////////////////////////
		if(nSkip & SKIP_PASCAL_COMMENT)
		{
			if(word == 0x2A28)
			{
				_EcxSkipBlockComment(&tmp, SKIP_PASCAL_COMMENT);
				g_nLastSkipNonInfoType = SKIP_PASCAL_COMMENT;
				continue;
			}
		}

		/////////////////////////////////////
		//   :    { COMMENT }
		// SASPlanet Script (GetUrlScript.txt)
		// Pascal
		/////////////////////////////////////
		if(nSkip & SKIP_COMMENT3)
		{
			if(*tmp == 0x7B)
			{
				_EcxSkipBlockComment(&tmp, SKIP_COMMENT3);
				g_nLastSkipNonInfoType = SKIP_COMMENT3;
				continue;
			}
		}

		/////////////////////////////////////
		//   :    <- COMMENT ->
		// Gentee Text (.gt)
		/////////////////////////////////////
		if(nSkip & SKIP_COMMENT4)
		{
			if(word == 0x2D3C)
			{
				_EcxSkipBlockComment(&tmp, SKIP_COMMENT4);
				g_nLastSkipNonInfoType = SKIP_COMMENT4;
				continue;
			}
		}

		/////////////////////////////////////
		//     :   \EOL  \EOT
		/////////////////////////////////////
		if(nSkip & SKIP_NEWLINE_BREAK)
		{
			//   \EOL
			if(word == 0x0D5C || word == 0x0A5C)
			{
				tmp++;
				skip_eol(tmp);
				g_nLastSkipNonInfoType = SKIP_NEWLINE_BREAK;
				g_isNewLineBreakPresent = TRUE;
				continue;
			}

			//   \EOT
			if(word == 0x005C || word == 0x1A5C)
			{
				tmp++;
				g_nLastSkipNonInfoType = SKIP_NEWLINE_BREAK;
				g_isNewLineBreakPresent = TRUE;
				continue;
			}
		}

		/////////////////////////////////////
		//   
		/////////////////////////////////////
		if(nSkip & SKIP_USE_STATE_FUNC)
		{
			if(_EcxSkipNonInfoProc(&tmp, nSkip)) continue;
		}

		/////////////////////////////////////
		//  
		/////////////////////////////////////
		if(nSkip & SKIP_USE_CUSTOM_FUNC)
		{
			if(pfnSkip != NULL)
			{
				if(pfnSkip(&tmp, nSkip)) continue;
			}
		}

		/////////////////////////////////////
		//   
		/////////////////////////////////////
		if(nSkip & (SKIP_STRING | SKIP_STRING_CAT))
		{
			if(*tmp == '\x22' || *tmp == '\x27')
			{
				_EcxSkipString(&tmp, nSkip & SKIP_STRING_CAT);
				g_nLastSkipNonInfoType = SKIP_STRING;
				continue;
			}
		}

		break;
	}

	*ppBuf = tmp;
	return res;
}



////////////////////////////////////////////////////////////////////////////
//     
////////////////////////////////////////////////////////////////////////////
inline int _EcxClearNonInformation(char* dest, char* src, PFN_SKIPNONINFOPROC pfnSkip = _EcxSkipNonInfoProc, int nSkip = SKIP_BASE, BOOL isIgnoreStrings = FALSE)
{
	int nLength = 0;
	char* tmp = dest;

	//    -     
	if(isIgnoreStrings) _remove_flag(nSkip, SKIP_STRING|SKIP_STRING_CAT);

	while(1)
	{
		//   
		if(_EcxSkipNonInformation(&src, pfnSkip, nSkip)) break;

		//        
		if(*src == '\x22' && isIgnoreStrings)
		{
			//  
			int n = _EcxGetString(&src, tmp, TRUE, FALSE, FALSE, NULL, NULL, TRUE);
			tmp += n;
			nLength += n;
			continue;
		}

		*tmp++ = *src++;
		nLength++;
	}
	dest[nLength] = '\0';

	return nLength;
}


////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////
inline int _EcxGetString(char** ppBuf, char* dest, BOOL isCat, BOOL isExpEsc, BOOL isExpEnv, PFN_EXPANDENVIRONMENTPROC ExpandEnvironmentProc, void* pData, BOOL isQuote)
{
	//   
	int nLength = 0;

	//    
	BOOL isDest = (dest != GSTR_SKIP) && (dest != GSTR_LENGTH);

	char* tmp = *ppBuf;
	char* d0 = dest;
	if(isDest) dest[0] = '\0';

	// ;    ;  
	char ch = *tmp;
	if(ch != '\x22' && ch != '\x27' && ch != '<') return NULL;
	tmp++;
	char ch1 = (ch == '<') ? '>' : ch;

	//    -     
	if(isDest && isQuote) dest++;

	while(1)
	{
		if(isDest) *dest = *tmp;

		//  
		if(is_eot(tmp)) break;

		//     ( "  '  > )
		if(*tmp == ch1)
		{
			if(isDest) *dest = '\0';

SKIP_PLUS:
			tmp++;
			char* p0 = tmp;
			_EcxSkipNonInformation(&p0, NULL, SKIP_BASE);

			//   ("str1" + "str2" + ... + "strN")
			//   ("str1" & "str2" & ... & "strN")
			if(isCat && (*p0 == '+' || *p0 == '&'))
			{
				goto SKIP_PLUS;
			}

			//   ("str1" "str2" ... "strN")
			if(isCat && *p0 == ch)
			{
				tmp = p0;
				tmp++;
				continue;
			}

			//    -     
			if(isDest && isQuote)
			{
				*d0 = ch;
				*dest++ = ch;
				*dest = '\0';
				nLength += 2;
			}
			break;
		}

		//        
		if(*tmp == '%' && isExpEnv)
		{
			//    
			char* v0 = tmp++;
			skip_to_char(tmp, '%');
			if(*tmp == '%') tmp++;

			//    
			if(dest == GSTR_SKIP) continue;

			char* env =  new char[MAX_PATH];
			int exp_env_len, env_len = tmp - v0;
			memcpy(env, v0, env_len);
			env[env_len] = '\0';

			exp_env_len = _EcxExpandEnvironment(env, NULL, NULL, ExpandEnvironmentProc, pData);

			//    
			if(isDest)
			{
				char* exp_env = new char[exp_env_len + 1];
				exp_env_len = _EcxExpandEnvironment(env, exp_env, NULL, ExpandEnvironmentProc, pData);

				memcpy(dest, exp_env, exp_env_len);
				dest += exp_env_len;

				delete [] exp_env;
			}

			nLength += exp_env_len;
			delete [] env;
			continue;
		}

		//  Esc-
		if(*tmp == '\\')
		{
			//     
			_EcxSkipNonInformation(&tmp, NULL, SKIP_NEWLINE_BREAK);
			if(*tmp != '\\') continue;
//			if(g_isNewLineBreakPresent) continue;
			
			//    Esc-
			if(isExpEsc)
			{
				BYTE esc = _EcxEscapeToChar(&tmp);
				if(isDest) *dest++ = esc;
				nLength++;
				continue;
			}
		}

		if(isDest) dest++;
		tmp++;
		nLength++;
	}

	if(dest != GSTR_LENGTH) *ppBuf = tmp;
	return nLength;
}


////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////
inline BOOL _EcxSkipString(char** ppBuf, BOOL isCat)
{
	char* tmp = *ppBuf;
	char* p0 = tmp;
	_EcxGetString(&tmp, GSTR_SKIP, isCat, TRUE, FALSE);
	*ppBuf = tmp;
	return (p0 != tmp);
}




////////////////////////////////////////////////////////////////////////////
//      lpString
////////////////////////////////////////////////////////////////////////////
inline BOOL _EcxSkipFindedString(char** ppBuf, LPCSTR lpString)
{
	char* tmp = strstr(*ppBuf, lpString);
	if(tmp)
	{
		tmp += strlen(lpString);
		*ppBuf = tmp;
	}
	return (BOOL)tmp;
}






/////////////////////////////////////////////////////////////////////////////
// 
/////////////////////////////////////////////////////////////////////////////
inline char _EcxTestReal(char* buf, int* pType, BOOL isAst = FALSE)
{
	char *tmp = buf;
	BOOL isS = FALSE, isP = FALSE, isN = FALSE, isF = FALSE, isE = FALSE;
	int type = 0;
	if(pType) *pType = type;
	if(is_eot(tmp)) return 0;

	//   +-
	if(*tmp == '+' || *tmp == '-')
	{
		isS = TRUE;
		tmp++;
		if(is_eot(tmp)) return 0;
	}
	//   Dn
	if(is_char(*tmp, CCM_DEC))
	{
		isN = TRUE;
		skip_class(tmp, CCM_DEC);
		if(is_eot(tmp)) return (isS) ? 'N' : 'U';
	}
	//  .
	if(*tmp == '.')
	{
		isP = TRUE;
		tmp++;
		if(isN)
		{
			type = 1;
		}
		if(is_eot(tmp))
		{
			if(type)
			{
				if(pType) *pType = type;
				return 'R';
			}
			return 0;
		}
	}
	//   Df
	if(is_char(*tmp, CCM_DEC))
	{
		isF = TRUE;
		skip_class(tmp, CCM_DEC);
		if(isP)
		{
			type = 2;
			if(isN)
			{
				type = 3;
			}
		}
		if(is_eot(tmp))
		{
			if(type)
			{
				if(pType) *pType = type;
				return 'R';
			}
			return 0;
		}
	}
	//  real-: (0) Dn; (3) [+/-]Dn.Df; (2) [+/-]Dn.; (1) [+/-].Df; (+4) e/E[+/-]De;
	if(type || isN)
	{
		//  
		if(*tmp == 'e' || *tmp == 'E')
		{
			isE = TRUE;
			tmp++;
			if(is_eot(tmp)) return 0;
		}
		//  
		if(*tmp == '+' || *tmp == '-')
		{
			tmp++;
			if(is_eot(tmp)) return 0;
		}
		//  De
		if(is_char(*tmp, CCM_DEC))
		{
			skip_class(tmp, CCM_DEC);
			type += 4;
			if(is_eot(tmp))
			{
				if(isE)
				{
					if(pType) *pType = type;
					return 'E';
				}
			}
		}
	}
	// *
	if(isAst)
	{
		if(type || isN)
		{
			if(*tmp == '*')
			{
				while(*tmp == '*') tmp++;
				if(is_eot(tmp))
				{
					if(type)
					{
						if(pType) *pType = type;
						return (type >= 4) ? 'E' : 'R';
					}
					if(isN)
					{
						return (isS) ? 'N' : 'U';
					}
				}
			}
		}
	}
	return 0;

}

inline char _EcxTestInteger(char* buf, int& prefsuff)
{
	char *p, *p1, *tmp = buf;
	char ch[2];
	ch[1] = '\0';
	prefsuff = 0;

	//    
	if(*tmp == '0')
	{
		//  
		*ch = *(tmp + 1);

		//   : b - bin; o,c - oct; d - dec; x - hex
		if(strpbrk("bocdx", ch) != NULL)
		{
			tmp += 2;
			prefsuff = 1;
		}
	}

	//   
	p = tmp;
	while(!is_char(*p, CCM_SPACE|CCM_SYMBOL|CCM_EOT)) p++;

	//   
	if(prefsuff == 1)
	{
		p1 = p;
		goto LABEL_SCAN1;
	}

	//  
	*ch = *(p - 1);

	//   
	if(strpbrk("bocdh", ch) != NULL)
	{
		prefsuff = 2;
	}

	//   ,  
	if(prefsuff == 0) goto LABEL_SCAN2;


LABEL_SCAN1:

	if(p == tmp) return 0;

	//     bin
	if(*ch == 'b')
	{
		p = tmp;
		skip_class(p, CCM_BIN);
	}

	//     oct
	if(*ch == 'o' || *ch == 'c')
	{
		p = tmp;
		skip_class(p, CCM_OCT);
	}

	//     dec
	if(*ch == 'd')
	{
		p = tmp;
		skip_class(p, CCM_DEC);
	}

	//     hex
	if(*ch == 'h' || *ch == 'x')
	{
		p = tmp;
		skip_class(p, CCM_HEX);
	}

	if(prefsuff == 2)
	{
		if(*ch == *p) goto LABEL_RETURN;
	}
	if(prefsuff == 1)
	{
		if(p == p1) goto LABEL_RETURN;
	}


LABEL_SCAN2:

	//   ,  
	prefsuff = 0;

	//   dec
	tmp = buf;
	skip_class(tmp, CCM_DEC);
	if(tmp == p) return 'd';

	//   bin
	tmp = buf;
	skip_class(tmp, CCM_BIN);
	if(tmp == p) return 'b';

	//    oct
	if(*buf == '0')
	{
		//   oct
		tmp = buf;
		skip_class(tmp, CCM_OCT);
		if(tmp == p) return 'o';
	}

	//   hex
	tmp = buf;
	skip_class(tmp, CCM_HEX);
	if(tmp == p) return 'x';

	return 0;

LABEL_RETURN:

	if(*ch == 'c') *ch = 'o';
	if(*ch == 'h') *ch = 'x';

	return *ch;
}


/////////////////////////////////////////////////////////////////////////////
//     lpszBuffer
/////////////////////////////////////////////////////////////////////////////
inline int _EcxScanV(LPCSTR lpszBuffer, LPCSTR lpszFormat, va_list arglist)
{
	int i, j, k;
	char* tmp;
	char* szFormat;

	EBitArray ba;
	int isWidth;                //   
	int isPrecision;            //   
	int nWidth;                 //  
	int nPrecision;             //  
	int nLongone;               // 0 == short; 1 == long; 2 == ???
	                            // 0 == ???; 1 == float; 2 == double

	int isInteger64;            // 1 == 64-  
	int isNegative;             //   
	int isAssign;               //      
	int isGoodNumber;           //   
	int nNumPrefSuff;           // 0 == ; 1 ==  ; 2 ==  
	char chIntType;
	char chDecimalPoint;
	unsigned __int64 qNumber;   // temp for 64-bit integers          
	char* szBuffer;				// ASCII buffer for floats and other          
	int isWideChar;             // 0 = char, 1 = wchar_t
	int isString;               //     
	int isCat;
	int isExpEsc;
	int isExpEnv;


	register int chBuffer;      //    
	register int chFormat;      //   
	void* pointer;              //    
	int nAssigned;              //    
	int nMatch;                 //     (0 -   )
	int nCharCount;             //   
	va_list arglistsave;        //   arglist





	nCharCount = 1;
	nWidth = nPrecision = 0;
	isWidth = isPrecision = FALSE;
	tmp = (char*)lpszFormat;

	while(*tmp)
	{
		skip_spaces(tmp);
		if(*tmp++ == '%')
		{
			if(*tmp == '*') tmp++;
			__scan_number(tmp, isWidth, nWidth);
			if(*tmp == '.')
			{
				tmp++;
				__scan_number(tmp, isPrecision, nPrecision);
			}

			if(*tmp == '[')
			{
				nCharCount += (isPrecision) ? nPrecision : 0;
				i = ba.GetSetLength(tmp) * 3;
				tmp += i;
				if(is_char(*tmp, CCM_EOT)) break;
			}
		}
		tmp++;
	}

	szFormat = new char[2 * nCharCount * strlen(lpszFormat) + MAX_PATH];
	strcpy(szFormat, lpszFormat);
	char* format = szFormat;
	char* pBuffer = (char*)lpszBuffer;

	nAssigned = nCharCount = nMatch = 0;
	szBuffer = new char[400];


	//---------------------------------------------------------------------------------
	//   
	//---------------------------------------------------------------------------------
	while(*format)
	{
		__scan_skip_spaces(pBuffer, nCharCount);
		skip_spaces(format);

		chFormat = *format;
		format++;

		//---------------------------------------------------------------------------------
		//     ( )
		//---------------------------------------------------------------------------------
		if(chFormat == '%')
		{
			qNumber = 0;
			nWidth = nPrecision = 0;
			isWidth = isPrecision = FALSE;
			isGoodNumber = FALSE;
			chDecimalPoint = '.';
			nNumPrefSuff = 0;
			isNegative = FALSE;
			isAssign = TRUE;
			isWideChar = FALSE;
			isString = FALSE;
			nLongone = 1;
			isInteger64 = 0;
			isCat = isExpEsc = isExpEnv = 0;

			//  
			chFormat = *format;


			//---------------------------------------------------------------------------------
			//  ,   c
			//---------------------------------------------------------------------------------

			//     
			if(chFormat == '*')
			{
				isAssign = FALSE;
				format++;
			}

			//   ( )
			__scan_number(format, isWidth, nWidth);
			chFormat = *format;

			//  -  
			if(chFormat == '.')
			{
				//  
				format++;

				//   ( )
				__scan_number(format, isPrecision, nPrecision);
				chFormat = *format;
			}

			format++;

			//  
			switch(chFormat)
			{
				//  h (short)
				case 'h':
				{
					nLongone = 0;
					isWideChar = FALSE;
					break;
				}

				//  I64 (int64)
				case 'I':
				{
					if(*(WORD*)format == 0x3436)  // 64
					{
						format += 2;
						isInteger64 = 1;
						qNumber = 0;
						break;
					}
					break;
				}

				//  f
				// +-----+------------+
				// | bit |  |
				// +-----+------------+
				// |  0  |  isExpEnv  |
				// |  1  |  isExpEsc  |
				// |  2  |  isCat     |
				// +-----+------------+
				case 'f':
				{
					__scan_number(format, j, i);
					if(i & 1) isExpEnv = TRUE;
					if(i & 2) isExpEsc = TRUE;
					if(i & 4) isCat = TRUE;
					break;
				}

				//  L,l (long)
				case 'L':
				case 'l':
					nLongone = 2;
					isWideChar = TRUE;
					break;

				default:
					format--;
			}
			

			//---------------------------------------------------------------------------------
			//      
			//---------------------------------------------------------------------------------
			if(isAssign)
			{
				arglistsave = arglist;
				pointer = va_arg(arglist, void*);
			}
			
			//  
			//    
			chFormat = *format;
			if(is_char(chFormat, CCM_ALPHA)) chFormat |= 0x20;

			// 
			if(chFormat != 'n')
			{
				__scan_inc(pBuffer, chBuffer, nCharCount);
			}


			//---------------------------------------------------------------------------------
			//   
			//---------------------------------------------------------------------------------
			if(!isWidth || nWidth)
			{
				//   
				switch(chFormat)
				{

					//---------------------------------------------------------------------------------
					//  
					//---------------------------------------------------------------------------------
					case 'c':
					{
						isWidth = TRUE;
						nWidth = 1;
						
						// %c  %*c
						k = 200;
						szBuffer[k++] = '%';
						if(!isAssign) szBuffer[k++] = '*';
						strcpy(szBuffer + k, "c");

						// %.Pc --> %c%c%c...
						// %*.Pc --> %*c%*c%*c...
						_EcxRepBuffer(format + 1, szBuffer + 200, nPrecision - 1);

						ba.Filling();
						goto LABEL_SCAN_STRING;
					}
					

					//---------------------------------------------------------------------------------
					//  
					//---------------------------------------------------------------------------------
					case 's':
					{
						// %s  %*s  %fNs  %*fNs
						k = 200;
						szBuffer[k++] = '%';
						if(!isAssign) szBuffer[k++] = '*';
						i = (isCat << 2) + (isExpEsc << 1) + isExpEnv;
						if(i > 0)
						{
							szBuffer[k++] = 'f';
							szBuffer[k++] = '0' + i;
						}
						strcpy(szBuffer + k, "s");
						
						// %.Ps --> %s%s%s...
						// %*.Ps --> %*s%*s%*s...
						// %.PfNs --> %fNs%fNs%fNs...
						// %*.PfNs --> %*fNs%*fNs%*fNs...
						_EcxRepBuffer(format + 1, szBuffer + 200, nPrecision - 1);

						//     
						if(chFormat == 's' && chBuffer == '\"')
						{
							__scan_dec(pBuffer, chBuffer, nCharCount);
							tmp = pBuffer;
							i = _EcxGetString(&tmp, GSTR_LENGTH, isCat, isExpEsc, isExpEnv);
							if(i >= 400)
							{
								delete [] szBuffer;
								szBuffer = new char[i + 1];
							}
							_EcxGetString(&tmp, szBuffer, isCat, isExpEsc, isExpEnv);
							isString = TRUE;
							tmp = pBuffer;
							pBuffer = szBuffer;
							__scan_inc(pBuffer, chBuffer, nCharCount);
						}

						ba.Filling();
						goto LABEL_SCAN_STRING;
					}
					

					//---------------------------------------------------------------------------------
					//  
					//---------------------------------------------------------------------------------
					case '[':
					{
						//   [set]
						i = ba.GetSetLength(format);
						memcpy(szBuffer, format, i);
						szBuffer[i] = '\0';
						
						// %[set]  %*[set]
						k = 200;
						szBuffer[k++] = '%';
						if(!isAssign) szBuffer[k++] = '*';
						strcpy(szBuffer + k, szBuffer);

						// %.P[set] --> %[set]%[set]%[set]...
						// %*.P[set] --> %*[set]%*[set]%*[set]...
						_EcxRepBuffer(format + i, szBuffer + 200, nPrecision - 1);

						// %[~delim] --> %[^delim]%*[delim]
						if(szBuffer[1] == '~')
						{
							szBuffer[1] = '^';
							int len = strlen(format) - i + 1;
							memmove(format + 2 * i + 1, format + i, len);
							memcpy(format + i + 1, szBuffer, i);
							memcpy(format + i, "%*[", 3);
						}

						format += i - 1;
						ba.ParseSet(szBuffer);

//  
LABEL_SCAN_STRING:
						__scan_dec(pBuffer, chBuffer, nCharCount);

						//   
						while(!isWidth || nWidth--)
						{
							__scan_inc(pBuffer, chBuffer, nCharCount);


							if(!is_char(chBuffer, CCM_EOT) && ba.IsBit(chBuffer))
							{
								//      
								if(isAssign)
								{
									if(isWideChar)
									{
										char temp[2];
										temp[0] = (char)chBuffer;
										__scan_inc(pBuffer, temp[1], nCharCount);

										*(wchar_t*)pointer = *(wchar_t*)&temp;
										pointer = (wchar_t*)pointer + 1;
									}
									else
									{
										*(char*)pointer = (char)chBuffer;
										pointer = (char*)pointer + 1;
									}
								}
							}
							else
							{
								__scan_dec(pBuffer, chBuffer, nCharCount);
								break;
							}
						}
						
						//      
						if(isAssign)
						{
							nAssigned++;

							//   
							if(chFormat != 'c')
								// null-terminate strings
								if(isWideChar)
									*(wchar_t*)pointer = L'\0';
								else
									*(char*)pointer = '\0';
						}

						//     
						if(isString)
						{
							pBuffer = tmp;
							_EcxSkipString(&pBuffer, isCat);
							chBuffer = *pBuffer;
						}
						break;
					}
					

					//---------------------------------------------------------------------------------
					//  
					//---------------------------------------------------------------------------------
					case 'i': // bin, oct, dec, hex -  
					case 'b': // binary
					case 'o': // octal
					case 'u': // unsigned dec
					case 'd': // dec
					case 'x': // hexadecimal
					{
						//   
						if(chBuffer == '-') isNegative = TRUE;
						if(chFormat == 'u') isNegative = FALSE;
						if(isNegative || chBuffer == '+')
						{
							//    
							__scan_dec_width(isWidth, nWidth, 1);
							if(isWidth && !nWidth) goto LABEL_WIDTH_NUMBER;

							__scan_inc(pBuffer, chBuffer, nCharCount);
						}

						//       
						chIntType = _EcxTestInteger(pBuffer - 1, nNumPrefSuff);
						if(chFormat == 'i') chFormat = chIntType;

						//    (0b,0o,0c,0d,0x)
						if(nNumPrefSuff == 1)
						{
							nNumPrefSuff = 0;

							//    
							__scan_dec_width(isWidth, nWidth, 2);
							if(isWidth && !nWidth) goto LABEL_WIDTH_NUMBER;

							__scan_inc(pBuffer, chBuffer, nCharCount);
							__scan_inc(pBuffer, chBuffer, nCharCount);
						}

						//   
						while(1)
						{
							//   (hex)
							if(chFormat == 'x')
							{
								//     
								if(!is_char(chBuffer, CCM_HEX)) break;

								qNumber <<= 4;
								chBuffer = _EcxConvertHex(chBuffer);
							}
							else
							{
								//     
								if(!is_char(chBuffer, CCM_DIGIT)) break;

								//   (dec)
								if(chFormat == 'd')
									qNumber = mul_10(qNumber);
								
								//   (bin)
								if(chFormat == 'b')
								{
									//     
									if(!is_char(chBuffer, CCM_BIN)) break;
									qNumber <<= 1;
								}

								//   (oct)
								if(chFormat == 'o')
								{
									//     
									if(!is_char(chBuffer, CCM_OCT)) break;
									qNumber <<= 3;
								}
							}
							
							isGoodNumber = TRUE;
							qNumber += chBuffer - '0';

							//    
							__scan_dec_width(isWidth, nWidth, 1);
							if(isWidth && !nWidth) goto LABEL_WIDTH_NUMBER;
							
							__scan_inc(pBuffer, chBuffer, nCharCount);
						}

						//    ,  
						if(nNumPrefSuff == 0)
							__scan_dec(pBuffer, chBuffer, nCharCount);

//   
LABEL_WIDTH_NUMBER:
						//   
						if(isNegative)
							qNumber = (unsigned __int64 )(-(__int64)qNumber);

						//    
						if(isGoodNumber)
						{
							//      
							if(isAssign)
							{
								//    
								nAssigned++;
//   
LABEL_ASSIGN_NUMBER:
								//  64-  
								if(isInteger64)
									*(__int64*)pointer = (unsigned __int64)qNumber;
								else
								{
									if(nLongone)
										*(long*)pointer = (unsigned long)qNumber;
									else
										*(short*)pointer = (unsigned short)qNumber;
								}
							}
						}
						else
							goto ERROR_RETURN;
						break;
					}


					//---------------------------------------------------------------------------------
					//   
					//---------------------------------------------------------------------------------
					case 'n':
					{
						qNumber = nCharCount;

						//      
						if(isAssign) goto LABEL_ASSIGN_NUMBER;
						break;
					}
					

					//---------------------------------------------------------------------------------
					//     ()
					//---------------------------------------------------------------------------------
					case 'f': // float
					case 'e': // double
					{
						tmp = szBuffer;
						
						//  
						if(chBuffer == '-')
						{
							isNegative = TRUE;
							*tmp++ = '-';
						}

						if(isNegative || chBuffer == '+')
						{
							//    
							__scan_dec_width(isWidth, nWidth, 1);
							if(isWidth && !nWidth) goto LABEL_WIDTH_FLOAT;

							__scan_inc(pBuffer, chBuffer, nCharCount);
						}

						//        
						if(!isWidth || nWidth > 349)
						{
							nWidth = 349;
						}

						//   
						while(is_char(chBuffer, CCM_DIGIT) && nWidth--)
						{
							isGoodNumber = TRUE;
							*tmp++ = (char)chBuffer;
							__scan_inc(pBuffer, chBuffer, nCharCount);
						}
						
						//    
						if((char)chBuffer == chDecimalPoint && nWidth--)
						{
							*tmp++ = chDecimalPoint;
							__scan_inc(pBuffer, chBuffer, nCharCount);

							//   
							while(is_char(chBuffer, CCM_DIGIT) && nWidth--)
							{
								isGoodNumber = TRUE;
								*tmp++ = (char)chBuffer;
								__scan_inc(pBuffer, chBuffer, nCharCount);
							}
						}
							
						//       
						if(isGoodNumber && (chBuffer == 'e' || chBuffer == 'E') && nWidth--)
						{
							*tmp++ = 'e';
							__scan_inc(pBuffer, chBuffer, nCharCount);

							//  
							if(chBuffer == '-')
							{
								isNegative = TRUE;
								*tmp++ = '-';
							}

							if(isNegative || chBuffer == '+')
							{
								//    
								__scan_dec_width(isWidth, nWidth, 1);
								if(isWidth && !nWidth) goto LABEL_WIDTH_FLOAT;

								__scan_inc(pBuffer, chBuffer, nCharCount);
							}

							//  
							while(is_char(chBuffer, CCM_DIGIT) && nWidth--)
							{
								isGoodNumber = TRUE;
								*tmp++ = (char)chBuffer;
								__scan_inc(pBuffer, chBuffer, nCharCount);
							}
						}

						__scan_dec(pBuffer, chBuffer, nCharCount);
						
//   
LABEL_WIDTH_FLOAT:
						//    
						if(isGoodNumber)
						{
							//      
							if(isAssign)
							{
								nAssigned++;
								*tmp = '\0';

								double d = atof(szBuffer);

								//   
								if(isPrecision)
								{
									//    nPrecision
									d = _round(d, nPrecision);
								}

								if(nLongone == 1 && chFormat == 'f')
								{
									*(float*)pointer = (float)d;
								}
								else
								{
									*(double*)pointer = d;
								}
							}
						}
						else
							goto ERROR_RETURN;
						break;
					}
					

					//---------------------------------------------------------------------------------
					//   
					//---------------------------------------------------------------------------------
					default:
					{
						// either found '%' or something else
						if((int)*format != (int)chBuffer)
						{
							__scan_dec(pBuffer, chBuffer, nCharCount);
							goto ERROR_RETURN;
						}
						else
							nMatch--; // % found, compensate for inc below

						//      
						if(isAssign)
							arglist = arglistsave;
					}
				}
					
				nMatch++; // matched a format field - set flag
			}
			else
			{
				// zero-nWidth field in format string
				__scan_dec(pBuffer, chBuffer, nCharCount);
				goto ERROR_RETURN;
			}
		}

		//---------------------------------------------------------------------------------
		//     (*format != '%')
		//---------------------------------------------------------------------------------
		else
		{
			__scan_inc(pBuffer, chBuffer, nCharCount);
			if((int)chFormat == (int)chBuffer) continue;
			goto ERROR_RETURN;
		}

		format++;
		if(is_char(chBuffer, CCM_EOT) && ((*format != '%') || (*(format + 1) != 'n'))) break;
	
	}  // while(*format)

ERROR_RETURN:

	delete [] szFormat;
	delete [] szBuffer;

	if(is_char(chBuffer, CCM_EOT))
		//If any fields were matched or assigned, return nAssigned
		return ( (nAssigned || nMatch) ? nAssigned : -1 );
	else
		return nAssigned;
}


/////////////////////////////////////////////////////////////////////////////
//     lpszBuffer ( sscanf)
/////////////////////////////////////////////////////////////////////////////
inline int _EcxScan(LPCSTR lpszBuffer, LPCSTR lpszFormat, ...)
{
	va_list arglist;
	va_start(arglist, lpszFormat);
	int nAssigned = _EcxScanV(lpszBuffer, lpszFormat, arglist);
	va_end(arglist);
	return nAssigned;
}



/////////////////////////////////////////////////////////////////////////////
//     pBuf
/////////////////////////////////////////////////////////////////////////////
inline int _EcxTxtGetLinesCount(BYTE* pBuf)
{
	int br, count = 0;
	BYTE* tmp = pBuf;

	//   VLL-
	while(*tmp)
	{
		//    
		if((br = is_eol_eot(tmp)))
		{
			count++;
			tmp += br;
			continue;
		}
		tmp++;
	}
	
	return (++count);
}


/////////////////////////////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////////////////////////////
inline BOOL _EcxIsValidIdentifier(LPCSTR szBuffer, LPCSTR szSet1 = NULL, LPCSTR szSet2 = NULL, int nMaxCount = -1)
{
	char* tmp = (char*)szBuffer;
	int i = 0;

	while(*tmp)
	{
		if(nMaxCount > 0 && nMaxCount == i) break;

		if(i > 0)
		{
			if(!is_char(*tmp, CCM_WORD) && !_EcxIsCharInSet(*tmp, szSet2)) return FALSE;
		}
		else
		{
			if(!is_char(*tmp, CCM_ALPHA) && *tmp != '_' && !_EcxIsCharInSet(*tmp, szSet1)) return FALSE;
		}
		tmp++;
	}
	return TRUE;
}













////////////////////////////////////////////////////////////////////////////
#endif // __TEXTPARSINGFUNC_H__
